c
c  Subroutine to compute weights associated with Delauney triangle raingauges.
c
c  This version finds the triangle itself so does not need a triangle index grid.
c
	subroutine rainweight(nb,basingrid,narr,warr,ngmax,ntmax,
     1        nrows,ncols,trindexgrid,dotg,bndbox,cellsize,annraingrid,
     2        sites,x,y,trilist,agbndbox,agcsize,agnx,agny)
      !DEC$ ATTRIBUTES DLLEXPORT::rainweight
c arguments
	integer trilist(ntmax,3),nb,ngmax,igy,igx,nrows,ncols,agnx,agny
	integer*2 basingrid(nrows,ncols),
     1   trindexgrid(nrows,ncols)
	integer narr(nb),dotg
	character*80 trifilename 
	real*4 warr(nb,ngmax),annraingrid(agny,agnx)
	real*8 bndbox(4),cellsize,agbndbox(4),agcsize,x(ngmax),y(ngmax)
	integer*4 sites(ngmax)
c inputs
c		nb				= number of basins
c		basingrid(i,j)	= sub-basin number at cell i,j
c		ngmax			= maximum possible number of raingauges
c		igy,igx			= maximum no of rows and columns in a grid
c		nrows, ncols	= actual number of rows and columns in this grid
c		bndbox(1:4)		= xmin,ymin,xmax,ymax 
c		cellsize		= width of a cell (in metres)
c		annraingrid(i,j) = mean annual rain at cell i,j
c					(data generated by Matlab m-file annraingrid.m)
c		trifilename = name of file containing info about triangles
c					(data generated by Matlab m-file trindex.m)
c outputs
c		narr(1:nb)		= number of cells in basin isub
c		warr(1:nb,1:ngmax)	= weight to apply for gauge ig in basin isub
c		trindexgrid		= index of triangle to use at cell i,j
c		sites(1:ngmax) = site numbers of the raingauges
c locals
	real*8 w(3)
	real*8 del,x1,x2,x3,y1,y2,y3,xi,yi
	integer i,j,ig,jg,lun,ind,isub,k,ngauge,ntri
	integer kg(3)
	
	logical found
	ngauge=ngmax
	ntri=ntmax
c
c read in the list of triangles for raingauge weighting
c the trindexgrid array points to triangles defined by lines in file
cdgt	lun=9
cdgt      close(lun)
c	write(6,*)trifilename
cdgt	open(unit=lun,file=trifilename)
cdgt	read(lun,*)ngauge
cdgt	 if(ngauge .gt. ngmax)write(6,*)"Error ngauge arrays too small"
cdgt	read(lun,*) (sites(k),k=1,ngauge)  !RPI & RAW introduced reading site numbers 25/8/00
cdgt	read(lun,*)(x(k),k=1,ngauge)   !raingauge x co-ords
cdgt	read(lun,*)(y(k),k=1,ngauge)   !raingauge y co-ords
cdgt	read(lun,*) ntri
cdgt	  if(ntri .gt. ntmax)write(6,*)"Error ntri arrays too small"
cdgt	if(ntri .gt. 0)read(lun,*)((trilist(i,j),j=1,3),i=1,ntri)
cdgt	close(lun)
cdgt      do 232 i = 1,ntri
cdgt	write(6,*)(trilist(i,j),j=1,3)
cdgt 232  continue
c
c we must set warr and narr to 0 before we start 
	do 5 isub=1,nb				       ! nb is number of sub-basins
	  narr(isub) = 0				   ! narr is number of cells in sub-basin 
	  do 5 ig=1,ngauge				   ! warr is weight for ig'th gauge
    5	    warr(isub,ig) = 0.			   ! to get rain in basin isub 
c now loop over all grid cells collecting weight information
      notfound = 0   !  Count how many cells do not fall within triangles
	do 30 i = 1, nrows
	  yi=bndbox(4)-(i-1+.5)*cellsize   ! y co-ord of cell center
	  do 20 j = 1, ncols
          if(dotg .eq. 1)trindexgrid(i,j) = -9999  ! initialize triangle grid
	    isub = basingrid(i,j)		   ! which basin is this cell in
c	if(isub .eq. 148)then
c	isub=isub
c	endif
          if ( isub .gt. 0 ) then
	      xi=bndbox(1)+(j-1+.5)*cellsize ! x co-ord of cell center
            found = .false. 
		  lastind=-1
            do 25 ind = 1,ntri   ! search through triangles for one that has all weights positive
	        do 10 k = 1, 3			   ! kg has indices of the 3 
   10	          kg(k)=trilist(ind,k)	   ! gauges to use for this cell 
c
              x1=x(kg(1))				   ! x1,y1,x2,y2,x3,y3 are
	        y1=y(kg(1))				   ! co-ords of the 3 gauges
              x2=x(kg(2))				   ! which influence this cell
	        y2=y(kg(2))
              x3=x(kg(3))				   ! w1,w2,w3 are weights for
	        y3=y(kg(3))				   ! those gauges
c these formulae adapted from the 'linear' subfunction in matlab 5.1's
c griddata.m function.  w(1,2,3) are weights for gauges 1,2,3
              del  =  (x2-x1) * (y3-y1) - (x3-x1) * (y2-y1)
	        w(3) = ((x1-xi) * (y2-yi) - (x2-xi) * (y1-yi)) / del
	        w(2) = ((x3-xi) * (y1-yi) - (x1-xi) * (y3-yi)) / del
	        w(1) = ((x2-xi) * (y3-yi) - (x3-xi) * (y2-yi)) / del
	        if(w(1) .ge. 0. .and. w(2) .ge. 0. .and. w(3) .ge. 0.)then  ! found triangle
c we expect to find each pixel in exactly one triangle
c if a pixel is right on a vertex or right on a line, it will seem to belong to several triangles
c that is ok, and we'll just use the first triangle we find
c when this happens we get one or two of the w's=0
			    if(found) then 
c don't want to do any work if we find a second triangle
c however we get upset if we find two triangles, and are not at a vertex or on a line
     				  if (w(1)*w(2)*w(3).gt.0) 
     1				  write(6,*)"Error in triangle coords",
     2										" - Multiple triangles"
				else
					found = .true.
					lastind=ind
					if(dotg .eq. 1)trindexgrid(i,j) = ind  ! record which triangle the cell is in
					iag = 1 + int( (agbndbox(4)-yi) / agcsize ) ! row/col in annual grid
					jag = 1 + int( (xi-agbndbox(1)) / agcsize )
					do 15 k = 1, 3
					   ig = 1 + int( (agbndbox(4)-y(kg(k))) / agcsize ) !row/col of gauge 
					   jg = 1 + int( (x(kg(k))-agbndbox(1)) / agcsize )
c each weight depends on the proximity of gauges (the w weights)
c and the ratio of the cell mean annual rain to raingauge mean annual rain
c	              warr(isub,kg(k)) = warr(isub,kg(k)) +
c     1	          w(k)*annraingrid(iag,jag)/annraingrid(ig,jg)
					   warr(isub,kg(k)) = warr(isub,kg(k)) +
     1				   w(k)*ratio(annraingrid,iag,jag,ig,jg,agny,agnx)
c   Use ratio function to implement the above with error checking
   15					continue					   ! in this basin
				endif
              endif
   25       continue
            if(.not. found)then
c   Use the nearest gauge with weight 1
	          iag = 1 + int( (agbndbox(4)-yi) / agcsize ) ! row/col in annual grid
		      jag = 1 + int( (xi-agbndbox(1)) / agcsize )
	          nn=nrest(xi,yi,x,y,ngauge)
	          ig = 1 + int( (agbndbox(4)-y(nn)) / agcsize ) !row/col of gauge 
	          jg = 1 + int( (x(nn)-agbndbox(1)) / agcsize )
	          warr(isub,nn)=warr(isub,nn)
     1            +ratio(annraingrid,iag,jag,ig,jg,agny,agnx)
c RAW found that last two arguments were in the wrong order, and fixed it 11/10/99
c     1            +ratio(annraingrid,iag,jag,ig,jg,agnx,agny)
		      notfound = notfound+1
                if(dotg .eq. 1)trindexgrid(i,j) = -nn  ! use - gauge number
c                     to record this in triangle grid
	      endif
		  narr(isub) = narr(isub) + 1  ! total up the # of cells
	    endif
   20	  continue
   30	continue
c      if(notfound .gt. 0)write(6,100)Notfound
c  100       format(1x,'Warning:',i8,
c     1        ' grid cells not within specfied triangles')
c all data collection is done: now we normalise by sub-basin area
	do 50 isub=1,nb
	  do 50 ig=1,ngauge
	    if ( narr(isub) .gt. 0 ) then
		  warr(isub,ig) = warr(isub,ig)/narr(isub)
          else
c	      write(*,*)'rainweight error: No cells for sub-basin ',isub
	    endif
   50	continue
	return
	end
c
      function ratio(annraingrid,iag,jag,ig,jg,agny,agnx)
	integer agny, agnx
	real *4 annraingrid(agny,agnx)
	if((iag .lt. 1) .or. (iag .gt. agny) 
     1   .or. (jag .lt. 1) .or. (jag .gt. agnx))then
c	    write(6,*)"Basin grid outside annual grid array"
c	    write(6,*)"Weight will not be scaled by annrain grid"
		ratio = 1.
	    return
	endif
      if((ig .lt. 1) .or. (ig .gt. agny) 
     1  .or. (jg .lt. 1) .or. (jg .gt. agnx))then
c	    write(6,*)"Raingauge outside annual grid array"
c	    write(6,*)"Weight will not be scaled by annrain grid"
	    ratio = 1.
	    return
	endif
	if(annraingrid(ig,jg) .le. 0. .or. 
     +   annraingrid(iag,jag) .le. 0.)then
c	    write(6,*)"Annual rainfall grid is zero at a gauge "
c	  	write(6,*)"Weight will not be scaled by annrain grid"
		ratio = 1.
		return
	endif 
	ratio = real(annraingrid(iag,jag))/real(annraingrid(ig,jg))
	return
	end

c   function to return nearest guage to location xi, yi	     
	function nrest(xi,yi,x,y,ngauge)
	real *8 x(ngauge),y(ngauge),xi,yi,dist,distn
	distn = sqrt((xi-x(1))**2 + (yi - y(1))**2)
	nrest = 1
	do 2 j = 2,ngauge
	  dist = sqrt((xi-x(j))**2 + (yi - y(j))**2)
	  if(dist .lt. distn)then
	    distn = dist
		nrest = j
	  endif
    2 continue
      return
	end 