
c  Copyright (C) 2000  David Tarboton, Utah State University

c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License 
c version 2, 1991 as published by the Free Software Foundation.

c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c GNU General Public License for more details.

c A copy of the full GNU General Public License is included in file 
c gpl.html. This is also available at:
c http://www.gnu.org/copyleft/gpl.html
c or from:
c The Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
c Boston, MA  02111-1307, USA.

c If you wish to use or incorporate this program (or parts of it) into 
c other software that does not meet the GNU General Public License 
c conditions contact the author to request permission.
c David G. Tarboton                                                      
c Utah State University 
c 8200 Old Main Hill 
c Logan, UT 84322-8200 
c USA 
c http://www.engineering.usu.edu/dtarb/ 
c email:  dtarb@cc.usu.edu 
C
C   Subroutine to extract channel network from DEM area and pointer files
C
C       CREATED BY DAVID G TARBOTON
C       Utah State University
C
C    Based on the support area threshold this program identifies each link in
C    the channel network and sets up the tree file defining its topological
C    structure. The (I,J) row and column numbers of each pixel along each link
C    are saved for later reading by NETPROP.FOR. 
C 
	

       subroutine NETEX(dir,area,treefile,coordfile,ordfile,nx,ny,
     +   itresh,icr,icend,dx,dy,bndbox,csize,iftype,err,inodes,jnodes,
     +   nnodes,idnodes)
	!DEC$ ATTRIBUTES DLLEXPORT::NETEX
c	PARAMETER(MNL=200000)
c       PARAMETER(MNL=300000) 
c      PARAMETER(IGX=100,IGY=100,MNL=5000) 
      CHARACTER*4096 TREEFILE,COORDFILE, ordfile, pointfile,areafile
C---with MNL = 50000 the number of links can exceed 32767 the max two
C    bit integer value so link pointers have to be 4 bit integers
C     (NEXTL,PREVL1,PREVL2,IPOINT)
      INTEGER AREA(ny,nx),IPOINT(7)
	integer, allocatable :: NEXTL(:),PREVL1(:),PREVL2(:)
      INTEGER*2 DIR(ny,nx),D1(8),D2(8),err,IORDUP(7)
	integer*2, allocatable :: IST(:),JST(:),IORD(:),
     &ISTART(:),JSTART(:),IEND(:),JEND(:),MAG(:)
      integer*2 inodes(nnodes),jnodes(nnodes)
	integer idnodes(nnodes)
	parameter(ishrdt = 1, iintdt = 2, ifltdt=3)
	double precision BNDBOX(4),csize, dx, dy   ! DGT 5/5/09  Made dx and dy double
      LOGICAL strt
      DATA D1/0,-1,-1,-1,0,1,1,1/
      DATA D2/1,1,0,-1,-1,-1,0,1/
      
C
C    READ INPUT
C
      igy=ny
	igx=nx
	err=0
C
C       MEANING OF POINTERS IS -------------
C                              I 4 I 3 I 2 I
C        0 = POINTS TO SELF    -------------
C            I.E. UNRESOLVED   I 5 I 0 I 1 I
C        -1= BOUNDARY PIXEL    -------------
C                              I 6 I 7 I 8 I
C                              -------------                      
C
C----FIRST FIND ALL START PIXELS
C
      N=0 
c	open(8,file="d:\tt.dat")          
c     count start grid cells
      nmax=(mnl+1)/2
      DO 2 I=2,NY-1
        DO 2 J=2,NX-1                                             
          IF(strt(I,J,AREA,DIR,NX,NY,IGX,IGY,ITRESH))THEN
            N=N+1
c            if(n.le.nmax)then
c            IST(N)=I
c            JST(N)=J
c            endif
          ENDIF       
 2    CONTINUE   
      nmax=n
      mnl=2*n+1+nnodes   ! 11/17/04  DGT added nnodes to avoid memory overflow
c       when added nodes increase the number of links
	allocate(NEXTL(mnl),PREVL1(mnl),PREVL2(mnl),stat=ierr) 
	if(ierr .ne. 0)go to 99
	allocate(ist(nmax),jst(nmax),iord(mnl),stat=ierr)
	if(ierr .ne. 0)go to 99
	allocate(istart(mnl),jstart(mnl),iend(mnl),jend(mnl),stat=ierr)
	if(ierr .ne. 0)go to 99
	allocate(mag(mnl),stat=ierr)
	if(ierr .ne. 0)go to 99
	n=0
      DO 22 I=2,NY-1
        DO 22 J=2,NX-1                                             
          IF(strt(I,J,AREA,DIR,NX,NY,IGX,IGY,ITRESH))THEN
            N=N+1
            if(n.le.nmax)then
            IST(N)=I
            JST(N)=J
            endif
          ENDIF       
 22    CONTINUE   

c      WRITE(8,*)'MAGNITUDE',N
      if(n.gt.nmax) then
		err=2 !stop 'too big'
	    RETURN
	endif                   
C
C---ZERO AREA ARRAY
C
      DO 12 I=1,NY
       DO 12 J=1,NX
 12      AREA(I,J)=0
C
C----TRACE STREAMS DOWNWARDS ADDING 1 TO MAGNITUDE OF EACH PIXEL
C      (MAGNITUDE STORED IN AREA ARRAY)
      DO 3 IS=1,N
        I=IST(IS)
        J=JST(IS)
c	write(8,*)i,j
c 4	  AREA(I,J)=AREA(I,J)+1  ! 8/13/04  moved outside loop to catch edge situation
  4      IF(DIR(I,J).GT.0)THEN
          AREA(I,J)=AREA(I,J)+1
          INEXT=I+D1(DIR(I,J))
          JNEXT=J+D2(DIR(I,J))
          I=INEXT
          J=JNEXT
c	x=dx*j+bndbox(1)-dx/2.
c	y=bndbox(4)-dy*i+dy/2.
c	write(8,*)i,j,x,y
          GO TO 4
        ENDIF
 3    CONTINUE
C
C----IDENTIFY LINKS BY DIFFERENT MAGNITUDES
C
      ILINK=1
      DO 5 IS=1,N
        ISTART(ILINK)=IST(IS)
        JSTART(ILINK)=JST(IS)
C---INITIALISE POINTERS
        PREVL1(ILINK)=0
        PREVL2(ILINK)=0      
        I=IST(IS)
        J=JST(IS)
        MAG(ILINK)=AREA(I,J)
        IORD(ILINK)=1
 6      INEXT=I+D1(ABS(DIR(I,J)))
        JNEXT=J+D2(ABS(DIR(I,J)))
C----CHECK FOR END
        IF(DIR(INEXT,JNEXT).EQ.0)then     ! END OF PATH
			iend(ilink)=i
			jend(ilink)=j
			nextl(ilink)= -1
		    if(is .lt. n)ilink=ilink+1  ! if not last on list increment link counter
	  else
        MNEXT=AREA(INEXT,JNEXT)
        I=INEXT
        J=JNEXT
        IEND(ILINK)=I
        JEND(ILINK)=J
c   mods allow insertion of nodes   DGT 7/17/02
	  if(isnode(mnext,mag(ilink),i,j,inodes,jnodes,nnodes))then	
c  Check here that this is not the end of a path because then it will be a node anyway
            intemp=i+d1(abs(dir(i,j)))
	      jntemp=j+d2(abs(dir(i,j)))
	      if(.not.(dir(intemp,jntemp).eq.0))then
			ilink=ilink+1
			istart(ilink)=i
			jstart(ilink)=j
			prevl1(ilink)=ilink-1
			prevl2(ilink)=0
			nextl(ilink-1)=ilink
			mag(ilink)=mag(ilink-1)
			iord(ilink)=iord(ilink-1)
			iend(ilink)=i
			jend(ilink)=j
		  endif
	  endif
c  end mods to allow insertion of nodes  DGT
        IF(MNEXT.EQ.MAG(ILINK))GO TO 6
C---CONTINUE HERE FOR NEW LINK
C----CHECK IF JUNCTION ALREADY REACHED (FLAGGED BY NEGATIVE DIRECTION)
          IF(DIR(I,J).LT.0)THEN
C----CHECK IF ALL LINKS CONVERGING HERE HAVE BEEN DONE BY SUMMING
C           MAGNITUDE
            MSUM=0
            IORDM=0
            ICONV=0
            DO 7 IL=1,ILINK
              IF(IEND(IL).EQ.I.AND.JEND(IL).EQ.J)THEN
                ICONV=ICONV+1      ! COUNTER OF NUMBER OF LINKS THAT CONVERGE
                IPOINT(ICONV)=IL
                IORDUP(ICONV)=IORD(IL)
                MSUM=MSUM+MAG(IL)                  
              ENDIF
  7         CONTINUE
            IF(MSUM.EQ.MNEXT)THEN     ! YES ALL LINKS HAVE BEEN PROCESSED
C---SORT IORDUP,IPOINT INTO DECENDING STREAM ORDER
              DO 11 IC=1,ICONV-1
                DO 11 IIC=IC+1,ICONV
                  IF(IORDUP(IIC).GT.IORDUP(IC))THEN   ! SWITCH THESE
                    ITEMP=IORDUP(IIC)
                    IORDUP(IIC)=IORDUP(IC)
                    IORDUP(IC)=ITEMP
                    ITEMP=IPOINT(IIC)
                    IPOINT(IIC)=IPOINT(IC)
                    IPOINT(IC)=ITEMP
                  ENDIF
 11           CONTINUE
              DO 17 IC=1,ICONV-1
                ILINK=ILINK+1
                ISTART(ILINK)=I
                JSTART(ILINK)=J  
                PREVL1(ILINK)=IPOINT(IC)
                PREVL2(ILINK)=IPOINT(IC+1)
                NEXTL(IPOINT(IC))=ILINK
                NEXTL(IPOINT(IC+1))=ILINK
                MAG(ILINK)=MAG(PREVL1(ILINK))+MAG(PREVL2(ILINK))
                IORD(ILINK)=MAX(IORDUP(1),IORDUP(2)+1)
                IPOINT(IC+1)=ILINK
                IEND(ILINK)=I
                JEND(ILINK)=J
 17           CONTINUE
              GO TO 6       ! CONTINUE TRACING DOWN
            ELSE
              ILINK=ILINK+1
            ENDIF
          ELSE
            DIR(I,J)=-DIR(I,J)
            ILINK=ILINK+1
          ENDIF
	  endif     !  END OF PATH ENDIF
 5    CONTINUE   
c	err = 1 
c	RETURN
cc      WRITE(6,*)'LOGIC ERROR LINK LOOP ENDED'
c      STOP
C
C----WRITE TREE FILE
C     
 90   OPEN(UNIT=10,FILE=COORDFILE,STATUS='unknown')
      OPEN(UNIT=12,FILE=TREEFILE,STATUS='unknown') 
c
c   reinitialize area array - for output it will contain order
c	
      do 73 i=1,ny
	  do 73 j = 1,nx
 73       area(i,j)=0
      ICORD=0
C---WRITE ROOT LINK FIRST
      DO 10 IL=ILINK,ILINK
        I=ISTART(IL)
        J=JSTART(IL)
        ICS=ICORD
 20     WRITE(10,*)I,J
        area(i,j)=max(iord(il),area(i,j))
        ICEND=ICORD
        ICORD=ICORD+1
        IF(.NOT.(I.EQ.IEND(IL).AND.J.EQ.JEND(IL)))THEN
          INEXT=I+D1(ABS(DIR(I,J)))
          JNEXT=J+D2(ABS(DIR(I,J)))
          I=INEXT
          J=JNEXT
          GO TO 20
        ENDIF
	  inodeid=0   !  This is the first one so it will be the most downstream
	  if(isnode2(iend(il),jend(il),inodes,jnodes,
     +    nnodes,nodeno))then
          if(idnodes(nodeno) .ge. 0) then 
		   inodeid=idnodes(nodeno)
	       idnodes(nodeno)=-1   ! This logic to pick only the first one if there are multiple at a junction
	    endif
	  endif
        WRITE(12,122)0,ICS,ICEND,-1,PREVL1(IL)
     &  ,PREVL2(IL),IORD(IL),inodeid
 10   CONTINUE         
c      WRITE(6,*)'ROOT AT COORD. NUMBER ',ICORD
	icr=icord
c      WRITE(11,*)ICORD
C---WRITE REMAINDER OF LINKS
      DO 110 IL=1,ILINK-1
        I=ISTART(IL)
        J=JSTART(IL)
        ICS=ICORD	
 120     WRITE(10,*)I,J
         area(i,j)=max(iord(il),area(i,j))
        ICEND=ICORD
        ICORD=ICORD+1
        IF(.NOT.(I.EQ.IEND(IL).AND.J.EQ.JEND(IL)))THEN
          INEXT=I+D1(ABS(DIR(I,J)))
          JNEXT=J+D2(ABS(DIR(I,J)))
          I=INEXT
          J=JNEXT
          GO TO 120
        ENDIF
        IF(NEXTL(IL).EQ.ILINK)NEXTL(IL)=0
	  inodeid=-1
	  if(nextl(il) .lt. 0)inodeid=0   ! this is a most downstream link so flagged by inodeid=0
	  if(isnode2(iend(il),jend(il),inodes,jnodes,
     +    nnodes,nodeno))then
          if(idnodes(nodeno) .ge. 0) then 
		   inodeid=idnodes(nodeno)
	       idnodes(nodeno)=-1   ! This logic to pick only the first one if there are multiple at a junction
	    endif
	  endif
        WRITE(12,122)IL,ICS,ICEND,NEXTL(IL),PREVL1(IL)
     &  ,PREVL2(IL),IORD(IL),inodeid
 122    FORMAT(8(1x,I9))
 110   CONTINUE
c       write(11,*)icend
      close(10)
	close(12)
99    if(ierr .ne. 0)	err=33   ! fortran allocate error
	deallocate(nextl,prevl1,prevl2,stat=ierr) 
	deallocate(ist,jst,iord,stat=ierr)
	deallocate(istart,jstart,iend,jend,stat=ierr)
	deallocate(mag,stat=ierr)
cc	call fgridwrite(ordfile,area,2,nx,ny,dx,dy,bndbox,csize,-1.,
cc     &iftype,igy)
	return
      END  
c
c   Function to indicate whether pixel being considered is a node being inserted into the network 
c   Need to make sure it is not a junction already because junctions have to wait for all paths 
c   to them to be processed	
	logical function isnode(mnext,mag,i,j,inodes,jnodes,nnodes)
	integer*2 inodes(nnodes),jnodes(nnodes),mag
	isnode=.false.
	do k = 1,nnodes
	if((inodes(k)+1) .eq. i .and. (jnodes(k)+1) .eq. j)then  !found one  
c                +1 is because arrays came from C
c         nodeno=k    ! for return to use in indexing
	   isnode=.true.
	    IF(.not. (MNEXT.eq.MAG))isnode=.false.  ! false alarm it is a junction
		return         
	endif
	enddo
c   a drop through return here should be false
	return
	end 
	
c
c   Function to indicate whether pixel being considered is a node being inserted into the network 
c   This version is used for output and returns true for junctions too	
	logical function isnode2(i,j,inodes,jnodes,nnodes,nodeno)
	integer*2 inodes(nnodes),jnodes(nnodes),i,j
	isnode2=.false.
	do k = 1,nnodes
	if((inodes(k)+1) .eq. i .and. (jnodes(k)+1) .eq. j)then  !found one  
c                +1 is because arrays came from C
         nodeno=k    ! for return to use in indexing
	   isnode2=.true.
		return         
	endif
	enddo
c   a drop through return here should be false
	return
	end                       
                      
C
C     THIS FUNCTION RETURNS TRUE IF THE PIXEL IS A STREAM SOURCE
C      ACCORDING TO THE THRESHOLD
C          
      LOGICAL FUNCTION strt(I,J,AREA,DIR,NX,NY,IGX,IGY,ITRESH)
      INTEGER*2 DIR(IGY,IGX)
      INTEGER AREA(IGY,IGX),D1(8),D2(8)
      DATA D1/0,-1,-1,-1,0,1,1,1/
      DATA D2/1,1,0,-1,-1,-1,0,1/
      strt=.TRUE.   
      IF(AREA(I,J).LT.ITRESH .or. dir(i,j) .lt. 0)THEN
        strt=.FALSE.
        IF(AREA(I,J).LE.0)DIR(I,J)=0   ! ZERO DIRECTIONS OUTSIDE AREA
	  return
      ELSE    ! CHECK UPSTREAM PIXELS 
        DO 2 K=1,8
          IN=I+D1(K)    ! NEIGHBOUR PIXEL
          JN=J+D2(K)       
          IF(DIR(IN,JN).GT.0)THEN
          IND=IN+D1(DIR(IN,JN))   ! PIXEL DOWNSTREAM FROM NEIGHBOUR
          JND=JN+D2(DIR(IN,JN))
          IF(IND.EQ.I.AND.JND.EQ.J)THEN   !NEIGHBOUR DRAINS INTO (I,J)
            IF(AREA(IN,JN).GE.ITRESH)strt=.FALSE.
          ENDIF
          ENDIF
 2      CONTINUE
c    Do not allow sources that drain off the raster set i.e. a link of 0 length
        in=i+d1(dir(i,j))
	  jn=j+d2(dir(i,j))
	  if(area(in,jn) .lt. itresh)strt=.false.
      ENDIF
      RETURN
      END     
C                                   
C
        
C   Subroutine to compute channel network properties for each pixel on the channel
C   network
C
C       CREATED BY DAVID G TARBOTON
C
C
C    This subroutine follows after NETEX.FOR and computes the X and Y coordinates
C    of pixels on the channel network from row and column numbers.
C    It also writes elevations, distances to
C    the outlet along the streams and contributing area for each pixel to the
C    coordinate file.
C
C
       subroutine NETPROP(dir,area,elev,coordfile,
     + icr,icmax,dx,dy,nx,ny,bndbox,csize,err)
	!DEC$ ATTRIBUTES DLLEXPORT::NETPROP
c       PARAMETER(MC=2000000)
c      parameter(ishrdt = 1, iintdt = 2, ifltdt=3) 
       CHARACTER*4096 COORDFILE,ELEVFILE,AREAFILE,POINTFILE
      INTEGER AREA(ny,nx),err
      REAL, allocatable :: RAREA(:),LENGTH(:),elv(:)
      real elev(ny,nx)
      INTEGER*2 DIR(ny,nx),D1(8),D2(8)
	integer*2, allocatable :: IA(:),JA(:)
      double precision BNDBOX(4),csize, dx, dy,x,y  ! DGT 5/5/09  Made dx, dy, x,y double
      DATA D1/0,-1,-1,-1,0,1,1,1/
      DATA D2/1,1,0,-1,-1,-1,0,1/

	mc=icmax+1
	allocate(rarea(mc),length(mc),elv(mc),ia(mc),ja(mc),stat=ierr)
	if(ierr .ne. 0)go to 99
C
C    READ INPUT
C
c     READ(11,*)ICR       ! ICR IS COORD OF ROOT
c       read(11,*)icmax
c       if(icmax.gt.mc)then
c		err=1
c		RETURN
c         write(6,*)'Coord file dimension not big enough'
c         stop
c       endif
       OPEN(UNIT=10,FILE=coordfile,STATUS='OLD')
       DO 5 N=1,MC
       READ(10,*,END=110)IA(N),JA(N)
 5     CONTINUE
	 
CC      WRITE(6,*)'TOO MANY COORDS'
 110   CLOSE(10)
       N=N-1              
C
C---COMPUTE AREAS
C
C       CALL demread(AREA,AREAFILE,NX,NY,IGY,dx,dy)
c        call fgridread(AREAFILE,AREA, IINTDT, NX, NY,
c     &	 DX, DY, BNDBOX,CSIZE, XNDV,IFTYPE, IGY)
       DO 6 IC=1,N
 6       RAREA(IC)=FLOAT(AREA(IA(IC),JA(IC)))*dx*dy
C
C---READ ELEVATIONS
C
C      CALL demread(DIR,ELEVFILE,NX,NY,IGY,dx,dy)
c       call fgridread(ELEVFILE,elev, IfltDT, NX, NY,
c     &	 DX, DY, BNDBOX,CSIZE, XNDV,IFTYPE, IGY)

       DO 7 IC=1,N
         ELV(IC)=elev(IA(IC),JA(IC))
 7     CONTINUE 
       IROOT=IA(ICR)
       JROOT=JA(ICR)
c	write(6,*)iroot,jroot
C
C---READ POINTERS AND COMPUTE LENGTHS
C
C       CALL demread(DIR,POINTFILE,NX,NY,IGY,dx,dy)
c	   call fgridread(POINTFILE,DIR, ISHRDT, NX, NY,
c     &	 DX, DY, BNDBOX,CSIZE, XNDV,IFTYPE, IGY)

C
C       MEANING OF POINTERS IS -------------
C                              I 4 I 3 I 2 I
C        0 = POINTS TO SELF    -------------
C            I.E. UNRESOLVED   I 5 I 0 I 1 I
C        -1= BOUNDARY PIXEL    -------------
C                              I 6 I 7 I 8 I
C                              -------------                      
C
C----TRACE STREAMS DOWNWARDS
      DO 3 IC=1,N
        LENGTH(IC)=0.
        I=IA(IC)
        J=JA(IC)
	  INEXT=I+D1(DIR(I,J))
        JNEXT=J+D2(DIR(I,J))
c        IF(.NOT.(I.EQ.IROOT.AND.J.EQ.JROOT))THEN      ! LOOP
        if(dir(inext,jnext) .ne. 0)then   !  not yet end of path
 4      DXx=dx*FLOAT(J-JNEXT)
        DYy=dy*FLOAT(I-INEXT)
        LENGTH(IC)=LENGTH(IC)+SQRT(DXx*DXx+DYy*DYy)
	  i=inext
	  j=jnext
	  inext=i+d1(dir(i,j))
	  jnext=j+d2(dir(i,j))
c	write(6,*)inext,jnext,dir(i,j),dir(inext,jnext)
	  if(dir(inext,jnext) .ne. 0)go to 4
        ENDIF
 3    CONTINUE
C--WRITE OUTPUT
      OPEN(UNIT=10,FILE=COORDFILE,STATUS='UNKNOWN')
      DO 10 IC=1,N
        X=(JA(IC)-1)*dx+bndbox(1)+dx*0.5
        Y=dy*(NY-IA(IC))+bndbox(2)+dy*0.5
        WRITE(10,'(5g)')X,Y,LENGTH(IC),ELV(IC),RAREA(IC)
 10   CONTINUE
	close(10)
99    if(ierr .ne. 0)	err=33   ! fortran allocate error
	deallocate(rarea,length,elv,ia,ja,stat=ierr) 
      return
      END                                      

