/*  DistToStream.cpp. 

  Distance to Stream.  
  
  David G Tarboton
  Utah State University   
  
  3/26/07
   */
  
#include "stdafx.h"
#include "gridio.h"

/*  Globals   */
 //	int iy,jy,ccheck;  // Could make these local variables. (Not necessary to declare here because already a global in tardemlib.cpp)
 //  float aref[10];  // Not necessary to declare here because already a global in tardemlib.cpp
   float dist[9];
// int dd1[8],dd2[8];	// Not necessary to declare here because already a global in tardemlib.cpp			
/* Offset pointers d1dts and d2 */
int *d1dts = dd1-1;
int *d2dts = dd2-1;
//int nx,ny;  // Not necessary to declare here because already a global in tardemlib.cpp
float idv;
int gweight;

float prop(float,int);
void adistrec(int,int);
void ldistrec(int,int);
void sdistrec(int,int);

/*  Global grids  */
fgrid ang;
fgrid wgrd;
fgrid dts;
sgrid src;

int disttostreamgrd(char *angfile, char *wfile, char *srcfile, char *dtsfile, int method, int useweight, int concheck)
{
   int err,i,j,filetype;
 
   /* read angles */
	err=gridread(angfile,&ang,&filetype);
    if(err != 0)goto ERROR1; 
	nx=ang.head.nx;
	ny=ang.head.ny;

 	/* read Stream Raster Grid */
	err=gridread(srcfile,&src,&filetype);
    if(err != 0)goto ERROR2;  
	//Check that grid dimensions are the same
	if(nx != src.head.nx || ny != src.head.ny)
	{
		printf("Grid dimensions are inconsistent\n");
		printf("%d %d %s\n",nx,ny,angfile);
		printf("%d %d %s\n",src.head.nx,src.head.ny,srcfile);
		goto ERROR2;
	}
  /* read weights if necessary */
	if(useweight == 1)
	{
	err=gridread(wfile,&wgrd,&filetype);
    if(err != 0)goto ERROR3;  
	//Check that grid dimensions are the same
	if(nx != wgrd.head.nx || ny != wgrd.head.ny)
	{
		printf("Grid dimensions are inconsistent\n");
		printf("%d %d %s\n",nx,ny,angfile);
		printf("%d %d %s\n",wgrd.head.nx,wgrd.head.ny,wfile);
		goto ERROR3;
	}
	}
	gweight=useweight;   //  Assign to global for recursion
	ccheck=concheck;   // Another global for recursion

/*  Allocate memory  */
	err=allocategrid(&dts,ang.head,-2.);
    if(err != 0)goto ERROR3;  

/* Define directions */

d1dts[1]=0; d1dts[2]= -1; d1dts[3]= -1; d1dts[4]= -1; d1dts[5]=0; d1dts[6]=1; d1dts[7]=1; d1dts[8]=1;
d2dts[1]=1; d2dts[2]=1; d2dts[3]=0; d2dts[4]= -1; d2dts[5]= -1; d2dts[6]= -1; d2dts[7]=0; d2dts[8]=1;

/* Calculate angles for interpolation */                                      
aref[0]= -atan2(ang.head.dy,ang.head.dx);
aref[1]=0.;
aref[2]= -aref[0];
aref[3]=0.5*PI;
aref[4]=PI-aref[2];
aref[5]=PI;
aref[6]=PI+aref[2];
aref[7]=1.5*PI;
aref[8]=2.*PI-aref[2];
aref[9]=2.*PI;

//  Calculate distances in each direction
int k;
for(k=1; k<=8; k++)
{
	dist[k]=sqrt(pow(ang.head.dx*d2dts[k],2)+pow(ang.head.dy*d1dts[k],2));
}
idv = -1;
/*  Initialize distance to idv, an initial data value that is not the same as the no data value
    to indicate that this has not been done yet.  On same parse through initialize dts values on streams  */
    for(i=0; i<ny; i++)
      for(j=0; j<nx; j++)
	  {
         dts.d[j][i]= idv;  // Use idv to indicate that this has not been done yet.
		 if(src.d[j][i] >=1)
		 {
			 dts.d[j][i]=0.;  // Initialize dts values on stream to 0.
		 }
	  }

switch (method)
{
case 0:
/* "lower block" */
for(i=ny/2; i<ny-1; i++)
   {for(j=nx/2; j<nx-1; j++)
      adistrec(i,j);
    for(j=nx/2-1; j>=1; j--)
      adistrec(i,j);
   }
 /* "upper block" */
for(i=ny/2-1; i>=1; i--)
   {for(j=nx/2; j<nx-1; j++)
      adistrec(i,j);
    for(j=nx/2-1; j>=1; j--)
      adistrec(i,j);
   }
break;
case 1:
	/* "lower block" */
for(i=ny/2; i<ny-1; i++)
   {for(j=nx/2; j<nx-1; j++)
      ldistrec(i,j);
    for(j=nx/2-1; j>=1; j--)
      ldistrec(i,j);
   }
 /* "upper block" */
for(i=ny/2-1; i>=1; i--)
   {for(j=nx/2; j<nx-1; j++)
      ldistrec(i,j);
    for(j=nx/2-1; j>=1; j--)
      ldistrec(i,j);
   }
break;
case 2:
	/* "lower block" */
for(i=ny/2; i<ny-1; i++)
   {for(j=nx/2; j<nx-1; j++)
      sdistrec(i,j);
    for(j=nx/2-1; j>=1; j--)
      sdistrec(i,j);
   }
 /* "upper block" */
for(i=ny/2-1; i>=1; i--)
   {for(j=nx/2; j<nx-1; j++)
      sdistrec(i,j);
    for(j=nx/2-1; j>=1; j--)
      sdistrec(i,j);
   }
break;
}
//  Parse through the grid again setting unchanged initial data values to no data
    for(i=0; i<ny; i++)
      for(j=0; j<nx; j++)
	  {
         if(dts.d[j][i]== idv)dts.d[j][i]=dts.nodata;  
	  }


	/*  Write out results  */ 
	err= gridwrite(dtsfile,dts,filetype);
 
/*    err = gridwrite(rtrfile,(void **)rtr,RPFLTDTYPE,nx,ny,dx,dy,
             bndbox,csize,-1.,filetype);  */
	free(dts.d[0]);
    free(dts.d); 
ERROR3:
    free(src.d[0]);
    free(src.d);
ERROR2:
    free(ang.d[0]);
    free(ang.d);
ERROR1:	
	return(err);
} 


/* function to compute average distance to stream recursively.  */
void adistrec(int i, int j)
{
	int in,jn,k,con=0;
	float p,sumn,sump;
	if(dts.d[j][i] == idv)  // idv (initial data value) is used to flag that the cell has not been done
	{
		/*  Make sure the cell is in the domain   */
		if(i!=0 && i!=ny-1 && j!=0 && j!=nx-1 && ang.d[j][i] > ang.nodata)
		{
			/*    look in all directions  */
			sumn=0.;
			sump=0.;
			float wf=1.;
			if(gweight == 1)
			{
				wf=wgrd.d[j][i];
			}
			for(k=1; k<=8; k++)
			{
				in = i + d1dts[k];
				jn = j + d2dts[k];
				/* Find the proportion draining to a downslope neighboring cell  */
				p=prop(ang.d[j][i],k);
				if(p>1e-6)  /*  Recurse to downslope neighbor only for grid cells 
					with proportion greater than 0.  Numerically the threshold 1e-6 guards against
					numerical loops due to angle rounding*/	
				{
					adistrec(in,jn);
					if(dts.d[jn][in] <= dts.nodata)
					{
						con = -1;
					}
					else
					{
						sumn=sumn+p*(dist[k]*wf+dts.d[jn][in]);
						sump=sump+p;
					}
				}
				if(ang.d[jn][in] <= ang.nodata) con= -1;
            }
			if(sump > 0.)
			{
				dts.d[j][i]=sumn/sump;
			}
			else
				dts.d[j][i]=dts.nodata;
			if (con == -1 && ccheck == 1)
			{
				dts.d[j][i]=dts.nodata;
			}
		}
		else
		dts.d[j][i]=dts.nodata;
	}
} 

/* function to compute longest drop to stream recursively.  */
void ldistrec(int i, int j)
{
	int in,jn,k,con=0;
	float p,sumn,sump;
	if(dts.d[j][i] == idv)  // idv (initial data value) is used to flag that the cell has not been done
	{
		/*  Make sure the cell is in the domain   */
		if(i!=0 && i!=ny-1 && j!=0 && j!=nx-1 && ang.d[j][i] > ang.nodata)
		{
			/*    look in all directions  */
			sumn=0.;
			sump=0.;
			float wf=1.;
			if(gweight == 1)
			{
				wf=wgrd.d[j][i];
			}
			for(k=1; k<=8; k++)
			{
				in = i + d1dts[k];
				jn = j + d2dts[k];
				/* Find the proportion draining to a downslope neighboring cell  */
				p=prop(ang.d[j][i],k);
				if(p>1e-6)  /*  Recurse to downslope neighbor only for grid cells 
					with proportion greater than 0.  Numerically the threshold 1e-6 guards against
					numerical loops due to angle rounding*/	
				{
					ldistrec(in,jn);
					if(dts.d[jn][in] <= dts.nodata)
					{
						con = -1;
					}
					else
					{
						float ed=wf*dist[k]+dts.d[jn][in];
						if(ed > sumn)sumn=ed;  //  Keep track of longest distance
						sump=sump+p;
					}
				}
				if(ang.d[jn][in] <= ang.nodata) con= -1;
            }
			if(sump > 0)
			{
				dts.d[j][i]=sumn;
			}
			else
				dts.d[j][i]=dts.nodata;
			if (con == -1 && ccheck == 1)
			{
				dts.d[j][i]=dts.nodata;
			}
		}
		else
		dts.d[j][i]=dts.nodata;
	}
} 

/* function to compute shortest distance to stream recursively.  */
void sdistrec(int i, int j)
{
	int in,jn,k,con=0;
	float p,sumn;
	if(dts.d[j][i] == idv)  // idv (initial data value) is used to flag that the cell has not been done
	{
		/*  Make sure the cell is in the domain   */
		if(i!=0 && i!=ny-1 && j!=0 && j!=nx-1 && ang.d[j][i] > ang.nodata)
		{
			/*    look in all directions  */
			float wf=1.;
			if(gweight == 1)
			{
				wf=wgrd.d[j][i];
			}
			int ik=0;  //  This records the number of positive p values
			for(k=1; k<=8; k++)
			{
				in = i + d1dts[k];
				jn = j + d2dts[k];
				/* Find the proportion draining to a downslope neighboring cell  */
				p=prop(ang.d[j][i],k);
				if(p>1e-6)  /*  Recurse to downslope neighbor only for grid cells 
					with proportion greater than 0.  Numerically the threshold 1e-6 guards against
					numerical loops due to angle rounding*/	
				{
					sdistrec(in,jn);
					if(dts.d[jn][in] <= dts.nodata)
					{
						con = -1;
					}
					else
					{
						ik=ik+1;
						float ed=wf*dist[k]+dts.d[jn][in];
						if(ik==1)  //  first pass through
						{
							sumn=ed;
						}
						else
						{
							if(ed < sumn)sumn=ed;  //  Keep track of shortest distance
						}
					}
				}
				if(ang.d[jn][in] <= ang.nodata) con= -1;
            }
			if(ik > 0)
			{
				dts.d[j][i]=sumn;
			}
			else
				dts.d[j][i]=dts.nodata;
			if (con == -1 && ccheck == 1)
			{
				dts.d[j][i]=dts.nodata;
			}
		}
		else
		dts.d[j][i]=dts.nodata;
	}
} 

/* float prop(float a, int k)
{
float p=0.;
if(k == 1 && a > PI)a=a-2.*PI;
if(a > aref[k-1] && a < aref[k+1]){
  if( a > aref[k])
       p=(aref[k+1]-a)/(aref[k+1]-aref[k]);
  else 
      p=(a-aref[k-1])/(aref[k]-aref[k-1]);
  }
return(p);
}  
      */          
 
   
