/***********************************************************/
/*                                                         */
/* gridio.c                                                */
/*                                                         */
/* I/O routines for terrain stability mapping              */
/*                                                         */
/***********************************************************/

/*  Copyright (C) 2000  David Tarboton, Utah State University

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License 
version 2, 1991 as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

A copy of the full GNU General Public License is included in file 
gpl.html. This is also available at:
http://www.gnu.org/copyleft/gpl.html
or from:
The Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
Boston, MA  02111-1307, USA.

If you wish to use or incorporate this program (or parts of it) into 
other software that does not meet the GNU General Public License 
conditions contact the author to request permission.
David G. Tarboton                                                      
Utah State University 
8200 Old Main Hill 
Logan, UT 84322-8200 
USA 
http://www.engineering.usu.edu/dtarb/ 
email:  dtarb@cc.usu.edu 
*/

#include "stdafx.h"
#include "gridio.h"
#include "TDErrorCodes.h"
#include <afxwin.h>


char * projection = new char[MAX_STRING_LENGTH + 1];
char * notes = new char[MAX_STRING_LENGTH + 1];


char *CAllocate1 (int	number ,
                    int	size )

{
	return (char *)calloc(number, size);
}

void CFree1 (char	*ptr)
{
	free(ptr);
}

/*  Nameadd(..)  Utility for adding suffixes to file names prior to
   "." extension   */
int nameadd(char *full,char *arg,char *suff)
  {
	char *ext;
	int nmain;
    ext=strrchr(arg,'.');
    if(ext == NULL)
	{
		nmain=strlen(arg);
		sprintf(full,"%s%s",arg,suff);
	}
	else
	{
		nmain=strlen(arg)-strlen(ext);
		strcpy(full,"");
		strncat(full,arg,nmain);
		strcat(full,suff);
		strcat(full,ext);
	}
	return(nmain);
  }

int readline(FILE *fp,char *fline)
{
  int i = 0, ch;

  for(i=0; i< MAXLN; i++)
  {
    ch = getc(fp);

    if(ch == EOF) { *(fline+i) = '\0'; return(EOF); }
    else          *(fline+i) = (char)ch;

    if((char)ch == '\n') { *(fline+i) = '\0'; return(0); }

  }
  return(1);
}

/*
  matalloc(...) allocates memory for matrix navigation pointer
  and for matrix data, then returns matrix navigation pointer
  Modification of matrixalloc by DGT to not use so many pointers
  7/1/97
*/
void **matalloc(int nx,int ny,int datatype)
{
    int i,arrsize;
    void **mat;
    void *data;
    int **imat;
    short **smat;
    float **fmat;

    /* allocate memory for array navigation pointers */
    if(datatype == 1)
    {
      mat = (void **)malloc(sizeof(short *)*(nx));
      arrsize = sizeof(short)*(nx)*(ny);
    }
    else if(datatype == 2)
    {
      mat = (void **)malloc(sizeof(int *)*(nx));
      arrsize = sizeof(int)*(nx)*(ny);
    }
    else if(datatype == 3)
    {
      mat = (void **)malloc(sizeof(float *)*(nx));
      arrsize = sizeof(float)*(nx)*(ny);
    }

    if(mat == NULL)
    {
//      printf("\nError: Cannot allocate memory for matrix navigation pointers");
//      printf("\n       nx = %d, ny = %d\n\n",nx,ny);
//      exit(2);
		return NULL;
	}
    
    data = malloc(arrsize);
    
    if(data == NULL)
    {
//      printf("\nError: Cannot allocate memory for matrix of dimension");
//      printf("\n       nx = %d, ny = %d\n\n",nx,ny);
//      exit(3);
		return NULL;
    }
    
    switch(datatype)
    {
      case 1:
        smat = (short **)mat;
        for(i=0; i<(nx); i++)
        {
          smat[i] = &(((short *)(data))[i*(ny)]);
        }
        break;
      case 2:
        imat = (int **)mat;
        for(i=0; i<(nx); i++)
        {
          imat[i] = &(((int *)(data))[i*(ny)]);
        }
        break;
      case 3:
        fmat = (float **)mat;
        for(i=0; i<(nx); i++)
        {
          fmat[i] = &(((float *)(data))[i*(ny)]);
        }
        break;
      default:
        break;
    }
    return(mat);
}



//New method overloading:Ajay
//New  gridread functions.FLOAT GRID
//====================================================
// Float datatype
int gridread(char *file, fgrid *dem, int *filetype)
{

	FILE *fp;
	int channel1,type,iesri;
	double *csize = new double;
    int i, j, hdrlines = 0;
    float value;
    char fline[MAXLN], keyword[21], utmetag, utmntag;
    float **farr;
    double adjbndbox[4],utme,utmn;
    CELLTYPE *rowbuf;
    float floatNull;
	double xllcenter=0,yllcenter=0;
//=========================================
	//declaring and assigning  variables
	double *dx,*dy;
	int *nx,*ny;
	void ***data;
	int datatype;
	float *nodata;

	dx=&(dem->head.dx);
	dy=&(dem->head.dy);
	nx=&(dem->head.nx);
	ny=&(dem->head.ny);
	data=(void ***)&(dem->d);
	datatype=RPFLTDTYPE;
	nodata=&dem->nodata;
	

//=======================================
    if( iesri = GridIOSetup() >= 0)
        /*
        Open input cell layer with no evaluation        (5)
        */
        if ((channel1 = 
              CellLayerOpen(file,READONLY,ROWIO,
                            &type,csize)) >= 0 )
    {
/*  The success of these means the file could be opened as an ESRI grid file  */
        *filetype=1;
/*  File types are
    0=ASCII   
    1= ARCVIEW grid via the ESRI Application Programmers Interface  */ 
/*  here is the ESRI grid file read stuff - following copyrow example */
        *dx = *dy = (float) *csize;
/*
        
        /*
        Get the bounding box of the input cell layer            (7)
        */
        if (BndCellRead (file, dem->head.bndbox) < 0)
        {
//                printf ("could not read bounding box of input grid\n");
                CellLyrClose(channel1);
                GridIOExit();
                return(1);
        }
/*        printf("%f %f %f %f %g\n",bndbox[0],bndbox[1],bndbox[2],bndbox[3],*nodata) */; 
/*  Bounding box is xllcorner, yllcorner, xurcorner, yurcorner   */
      
        /*
        Set the Window to the output bounding box and cellsize  (9)
        */
        if (AccessWindowSet (dem->head.bndbox, *csize, adjbndbox) < 0)
        {
//                printf ("Could not set Window\n");
                CellLyrClose(channel1);
                GridIOExit();
                return(2);
        }
        /*
        Get the number of rows and columns                      (10)
        in the window
        */
        *nx = WindowCols();
        *ny = WindowRows();

        /*  Allocate memory and set all type pointers  */
        *data = matalloc(*nx, *ny, datatype);
		if(*data == NULL)return(1);  //DGT 9/13/2007 Bail of not able to allocate memory
        farr = (float **) (*data);
        GetMissingFloat(&floatNull);
		
      //  *nodata = (datatype == RPFLTDTYPE) ? floatNull: -9999.; 
		*nodata=floatNull;
		        /*
        Allocate row buffer                                     (11)
        */
        if ((rowbuf = (CELLTYPE *)
                        CAllocate1 ( *nx, sizeof(CELLTYPE)))
                        == NULL )
        {
//                printf ("Could not allocate memory\n");
                CellLyrClose(channel1);
                GridIOExit();
                return(3);
        }
        /*
        Now copy row into array                 (12)
        */
		/*     This is all repeated to get right the typecasting of data from ESRI file into the format 
       we want   */
        for (i=0; i < *ny; i++)
        {
             GetWindowRow(channel1, i, rowbuf);
             if(type == CELLINT)
             {
                register int *buf = (int *)rowbuf;
	            
                   for(j=0; j < *nx; j++)
                   {
                      farr[j][i]=(buf[j] == MISSINGINT ) ? *nodata: (float) buf[j];  
                   }
            
             }
			 
             else if(type==CELLFLOAT)
             {
                register float *buf = (float *)rowbuf;
								

                
                   for(j=0; j < *nx; j++)
                   {
						farr[j][i]= buf[j];
                   }
              }
			 
       }

        /*
        Free row buffer                                         (13)
        */
        CFree1 ((char *)rowbuf);
        /*
        Close handles                                           (14)
        */
        CellLyrClose(channel1);  
        /*
        Done with the library                                   (15)
        */
        GridIOExit();
        return(0);
    }

/*  Here assume file is ASCII or Binary.  Close ESRI stuff. */
    CellLyrClose(channel1);
    GridIOExit();
//=========================================================
//Code for bgd .Ajay

    
	CString strFileName= CString(file); 
	CString strFileExtension= strFileName.Right(4); 

	if (strFileExtension.CompareNoCase(CString(".bgd"))==0)
		
{
    //Success means bgd 
	

		*filetype=2;
		fp=fopen(file,"rb");

			if(fp == NULL)
			{
		//        printf("\nERROR: Cannot open input file (%s).\n\n",file);
				return(1);
			}
    
        rewind(fp);
			
	//  CODE TO ASSIGN VALUES READ TO APPROPRIATE GRID HEADER VARIABLES
		fread(nx, sizeof(int),1,fp);
		fread(ny, sizeof(int),1,fp);
		fread(dx, sizeof(double),1,fp);
		fread(dy, sizeof(double),1,fp);
	//Bound box
		fread(&xllcenter, sizeof(double),1,fp);
		fread(&yllcenter, sizeof(double),1,fp);
	 // COMPUTE BNDBOX FROM XLLCENTER, YLLCENTER.
		dem->head.bndbox[0]=xllcenter-(*dx/2);
		dem->head.bndbox[1]=yllcenter-(*dy/2);
		dem->head.bndbox[2]=dem->head.bndbox[0] + *dx * (*nx);
		dem->head.bndbox[3]=dem->head.bndbox[1] + *dy * (*ny);
	
		fread(&datatype, sizeof(DATA_TYPE),1,fp);

		short snodata;
		if(datatype==SHORT_TYPE)
		{
			fread(&snodata,sizeof(short),1,fp);
			*nodata=(float)snodata;
		}
		long lnodata;
		if(datatype==LONG_TYPE)
		{
			fread(&lnodata,sizeof(long),1,fp);
			*nodata=(float)lnodata;
		}
		if(datatype==FLOAT_TYPE)fread(nodata,sizeof(float),1,fp);

		
		fread(projection,sizeof(char), MAX_STRING_LENGTH, fp);
        fread(notes, sizeof(char), MAX_STRING_LENGTH, fp);

			if( *nx < 0 || *ny < 0 || 
				*dx< 0 || *dy < 0 ) 
				{	
					fclose( fp );
					
					return 1;
				}

			farr =(float **)matalloc(*nx, *ny, FLOAT_TYPE + 1);
			if(farr == NULL)return(1);  //DGT 9/13/2007 Bail of not able to allocate memory

			//Check for datatype and typecasting        
			if(datatype==SHORT_TYPE)
				{	short shortarr;
					
					for( int i = 0; i < *ny; i++ )
					{	for( int j = 0; j < *nx; j++ )
						{	
							if( feof(fp) )
							{	
								fclose( fp );
								return 1;
							}
						
						fread(&shortarr,sizeof(short),1,fp );
						farr[j][i]=(float)shortarr;
						}
					}
				}
				else if(datatype==LONG_TYPE)
				{	
					int intarr;
			
					for( int i = 0; i < *ny; i++ )
					{	for( int j = 0; j < *nx; j++ )
						{	
							if( feof(fp) )
							{	
								fclose( fp );
								return 1;
							}
						
						fread( &intarr,sizeof(long),1,fp );
						farr[j][i]=(float)intarr;
						}
					}
				}
				else if(datatype==FLOAT_TYPE)
				{	
					
					for( int i = 0; i < *ny; i++ )
					{	for( int j = 0; j < *nx; j++ )
						{	
							if( feof(fp) )
							{	
								fclose( fp );
								return 1;
							}
						
						fread( &farr[j][i],sizeof(float),1,fp );
						}
					}
				}
				else return 1;
////////////////////////////////////////////////////////////////////
			*data = (void **) farr;
			fclose(fp);
			return 0;			

		 

	}

//================================================================
//================================================================
//If the file is ascii.
	*filetype=0;  
    fp = fopen(file,"r");
    if(fp == NULL)
    {
//        printf("\nERROR: Cannot open input file (%s).\n\n",file);
        return(1);
    }
    
	
    /* read ARC-Info header */
    while(1)
    {   
        readline(fp, fline);       
        if(!isalpha(*fline) || *fline == '-')
            break;       
        
        hdrlines++;

        sscanf(fline,"%s %f",keyword,&value);
        
	if(strcmp(keyword,"ncols") == 0 || strcmp(keyword,"NCOLS") == 0)
	    *nx = (int)value;
	else if(strcmp(keyword,"nrows") == 0 || strcmp(keyword,"NROWS") == 0)
	    *ny = (int)value;
	else if(strcmp(keyword,"xllcenter") == 0 || strcmp(keyword,"XLLCENTER") == 0)
	{
	    utmetag = 'c';
	    utme = value;
	}
	else if(strcmp(keyword,"xllcorner") == 0 || strcmp(keyword,"XLLCORNER") == 0)
	{
	    utmetag = 'e';
	    utme = value;
	}
	else if(strcmp(keyword,"yllcenter") == 0 || strcmp(keyword,"YLLCENTER") == 0)
	{
	    utmntag = 'c';
	    utmn = value;
	}
	else if(strcmp(keyword,"yllcorner") == 0 || strcmp(keyword,"YLLCORNER") == 0)
	{
	    utmntag = 'e';
	    utmn = value;
	}
	else if(strcmp(keyword,"cellsize") == 0 || strcmp(keyword,"CELLSIZE") == 0)
	{
	    *dx = *dy = value;
	    *csize = (double) value;
	}
	else if(strcmp(keyword,"nodata_value") == 0 || strcmp(keyword,"NODATA_VALUE") == 0 ||
		strcmp(keyword,"NODATA_value") == 0)
	    *nodata = (float)value;
    }
    
    /* adjust utme and utmn if necessary (we store center of reference cell) */
    if(utmetag == 'e') utme = utme + *dx/2;
    if(utmntag == 'e') utmn = utmn + *dy/2;
     dem->head.bndbox[0] = utme - *csize/2.;   
     dem->head.bndbox[1] = utmn - *csize/2.;
     dem->head.bndbox[2] = dem->head.bndbox[0] + *csize * (*nx);
     dem->head.bndbox[3] = dem->head.bndbox[1] + *csize * (*ny);
    /* position file pointer for ARC-Info file to beginning of image data */
    rewind(fp);
    for(i=0; i<hdrlines; i++) readline(fp, fline);
    
    /* convert depending on datatype */
     
       farr = (float **) matalloc(*nx, *ny, datatype);
	   if(farr == NULL)return(1);  //DGT 9/13/2007 Bail of not able to allocate memory

        /* read in the ARC-Info file */
        for(i=0; i< *ny; i++)
        {
            for(j=0; j< *nx; j++)
			{

                fscanf(fp,"%f",&farr[j][i]);
			}
        }
        *data = (void **) farr;   
   
    fclose(fp);
    return(0);

	
}
//INT GRID
//============================================================
int gridread(char *file, igrid *dem ,int *filetype)
{

FILE *fp;
	int channel1,type,iesri;
	double *csize = new double;
    float value;
	float floatNull;
	double xllcenter,yllcenter;
    char fline[MAXLN], keyword[21], utmetag, utmntag;
    int **iarr, *nx,*ny,datatype,*nodata,i, j, hdrlines = 0;
 	double adjbndbox[4],utme,utmn,*dx,*dy;
	void ***data;
	CELLTYPE *rowbuf;
//=========================================
	//assigning to variables
	dx=&(dem->head.dx);
	dy=&(dem->head.dy);
	nx=&(dem->head.nx);
	ny=&(dem->head.ny);
	data=(void ***)&(dem->d);
	datatype=RPINTDTYPE;
	nodata=&dem->nodata;
//===============================================================

    if( iesri = GridIOSetup() >= 0)
        /*
        Open input cell layer with no evaluation        (5)
        */
        if ((channel1 = 
              CellLayerOpen(file,READONLY,ROWIO,
                            &type,csize)) >= 0 )
    {
/*  The success of these means the file could be opened as an ESRI grid file  */
        *filetype=1;
/*  File types are
    0=ASCII   
    1= ARCVIEW grid via the ESRI Application Programmers Interface  */ 
/*  here is the ESRI grid file read stuff - following copyrow example */
        *dx = *dy = (float) *csize;
/*
        
        /*
        Get the bounding box of the input cell layer            (7)
        */
        if (BndCellRead (file, dem->head.bndbox) < 0)
        {
//                printf ("could not read bounding box of input grid\n");
                CellLyrClose(channel1);
                GridIOExit();
                return(1);
        }
/*        printf("%f %f %f %f %g\n",bndbox[0],bndbox[1],bndbox[2],bndbox[3],*nodata) */; 
/*  Bounding box is xllcorner, yllcorner, xurcorner, yurcorner   */
      
        /*
        Set the Window to the output bounding box and cellsize  (9)
        */
        if (AccessWindowSet (dem->head.bndbox, *csize, adjbndbox) < 0)
        {
//                printf ("Could not set Window\n");
                CellLyrClose(channel1);
                GridIOExit();
                return(2);
        }
        /*
        Get the number of rows and columns                      (10)
        in the window
        */
        *nx = WindowCols();
        *ny = WindowRows();

        /*  Allocate memory and set all type pointers  */
        *data = matalloc(*nx, *ny, datatype);
	    if(*data == NULL)return(1);  //DGT 9/13/2007 Bail of not able to allocate memory

        iarr = (int **) (*data);
        GetMissingFloat(&floatNull);
		*nodata=MISSINGINT;
		     /*
        Allocate row buffer                                     (11)
        */
        if ((rowbuf = (CELLTYPE *)
                        CAllocate1 ( *nx, sizeof(CELLTYPE)))
                        == NULL )
        {
//                printf ("Could not allocate memory\n");
                CellLyrClose(channel1);
                GridIOExit();
                return(3);
        }
        /*
        Now copy row into array                 (12)
        */
        for (i=0; i < *ny; i++)
        {
             GetWindowRow (channel1, i, rowbuf);
             if(type == CELLFLOAT)
				 /*     This is to typecast from ESRI float to int for a int grid.   */
             {  
				 register float *buf = (float *)rowbuf;
                
                 
                   for(j=0; j < *nx; j++)
                   {
                      iarr[j][i]=(buf[j] == floatNull) ?  *nodata : (int)buf[j]; 
                   }
                    
	           }
             else if(type==CELLINT)
             {
                register int *buf = (int *)rowbuf;
								

                   for(j=0; j < *nx; j++)
                   {
                      iarr[j][i]= buf[j] ;
                   }
 	           }
			 }
	
        /*
        Free row buffer                                         (13)
        */
        CFree1 ((char *)rowbuf);
        /*
        Close handles                                           (14)
        */
        CellLyrClose(channel1);  
        /*
        Done with the library                                   (15)
        */
        GridIOExit();
        return(0);
    }

	
	
/*  Here assume file is ASCII or Binary.  Close ESRI stuff. */
   


	CellLyrClose(channel1);
    GridIOExit();

//=========================================================
//Code for bgd .Ajay

//Checking the extension of the input file

    CString strFileName= CString(file); 
	CString strFileExtension= strFileName.Right(4); 

	if (strFileExtension.CompareNoCase(CString(".bgd"))==0)
{
    //Success means bgd 
		*filetype=2;
		fp=fopen(file,"rb");

			if(fp == NULL)
			{
		//        printf("\nERROR: Cannot open input file (%s).\n\n",file);
				return(1);
			}
    
        rewind(fp);
			
	//  CODE TO ASSIGN VALUES READ TO APPROPRIATE GRID HEADER VARIABLES
		fread(nx, sizeof(int),1,fp);
		fread(ny, sizeof(int),1,fp);
		fread(dx, sizeof(double),1,fp);
		fread(dy, sizeof(double),1,fp);
	//Bound box
		fread(&xllcenter, sizeof(double),1,fp);
		fread(&yllcenter, sizeof(double),1,fp);
	 // COMPUTE BNDBOX FROM XLLCENTER, YLLCENTER.
			dem->head.bndbox[0]=xllcenter-(*dx/2);
			dem->head.bndbox[1]=yllcenter-(*dy/2);
			dem->head.bndbox[2]=dem->head.bndbox[0] + *dx * (*nx);
			dem->head.bndbox[3]=dem->head.bndbox[1] + *dy * (*ny);
	/////////////////////////////////////////////
		fread(&datatype, sizeof(DATA_TYPE),1,fp);
	
		
		if(datatype==SHORT_TYPE)
		{	short snodata;
			fread(&snodata,sizeof(short),1,fp);
			*nodata=(int)snodata;
		}
		float fnodata;
		if(datatype==FLOAT_TYPE)
		{	
			fread(&fnodata,sizeof(float),1,fp);
			*nodata=MISSINGINT;
		}
		if(datatype==LONG_TYPE)fread(nodata,sizeof(long),1,fp);
				
		fread(projection,sizeof(char), MAX_STRING_LENGTH, fp);
		fread(notes, sizeof(char), MAX_STRING_LENGTH, fp);

			if( *nx < 0 || *ny < 0 || 
				*dx< 0 || *dy < 0)
				{	
					fclose( fp );
					return 1;
				}
			// here ,1 is  added to the datatype as fread returns datatype 1 less than matalloc datatypes.
				
	
			iarr =(int **) matalloc(*nx, *ny, LONG_TYPE + 1);
			if(iarr == NULL)return(1);  //DGT 9/13/2007 Bail of not able to allocate memory

			if(datatype==SHORT_TYPE)
				{
				short shortarr;
				
					for( int i = 0; i < *ny; i++ )
					{	for( int j = 0; j < *nx; j++ )
						{	
							if( feof(fp) )
							{	
								fclose( fp );
								return 1;
							}
						
						fread( &shortarr,sizeof(short),1,fp );
						iarr[j][i]=(int)shortarr;
						}
					}
				}
				else if(datatype==FLOAT_TYPE)
				{
					
				float floatarr;	
				
					for( int i = 0; i < *ny; i++ )
					{	for( int j = 0; j < *nx; j++ )
						{	
							if( feof(fp) )
							{	
								fclose( fp );
								return 1;
							}
						
						fread( &floatarr,sizeof(float),1,fp );
						iarr[j][i]=(floatarr == fnodata) ?  *nodata : (int)floatarr;
						}
					}
				}
				else if(datatype==LONG_TYPE)
				{
					
					for( int i = 0; i < *ny; i++ )
					{	for( int j = 0; j < *nx; j++ )
						{	
							if( feof(fp) )
							{	
								fclose( fp );
								return 1;
							}
								fread(&iarr[j][i],sizeof(long),1,fp);
								
						}
					}
				}
				else return 1;

			*data = (void **) iarr;
			fclose(fp);
			return 0;			

		 

	}



//================================================================
//If the file is ascii.
	*filetype=0;  
    fp = fopen(file,"r");
    if(fp == NULL)
    {
//        printf("\nERROR: Cannot open input file (%s).\n\n",file);
        return(1);
    }
    
	
    /* read ARC-Info header */
    while(1)
    {   
        readline(fp, fline);       
        if(!isalpha(*fline) || *fline == '-')
            break;       
        
        hdrlines++;

        sscanf(fline,"%s %f",keyword,&value);
        
	if(strcmp(keyword,"ncols") == 0 || strcmp(keyword,"NCOLS") == 0)
	    *nx = (int)value;
	else if(strcmp(keyword,"nrows") == 0 || strcmp(keyword,"NROWS") == 0)
	    *ny = (int)value;
	else if(strcmp(keyword,"xllcenter") == 0 || strcmp(keyword,"XLLCENTER") == 0)
	{
	    utmetag = 'c';
	    utme = value;
	}
	else if(strcmp(keyword,"xllcorner") == 0 || strcmp(keyword,"XLLCORNER") == 0)
	{
	    utmetag = 'e';
	    utme = value;
	}
	else if(strcmp(keyword,"yllcenter") == 0 || strcmp(keyword,"YLLCENTER") == 0)
	{
	    utmntag = 'c';
	    utmn = value;
	}
	else if(strcmp(keyword,"yllcorner") == 0 || strcmp(keyword,"YLLCORNER") == 0)
	{
	    utmntag = 'e';
	    utmn = value;
	}
	else if(strcmp(keyword,"cellsize") == 0 || strcmp(keyword,"CELLSIZE") == 0)
	{
	    *dx = *dy = value;
	    *csize = (double) value;
	}
	else if(strcmp(keyword,"nodata_value") == 0 || strcmp(keyword,"NODATA_VALUE") == 0 ||
		strcmp(keyword,"NODATA_value") == 0)
	    *nodata = (int) value;
    }
    
    /* adjust utme and utmn if necessary (we store center of reference cell) */
    if(utmetag == 'e') utme = utme + *dx/2;
    if(utmntag == 'e') utmn = utmn + *dy/2;
     dem->head.bndbox[0] = utme - *csize/2.;   
     dem->head.bndbox[1] = utmn - *csize/2.;
     dem->head.bndbox[2] = dem->head.bndbox[0] + *csize * (*nx);
     dem->head.bndbox[3] = dem->head.bndbox[1] + *csize * (*ny);
    /* position file pointer for ARC-Info file to beginning of image data */
    rewind(fp);
    for(i=0; i<hdrlines; i++) readline(fp, fline);
    
       iarr = (int **) matalloc(*nx, *ny, datatype);
 	   if(iarr == NULL)return(1);  //DGT 9/13/2007 Bail of not able to allocate memory
       
        for(i=0; i< *ny; i++)
        {
            for(j=0; j< *nx; j++)
			{
               if(EOF==fscanf(fp,"%d",&iarr[j][i])) return(1);  // error trapping for float data
			}
        }
        *data = (void **) iarr;
     
    
    fclose(fp);
    return(0);

}

	

//=============================================================
//gridread for short datatype:Ajay
//=============================================================
int gridread(char *file, sgrid *dem ,int *filetype)
{
	FILE *fp;
	int channel1,type,iesri;
	double *csize = new double,*dx,*dy;
    int i, j, hdrlines = 0,*nx,*ny,datatype;
    float value;
	float floatNull;
    char fline[MAXLN], keyword[21], utmetag, utmntag;
    short **sarr,*nodata;
    double adjbndbox[4],utme,utmn,xllcenter=0,yllcenter=0;
    CELLTYPE *rowbuf;
	void ***data;
//=========================================
	//assigning to variables
	
///////////////////////
	
	dx=&(dem->head.dx);
	dy=&(dem->head.dy);
	nx=&(dem->head.nx);
	ny=&(dem->head.ny);
	data=(void ***)&(dem->d);
	datatype=RPSHRDTYPE;
	nodata=&dem->nodata;
	
//===============================================================

      
    if( iesri = GridIOSetup() >= 0)
        /*
        Open input cell layer with no evaluation        (5)
        */
        if ((channel1 = 
              CellLayerOpen(file,READONLY,ROWIO,
                            &type,csize)) >= 0 )
    {
/*  The success of these means the file could be opened as an ESRI grid file  */
        *filetype=1;
/*  File types are
    0=ASCII   
    1= ARCVIEW grid via the ESRI Application Programmers Interface  */ 
/*  here is the ESRI grid file read stuff - following copyrow example */
        *dx = *dy = (double) *csize;
/*
        
        /*
        Get the bounding box of the input cell layer            (7)
        */
        if (BndCellRead (file, dem->head.bndbox) < 0)
        {
//                printf ("could not read bounding box of input grid\n");
                CellLyrClose(channel1);
                GridIOExit();
                return(1);
        }
/*        printf("%f %f %f %f %g\n",bndbox[0],bndbox[1],bndbox[2],bndbox[3],*nodata) */; 
/*  Bounding box is xllcorner, yllcorner, xurcorner, yurcorner   */
      
        /*
        Set the Window to the output bounding box and cellsize  (9)
        */
        if (AccessWindowSet (dem->head.bndbox, *csize, adjbndbox) < 0)
        {
//                printf ("Could not set Window\n");
                CellLyrClose(channel1);
                GridIOExit();
                return(2);
        }
        /*
        Get the number of rows and columns                      (10)
        in the window
        */
        *nx = WindowCols();
        *ny = WindowRows();

        /*  Allocate memory and set all type pointers  */
        *data = matalloc(*nx, *ny, datatype);
		if(*data == NULL)return(1);  //DGT 9/13/2007 Bail of not able to allocate memory

        sarr = (short **) (*data);
        GetMissingFloat(&floatNull);
        *nodata=MISSINGSHORT;  
		
        /*
        Allocate row buffer                                     (11)
        */
        if ((rowbuf = (CELLTYPE *)
                        CAllocate1 ( *nx, sizeof(CELLTYPE)))
                        == NULL )
        {
//                printf ("Could not allocate memory\n");
                CellLyrClose(channel1);
                GridIOExit();
                return(3);
        }
        /*
        Now copy row into array                 (12)
        */
        for (i=0; i < *ny; i++)
        {
             GetWindowRow (channel1, i, rowbuf);
             if(type == CELLFLOAT)
             {
				 register float *buf = (float *)rowbuf;
                
                   for(j=0; j < *nx; j++)
                   {
					    sarr[j][i]=(buf[j] == floatNull ) ? *nodata : (short) buf[j];   
                       
                   }
              }
             else if(type==CELLINT)
			 {
				register int *buf = (int *)rowbuf;
				for(j=0; j < *nx; j++)
                   
				{
					  sarr[j][i]=(buf[j] == MISSINGINT) ?  *nodata : (short) buf[j];   
                      
                   }
				
			 }
			}

        /*
        Free row buffer                                         (13)
        */
        CFree1 ((char *)rowbuf);
        /*
        Close handles                                           (14)
        */
        CellLyrClose(channel1);  
        /*
        Done with the library                                   (15)
        */
        GridIOExit();
        return(0);
    }

	
	
/*  Here assume file is ASCII or Binary.  Close ESRI stuff. */

    CellLyrClose(channel1);
    GridIOExit();
//=========================================================
//test code for bgd .Ajay
   
	CString strFileName= CString(file); 
	CString strFileExtension= strFileName.Right(4); 

	if (strFileExtension.CompareNoCase(CString(".bgd"))==0)
		
	{
	//Success means bgd 
		*filetype=2;
		fp=fopen(file,"rb");
			if(fp == NULL)
			{
		//        printf("\nERROR: Cannot open input file (%s).\n\n",file);
				return(1);
			}
    
        rewind(fp);
		//  CODE TO ASSIGN VALUES READ TO APPROPRIATE GRID HEADER VARIABLES

		fread(nx, sizeof(int),1,fp);
		fread(ny, sizeof(int),1,fp);
		fread(dx, sizeof(double),1,fp);
		fread(dy, sizeof(double),1,fp);
		fread(&xllcenter, sizeof(double),1,fp);
		fread(&yllcenter, sizeof(double),1,fp);
	 // COMPUTE BNDBOX FROM XLLCENTER, YLLCENTER.
		dem->head.bndbox[0]=xllcenter-(*dx/2);
		dem->head.bndbox[1]=yllcenter-(*dy/2);
		dem->head.bndbox[2]=dem->head.bndbox[0] + *dx * (*nx);
		dem->head.bndbox[3]=dem->head.bndbox[1] + *dy * (*ny);
		fread(&datatype, sizeof(DATA_TYPE),1,fp);
	//	fread(nodata, sizeof(short),1,fp);
		  long lnodata;
		if(datatype==LONG_TYPE)
		{ 
			fread(&lnodata,sizeof(long),1,fp);
			*nodata=MISSINGSHORT;
		}
		 float fnodata;
		if(datatype==FLOAT_TYPE)
		{  
			fread(&fnodata,sizeof(float),1,fp);
			*nodata=MISSINGSHORT;
		}
		if(datatype==SHORT_TYPE)fread(nodata,sizeof(short),1,fp);

		fread(projection,sizeof(char),MAX_STRING_LENGTH,fp);
		fread(notes, sizeof(char), MAX_STRING_LENGTH, fp);
  //CODE to check type is correct - 
	
			if( *nx < 0 || *ny < 0 || 
				*dx< 0 || *dy < 0)
				{	fclose( fp );
					return 1;
				}

		// here ,1 is  added to the datatype as fread returns datatype 1 less than matalloc datatypes.
			
	sarr= (short **)matalloc(*nx, *ny,SHORT_TYPE + 1);
	if(sarr == NULL)return(1);  //DGT 9/13/2007 Bail of not able to allocate memory

	if(datatype==LONG_TYPE)
	{		
			int intarr;
			
			for( int i= 0; i < *ny; i++ )
			{
				for( int j = 0; j < *nx ;j++ )
				{	
				if( feof(fp) )
					{	fclose( fp );
						return 1;
					}
				{
					fread( &intarr,sizeof(long),1,fp );
					 sarr[j][i]=(intarr == lnodata) ?  *nodata : (short)intarr;   
				}

				}
			}
	}
	else if(datatype==FLOAT_TYPE)
	{
		float floatarr;
		
			for( int i= 0; i < *ny; i++ )
			{
				for( int j = 0; j < *nx ;j++ )
				{	
				if( feof(fp) )
					{	fclose( fp );
						return 1;
					}
					{
						fread( &floatarr,sizeof(float),1,fp );
						sarr[j][i]=(floatarr == fnodata) ?  *nodata : (short)floatarr; 
					}

				}

			}
	}
	else if(datatype==SHORT_TYPE)
	{
		
		for( int i= 0; i < *ny; i++ )
			{
				for( int j = 0; j < *nx ;j++ )
				{	
				if( feof(fp) )
					{	fclose( fp );
						return 1;
					}
					{
						fread( &sarr[j][i],sizeof(short),1,fp );
					}

				}
			}
	}
	else return 1;


			*data = (void **) sarr;
			fclose(fp);
			return 0;			

}

//else assuming the file to be ascii
//================================================================
//If the file is ascii.

	*filetype=0;  
    fp = fopen(file,"r");
    if(fp == NULL)
    {
//        printf("\nERROR: Cannot open input file (%s).\n\n",file);
        return(1);
    }
    
	
    /* read ARC-Info header */
    while(1)
    {   
        readline(fp, fline);       
        if(!isalpha(*fline) || *fline == '-')
            break;       
        
        hdrlines++;

        sscanf(fline,"%s %f",keyword,&value);
        
	if(strcmp(keyword,"ncols") == 0 || strcmp(keyword,"NCOLS") == 0)
	    *nx = (int)value;
	else if(strcmp(keyword,"nrows") == 0 || strcmp(keyword,"NROWS") == 0)
	    *ny = (int)value;
	else if(strcmp(keyword,"xllcenter") == 0 || strcmp(keyword,"XLLCENTER") == 0)
	{
	    utmetag = 'c';
	    utme = value;
	}
	else if(strcmp(keyword,"xllcorner") == 0 || strcmp(keyword,"XLLCORNER") == 0)
	{
	    utmetag = 'e';
	    utme = value;
	}
	else if(strcmp(keyword,"yllcenter") == 0 || strcmp(keyword,"YLLCENTER") == 0)
	{
	    utmntag = 'c';
	    utmn = value;
	}
	else if(strcmp(keyword,"yllcorner") == 0 || strcmp(keyword,"YLLCORNER") == 0)
	{
	    utmntag = 'e';
	    utmn = value;
	}
	else if(strcmp(keyword,"cellsize") == 0 || strcmp(keyword,"CELLSIZE") == 0)
	{
	    *dx = *dy = value;
	    *csize = (double) value;
	}
	else if(strcmp(keyword,"nodata_value") == 0 || strcmp(keyword,"NODATA_VALUE") == 0 ||
		strcmp(keyword,"NODATA_value") == 0)
	    *nodata = (short)value;
    }
    
    /* adjust utme and utmn if necessary (we store center of reference cell) */
    if(utmetag == 'e') utme = utme + *dx/2;
    if(utmntag == 'e') utmn = utmn + *dy/2;
     dem->head.bndbox[0] = utme - *csize/2.;   
     dem->head.bndbox[1] = utmn - *csize/2.;
     dem->head.bndbox[2] = dem->head.bndbox[0] + *csize * (*nx);
     dem->head.bndbox[3] = dem->head.bndbox[1] + *csize * (*ny);
    /* position file pointer for ARC-Info file to beginning of image data */
    rewind(fp);
    for(i=0; i<hdrlines; i++) readline(fp, fline);
    
        sarr = (short **) matalloc(*nx, *ny, datatype);
	    if(sarr == NULL)return(1);  //DGT 9/13/2007 Bail of not able to allocate memory
             
        /* read in the ARC-Info file */        
        for(i=0; i< *ny; i++)
        {
            for(j=0; j< *nx; j++)
               if( EOF == fscanf(fp,"%hd",&sarr[j][i])) return(1);
				
        }
        *data = (void **) sarr;
    fclose(fp);
    return(0);

}

//==========================================================
//New gridwrite methods(overloading).
//For Float Grid:ajay

//==========================================================================
int gridwrite(char *file,fgrid dem,int infiletype)
{
FILE *fp;

    int i, j,nx,ny,datatype,filetype;
    char fline[MAXLN];
    float **farr,nodata;
    double adjbndbox[4],utme,utmn,dx,dy;
    int channel1,type;
    CELLTYPE *rowbuf;
	double csize,xllcenter,yllcenter;
	void **data;
	
//=========================================
	//assigning to variables
	dx=dem.head.dx;
	dy=dem.head.dy;
	nx=dem.head.nx;
	ny=dem.head.ny;
	data=(void **)(dem.d);
	datatype=RPFLTDTYPE;
	nodata=dem.nodata;
//===============================================================


// assinging csize with dx
	csize=dx;
/*  File types are
    0=ASCII   
    1= ARCVIEW grid via the ESRI Application Programmers Interface  
	2= USU binarg grid   */
//DGT added 3/19/05
//  Rather than use the filetype variable check file name for on the fly type conversions
	filetype=infiletype;  // First default is from the calling program
	CString strFileName= CString(file); 
	CString strFileExtension= strFileName.Right(4); 
	CString strLetter=strFileExtension.Left(1);
//  if there is an extension the filetype is ASCII by default
	if (strLetter.CompareNoCase(CString("."))==0)filetype=0;
//  if the extension is exactly .bgd the filetype is USU bgd
	if (strFileExtension.CompareNoCase(CString(".bgd"))==0)filetype=2;
//  The only way a file can be ESRI is if the calling program indicated this and there is not a . 4 characters from the right.

  if(filetype == 0)   /*  ASCII FILE  */
  {
    /* open ASCII file */
    fp = fopen(file,"w");
    if(fp == NULL)
    {
//        printf("\nERROR: Cannot open output file (%s).\n\n",file);
        return(4);
    }
    
   
    /* write ARC-Info header */
    fprintf(fp,"ncols\t%d\n",nx);
    fprintf(fp,"nrows\t%d\n",ny);
     utme=dem.head.bndbox[0]+dx*0.5;
     utmn=dem.head.bndbox[1]+dy*0.5;
    fprintf(fp,"xllcenter\t%f\n",utme);
    fprintf(fp,"yllcenter\t%f\n",utmn);
    fprintf(fp,"cellsize\t%g\n",csize);
	//  DGT 4/25/05   Use -9999 for no data if no data is less than -9999 and min is greater than -9999.
	float fmin= 0.,nodatap = -9999.;
    farr = (float **) data;
	for(i=0; i<ny; i++)
		for(j=0; j<nx; j++)
			if(farr[j][i] != nodata && farr[j][i] < fmin)fmin=farr[j][i];

	if(nodata > nodatap)nodatap=nodata;
	if(fmin < nodatap)nodatap=nodata;
    fprintf(fp,"nodata_value\t%g\n",nodatap);
    
    /* write raster data to ARC/INFO file */
    /* convert depending on datatype */
    
    
 
			for(i=0; i< ny; i++)
			{
				for(j=0; j< nx; j++)
			{
				if(farr[j][i] == nodata)fprintf(fp,"%g ",nodatap);
				else fprintf(fp,"%g ",farr[j][i]);
				if((j+1) % LINELEN == 0)fprintf(fp,"\n");
			}
			fprintf(fp,"\n");
			}   
    
    
    fclose(fp);
  }
  else if (filetype==2)
// bgd file	
  {
		fp = fopen(file,"wb");
		if( !fp )
		{	
			return 1;
		}
		

		fwrite( &nx, sizeof(int),1,fp);
		fwrite( &ny, sizeof(int),1,fp);
		fwrite( &dx, sizeof(double),1,fp);
		fwrite( &dy, sizeof(double),1,fp);
		xllcenter=dem.head.bndbox[0]+dx*0.5;
		yllcenter=dem.head.bndbox[1]+dy*0.5;
		fwrite( &xllcenter, sizeof(double),1,fp);
		fwrite( &yllcenter, sizeof(double),1,fp);
		DATA_TYPE type = FLOAT_TYPE;
		fwrite( &type, sizeof(DATA_TYPE),1,fp);
		fwrite( &nodata, sizeof(float),1,fp);
		for( int p = 0; p < MAX_STRING_LENGTH; p++ )
		{		projection[p] = 'A';
		}
		fwrite(&projection, sizeof(char), MAX_STRING_LENGTH, fp );
		for(p = 0; p < MAX_STRING_LENGTH; p++ )
		{		notes[p] = 'A';
		}	
		fwrite(&notes, sizeof(char), MAX_STRING_LENGTH, fp );
		
			farr=(float **)data;
			
			for( int i = 0; i < ny; i++ )
			{	for( int j = 0; j < nx; j++ )
				{						
					fwrite(&farr[j][i],sizeof(float),1,fp);
				}				
			}
			fclose( fp );
			return 0;			
				
  }

  else
  {
/*  ESRI grid file  */
     GridIOSetup();
	 AccessWindowClear();  /*  Forget about any window it used to have  */
     if (CellLyrExists(file))
     {
/*  Save old file as a backup and use new file name  */
        sprintf(fline,"%s%s",file,"_b");
        if (CellLyrExists(fline))GridDelete(fline);
        GridRename(file,fline);
//        printf("Output grid %s exists and will be overwritten.\n",file);
//        printf("The old grid is preserved as %s\n",fline);
        GridDelete(file);
     } 

	 type= CELLFLOAT;

     if((channel1 = CellLayerCreate(file,WRITEONLY,ROWIO,type,csize,dem.head.bndbox)) < 0)
     {
//        printf ("Could not cread output grid %s\n",file);
        CellLyrClose(channel1);
        GridIOExit();
        return(5);
      }
     if(AccessWindowSet(dem.head.bndbox, csize, adjbndbox) < 0)
     {
//        printf ("Could not set Window\n");
       CellLyrClose(channel1);
       CellLyrDelete (file);
       GridIOExit();
        return(5);
     }
/*  Allocate row buffer  */
    if((rowbuf = (CELLTYPE *)CAllocate1(nx,sizeof(CELLTYPE))) == NULL)
    {
//        printf("Could not allocate buffer memory\n");
        CellLyrClose(channel1);
        CellLyrDelete (file);
        GridIOExit();
        return(5);
     }
     
          register float *buf = (float *)rowbuf;
          float floatNull;

          GetMissingFloat(&floatNull);
          farr = (float **)data;
          for(i=0; i < ny; i++)
          {
                   for(j=0; j < nx; j++)
                   {
                      buf[j] = farr[j][i];
                      if(buf[j] == nodata) buf[j]=floatNull;
                   }
             PutWindowRow (channel1, i, rowbuf);
          }
       
       CFree1 ((char *)rowbuf);
       CellLyrClose(channel1);
       GridIOExit();
   }
   return(0);

}

//========================
//New Function for gridwrite:Ajay
//For Integer Grid
//========================

int gridwrite(char *file,igrid dem,int infiletype)
{
FILE *fp;

    int i, j,filetype;
    char fline[MAXLN];
    int **iarr;
    double adjbndbox[4],utme,utmn;
    int channel1,type,datatype,nx,ny;
	int nodata;
    CELLTYPE *rowbuf;
	double csize,dx,dy,xllcenter,yllcenter;
	void **data;
//=========================================
	//assigning to variables
	dx=dem.head.dx;
	dy=dem.head.dy;
	nx=dem.head.nx;
	ny=dem.head.ny;
	data=(void **)(dem.d);
	datatype=RPINTDTYPE;
	nodata=dem.nodata;

//===============================================================

	csize=dx;
/*  File types are
    0=ASCII   
    1= ARCVIEW grid via the ESRI Application Programmers Interface     
	2= USU binarg grid   */
//DGT added 3/19/05
//  Rather than use the filetype variable check file name for on the fly type conversions
	filetype=infiletype;  // First default is from the calling program
	CString strFileName= CString(file); 
	CString strFileExtension= strFileName.Right(4); 
	CString strLetter=strFileExtension.Left(1);
//  if there is an extension the filetype is ASCII by default
	if (strLetter.CompareNoCase(CString("."))==0)filetype=0;
//  if the extension is exactly .bgd the filetype is USU bgd
	if (strFileExtension.CompareNoCase(CString(".bgd"))==0)filetype=2;
//  The only way a file can be ESRI is if the calling program indicated this and there is not a . 4 characters from the right.

	if(filetype == 0)   /*  ASCII FILE  */
  {
    /* open ARC-Info file */
    fp = fopen(file,"w");
    if(fp == NULL)
    {
//        printf("\nERROR: Cannot open output file (%s).\n\n",file);
        return(4);
    }
    
   
    /* write ARC-Info header */
    fprintf(fp,"ncols\t%d\n",nx);
    fprintf(fp,"nrows\t%d\n",ny);
	utme=dem.head.bndbox[0]+dx*0.5;
	utmn=dem.head.bndbox[1]+dy*0.5;
    fprintf(fp,"xllcenter\t%f\n",utme);
    fprintf(fp,"yllcenter\t%f\n",utmn);
    fprintf(fp,"cellsize\t%g\n",csize);
    fprintf(fp,"nodata_value\t%d\n",nodata);
    
    /* write raster data to ARC/INFO file */
    /* convert depending on datatype */
   
        iarr = (int **) data;
        for(i=0; i< ny; i++)
        {
            for(j=0; j< nx; j++)
            {
                fprintf(fp,"%d ",iarr[j][i]);
                if((j+1) % LINELEN == 0)fprintf(fp,"\n");
            }
            fprintf(fp,"\n");
        }
   
   
    fclose(fp);
  }
 else if (filetype==2)
	 
	
  {

		
	
		fp = fopen(file,"wb");
		if( !fp )
		{	
			return 1;
		}
		
		rewind(fp);
		fwrite( &nx, sizeof(int),1,fp);
		fwrite( &ny, sizeof(int),1,fp);
		fwrite( &dx, sizeof(double),1,fp);
		fwrite( &dy, sizeof(double),1,fp);
		xllcenter = dem.head.bndbox[0] + dx * 0.5;
		yllcenter = dem.head.bndbox[1] + dy * 0.5;
		fwrite( &xllcenter, sizeof(double),1,fp);
		fwrite( &yllcenter, sizeof(double),1,fp);
		DATA_TYPE type = LONG_TYPE;
		fwrite( &type, sizeof(DATA_TYPE),1,fp);
		fwrite( &nodata, sizeof(int),1,fp);
		
		for( int p = 0; p < MAX_STRING_LENGTH; p++ )
		{		projection[p] = 'A';
		}
		fwrite(&projection, sizeof(char), MAX_STRING_LENGTH, fp );
		for(p = 0; p < MAX_STRING_LENGTH; p++ )
		{		notes[p] = 'A';
		}	
		fwrite(&notes, sizeof(char), MAX_STRING_LENGTH, fp );

		iarr=(int **)data;



			for(  i = 0; i < ny; i++ )
			{	for(  j = 0; j < nx; j++ )
				{	
				    
					fwrite(&iarr[j][i],sizeof(long),1,fp);
				}				
			}

			

			fclose( fp );
			return 0;			
				
  }

  else
  {
/*  ESRI grid file  */
     GridIOSetup();
	 AccessWindowClear();  /*  Forget about any window it used to have  */
     if (CellLyrExists(file))
     {
/*  Save old file as a backup and use new file name  */
        sprintf(fline,"%s%s",file,"_b");
        if (CellLyrExists(fline))GridDelete(fline);
        GridRename(file,fline);
//        printf("Output grid %s exists and will be overwritten.\n",file);
//        printf("The old grid is preserved as %s\n",fline);
        GridDelete(file);
     } 
       type=CELLINT;
     if((channel1 = CellLayerCreate(file,WRITEONLY,ROWIO,type,csize,dem.head.bndbox)) < 0)
     {
//        printf ("Could not cread output grid %s\n",file);
        CellLyrClose(channel1);
        GridIOExit();
        return(5);
      }
     if(AccessWindowSet(dem.head.bndbox, csize, adjbndbox) < 0)
     {
//        printf ("Could not set Window\n");
       CellLyrClose(channel1);
       CellLyrDelete (file);
       GridIOExit();
        return(5);
     }
/*  Allocate row buffer  */
    if((rowbuf = (CELLTYPE *)CAllocate1(nx,sizeof(CELLTYPE))) == NULL)
    {
//        printf("Could not allocate buffer memory\n");
        CellLyrClose(channel1);
        CellLyrDelete (file);
        GridIOExit();
        return(5);
     }
     
          register int *buf = (int *)rowbuf;
          
             iarr = (int **)data;
             for(i=0; i< ny; i++)
             {
                   for(j=0; j < nx; j++)
                   {
                      buf[j] = (int) iarr[j][i];
					  if(buf[j] == nodata) buf[j]=MISSINGINT;
                   }
                   PutWindowRow (channel1, i, rowbuf);
             }
          
       
       
       CFree1 ((char *)rowbuf);
       CellLyrClose(channel1);
       GridIOExit();
   }
   return(0);
}



//=====================================
//New gridwrite function:Ajay
//For Short Grid
//======================================
int gridwrite(char *file,sgrid dem,int infiletype)
{
FILE *fp;

    int i, j,filetype;
    char fline[MAXLN];
    short **sarr,nodata;
    double csize,dx,dy,adjbndbox[4],utme,utmn,xllcenter,yllcenter;
    int nx,ny,channel1,type,datatype;
    CELLTYPE *rowbuf;
	void **data;
//=========================================
	//assigning to variables
	dx=dem.head.dx;
	dy=dem.head.dy;
	nx=dem.head.nx;
	ny=dem.head.ny;
	data=(void **)(dem.d);
	datatype=RPSHRDTYPE;
	nodata=dem.nodata;
	csize=dx;
/*  File types are
    0=ASCII   
    1= ARCVIEW grid via the ESRI Application Programmers Interface  
	2= USU binarg grid   */
//DGT added 3/19/05
//  Rather than use the filetype variable check file name for on the fly type conversions
	filetype=infiletype;  // First default is from the calling program
	CString strFileName= CString(file); 
	CString strFileExtension= strFileName.Right(4); 
	CString strLetter=strFileExtension.Left(1);
//  if there is an extension the filetype is ASCII by default
	if (strLetter.CompareNoCase(CString("."))==0)filetype=0;
//  if the extension is exactly .bgd the filetype is USU bgd
	if (strFileExtension.CompareNoCase(CString(".bgd"))==0)filetype=2;
//  The only way a file can be ESRI is if the calling program indicated this and there is not a . 4 characters from the right.

  if(filetype == 0)   /*  ASCII FILE  */
  {
    /* open ARC-Info file */
    fp = fopen(file,"w");
    if(fp == NULL)
    {
//        printf("\nERROR: Cannot open output file (%s).\n\n",file);
        return(4);
    }
    
   
    /* write ARC-Info header */
    fprintf(fp,"ncols\t%d\n",nx);
    fprintf(fp,"nrows\t%d\n",ny);
    utme=dem.head.bndbox[0]+dx*0.5;
    utmn=dem.head.bndbox[1]+dy*0.5;
    fprintf(fp,"xllcenter\t%f\n",utme);
    fprintf(fp,"yllcenter\t%f\n",utmn);
    fprintf(fp,"cellsize\t%g\n",csize);
    fprintf(fp,"nodata_value\t%d\n",nodata);
    
    /* write raster data to ARC/INFO file */
    /* convert depending on datatype */
     
        sarr = (short **) data;       
        for(i=0; i< ny; i++)
        {
            for(j=0; j< nx; j++)
            {
                fprintf(fp,"%hd ",sarr[j][i]);
                if((j+1) % LINELEN == 0)fprintf(fp,"\n");
            }
            fprintf(fp,"\n");
        }
    
   
    fclose(fp);
  }
 else if (filetype==2)
	
  {
		fp = fopen(file,"wb");
		if( !fp )
		{	
			return 1;
		}
		
	
		fwrite( &nx, sizeof(int),1,fp);
		fwrite( &ny, sizeof(int),1,fp);
		fwrite( &dx, sizeof(double),1,fp);
		fwrite( &dy, sizeof(double),1,fp);
		xllcenter = dem.head.bndbox[0] + dx * 0.5;
		yllcenter = dem.head.bndbox[1] + dy * 0.5;
		fwrite( &xllcenter, sizeof(double),1,fp);
		fwrite( &yllcenter, sizeof(double),1,fp);
		DATA_TYPE type = SHORT_TYPE;
		fwrite( &type, sizeof(DATA_TYPE),1,fp);
		fwrite( &nodata, sizeof(short),1,fp);
	
		for( int p = 0; p < MAX_STRING_LENGTH; p++ )
		{		projection[p] = 'A';
		}
		fwrite(&projection, sizeof(char), MAX_STRING_LENGTH, fp );
		for(p = 0; p < MAX_STRING_LENGTH; p++ )
		{		notes[p] = 'A';
		}	
		fwrite(&notes, sizeof(char), MAX_STRING_LENGTH, fp );

			sarr=(short **)data;
	
			for( int i = 0; i < ny; i++ )
			{	for( int j = 0; j < nx; j++ )
				{						
					fwrite(&sarr[j][i],sizeof(short),1,fp);
				}				
			}
			fclose( fp );
			return 0;			
				
  }

  else
  {
/*  ESRI grid file  */
     GridIOSetup();
	 AccessWindowClear();  /*  Forget about any window it used to have  */
     if (CellLyrExists(file))
     {
/*  Save old file as a backup and use new file name  */
        sprintf(fline,"%s%s",file,"_b");
        if (CellLyrExists(fline))GridDelete(fline);
        GridRename(file,fline);
//        printf("Output grid %s exists and will be overwritten.\n",file);
//        printf("The old grid is preserved as %s\n",fline);
        GridDelete(file);
     } 

       type=CELLINT;

     if((channel1 = CellLayerCreate(file,WRITEONLY,ROWIO,type,csize,dem.head.bndbox)) < 0)
     {
//        printf ("Could not cread output grid %s\n",file);
        CellLyrClose(channel1);
        GridIOExit();
        return(5);
      }
     if(AccessWindowSet(dem.head.bndbox, csize, adjbndbox) < 0)
     {
//        printf ("Could not set Window\n");
       CellLyrClose(channel1);
       CellLyrDelete (file);
       GridIOExit();
        return(5);
     }
/*  Allocate row buffer  */
    if((rowbuf = (CELLTYPE *)CAllocate1(nx,sizeof(CELLTYPE))) == NULL)
    {
//        printf("Could not allocate buffer memory\n");
        CellLyrClose(channel1);
        CellLyrDelete (file);
        GridIOExit();
        return(5);
     }
    
          register int *buf = (int *)rowbuf;
          
             sarr = (short **)data;
            for(i=0; i < ny; i++)
            {
                   for(j=0; j < nx; j++)
                   {
					   buf[j] = (int)sarr[j][i];

                      if(buf[j] == nodata)buf[j]=MISSINGINT;
                   }
                   PutWindowRow (channel1, i, rowbuf);
            }
          
       
     
	 
		CFree1 ((char *)rowbuf);
       CellLyrClose(channel1);
       GridIOExit();
   }
   return(0);



}


void eol(FILE *fp)
{
    char ch;
    
    while(1)
    {
      if((ch = getc(fp)) == '\n') return;
      
      if(ch == EOF) { fseek(fp, -1L, SEEK_CUR); return; }
    }
}



//Code added here:Ajay
//Start creation of grid - put in allocategrid function part of gridio.cpp
//	  purpose is to replace matalloc within the code..tardemlib.cpp


//
//......................................-----------------============================
//Allocate function which allocates nodata  value to grids.Just like grid read it should take float,int and short type
////----------------------------------************************-----------------------------	
	

	int allocategrid(fgrid *grid, ghead head, float nodatavalue)
	  {
		  int i,j;
		  grid->head = head;
		  grid->d = (float **)  matalloc(head.nx,head.ny,RPFLTDTYPE);

		  //  check that matalloc worked and if not return 1
		  

		  if(grid->d == NULL) return(1);

		  grid->nodata= nodatavalue;
	  for(i=0; i<head.ny; i++)
	  {
	   for(j=0; j<head.nx; j++)
		{
		   grid->d[j][i]= nodatavalue;
		}
	  }
	  return 0;
	  }

	int allocategrid(igrid *grid, ghead head, int nodatavalue)
	  {
		  int i,j;
		  grid->head = head;
		  grid->d = (int **)  matalloc(head.nx,head.ny,RPINTDTYPE);

		  //  check that matalloc worked and if not return 1

		  if(grid->d == NULL)return(1);

		  grid->nodata= nodatavalue;
      for(i=0; i<head.ny; i++)
	  {
       for(j=0; j<head.nx; j++)
        {
           grid->d[j][i]= nodatavalue;
        }
	  }
	  return 0;
	  } 
int allocategrid(sgrid *grid, ghead head, short nodatavalue)
	  {
		  int i,j;
		  grid->head = head;
		  grid->d = (short **)  matalloc(head.nx,head.ny,RPSHRDTYPE);

		  //  check that matalloc worked and if not return 1

		  if(grid->d == NULL)return(1);

		  grid->nodata= nodatavalue;
      for(i=0; i<head.ny; i++)
	  {
       for(j=0; j<head.nx; j++)
        {
           grid->d[j][i]= nodatavalue;
        }
	  }
	  return 0;
	  } 




//============================================================================
//New Header read function written by Ajay
//============================================================================
int readHeader(char *ipFile,ghead *head,short *shortnodata,int *intnodata,float *floatnodata,int *datatype,int *filetype)
{
	int nx=0,ny=0, hdrlines = 0;
    int channel1,iesri;
	double dx,dy,xllcenter=0,yllcenter=0,adjbndbox[4],utme,utmn;
	double *csize=new double;
	FILE *ipf;
	float value;
	char fline[MAXLN], keyword[21], utmetag, utmntag;


	float floatNull;
	if( iesri = GridIOSetup() >= 0)
	{
		if ((channel1 = 
				  CellLayerOpen(ipFile,READONLY,ROWIO,
								datatype,csize)) >= 0 )
		{
			 head->dx = head->dy = (double) *csize;
			 
			if (BndCellRead (ipFile, head->bndbox) < 0)
			{
	//                printf ("could not read bounding box of input grid\n");
					CellLyrClose(channel1);
					GridIOExit();
					return(1);
			}
			iesri=AccessWindowSet (head->bndbox, *csize, adjbndbox);
			if ( iesri< 0)
			{
//                printf ("Could not set Window\n");
                CellLyrClose(channel1);
                GridIOExit();
                return(2);
			}
			GetMissingFloat(&floatNull);

			if (*datatype==0) *shortnodata=MISSINGSHORT;
			if (*datatype==1) *intnodata=MISSINGINT;
			if (*datatype==2) *floatnodata=floatNull;

			head->nx = WindowCols();
			head->ny = WindowRows();
            CellLyrClose(channel1);  
        /*
        Done with the library                                   (15)
        */
            GridIOExit();
			*filetype=1;
	}
	else
	{
		CString strFileName= CString(ipFile); 
		CString strFileExtension= strFileName.Right(4); 

		if (strFileExtension.CompareNoCase(CString(".bgd"))==0)
		
		{
		ipf=fopen(ipFile,"rb");
		if (ipf==NULL) return (1);

			fread(&nx, sizeof(int),1,ipf);
			head->nx=nx;
			fread(&ny, sizeof(int),1,ipf);
			head->ny=ny;
			fread(&dx, sizeof(double),1,ipf);
			head->dx=dx;
			fread(&dy, sizeof(double),1,ipf);
			head->dy=dy;
			fread(&xllcenter, sizeof(double),1,ipf);
			fread(&yllcenter, sizeof(double),1,ipf);
			

			head->bndbox[0] = xllcenter-(dx/2);
			head->bndbox[1] = yllcenter-(dy/2);
			head->bndbox[2] = head->bndbox[0] + dx * (nx);
			head->bndbox[3] = head->bndbox[1] + dy * (ny);

	 		fread(datatype, sizeof(DATA_TYPE),1,ipf);
			
			if (*datatype==0) fread(shortnodata,sizeof(short),1,ipf);
				
			if (*datatype==1) fread(intnodata,sizeof(long),1,ipf);
				
			if (*datatype==2) fread(floatnodata,sizeof(float),1,ipf);
			*filetype=2;
				
		}
		
		else if (strFileExtension.CompareNoCase(CString(".asc"))==0)
		{
			*datatype=2;
	*filetype=0;  
	FILE *fp;

    fp = fopen(ipFile,"r");
    if(fp == NULL)
    {
//        printf("\nERROR: Cannot open input file (%s).\n\n",file);
        return(1);
    }
    

			
	while(1)
    {   
        readline(fp, fline);       
        if(!isalpha(*fline) || *fline == '-')
            break;       
        
        hdrlines++;

        sscanf(fline,"%s %f",keyword,&value);
        
	if(strcmp(keyword,"ncols") == 0 || strcmp(keyword,"NCOLS") == 0)
	    nx = (int)value;
	else if(strcmp(keyword,"nrows") == 0 || strcmp(keyword,"NROWS") == 0)
	    ny = (int)value;
	else if(strcmp(keyword,"xllcenter") == 0 || strcmp(keyword,"XLLCENTER") == 0)
	{
	    utmetag = 'c';
	    utme = value;
	}
	else if(strcmp(keyword,"xllcorner") == 0 || strcmp(keyword,"XLLCORNER") == 0)
	{
	    utmetag = 'e';
	    utme = value;
	}
	else if(strcmp(keyword,"yllcenter") == 0 || strcmp(keyword,"YLLCENTER") == 0)
	{
	    utmntag = 'c';
	    utmn = value;
	}
	else if(strcmp(keyword,"yllcorner") == 0 || strcmp(keyword,"YLLCORNER") == 0)
	{
	    utmntag = 'e';
	    utmn = value;
	}
	else if(strcmp(keyword,"cellsize") == 0 || strcmp(keyword,"CELLSIZE") == 0)
	{
	    dx = dy = value;
	    *csize = (double) value;
	}
	else if(strcmp(keyword,"nodata_value") == 0 || strcmp(keyword,"NODATA_VALUE") == 0 ||
		strcmp(keyword,"NODATA_value") == 0)
	    *floatnodata = (float)value;
    }
    
    /* adjust utme and utmn if necessary (we store center of reference cell) */
    if(utmetag == 'e') utme = utme + dx/2;
    if(utmntag == 'e') utmn = utmn + dy/2;
     head->bndbox[0] = utme - *csize/2.;   
     head->bndbox[1] = utmn - *csize/2.;
     head->bndbox[2] = head->bndbox[0] + *csize * (nx);
     head->bndbox[3] = head->bndbox[1] + *csize * (ny);
	fclose (fp);




		}
		else 
			return(1);
		}
	}

	return (0);
	
}
int griddelete(char* ipfile)
{
	double *csize = new double;
	int type,channel1,iesri;

	
if( iesri = GridIOSetup() >= 0)
{      /*
        Open input cell layer with no evaluation        (5)
        */
        if ((channel1 = 
              CellLayerOpen(ipfile,READONLY,ROWIO,
                            &type,csize)) >= 0 )
		{
			if(GridDelete(ipfile)!=0) 
			{
				CellLyrClose(channel1);
				GridIOExit();
				return 1;
			}
			CellLyrClose(channel1);
			GridIOExit();
			return 0;
		}
}
		CString strFileName= CString(ipfile); 
		CString strFileExtension= strFileName.Right(4); 

		if(strFileExtension.CompareNoCase(CString(".bgd"))==0 || strFileExtension.CompareNoCase(CString(".asc"))==0)
		{
			if(!DeleteFile(ipfile)) return 1;
		}
		return 0;
}

int gridrename(char* ipfile,char* opfile)
{
	double *csize = new double;
	int type,channel1,iesri;

	
if( iesri = GridIOSetup() >= 0)
{   /*
        Open input cell layer with no evaluation        (5)
        */
        if ((channel1 = 
              CellLayerOpen(ipfile,READONLY,ROWIO,
                            &type,csize)) >= 0 )
		{
			if(GridRename(ipfile,opfile)!=0) return 1;
			CellLyrClose(channel1);
			GridIOExit();
			return 0;
		}
}
		
		CString strFileName= CString(ipfile); 
		CString strFileExtension= strFileName.Right(4); 

		if(strFileExtension.CompareNoCase(CString(".bgd"))==0 || strFileExtension.CompareNoCase(CString(".asc"))==0)
		{
			if(rename(ipfile,opfile)!=0) return 1;
			
		}
		return 0;
}
int gridexists(char* ipfile)
{
	double *csize = new double;
	int type,channel1,iesri;
if( iesri = GridIOSetup() >= 0)
{
	if ((channel1 = 
              CellLayerOpen(ipfile,READONLY,ROWIO,
                            &type,csize)) >= 0 )
		{
			if(GridExists(ipfile)==0) return 1;
			CellLyrClose(channel1);
			GridIOExit();
			return 0;
		}
}
		CString strFileName= CString(ipfile); 
		CString strFileExtension= strFileName.Right(4); 

		if(strFileExtension.CompareNoCase(CString(".bgd"))==0 || strFileExtension.CompareNoCase(CString(".asc"))==0)
		{   FILE * fp;
			fp=fopen(ipfile,"r");
			if(fp) 
			{
				fclose(fp);
				return 1;
			}
			
		}
		return 0;

}
///////////////////////////////////////////////////////////////////////////
int creategrid(char* demfile,char* opgridfile,int gridtype,int cellvalue)
{
	fgrid fopg;
	igrid iopg;
	sgrid sopg;
	int ftype;
	fgrid demgrid;
	
	if(gridread(demfile,&demgrid,&ftype) !=0) return 1;
	

	if (gridtype==1)
	{
	sopg.head=demgrid.head;
	if(allocategrid(&sopg,demgrid.head,0)!=0) return 1;

	for( int i = 0; i < demgrid.head.ny; i++ )
		{	for( int j = 0; j < demgrid.head.nx; j++ )
			{
			 sopg.d[j][i]=(demgrid.d[j][i]==demgrid.nodata)? MISSINGSHORT :(short)cellvalue;
			}				
		}
	sopg.nodata=MISSINGSHORT;
	
	if (gridwrite(opgridfile,sopg,ftype)!=0) return 1;
	}	
	else if(gridtype==2)
	{	iopg.head=demgrid.head;
	
		if(allocategrid(&iopg,demgrid.head,0)!=0) return 1;
			for( int i = 0; i < demgrid.head.ny; i++ )
		{	for( int j = 0; j < demgrid.head.nx; j++ )
			{
			 iopg.d[j][i]=(demgrid.d[j][i]==demgrid.nodata)? MISSINGINT :cellvalue;
			}				
		}
		iopg.nodata=MISSINGINT;
		if (gridwrite(opgridfile,iopg,ftype)!=0) return 1;
	}
	else if(gridtype==3)
	{	
		fopg.head=demgrid.head;
		if(allocategrid(&fopg,demgrid.head,0.)!=0) return 1;
			for( int i = 0; i < demgrid.head.ny; i++ )
		{	for( int j = 0; j < demgrid.head.nx; j++ )
			{
			 fopg.d[j][i]=(demgrid.d[j][i]==demgrid.nodata)? demgrid.nodata :(float)cellvalue;
			}				
		}
		fopg.nodata=demgrid.nodata ;
		if (gridwrite(opgridfile,fopg,ftype)!=0) return 1;
	}
	else
		return 1;

	

	return 0;


}
/*********************************************************************/

//Check datatype function :Ajay
int checkdatatype(char *ipFile)
{  
	int datatype,filetype,checkerror;
	ghead head;
	short shortnodata;
	int intnodata;
	float floatnodata;
	checkerror=readHeader(ipFile,&head,&shortnodata,&intnodata,&floatnodata,&datatype,&filetype);
	if (checkerror!=0) return (TD_FAILED_GRID_OPEN);
	return datatype;
}

int subsetgrid(char *bigGrid /*Path the big grid */, char *newgrid/*Path the new grid to be created*/, double *newbndbox/*Bound box to cut the big grid*/){
/*
This function added to be used in the VB program developed for Watershed classification. Given a big grid,
this function cuts the big grid according to the boundbox input.
*/	

	int filetype, err=0;
	sgrid sdem;
	igrid idem;
	fgrid fdem;
	filetype=checkdatatype(bigGrid);
	

	switch( filetype ) 
	{
		case 0:
			{
				if ( gridblockread(bigGrid,&sdem, newbndbox, &filetype) != 0)return TD_FAILED_GRID_OPEN	;
				else
				{
					if ( gridwrite(newgrid,sdem,filetype)==0 ){

						free(sdem.d[0]); free(sdem.d);
						return TD_NO_ERROR;
					}
					else return TD_FAILED_GRID_SAVE;
				}
			}
		case 1:
			{
				if ( gridblockread(bigGrid,&idem, newbndbox, &filetype) != 0)return TD_FAILED_GRID_OPEN	;
				else
				{
					if ( gridwrite(newgrid,idem,filetype)==0 ){
						free(idem.d[0]); free(idem.d);	
						return TD_NO_ERROR;
					}
					else return TD_FAILED_GRID_SAVE;
				}
			}
		case 2:
			{
				if ( gridblockread(bigGrid,&fdem, newbndbox, &filetype) != 0)return TD_FAILED_GRID_OPEN	;
				else
				{
					if ( gridwrite(newgrid,fdem,filetype)==0 ){
						free(fdem.d[0]); free(fdem.d);	
						return TD_NO_ERROR;
					}
					else return TD_FAILED_GRID_SAVE;
				}
			}
		default :
			return 1;
	}


	return 0;
}

//********************************************************************
//**********************gridblcread()..by Kiran***********************
//To read smaller blocks from grids. Eqvivalent to chipping of a grid
//********************************************************************
//INT grids
int  gridblockread(char *file, igrid* dem ,double blkbndbox[4],int *filetype)
{
FILE *fp;
	int channel1,type,iesri;
	double *csize = new double;
    float value;
	float floatNull;
	double xllcenter,yllcenter;
    char fline[MAXLN], keyword[21], utmetag, utmntag;
    int **iarr, *nx,*ny,datatype,*nodata,i, j, hdrlines = 0;
 	double adjbndbox[4],utme,utmn,*dx,*dy;
	void ***data;
	CELLTYPE *rowbuf;
//=========================================
	//assigning to variables
	dx=&(dem->head.dx);
	dy=&(dem->head.dy);
	nx=&(dem->head.nx);
	ny=&(dem->head.ny);
	data=(void ***)&(dem->d);
	datatype=RPINTDTYPE;
	nodata=&dem->nodata;
//===============================================================

    if( iesri = GridIOSetup() >= 0)
        /*
        Open input cell layer with no evaluation        (5)
        */
        if ((channel1 = 
              CellLayerOpen(file,READONLY,ROWIO,
                            &type,csize)) >= 0 )
    {
/*  The success of these means the file could be opened as an ESRI grid file  */
        *filetype=1;
/*  File types are
    0=ASCII   
    1= ARCVIEW grid via the ESRI Application Programmers Interface  */ 
/*  here is the ESRI grid file read stuff - following copyrow example */
        *dx = *dy = (float) *csize;
/*
        
        /*
        Get the bounding box of the input cell layer            (7)
        */
        if (BndCellRead (file, dem->head.bndbox) < 0)
        {
//                printf ("could not read bounding box of input grid\n");
                CellLyrClose(channel1);
                GridIOExit();
                return(1);
        }
/*        printf("%f %f %f %f %g\n",bndbox[0],bndbox[1],bndbox[2],bndbox[3],*nodata) */; 
/*  Bounding box is xllcorner, yllcorner, xurcorner, yurcorner   */
      
        /*
        Set the Window to the output bounding box and cellsize  (9)
        */
        if (AccessWindowSet (blkbndbox, *csize, adjbndbox) < 0)//Kiran...new nlkbndbox supplied
        {
//                printf ("Could not set Window\n");
                CellLyrClose(channel1);
                GridIOExit();
                return(2);
        }
        /*
        Get the number of rows and columns                      (10)
        in the window
        */
        *nx = WindowCols();
        *ny = WindowRows();

        /*  Allocate memory and set all type pointers  */
        *data = matalloc(*nx, *ny, datatype);
		if(*data == NULL)return(1);  //DGT 9/13/2007 Bail of not able to allocate memory

        iarr = (int **) (*data);
        GetMissingFloat(&floatNull);
		*nodata=MISSINGINT;
		dem->head.bndbox[0] = adjbndbox[0];
		dem->head.bndbox[1] = adjbndbox[1];
		dem->head.bndbox[2] = adjbndbox[2];
		dem->head.bndbox[3] = adjbndbox[3];
        /*
		     /*
        Allocate row buffer                                     (11)
        */
        if ((rowbuf = (CELLTYPE *)
                        CAllocate1 ( *nx, sizeof(CELLTYPE)))
                        == NULL )
        {
//                printf ("Could not allocate memory\n");
                CellLyrClose(channel1);
                GridIOExit();
                return(3);
        }
        /*
        Now copy row into array                 (12)
        */
        for (i=0; i < *ny; i++)
        {
             GetWindowRow (channel1, i, rowbuf);
             if(type == CELLFLOAT)
				 /*     This is to typecast from ESRI float to int for a int grid.   */
             {  
				 register float *buf = (float *)rowbuf;
                
                 
                   for(j=0; j < *nx; j++)
                   {
                      iarr[j][i]=(buf[j] == floatNull) ?  *nodata : (int)buf[j]; 
                   }
                    
	           }
             else if(type==CELLINT)
             {
                register int *buf = (int *)rowbuf;
								

                   for(j=0; j < *nx; j++)
                   {
                      iarr[j][i]= buf[j] ;
                   }
 	           }
			 }
	
        /*
        Free row buffer                                         (13)
        */
        CFree1 ((char *)rowbuf);
        /*
        Close handles                                           (14)
        */
        CellLyrClose(channel1);  
        /*
        Done with the library                                   (15)
        */
        GridIOExit();
        return(0);
    }

	
	
/*  Here assume file is ASCII or Binary.  Close ESRI stuff. */
   


	CellLyrClose(channel1);
    GridIOExit();

//=========================================================
//Code for bgd .Ajay

//Checking the extension of the input file

    CString strFileName= CString(file); 
	CString strFileExtension= strFileName.Right(4); 

	if (strFileExtension.CompareNoCase(CString(".bgd"))==0)
{
    //Success means bgd 
		*filetype=2;
		fp=fopen(file,"rb");

			if(fp == NULL)
			{
		//        printf("\nERROR: Cannot open input file (%s).\n\n",file);
				return(1);
			}
    
        rewind(fp);
			
	//  CODE TO ASSIGN VALUES READ TO APPROPRIATE GRID HEADER VARIABLES
		fread(nx, sizeof(int),1,fp);
		fread(ny, sizeof(int),1,fp);
		fread(dx, sizeof(double),1,fp);
		fread(dy, sizeof(double),1,fp);
	//Bound box
		fread(&xllcenter, sizeof(double),1,fp);
		fread(&yllcenter, sizeof(double),1,fp);
	 // COMPUTE BNDBOX FROM XLLCENTER, YLLCENTER.
			dem->head.bndbox[0]=xllcenter-(*dx/2);
			dem->head.bndbox[1]=yllcenter-(*dy/2);
			dem->head.bndbox[2]=dem->head.bndbox[0] + *dx * (*nx);
			dem->head.bndbox[3]=dem->head.bndbox[1] + *dy * (*ny);
	/////////////////////////////////////////////
		fread(&datatype, sizeof(DATA_TYPE),1,fp);
	
		
		if(datatype==SHORT_TYPE)
		{	short snodata;
			fread(&snodata,sizeof(short),1,fp);
			*nodata=(int)snodata;
		}
		float fnodata;
		if(datatype==FLOAT_TYPE)
		{	
			fread(&fnodata,sizeof(float),1,fp);
			*nodata=MISSINGINT;
		}
		if(datatype==LONG_TYPE)fread(nodata,sizeof(long),1,fp);
				
		fread(projection,sizeof(char), MAX_STRING_LENGTH, fp);
		fread(notes, sizeof(char), MAX_STRING_LENGTH, fp);

			if( *nx < 0 || *ny < 0 || 
				*dx< 0 || *dy < 0)
				{	
					fclose( fp );
					return 1;
				}
			// here ,1 is  added to the datatype as fread returns datatype 1 less than matalloc datatypes.
				
	
			iarr =(int **) matalloc(*nx, *ny, LONG_TYPE + 1);
			if(iarr == NULL)return(1);  //DGT 9/13/2007 Bail of not able to allocate memory

			if(datatype==SHORT_TYPE)
				{
				short shortarr;
				
					for( int i = 0; i < *ny; i++ )
					{	for( int j = 0; j < *nx; j++ )
						{	
							if( feof(fp) )
							{	
								fclose( fp );
								return 1;
							}
						
						fread( &shortarr,sizeof(short),1,fp );
						iarr[j][i]=(int)shortarr;
						}
					}
				}
				else if(datatype==FLOAT_TYPE)
				{
					
				float floatarr;	
				
					for( int i = 0; i < *ny; i++ )
					{	for( int j = 0; j < *nx; j++ )
						{	
							if( feof(fp) )
							{	
								fclose( fp );
								return 1;
							}
						
						fread( &floatarr,sizeof(float),1,fp );
						iarr[j][i]=(floatarr == fnodata) ?  *nodata : (int)floatarr;
						}
					}
				}
				else if(datatype==LONG_TYPE)
				{
					
					for( int i = 0; i < *ny; i++ )
					{	for( int j = 0; j < *nx; j++ )
						{	
							if( feof(fp) )
							{	
								fclose( fp );
								return 1;
							}
								fread(&iarr[j][i],sizeof(long),1,fp);
								
						}
					}
				}
				else return 1;

			*data = (void **) iarr;
			fclose(fp);
			return 0;			

		 

	}



//================================================================
//If the file is ascii.
	*filetype=0;  
    fp = fopen(file,"r");
    if(fp == NULL)
    {
//        printf("\nERROR: Cannot open input file (%s).\n\n",file);
        return(1);
    }
    
	
    /* read ARC-Info header */
    while(1)
    {   
        readline(fp, fline);       
        if(!isalpha(*fline) || *fline == '-')
            break;       
        
        hdrlines++;

        sscanf(fline,"%s %f",keyword,&value);
        
	if(strcmp(keyword,"ncols") == 0 || strcmp(keyword,"NCOLS") == 0)
	    *nx = (int)value;
	else if(strcmp(keyword,"nrows") == 0 || strcmp(keyword,"NROWS") == 0)
	    *ny = (int)value;
	else if(strcmp(keyword,"xllcenter") == 0 || strcmp(keyword,"XLLCENTER") == 0)
	{
	    utmetag = 'c';
	    utme = value;
	}
	else if(strcmp(keyword,"xllcorner") == 0 || strcmp(keyword,"XLLCORNER") == 0)
	{
	    utmetag = 'e';
	    utme = value;
	}
	else if(strcmp(keyword,"yllcenter") == 0 || strcmp(keyword,"YLLCENTER") == 0)
	{
	    utmntag = 'c';
	    utmn = value;
	}
	else if(strcmp(keyword,"yllcorner") == 0 || strcmp(keyword,"YLLCORNER") == 0)
	{
	    utmntag = 'e';
	    utmn = value;
	}
	else if(strcmp(keyword,"cellsize") == 0 || strcmp(keyword,"CELLSIZE") == 0)
	{
	    *dx = *dy = value;
	    *csize = (double) value;
	}
	else if(strcmp(keyword,"nodata_value") == 0 || strcmp(keyword,"NODATA_VALUE") == 0 ||
		strcmp(keyword,"NODATA_value") == 0)
	    *nodata = (int) value;
    }
    
    /* adjust utme and utmn if necessary (we store center of reference cell) */
    if(utmetag == 'e') utme = utme + *dx/2;
    if(utmntag == 'e') utmn = utmn + *dy/2;
     dem->head.bndbox[0] = utme - *csize/2.;   
     dem->head.bndbox[1] = utmn - *csize/2.;
     dem->head.bndbox[2] = dem->head.bndbox[0] + *csize * (*nx);
     dem->head.bndbox[3] = dem->head.bndbox[1] + *csize * (*ny);
    /* position file pointer for ARC-Info file to beginning of image data */
    rewind(fp);
    for(i=0; i<hdrlines; i++) readline(fp, fline);
    
       iarr = (int **) matalloc(*nx, *ny, datatype);
       if(iarr == NULL)return(1);  //DGT 9/13/2007 Bail of not able to allocate memory
        
        for(i=0; i< *ny; i++)
        {
            for(j=0; j< *nx; j++)
			{
               if(EOF==fscanf(fp,"%d",&iarr[j][i])) return(1);  // error trapping for float data
			}
        }
        *data = (void **) iarr;
     
    
    fclose(fp);
    return(0);

}
//blk read for short grid...Kiran
int gridblockread(char *file, sgrid* dem ,double blkbndbox[4], int *filetype)
{
	FILE *fp;
	int channel1,type,iesri;
	double *csize = new double,*dx,*dy;
    int i, j, hdrlines = 0,*nx,*ny,datatype;
    float value;
	float floatNull;
    char fline[MAXLN], keyword[21], utmetag, utmntag;
    short **sarr,*nodata;
    double adjbndbox[4],utme,utmn,xllcenter=0,yllcenter=0;
    CELLTYPE *rowbuf;
	void ***data;
//=========================================
	//assigning to variables
	
///////////////////////
	
	dx=&(dem->head.dx);
	dy=&(dem->head.dy);
	nx=&(dem->head.nx);
	ny=&(dem->head.ny);
	data=(void ***)&(dem->d);
	datatype=RPSHRDTYPE;
	nodata=&dem->nodata;
	
//===============================================================

      
    if( iesri = GridIOSetup() >= 0)
        /*
        Open input cell layer with no evaluation        (5)
        */
        if ((channel1 = 
              CellLayerOpen(file,READONLY,ROWIO,
                            &type,csize)) >= 0 )
    {
/*  The success of these means the file could be opened as an ESRI grid file  */
        *filetype=1;
/*  File types are
    0=ASCII   
    1= ARCVIEW grid via the ESRI Application Programmers Interface  */ 
/*  here is the ESRI grid file read stuff - following copyrow example */
        *dx = *dy = (double) *csize;
/*
        
        /*
        Get the bounding box of the input cell layer            (7)
        */
        if (BndCellRead (file, dem->head.bndbox) < 0)
        {
//                printf ("could not read bounding box of input grid\n");
                CellLyrClose(channel1);
                GridIOExit();
                return(1);
        }
/*        printf("%f %f %f %f %g\n",bndbox[0],bndbox[1],bndbox[2],bndbox[3],*nodata) */; 
/*  Bounding box is xllcorner, yllcorner, xurcorner, yurcorner   */
      
        /*
        Set the Window to the output bounding box and cellsize  (9)
        */
        if (AccessWindowSet (blkbndbox, *csize, adjbndbox) < 0)
        {
//                printf ("Could not set Window\n");
                CellLyrClose(channel1);
                GridIOExit();
                return(2);
        }
        /*
        Get the number of rows and columns                      (10)
        in the window
        */
        *nx = WindowCols();
        *ny = WindowRows();

        /*  Allocate memory and set all type pointers  */
        *data = matalloc(*nx, *ny, datatype);
		if(*data == NULL)return(1);  //DGT 9/13/2007 Bail of not able to allocate memory

        sarr = (short **) (*data);
        GetMissingFloat(&floatNull);
        *nodata=MISSINGSHORT;  
		dem->head.bndbox[0] = adjbndbox[0];
		dem->head.bndbox[1] = adjbndbox[1];
		dem->head.bndbox[2] = adjbndbox[2];
		dem->head.bndbox[3] = adjbndbox[3];
        /*
        Allocate row buffer                                     (11)
        */
        if ((rowbuf = (CELLTYPE *)
                        CAllocate1 ( *nx, sizeof(CELLTYPE)))
                        == NULL )
        {
//                printf ("Could not allocate memory\n");
                CellLyrClose(channel1);
                GridIOExit();
                return(3);
        }
        /*
        Now copy row into array                 (12)
        */
        for (i=0; i < *ny; i++)
        {
             GetWindowRow (channel1, i, rowbuf);
             if(type == CELLFLOAT)
             {
				 register float *buf = (float *)rowbuf;
                
                   for(j=0; j < *nx; j++)
                   {
					    sarr[j][i]=(buf[j] == floatNull ) ? *nodata : (short) buf[j];   
                       
                   }
              }
             else if(type==CELLINT)
			 {
				register int *buf = (int *)rowbuf;
				for(j=0; j < *nx; j++)
                   
				{
					  sarr[j][i]=(buf[j] == MISSINGINT) ?  *nodata : (short) buf[j];   
                      
                   }
				
			 }
			}

        /*
        Free row buffer                                         (13)
        */
        CFree1 ((char *)rowbuf);
        /*
        Close handles                                           (14)
        */
        CellLyrClose(channel1);  
        /*
        Done with the library                                   (15)
        */
        GridIOExit();
        return(0);
    }

	
	
/*  Here assume file is ASCII or Binary.  Close ESRI stuff. */

    CellLyrClose(channel1);
    GridIOExit();
//=========================================================
//test code for bgd .Ajay
   
	CString strFileName= CString(file); 
	CString strFileExtension= strFileName.Right(4); 

	if (strFileExtension.CompareNoCase(CString(".bgd"))==0)
		
	{
	//Success means bgd 
		*filetype=2;
		fp=fopen(file,"rb");
			if(fp == NULL)
			{
		//        printf("\nERROR: Cannot open input file (%s).\n\n",file);
				return(1);
			}
    
        rewind(fp);
		//  CODE TO ASSIGN VALUES READ TO APPROPRIATE GRID HEADER VARIABLES

		fread(nx, sizeof(int),1,fp);
		fread(ny, sizeof(int),1,fp);
		fread(dx, sizeof(double),1,fp);
		fread(dy, sizeof(double),1,fp);
		fread(&xllcenter, sizeof(double),1,fp);
		fread(&yllcenter, sizeof(double),1,fp);
	 // COMPUTE BNDBOX FROM XLLCENTER, YLLCENTER.
		dem->head.bndbox[0]=xllcenter-(*dx/2);
		dem->head.bndbox[1]=yllcenter-(*dy/2);
		dem->head.bndbox[2]=dem->head.bndbox[0] + *dx * (*nx);
		dem->head.bndbox[3]=dem->head.bndbox[1] + *dy * (*ny);
		fread(&datatype, sizeof(DATA_TYPE),1,fp);
	//	fread(nodata, sizeof(short),1,fp);
		  long lnodata;
		if(datatype==LONG_TYPE)
		{ 
			fread(&lnodata,sizeof(long),1,fp);
			*nodata=MISSINGSHORT;
		}
		 float fnodata;
		if(datatype==FLOAT_TYPE)
		{  
			fread(&fnodata,sizeof(float),1,fp);
			*nodata=MISSINGSHORT;
		}
		if(datatype==SHORT_TYPE)fread(nodata,sizeof(short),1,fp);

		fread(projection,sizeof(char),MAX_STRING_LENGTH,fp);
		fread(notes, sizeof(char), MAX_STRING_LENGTH, fp);
  //CODE to check type is correct - 
	
			if( *nx < 0 || *ny < 0 || 
				*dx< 0 || *dy < 0)
				{	fclose( fp );
					return 1;
				}

		// here ,1 is  added to the datatype as fread returns datatype 1 less than matalloc datatypes.
			
	sarr= (short **)matalloc(*nx, *ny,SHORT_TYPE + 1);
	if(sarr == NULL)return(1);  //DGT 9/13/2007 Bail of not able to allocate memory

	if(datatype==LONG_TYPE)
	{		
			int intarr;
			
			for( int i= 0; i < *ny; i++ )
			{
				for( int j = 0; j < *nx ;j++ )
				{	
				if( feof(fp) )
					{	fclose( fp );
						return 1;
					}
				{
					fread( &intarr,sizeof(long),1,fp );
					 sarr[j][i]=(intarr == lnodata) ?  *nodata : (short)intarr;   
				}

				}
			}
	}
	else if(datatype==FLOAT_TYPE)
	{
		float floatarr;
		
			for( int i= 0; i < *ny; i++ )
			{
				for( int j = 0; j < *nx ;j++ )
				{	
				if( feof(fp) )
					{	fclose( fp );
						return 1;
					}
					{
						fread( &floatarr,sizeof(float),1,fp );
						sarr[j][i]=(floatarr == fnodata) ?  *nodata : (short)floatarr; 
					}

				}

			}
	}
	else if(datatype==SHORT_TYPE)
	{
		
		for( int i= 0; i < *ny; i++ )
			{
				for( int j = 0; j < *nx ;j++ )
				{	
				if( feof(fp) )
					{	fclose( fp );
						return 1;
					}
					{
						fread( &sarr[j][i],sizeof(short),1,fp );
					}

				}
			}
	}
	else return 1;


			*data = (void **) sarr;
			fclose(fp);
			return 0;			

}

//else assuming the file to be ascii
//================================================================
//If the file is ascii.

	*filetype=0;  
    fp = fopen(file,"r");
    if(fp == NULL)
    {
//        printf("\nERROR: Cannot open input file (%s).\n\n",file);
        return(1);
    }
    
	
    /* read ARC-Info header */
    while(1)
    {   
        readline(fp, fline);       
        if(!isalpha(*fline) || *fline == '-')
            break;       
        
        hdrlines++;

        sscanf(fline,"%s %f",keyword,&value);
        
	if(strcmp(keyword,"ncols") == 0 || strcmp(keyword,"NCOLS") == 0)
	    *nx = (int)value;
	else if(strcmp(keyword,"nrows") == 0 || strcmp(keyword,"NROWS") == 0)
	    *ny = (int)value;
	else if(strcmp(keyword,"xllcenter") == 0 || strcmp(keyword,"XLLCENTER") == 0)
	{
	    utmetag = 'c';
	    utme = value;
	}
	else if(strcmp(keyword,"xllcorner") == 0 || strcmp(keyword,"XLLCORNER") == 0)
	{
	    utmetag = 'e';
	    utme = value;
	}
	else if(strcmp(keyword,"yllcenter") == 0 || strcmp(keyword,"YLLCENTER") == 0)
	{
	    utmntag = 'c';
	    utmn = value;
	}
	else if(strcmp(keyword,"yllcorner") == 0 || strcmp(keyword,"YLLCORNER") == 0)
	{
	    utmntag = 'e';
	    utmn = value;
	}
	else if(strcmp(keyword,"cellsize") == 0 || strcmp(keyword,"CELLSIZE") == 0)
	{
	    *dx = *dy = value;
	    *csize = (double) value;
	}
	else if(strcmp(keyword,"nodata_value") == 0 || strcmp(keyword,"NODATA_VALUE") == 0 ||
		strcmp(keyword,"NODATA_value") == 0)
	    *nodata = (short)value;
    }
    
    /* adjust utme and utmn if necessary (we store center of reference cell) */
    if(utmetag == 'e') utme = utme + *dx/2;
    if(utmntag == 'e') utmn = utmn + *dy/2;
     dem->head.bndbox[0] = utme - *csize/2.;   
     dem->head.bndbox[1] = utmn - *csize/2.;
     dem->head.bndbox[2] = dem->head.bndbox[0] + *csize * (*nx);
     dem->head.bndbox[3] = dem->head.bndbox[1] + *csize * (*ny);
    /* position file pointer for ARC-Info file to beginning of image data */
    rewind(fp);
    for(i=0; i<hdrlines; i++) readline(fp, fline);
    
        sarr = (short **) matalloc(*nx, *ny, datatype);
 	    if(sarr == NULL)return(1);  //DGT 9/13/2007 Bail of not able to allocate memory
            
        /* read in the ARC-Info file */        
        for(i=0; i< *ny; i++)
        {
            for(j=0; j< *nx; j++)
               if( EOF == fscanf(fp,"%hd",&sarr[j][i])) return(1);
				
        }
        *data = (void **) sarr;
    fclose(fp);
    return(0);

}

//block read for float data..Kiran
// Float datatype
int gridblockread(char *file, fgrid *dem,double blkbndbox[4], int *filetype)
{

	FILE *fp;
	int channel1,type,iesri;
	double *csize = new double;
    int i, j, hdrlines = 0;
    float value;
    char fline[MAXLN], keyword[21], utmetag, utmntag;
    float **farr;
    double adjbndbox[4],utme,utmn;
    CELLTYPE *rowbuf;
    float floatNull;
	double xllcenter=0,yllcenter=0;
//=========================================
	//declaring and assigning  variables
	double *dx,*dy;
	int *nx,*ny;
	void ***data;
	int datatype;
	float *nodata;

	dx=&(dem->head.dx);
	dy=&(dem->head.dy);
	nx=&(dem->head.nx);
	ny=&(dem->head.ny);
	data=(void ***)&(dem->d);
	datatype=RPFLTDTYPE;
	nodata=&dem->nodata;
	

//=======================================
    if( iesri = GridIOSetup() >= 0)
        /*
        Open input cell layer with no evaluation        (5)
        */
        if ((channel1 = 
              CellLayerOpen(file,READONLY,ROWIO,
                            &type,csize)) >= 0 )
    {
/*  The success of these means the file could be opened as an ESRI grid file  */
        *filetype=1;
/*  File types are
    0=ASCII   
    1= ARCVIEW grid via the ESRI Application Programmers Interface  */ 
/*  here is the ESRI grid file read stuff - following copyrow example */
        *dx = *dy = (float) *csize;
/*
        
        /*
        Get the bounding box of the input cell layer            (7)
        */
        if (BndCellRead (file, dem->head.bndbox) < 0)
        {
//                printf ("could not read bounding box of input grid\n");
                CellLyrClose(channel1);
                GridIOExit();
                return(1);
        }
/*        printf("%f %f %f %f %g\n",bndbox[0],bndbox[1],bndbox[2],bndbox[3],*nodata) */; 
/*  Bounding box is xllcorner, yllcorner, xurcorner, yurcorner   */
      
        /*
        Set the Window to the output bounding box and cellsize  (9)
        */
        if (AccessWindowSet (blkbndbox, *csize, adjbndbox) < 0)
        {
//                printf ("Could not set Window\n");
                CellLyrClose(channel1);
                GridIOExit();
                return(2);
        }
        /*
        Get the number of rows and columns                      (10)
        in the window
        */
        *nx = WindowCols();
        *ny = WindowRows();
		dem->head.bndbox[0] = adjbndbox[0];
		dem->head.bndbox[1] = adjbndbox[1];
		dem->head.bndbox[2] = adjbndbox[2];
		dem->head.bndbox[3] = adjbndbox[3];

        /*  Allocate memory and set all type pointers  */
        *data = matalloc(*nx, *ny, datatype);
		if(*data == NULL)return(1);  //DGT 9/13/2007 Bail of not able to allocate memory

        farr = (float **) (*data);
        GetMissingFloat(&floatNull);
		
      //  *nodata = (datatype == RPFLTDTYPE) ? floatNull: -9999.; 
		*nodata=floatNull;
		        /*
        Allocate row buffer                                     (11)
        */
        if ((rowbuf = (CELLTYPE *)
                        CAllocate1 ( *nx, sizeof(CELLTYPE)))
                        == NULL )
        {
//                printf ("Could not allocate memory\n");
                CellLyrClose(channel1);
                GridIOExit();
                return(3);
        }
        /*
        Now copy row into array                 (12)
        */
		/*     This is all repeated to get right the typecasting of data from ESRI file into the format 
       we want   */
        for (i=0; i < *ny; i++)
        {
             GetWindowRow(channel1, i, rowbuf);
             if(type == CELLINT)
             {
                register int *buf = (int *)rowbuf;
	            
                   for(j=0; j < *nx; j++)
                   {
                      farr[j][i]=(buf[j] == MISSINGINT ) ? *nodata: (float) buf[j];  
                   }
            
             }
			 
             else if(type==CELLFLOAT)
             {
                register float *buf = (float *)rowbuf;
								

                
                   for(j=0; j < *nx; j++)
                   {
						farr[j][i]= buf[j];
                   }
              }
			 
       }

        /*
        Free row buffer                                         (13)
        */
        CFree1 ((char *)rowbuf);
        /*
        Close handles                                           (14)
        */
        CellLyrClose(channel1);  
        /*
        Done with the library                                   (15)
        */
        GridIOExit();
        return(0);
    }

/*  Here assume file is ASCII or Binary.  Close ESRI stuff. */
    CellLyrClose(channel1);
    GridIOExit();
//=========================================================
//Code for bgd .Ajay

    
	CString strFileName= CString(file); 
	CString strFileExtension= strFileName.Right(4); 

	if (strFileExtension.CompareNoCase(CString(".bgd"))==0)
		
{
    //Success means bgd 
	

		*filetype=2;
		fp=fopen(file,"rb");

			if(fp == NULL)
			{
		//        printf("\nERROR: Cannot open input file (%s).\n\n",file);
				return(1);
			}
    
        rewind(fp);
			
	//  CODE TO ASSIGN VALUES READ TO APPROPRIATE GRID HEADER VARIABLES
		fread(nx, sizeof(int),1,fp);
		fread(ny, sizeof(int),1,fp);
		fread(dx, sizeof(double),1,fp);
		fread(dy, sizeof(double),1,fp);
	//Bound box
		fread(&xllcenter, sizeof(double),1,fp);
		fread(&yllcenter, sizeof(double),1,fp);
	 // COMPUTE BNDBOX FROM XLLCENTER, YLLCENTER.
		dem->head.bndbox[0]=xllcenter-(*dx/2);
		dem->head.bndbox[1]=yllcenter-(*dy/2);
		dem->head.bndbox[2]=dem->head.bndbox[0] + *dx * (*nx);
		dem->head.bndbox[3]=dem->head.bndbox[1] + *dy * (*ny);
	
		fread(&datatype, sizeof(DATA_TYPE),1,fp);

		short snodata;
		if(datatype==SHORT_TYPE)
		{
			fread(&snodata,sizeof(short),1,fp);
			*nodata=(float)snodata;
		}
		long lnodata;
		if(datatype==LONG_TYPE)
		{
			fread(&lnodata,sizeof(long),1,fp);
			*nodata=(float)lnodata;
		}
		if(datatype==FLOAT_TYPE)fread(nodata,sizeof(float),1,fp);

		
		fread(projection,sizeof(char), MAX_STRING_LENGTH, fp);
        fread(notes, sizeof(char), MAX_STRING_LENGTH, fp);

			if( *nx < 0 || *ny < 0 || 
				*dx< 0 || *dy < 0 ) 
				{	
					fclose( fp );
					
					return 1;
				}

			farr =(float **)matalloc(*nx, *ny, FLOAT_TYPE + 1);
			if(farr == NULL)return(1);  //DGT 9/13/2007 Bail of not able to allocate memory

			//Check for datatype and typecasting        
			if(datatype==SHORT_TYPE)
				{	short shortarr;
					
					for( int i = 0; i < *ny; i++ )
					{	for( int j = 0; j < *nx; j++ )
						{	
							if( feof(fp) )
							{	
								fclose( fp );
								return 1;
							}
						
						fread(&shortarr,sizeof(short),1,fp );
						farr[j][i]=(float)shortarr;
						}
					}
				}
				else if(datatype==LONG_TYPE)
				{	
					int intarr;
			
					for( int i = 0; i < *ny; i++ )
					{	for( int j = 0; j < *nx; j++ )
						{	
							if( feof(fp) )
							{	
								fclose( fp );
								return 1;
							}
						
						fread( &intarr,sizeof(long),1,fp );
						farr[j][i]=(float)intarr;
						}
					}
				}
				else if(datatype==FLOAT_TYPE)
				{	
					
					for( int i = 0; i < *ny; i++ )
					{	for( int j = 0; j < *nx; j++ )
						{	
							if( feof(fp) )
							{	
								fclose( fp );
								return 1;
							}
						
						fread( &farr[j][i],sizeof(float),1,fp );
						}
					}
				}
				else return 1;
////////////////////////////////////////////////////////////////////
			*data = (void **) farr;
			fclose(fp);
			return 0;			

		 

	}

//================================================================
//================================================================
//If the file is ascii.
	*filetype=0;  
    fp = fopen(file,"r");
    if(fp == NULL)
    {
//        printf("\nERROR: Cannot open input file (%s).\n\n",file);
        return(1);
    }
    
	
    /* read ARC-Info header */
    while(1)
    {   
        readline(fp, fline);       
        if(!isalpha(*fline) || *fline == '-')
            break;       
        
        hdrlines++;

        sscanf(fline,"%s %f",keyword,&value);
        
	if(strcmp(keyword,"ncols") == 0 || strcmp(keyword,"NCOLS") == 0)
	    *nx = (int)value;
	else if(strcmp(keyword,"nrows") == 0 || strcmp(keyword,"NROWS") == 0)
	    *ny = (int)value;
	else if(strcmp(keyword,"xllcenter") == 0 || strcmp(keyword,"XLLCENTER") == 0)
	{
	    utmetag = 'c';
	    utme = value;
	}
	else if(strcmp(keyword,"xllcorner") == 0 || strcmp(keyword,"XLLCORNER") == 0)
	{
	    utmetag = 'e';
	    utme = value;
	}
	else if(strcmp(keyword,"yllcenter") == 0 || strcmp(keyword,"YLLCENTER") == 0)
	{
	    utmntag = 'c';
	    utmn = value;
	}
	else if(strcmp(keyword,"yllcorner") == 0 || strcmp(keyword,"YLLCORNER") == 0)
	{
	    utmntag = 'e';
	    utmn = value;
	}
	else if(strcmp(keyword,"cellsize") == 0 || strcmp(keyword,"CELLSIZE") == 0)
	{
	    *dx = *dy = value;
	    *csize = (double) value;
	}
	else if(strcmp(keyword,"nodata_value") == 0 || strcmp(keyword,"NODATA_VALUE") == 0 ||
		strcmp(keyword,"NODATA_value") == 0)
	    *nodata = (float)value;
    }
    
    /* adjust utme and utmn if necessary (we store center of reference cell) */
    if(utmetag == 'e') utme = utme + *dx/2;
    if(utmntag == 'e') utmn = utmn + *dy/2;
     dem->head.bndbox[0] = utme - *csize/2.;   
     dem->head.bndbox[1] = utmn - *csize/2.;
     dem->head.bndbox[2] = dem->head.bndbox[0] + *csize * (*nx);
     dem->head.bndbox[3] = dem->head.bndbox[1] + *csize * (*ny);
    /* position file pointer for ARC-Info file to beginning of image data */
    rewind(fp);
    for(i=0; i<hdrlines; i++) readline(fp, fline);
    
    /* convert depending on datatype */
     
       farr = (float **) matalloc(*nx, *ny, datatype);
       if(farr == NULL)return(1);  //DGT 9/13/2007 Bail of not able to allocate memory

        /* read in the ARC-Info file */
        for(i=0; i< *ny; i++)
        {
            for(j=0; j< *nx; j++)
			{

                fscanf(fp,"%f",&farr[j][i]);
			}
        }
        *data = (void **) farr;   
   
    fclose(fp);
    return(0);
}

/*  headcheck.  Function to check compatibility of headers and if problem return error   */
int headcheck(ghead head1,ghead head2)
  {
	int err=TD_NO_ERROR;
    if(fabs(1-head1.dx/head2.dx)>0.001)err=TD_GRID_SIZE_MISMATCH;
	if(fabs(1-head1.dy/head2.dy)>0.001)err=TD_GRID_SIZE_MISMATCH;
    if(head1.nx != head2.nx)err=TD_GRID_SIZE_MISMATCH;
	if(head1.ny != head2.ny)err=TD_GRID_SIZE_MISMATCH;
	return(err);
  }