/***********************************************************/
/*                                                         */
/* gridio.h                                                */
/*                                                         */
/* Grid Data manipulation functions -- header file       */
/*                                                         */
/*                                                         */
/* David Tarboton                                           */
/* Utah Water Research Laboratory                          */
/* Utah State University                                   */
/* Logan, UT 84322-8200                                    */
/*                                                         */
/***********************************************************/
//#include "stdafx.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <limits.h>
#include <math.h>

#ifdef USEESRILIB
extern "C" {
#include "gioapi.h"
}
#else
#include "gioapi.h"
#endif


# ifndef MAX_STRING_LENGTH
# define MAX_STRING_LENGTH 255
# endif

# ifndef GRID_DATA_TYPE
# define GRID_DATA_TYPE
	
	enum DATA_TYPE
	{ SHORT_TYPE,
	  LONG_TYPE,
	  FLOAT_TYPE,
	  DOUBLE_TYPE,	 
	  UNKNOWN_TYPE,
	  INVALID_DATA_TYPE = -1
	};

# endif


/*  ESRI Application Programmers Interface include file  */

/*  Constants    */
#define NA (10.0*FLT_MIN)
#ifndef PI
#define PI 3.14159265359
#endif
#define LINELEN 40
#ifndef MAXLN
#define MAXLN 4096
#endif

/* data types */
#define RPSHRDTYPE  1
#define RPINTDTYPE  2
#define RPFLTDTYPE  3

/* byte sizes corresponding to data types above */
#define RPSHRSIZE (sizeof(short))
#define RPINTSIZE (sizeof(int))
#define RPFLTSIZE (sizeof(float))

#define	MISSINGINT	-2147483647	/* CELLMIN - 1 */
#define MISSINGSHORT -32767 

//#define DLL   //  Toggle this depending upon whether functions are to be exported or not   */

#ifdef DLL

#include <windows.h>

#define DllExport __declspec( dllexport )  

/* Functions in dgtgio.dll that are callable from outside   */

DllExport void fgridread(char *file,void *dem, int *datatype, int *nx, int *ny,
			   float *dx, float *dy, double bndbox[4],double *csize,
			   float *ndv,int *filetype, int *igy);
DllExport void fgridwrite(char *file,void *dem, int *datatype, int *nx, int *ny,
			   float *dx, float *dy, double bndbox[4],double *csize,
			   float *ndv,int *filetype, int *igy);    
DllExport int gridwrite(char *file, void **data, int datatype, int nx, int ny, float dx, 
 float dy, double bndbox[4], double csize, float ndv, int filetype);

DllExport int gridread(char *file, void ***data, int datatype, int *nx, int *ny,
 float *dx, float *dy, double bndbox[4], double *csize, float *ndv, int *filetype);

DllExport void eol(FILE *fp);

DllExport int readline(FILE *fp,char *fline);

DllExport void **matalloc(int nx,int ny,int datatype);

DllExport int nameadd(char *full,char *arg,char *suff);

#else


//New code added here:Ajay
//=====================================
//Class definitions.
/*# ifndef DATA_TYPES
# define DATA_TYPES
enum DATA_TYPE
	{ SHORT_TYPE,
	  LONG_TYPE,
	  FLOAT_TYPE,
	  DOUBLE_TYPE,	 
	  UNKNOWN_TYPE,
	  INVALID_DATA_TYPE = -1
	};
#endif*/
	

	
class ghead
{
	public:
	 int nx;
	 int ny;
	 double dx;
	 double dy;
	 double bndbox[4];

};

class fgrid
{
	public:
	 float **d;
	 float nodata;
	 ghead head;
	
};

class igrid
{
	public:
	 int **d;
	 int nodata;
	 ghead head;
	
	
};

class sgrid
{
	public:
	 short **d;
	 short nodata;
	 ghead head;
};

//=====================================
void fgridread(char *file,void *dem, int *datatype, int *nx, int *ny,
			   float *dx, float *dy, double bndbox[4],double *csize,
			   float *ndv,int *filetype, int *igy);
void fgridwrite(char *file,void *dem, int *datatype, int *nx, int *ny,
			   float *dx, float *dy, double bndbox[4],double *csize,
			   float *ndv,int *filetype, int *igy);    
//==================================================
//Old declaration
/*int gridwrite(char *file, void **data, int datatype, int nx, int ny, float dx, 
 float dy, double bndbox[4], double csize, float ndv, int filetype);*/

/*int gridwrite(char *file, void **data, int datatype, int nx, int ny, double dx, 
 double dy, double bndbox[4], float ndv, int filetype);
//==================================================
int gridread(char *file, void ***data, int datatype, int *nx, int *ny,
 double *dx, double *dy, double bndbox[4], float *ndv, int *filetype);*/

void eol(FILE *fp);
void eol2(FILE *fp);
int readline(FILE *fp,char *fline);  

void **matalloc(int nx,int ny,int datatype);

int nameadd(char *full,char *arg,char *suff);  
//Code added here:Ajay
//Overloading functions.Gridread,Gridwrite and allocategrid
//========================================
int gridread(char *file, sgrid* dem ,int *filetype);
int gridread(char *file, fgrid* dem ,int *filetype);
int gridread(char *file, igrid* dem ,int *filetype);

int gridblockread(char *file, sgrid* dem ,double blkbndbox[4], int *filetype);//blk bndbox[] is the boundbox of the subgrid (smaller grid)
int gridblockread(char *file, igrid* dem ,double blkbndbox[4],int *filetype);
int gridblockread(char *file, fgrid *dem, double blkbndbox[4],int *filetype);
int gridwrite(char *file, fgrid dem, int filetype);
int gridwrite(char *file, sgrid dem, int filetype);
int gridwrite(char *file, igrid dem, int filetype);

int allocategrid(fgrid *grid, ghead head, float nodatavalue);
int allocategrid(igrid *grid, ghead head, int nodatavalue);
int allocategrid(sgrid *grid, ghead head, short nodatavalue);
//===========================================
//New head read function Jan-28-2004:Ajay Prasad
int readHeader(char *ipFile,ghead *head,short *shortnodata,int *intnodata,float *floatnodata,int *datatype,int *filetype);
int headcheck(ghead head1,ghead head2);
int griddelete(char* ipfile);
int gridrename(char* ipfile,char* opfile);
int gridexists(char* ipfile);
int creategrid(char* demfile,char* opgridfile,int gridtype,int cellvalue);
//This function creates a new grid named by "newgrid", from the bigGrid with a new bound box
int subsetgrid(char *bigGrid, char *newgrid, double *newbndbox);
int checkdatatype(char *ipFile);
int headcheck(ghead head1,ghead head2);
#endif

