//  Tardemlib1.cpp,  is an extention of tardemlib.cpp for functions created by Kiran.

/* Created by David G Tarboton  */

/*  Copyright (C) 2000  David Tarboton, Utah State University

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License 
version 2, 1991 as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

A copy of the full GNU General Public License is included in file 
gpl.html. This is also available at:
http://www.gnu.org/copyleft/gpl.html
or from:
The Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
Boston, MA  02111-1307, USA.

If you wish to use or incorporate this program (or parts of it) into 
other software that does not meet the GNU General Public License 
conditions contact the author to request permission.
David G. Tarboton  
Utah State University 
8200 Old Main Hill 
Logan, UT 84322-8200 
USA 
http://www.engineering.usu.edu/dtarb/ 
email:  dtarb@cc.usu.edu 
*/

#include "stdafx.h"	
#include "tardemlib.h"
#include "TDErrorCodes.h"
#include <stdio.h>
#include <math.h>

//moment() is from Numerical Recpes (Press et al.) 13.1 p 475
int moment(float data[], int n,float *ave,float *adev,float *sdev,float *svar,float *skew, float *curt)
{
	int j;
	float s,p;
	if (n<= 1) return 1;
	s=0.0;
	for(j=0;j<n;j++)s += data[j];
	//float ave1,float adev1,float sdev1,float svar1,float skew1, float curt1;
	//ave1=(s/n);
	//ave=&ave1;
	*ave=(s/n);
	*adev=*svar=*skew=*curt=0.0;
	for(j=0;j<n;j++){
		*adev +=(float)fabs(s=data[j]-(*ave));
		*svar += (p=s*s);
		*skew += (p *= s);
		*curt += (p *= s);
	}
	*adev /= n;
	*svar /= (n-1);
	*sdev=(float)sqrt(*svar);
	if (*svar){
		*skew /= (float)(n*(*svar)*(*sdev));
		*curt=(float)((*curt)/(n*(*svar)*(*svar))-3.0);
	}else return 1;
	
	return 0;

}
//This following rountine is from Numerical recipies(Press et al.)  8.2 p247
void sort(int n,float raa[]){

	float *ra = raa-1;
	int l,j,ir,i;
	float rra;

	l=(n >> 1)+1;
	ir=n;
	for (;;) {
		if (l > 1)
			rra=ra[--l];
		else {
			rra=ra[ir];
			ra[ir]=ra[1];
			if (--ir == 1) {
				ra[1]=rra;
				return;
			}
		}
	
		i=l;
		j=l << 1;
		while (j <= ir) {
			if (j < ir && ra[j] < ra[j+1]) ++j;
			if (rra < ra[j]) {
				ra[i]=ra[j];
				j += (i=j);
			}
			else j=ir+1;
		}
		ra[i]=rra;
	}
	
}

//mdian1() is from Numerical Recipes (Press et al.) 13.2 p 477
void mdian1(float x[], int n, float *xmed){
	int n2,n2p;
	//void sort();
	sort(n,x);
	n2p=(n2=((n-1)/2))+1;
	*xmed=(float)((n-1)%2 ? 0.5 *(x[n2] + x[n2p]) : x[n2p] );
}

int gridstat(char *grdname , char *idgrid, long ind, double *statlist)
{
	int filetype, filetypeID, err=0;
	int nx, ny;
	double dx, dy;
	double csize;
	float ndvp;
	int i, j,k=0;
	long ncount=0;
	sgrid sgrd;
	igrid igrd;
	fgrid fgrd;

	sgrid IDsgrid;

	
	filetypeID =checkdatatype(idgrid);
	if(filetypeID == 1){ 
		//Find out how many cells are valid, i.e the cells without NODATA value
		if ( gridread(idgrid, &IDsgrid, &filetypeID) != 0)return TD_FAILED_GRID_OPEN;
		nx = IDsgrid.head.nx;
		ny = IDsgrid.head.ny;
		dx = IDsgrid.head.dx;
		dy = IDsgrid.head.dy;
		

		csize = dx;
		ndvp = IDsgrid.nodata;
		for(i=0; i< IDsgrid.head.ny; i++)
		{
			for(j=0; j< IDsgrid.head.nx; j++)
			{
				if(IDsgrid.d[j][i] == ind) ncount=ncount+1;//ncount will be the number of valid cells
			}
		}
				
	}
	else{
		
	}
	filetype=checkdatatype(grdname);
	

	switch( filetype ) 
	{
		case 0:
			{
				if ( gridread(grdname,&sgrd,&filetype) != 0)return TD_FAILED_GRID_OPEN;
				short* sdata = new short[ncount-1];  
				for(i=0; i< IDsgrid.head.ny; i++)
				{
					for(j=0; j< IDsgrid.head.nx; j++)
					{
						if(IDsgrid.d[j][i] == ind){
							sdata[k]= sgrd.d[j][i];//converts the 2D into single array
							k=k+1;
						}
					}
				}
				free(IDsgrid.d[0]); free(IDsgrid.d);//free the 2D idgrid
				free(sgrd.d[0]); free(sgrd.d);//free the 2D data grid
				//sort(ncount,fdata);
				float ave,adev,sdev,svar,skew,curt,xmed;
				moment((float *)sdata,ncount,&ave,&adev,&sdev,&svar,&skew,&curt);
				mdian1((float *)sdata,ncount,&xmed);
				statlist[0]=ave;
				statlist[1]=xmed;
				statlist[2]=sdev;
				statlist[3]=sdata[0];
				statlist[4]=sdata[ncount-1];
				statlist[5]=statlist[4]-statlist[3];
				sdata=NULL;
				return 0;
			}
		case 1:
			{
				if ( gridread(grdname,&igrd,&filetype) != 0)return TD_FAILED_GRID_OPEN;
				int* idata = new int[ncount-1];  
				for(i=0; i< IDsgrid.head.ny; i++)
				{
					for(j=0; j< IDsgrid.head.nx; j++)
					{
						if(IDsgrid.d[j][i] == ind){
							idata[k]= igrd.d[j][i];//converts the 2D into single array
						    k=k+1;
						}
					}
				}
				free(IDsgrid.d[0]); free(IDsgrid.d);//free the 2D idgrid
				free(igrd.d[0]); free(igrd.d);//free the 2D data grid
				//sort(ncount,fdata);
				float ave,adev,sdev,svar,skew,curt,xmed;
				moment((float *)idata,ncount,&ave,&adev,&sdev,&svar,&skew,&curt);
				mdian1((float *)idata,ncount,&xmed);
				statlist[0]=ave;
				statlist[1]=xmed;
				statlist[2]=sdev;
				statlist[3]=idata[0];
				statlist[4]=idata[ncount-1];
				statlist[5]=statlist[4]-statlist[3]; 
				idata=NULL;
				return 0;
				
			}
		case 2:
			{
				if ( gridread(grdname,&fgrd,&filetype) != 0)return TD_FAILED_GRID_OPEN;
				float* fdata = new float[ncount-1];  
				for(i=0; i< IDsgrid.head.ny; i++)
				{
					for(j=0; j< IDsgrid.head.nx; j++)
					{
						if(IDsgrid.d[j][i] == ind){
							fdata[k]= fgrd.d[j][i];//converts the 2D into single array
							k=k+1;
						}
					}
				}
				free(IDsgrid.d[0]); free(IDsgrid.d);//free the 2D idgrid
				free(fgrd.d[0]); free(fgrd.d);//free the 2D data grid
				float ave,adev,sdev,svar,skew,curt,xmed;
				moment(fdata,ncount,&ave,&adev,&sdev,&svar,&skew,&curt);
				mdian1(fdata,ncount,&xmed);
								
				statlist[0]=ave;
				statlist[1]=xmed;
				statlist[2]=sdev;
				statlist[3]=fdata[0];
				statlist[4]=fdata[ncount-1];
				statlist[5]=statlist[4]-statlist[3];

				//free(fdata[0]);
			//	free(fdata);
				fdata=NULL;
				//delete [] fdata;
				return 0;
				
			}
		default :
			return 1;
	}


	return 0;
}