//  Tardem lib.cpp, a combination of all TARDEM library source files into a single file
//  combined aread8.cpp, setdir.cpp, area.cpp, flood.cpp, netsetup.cpp, gridnet.cpp, 
//  subbasinsetup.cpp, depgrd

/* Created by David G Tarboton  */

/*  Copyright (C) 2000  David Tarboton, Utah State University

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License 
version 2, 1991 as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

A copy of the full GNU General Public License is included in file 
gpl.html. This is also available at:
http://www.gnu.org/copyleft/gpl.html
or from:
The Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
Boston, MA  02111-1307, USA.

If you wish to use or incorporate this program (or parts of it) into 
other software that does not meet the GNU General Public License 
conditions contact the author to request permission.
David G. Tarboton  
Utah State University 
8200 Old Main Hill 
Logan, UT 84322-8200 
USA 
http://www.engineering.usu.edu/dtarb/ 
email:  dtarb@cc.usu.edu 
*/

#include "stdafx.h"	
#include "lsm.h"
#include "TDErrorCodes.h"
#include "feedback.h"

#include <afx.h>
#include <string.h>
#include "Callback.h"
#include <math.h>
#include <stdio.h> //needed to create temporary files

#include "shapefile.h"
#include "feedback.h"
#include "gridio.h"

//prototypes for functions in AVCalls.c 



//Prototypes for Numerical Recipe Functions Used 

int **imatrix(int,int,int,int);
float **matrix(int,int,int,int);
int *ivector(int,int);
double *dvector(int,int);
void free_imatrix(int**,int,int,int,int);
void free_matrix(float**,int,int,int,int);
void larea(int,int,int);
void free_ivector(int*,int,int);

//Prototype for Netex
int Netex_c(short **dir, int **area, char *treefile, char *coordfile, char *ordfile, int nx, int ny, int itresh,
			 int& icr, int& icend, double dx, double dy, double bndbox, double csize, int iftype, 
			 short *inodes, short *jnodes, int nnodes, long *idnodes);

//Global variables and constants
#define PI2 1.5707963268
#ifndef PI
#define PI 3.14159265359
#endif

#define LINELEN 40
#ifndef MAXLN
#define MAXLN 4096
#endif

//data types 
#define RPSHRDTYPE  1
#define RPINTDTYPE  2
#define RPFLTDTYPE  3

//byte sizes corresponding to data types above 
#define RPSHRSIZE (sizeof(short))
#define RPINTSIZE (sizeof(int))
#define RPFLTSIZE (sizeof(float))

bool inRam; //whether or not to load grids into RAM
float **weight;  //Global weight array  
float **areaw;  //Global array with weighted results   
int ccheck;   //Global contamination check flag  
int useww;//Global use weights flag  
float ndva;   //Global angle no data value  
short **src;
int **arr;
int usecc;
int srcmethod;
float **ltp,**lengd;
double dist[9];
int **gorder, **aa, tresh;
float ndvp;
int iy,jy;
float **dep;
short **aw;
int **rtable; 
float **rprops; 
short **dir;  //flood, setdir 
short **apool;//flood 
float **elev; //flood, setdir 
float **slope;		//setdir 
float **ang;  //setdir, area 
float **sca;			//area 
int err;

short *ipool, *jpool, *tmp;		//flood 
short *dn, *is, *js;					//setdir, flood 

int nx, ny;										//ALL 
int npool, pooln, pstack;			//flood 
int nis, istack;						//flood, setdir 
int filetype;									//all IO 

/*  i1,n1 are the x range when tiling
i2,n2 are the y range when tiling
These are global so as to avoid duplication in recursive routines  */
int i1,i2,n1,n2;			//flood, setdir 

//char annot[MAXLN],units[MAXLN];			//flood, setdir 

double dx, dy;							//flood, setdir, area 
float emin, et;						//flood, setdir 
float utme,utmn,skew;			//setdir 
float aref[10];						//area 
float ndv;						//area, setdir 

double boundbox[4];

int nf;				//flood, setdir 
int dd1[8],dd2[8];				
//Offset pointers d1 and d2 
int *d1 = dd1-1;
int *d2 = dd2-1;

//Grid bound variables   
double bndbox[4],csize;
float mval;

// outlet list variables
short nout=0,maxno=0, *diffout;   //  Initialize to 0 for when not used
long *xout, *yout;
short SourceThreshold;

float cSolG; //global solubility for saturated limited loading accumulation (dsllarea)

char errmessage[1024];

//*************************************************************
  //Whole Code modified for new gridio file.Removed tkshp and tkgrd dependency.:Ajay Prasad 2003-2004
  //*************************************************************

/************** Grid Function Globals ***************/

//*******************************************************
//NEW DECLARATIONS:
//*******************************************************
//aread8 & area
fgrid fweight;
igrid larr;
fgrid fareaw;
sgrid sdir;
//area
fgrid fangle;
fgrid fsca;
//source
fgrid felevg;
fgrid fslopeg;
//distgrid
fgrid fdarr;

//sdir function
sgrid sapoolg;
//gridnet
igrid laag;
sgrid sordg;
fgrid fltpg,fleng;
//subbasinsetup
shapefile shp1;
sgrid sawg;
//dependence
fgrid fdepg;
igrid ldisturbg;
//tlaccum
fgrid ftcapg;
fgrid fcing;
fgrid fcoutg;
//dsaccum
fgrid fdisturbg;
fgrid fdmaxg;
//areadinfdecay
fgrid fdmg;
float dmndv;
//dsllarea 
fgrid fqg;
sgrid scontag;

//	FILE *fdbg;
	int recursedepth;
//*****************************************************


//*********************************************
//Callback Helper functions
//*********************************************
//To avoid repetition of code to clean up the BSTR strings,
//I have written the next two functions to save time
//and to avoid having to call the A2BSTR functions repeatedly elsewhere


inline void cbProgress(ItkCallback * callback, char * key, long percent, char *message)
{	
	if (callback == NULL)
		return;

	USES_CONVERSION; //needed for BSTR conversion
	BSTR newKey = A2BSTR(key);
	BSTR newMessage = A2BSTR(message);

	callback->Progress(newKey,percent,newMessage);

	::SysFreeString(newKey);
	::SysFreeString(newMessage);
}

inline void cbError(ItkCallback * callback, char * key, long ErrorCode, char * message)
{
	if (callback == NULL)
		return;

	USES_CONVERSION; //needed for BSTR conversion

	BSTR newKey = A2BSTR(key);
	BSTR newMessage = A2BSTR(message);

	callback->Error(newKey,ErrorCode,newMessage);

	::SysFreeString(newKey);
	::SysFreeString(newMessage);

}

//*********************************************
//end of callback helper functions
//*********************************************

// experimental eol to avoid end of file problem in topsetup 8/9/02
void eol2(FILE *fp)
{
char ch;

while(1)
{
  if((ch = getc(fp)) == '\n') return;
  
  if(ch == EOF) return;//{ fseek(fp, -1L, SEEK_CUR); return; }
}
}

void darea(int i,int j)
  { 

    int in,jn,k,con=0;
      // con is a flag that signifies possible contaminatin of area
		//due to edge effects  
    if(i!=0 && i!=ny-1 && j!=0 && j!=nx-1 && sdir.d[j][i] > -1)
                 //not on boundary  
    {
      if(larr.d[j][i]==0)  // not touched yet
      {
       larr.d[j][i]=1;
		if(useww == 1)fareaw.d[j][i]=fweight.d[j][i];
        for(k=1; k<=8; k++)
        {  in=i+d1[k];
           jn=j+d2[k];
//test if neighbor drains towards cell excluding boundaryies 
           if(sdir.d[jn][in]>0 && ( (sdir.d[jn][in]-k)==4 || (sdir.d[jn][in]-k)==-4))
             {
                darea(in,jn);
                if(larr.d[jn][in] < 0)con = -1;
                else larr.d[j][i]=larr.d[j][i]+larr.d[jn][in];
				if(useww == 1)
				{
					if(fareaw.d[jn][in] <= ndv  || fareaw.d[j][i] <=ndv)
					{  fareaw.d[j][i]=ndv;
					}
					else fareaw.d[j][i]=fareaw.d[j][i]+fareaw.d[jn][in];
				}
             }
           if(sdir.d[jn][in] < 0)con = -1;
        }
        if(con == -1 && ccheck == 1)
		{
			larr.d[j][i]=-1;
			if(useww == 1)fareaw.d[j][i]=ndv;
		}
      }
    }
	else larr.d[j][i]=-1;


}

  
int aread8(char *pfile,char *afile, double *x, double *y,long nxy, int doall, 
		   char *wfile, int usew, int contcheck)
{	
	int i,j,row,col;
	err = TD_NO_ERROR; 
	ccheck=contcheck;
	useww=usew;
	//define directions 
	d1[1]=0; d1[2]= -1; d1[3]= -1; d1[4]= -1; d1[5]=0; d1[6]=1; d1[7]=1; d1[8]=1;
	d2[1]=1; d2[2]=1; d2[3]=0; d2[4]= -1; d2[5]= -1; d2[6]= -1; d2[7]=0; d2[8]=1;
	
	//New Gridread function:	
	if (gridread(pfile,&sdir,&filetype) != 0) return (TD_FAILED_GRID_OPEN);

	




	nx = sdir.head.nx;
	ny = sdir.head.ny;
	dx = sdir.head.dx;
	dy = sdir.head.dy;

	csize = dx;
	ndvp = sdir.nodata;
	for (int bnd=0;bnd<4;bnd++) boundbox[bnd]=sdir.head.bndbox[bnd];
	if(usew == 1)
	{
		//New Gridread function gridio.cpp:
		if(gridread(wfile,&fweight,&filetype)==0)
			err=TD_NO_ERROR;
		else
			err=TD_FAILED_GRID_OPEN;
		if(err != TD_NO_ERROR) goto ERROR1;
		ndv=fweight.nodata;

		fareaw.nodata=ndv;
		fareaw.head=fweight.head;
	
	//read weights if necessary   

	/*  Allocate areaw memory  */
//======================================================================
//Code changed here:
//allocategrid function

//Start creation of grid - put in allocategrid function part of gridio.cpp
//purpose is to replace matalloc within the code	
	
		err=allocategrid(&fareaw,fweight.head,ndv);

		if(err != 0)goto ERROR2;
		
	}		
			larr.head.dx=dx;
			larr.head.dy=dy;
			larr.head.nx=nx;
			larr.head.ny=ny;
			for (bnd=0;bnd<4;bnd++) larr.head.bndbox[bnd]=sdir.head.bndbox[bnd];
			larr.nodata=-1;
			err=allocategrid(&larr,sdir.head,larr.nodata);
	if(err != 0)goto ERROR2;
	
	//Allocate area memory  
	//initialize area array to 0, -1 on boundary 
	for(i=0; i< sdir.head.ny; i++)
	{
		for(j=0; j< sdir.head.nx; j++)
        {
			if(i!=0 && i!=ny-1 && j!=0 && j!=nx-1 && sdir.d[j][i] > -1)
				larr.d[j][i]=0;

        }
	}
		
	//by default start at first cell of grid
	row=0; col=0;

	if(doall == 0)
	{
		for (int curXY = 0; curXY < nxy; curXY++)
		{
		 
			//translate the coordinates from a world location to grid cell index
			col= (int)floor((x[curXY]-boundbox[0])/csize); 
			row= (int)floor((boundbox[3]-y[curXY])/csize);
			if(row <0 || row > ny || col < 0 || col > nx)
			{
				//printf("Given coords out of area - whole area will be calculated\n");
				row=0; col=0;
			}
			//call drainage area subroutine for pixel to zero on  
					darea(row,col);
		}

		
    }
    else
    {
		
		//call drainage area subroutine for each area 
		//work from middle outwards to avoid deep recursions  
		for(i=ny/2; i<ny-1; i++)                    
		{  for(j=nx/2; j<nx-1; j++)
		    darea(i,j);
		for(j=nx/2-1; j>=1; j--)
			darea(i,j);
		}  
		for(i=ny/2-1; i>=1; i--)                    
		{  for(j=nx/2; j<nx-1; j++)
        darea(i,j);
		for(j=nx/2-1; j>=1; j--)
			darea(i,j);
		} 
	} 
	//write out areas New gridwrite functions :
	if(usew == 0)
	{
	
		if ( gridwrite(afile,larr,filetype)==0 )
			err=TD_NO_ERROR;
		else
		{
			err=TD_FAILED_GRID_SAVE;
			return err;
//			if (afile) //don't try to perform strcat this pointer if it is null
//				AfxMessageBox( LPCTSTR (strcat( "Failed to save file: ", afile) ));
		}

	}
	else
	{
		

		if ( gridwrite(afile,fareaw,filetype)==0 )
			err=TD_NO_ERROR;
		else{
			err=TD_FAILED_GRID_SAVE;
			return err;
		//	if (afile) //don't try to perform strcat this pointer if it is null
		//		AfxMessageBox( LPCTSTR (strcat( "Failed to save file: ", afile) ));
		}

	
		free(fareaw.d[0]); 
		free(fareaw.d);
	}

	free(larr.d[0]);
	free(larr.d);

if(usew == 1)
{
ERROR2:
	free(fweight.d[0]); free(fweight.d);
	}
ERROR1:
	free(sdir.d[0]);
	free(sdir.d);
	return(err);
 }     
 

//************************************************************************

void   VSLOPE(float E0,float E1, float E2,
  float D1,float D2,float DD,
  float *S,float *A)
{


//SUBROUTINE TO RETURN THE SLOPE AND ANGLE ASSOCIATED WITH A DEM PANEL 
	float S1,S2,AD;
	
	
	if(D1!=0)
		S1=(E0-E1)/D1;
	if(D2!=0)
		S2=(E1-E2)/D2;
	
	if(S2==0 && S1==0) *A=0;
	else
		*A= (float) atan2(S2,S1);
	AD= (float) atan2(D2,D1);
	if(*A  <   0.)
	{
		*A=0.;
		*S=S1;
	}
	else if(*A > AD)
	{
*A=AD;
*S=(E0-E2)/DD;
	}
	else
*S= (float) sqrt(S1*S1+S2*S2);


}



//***************************************************************************
// SUBROUTINE TO RETURN SLOPE AND ANGLE OF STEEPEST DECENT IF POSITIVE 

void   SET2(int I, int J,float *DXX,float DD)
{
	float SK[9];
	float ANGLE[9];
	float SMAX;
	int K;
	int KD;

  int ID1[]= {0,1,2,2,1,1,2,2,1 }; 
  int ID2[]= {0,2,1,1,2,2,1,1,2};
  int I1[] = {0,0,-1,-1,0,0,1,1,0 };
  int I2[] = {0,-1,-1,-1,-1,1,1,1,1};
  int J1[] = {0,1,0,0,-1,-1,0,0,1};
  int J2[] = {0,1,1,-1,-1,-1,-1,1,1};
   float  ANGC[]={0,0.,1.,1.,2.,2.,3.,3.,4.};
   float  ANGF[]={0,1.,-1.,1.,-1.,1.,-1.,1.,-1.};
 
   
   for(K=1; K<=8; K++)
   {
	   VSLOPE(felevg.d[J][I],
		   felevg.d[J+J1[K]][I+I1[K]],
		   felevg.d[J+J2[K]][I+I2[K]],
		   DXX[ID1[K]],DXX[ID2[K]],DD,&SK[K],&ANGLE[K]);
   }
   
   SMAX=0.;
   KD=0;
   fangle.d[J][I]=-1.;  //USE -1 TO INDICATE DIRECTION NOT YET SET 
   for(K=1; K<=8; K++)
   {
	   if(SK[K] >  SMAX)
	   {
		   SMAX=SK[K];
		   KD=K;
	   }
   }

   if(KD  > 0)
   {
	   fangle.d[J][I] = (float) (ANGC[KD]*PI2+ANGF[KD]*ANGLE[KD]) ;
   }
   fslopeg.d[J][I]=SMAX;
}


//************************************************************************

void sloped8(float nodata)
{

	int k,i,j,in,jn;
	float fact[9],ed;
	/*  Direction factors  */
	for(k=1; k<= 8; k++)
		fact[k]= (float) (1./sqrt(d1[k]*dy*d1[k]*dy+d2[k]*d2[k]*dx*dx));
 
	for(i=i2; i< n2; i++)for(j=i1; j<n1; j++)
	{
		if(sdir.d[j][i] > 0)
		{
			jn=j+d2[sdir.d[j][i]];
			in=i+d1[sdir.d[j][i]];
			ed = felevg.d[j][i] - felevg.d[jn][in];
			float tempvalue = ed*fact[sdir.d[j][i]];
			fslopeg.d[j][i]= tempvalue;	
		}
		else
			fslopeg.d[j][i]= nodata;
	}

}

//************************************************************************

 
void set2(int i,int j,float *fact,float *elev1, float *elev2, int iter, 
		  int **spos, short *s, short useflowfile)
{
/*  This function sets directions based upon secondary elevations for
  assignment of flow directions across flats according to Garbrecht and Martz
  scheme.  There are two possibilities: 
	A.  The neighbor is outside the flat set
	B.  The neighbor is in the flat set.
	In the case of A the elevation of the neighbor is set to 0 for the purposes
	of computing slope.  Since the incremental elevations are all positive there is 
	always a downwards slope to such neighbors, and if the previous elevation 
	increment had 0 slope then a flow direction can be assigned.*/

	float slope,slope2,smax,ed;
	int k,spn,sp,kflat=0;
	short in,jn;
	smax=0.;
	sp=spos[j][i];
	for(k=1; k<=8; k++)
	{
		jn=j+d2[k];
		in=i+d1[k];
		spn=spos[jn][in];
		if(iter <= 1)
		{
			ed = felevg.d[j][i] - felevg.d[jn][in];
		}
		else
		{
			ed = elev1[sp] - elev1[spn];
		}
		slope=fact[k]* ed;
		if(spn < 0 || s[spn] < 0)
		{
			/*  The neighbor is outside the flat set.  */
			ed=0.;
		}
		else
		{
			ed=elev2[spn];
		}
		slope2 =fact[k]*(elev2[sp]-ed);
			/*  Only if latest iteration slope is 
 //			positive and previous iteration slope flat  */		
 
//9/13/03  add check for opposing directions to resove a bug reported by Anna Katarina Mahlau
		
 		int adrdirnn=0;
		if(sdir.d[jn][in] > 0 && sdir.d[jn][in]<=8)adrdirnn=abs(sdir.d[jn][in]-k);  // only calculate if sdir has been really set

//		if(adrdirnn == 4)
//			in=in;
		if(slope2 > smax && slope >= 0. && (felevg.d[j][i] - felevg.d[jn][in])>=0. 
			&& adrdirnn!=4)
		/*  Only if latest iteration slope is 
			positive and previous iteration slope flat and real elevation difference not uphill
			The elevation check added 3/1/03 to resolve a bug reported by Anna Katarina Mahlau <au@ivu-umwelt.de> */
		{
			smax=slope2;
			sdir.d[j][i]=k;
		}
	}  /*  End of for  */

}

//************************************************************************

int dontcross(int k,int i,int j)
{
	int n1, n2, c1, c2, ans=0;
switch(k)
{
  case 2:
		n1=1; c1=4; n2=3; c2=8;		 
		if(sdir.d[j+d2[n1]][i+d1[n1]] == c1 ||
			sdir.d[j+d2[n2]][i+d1[n2]] == c2)
			  ans=1;
		break;
  case 4:
		n1=3; c1=6; n2=5; c2=2;		 
		if(sdir.d[j+d2[n1]][i+d1[n1]] == c1 ||
		sdir.d[j+d2[n2]][i+d1[n2]] == c2)
		   ans=1;
		break;
	  case 6:
		n1=7; c1=4; n2=5; c2=8;	
		if(sdir.d[j+d2[n1]][i+d1[n1]] == c1 ||
		sdir.d[j+d2[n2]][i+d1[n2]] == c2)
		   ans=1;
		break;
	  case 8:
		n1=1; c1=6; n2=7; c2=2;		 
		if(sdir.d[j+d2[n1]][i+d1[n1]] == c1 ||
			sdir.d[j+d2[n2]][i+d1[n2]] == c2)
			   ans=1;
		break;
}
return(ans);
}

//*************************************************************************

void incfall(int n, float *elev1, short *s1,int **spos,  
			 int iter, int *sloc)
{
	/* This routine implements drainage towards lower areas - stage 1 */
	int done=0,donothing,k,ip,ninc,nincold,spn;
	short st=1,i,j,in,jn;
	float ed;
	nincold= -1;
	while(done < 1)
	{
		done=1;
		ninc=0;
		for(ip=0; ip<n; ip++)
		{
/*			if 	adjacent to same level or lower that drains or 
				adjacent to pixel with s1 < st and dir not set
				do nothing  */
			donothing=0;
			j=js[sloc[ip]];
			i=is[sloc[ip]];
			for(k=1; k<=8; k++)
			{
		//7/28/04  DGT Dont cross check added
			  if(dontcross(k,i,j) ==0){   // Only examine neighbors that do not cross existing flow directions, e.g. due to burn in
				jn=j+d2[k];
				in=i+d1[k];
				spn=spos[jn][in];
				if(iter <= 1)
				{
					ed= felevg.d[j][i] - felevg.d[jn][in];
				}
				else
				{
					ed = elev1[sloc[ip]] - elev1[spn];
				}
				//9/13/03  DGT changed conditional below to >=1 <=8 rather than !=0 to also capture 9 case 
				//decided to revert back to old code.  This did not succeed in flagging the entire pit
				if(ed >= 0. && sdir.d[jn][in] != 0)
				{
					donothing = 1;  /* If neighbor drains */
				}
				if(spn >= 0) /* if neighbor is in flat   */
					if(s1[spn] >= 0 && s1[spn] < st   /*  If neighbor is not being  */
						&& (sdir.d[jn][in]  == 0 )){
							donothing = 1;   /*  Incremented  */
					}
			  }
 			}   //9/13/03  DGT added ==9 in the above for consistency
 			if(donothing == 0)
			{
				s1[sloc[ip]]++;
				ninc++;
				done=0;
			}
		}   /*  End of loop over all flats  */
		st=st+1;
//		printf("Incfall %d %d \n",ninc,n);
		if(ninc == nincold)
		{
			done = 1;
//			printf("There are pits remaining, direction will not be set\n");
/*  Set the direction of these pits to 9 to flag them   */
			for(ip=0; ip<n; ip++)  /*  loop 2 over all flats  */
			{
/*			if 	adjacent to same level or lower that drains or 
				adjacent to pixel with s1 < st and dir not set
				do nothing  */
				donothing=0;
				j=js[sloc[ip]];
				i=is[sloc[ip]];
				for(k=1; k<=8; k++)
				{
					jn=j+d2[k];
					in=i+d1[k];
					spn=spos[jn][in];
					if(iter <= 1)
					{
						ed= felevg.d[j][i] - felevg.d[jn][in];
					}
					else
					{
						ed= elev1[sloc[ip]]- elev1[spn];
					}
				//9/13/03  DGT changed conditional below to >=1 <=8 rather than !=0 to also capture 9 case
				//decided to revert back to old code.  This did not succeed in flagging the entire pit
					if(ed >= 0. && sdir.d[jn][in] != 0)donothing = 1;  /* If neighbor drains */
					if(spn >= 0) /* if neighbor is in flat   */
					if(s1[spn] >= 0 && s1[spn] < st   /*  If neighbor is not being  */
						&& (sdir.d[jn][in]  == 0 ))donothing = 1;   /*  Incremented  */
				}  //9/13/03  DGT added ==9 in the above for consistency
				if(donothing == 0)
				{
				   sdir.d[j][i] = 9;
			   			   
 				}
			}   /*  End of loop 2 over all flats  */
		}
		nincold=ninc;
	}  /*  End of while done loop  */
}

//************************************************************************

void incrise(int n, float *elev1, short *s2,int **spos, int iter, int *sloc)
{

	/*  This routine implements stage 2 drainage away from higher ground 
	dn is used to flag pixels still being incremented */
	int done=0,ip,k,ninc,nincold,spn;
	float ed;
	short i,j,in,jn;
	nincold=0;
	while(done < 1)
	{
		done=1;
		ninc=0; 
		for(ip=0; ip<n; ip++)
		{
			for(k=1; k<=8; k++)
			{
				j=js[sloc[ip]];
				i=is[sloc[ip]];
				jn=j+d2[k];
				in=i+d1[k];
				spn=spos[jn][in];

				if(iter <= 1)
				{
					ed = felevg.d[j][i] - felevg.d[jn][in];
				}
				else
				{
					ed= elev1[sloc[ip]] - elev1[spn];
				}
				if(ed < 0.)dn[sloc[ip]]=1;
				if(spn >=0)
					if(s2[spn] > 0)dn[sloc[ip]] = 1;
			}
		}
		for(ip=0; ip<n; ip++)
		{
			s2[sloc[ip]]=s2[sloc[ip]]+dn[sloc[ip]];
			ninc=ninc+ (dn[sloc[ip]]>0 ? 1: 0);   // DGT 9/13/03 changed to because now dn is not 0 or 1
			if(dn[sloc[ip]] == 0)done=0;  /*  if still some not being incremented continue 
									looping  */
		}
//		printf("incrise %d %d\n",ninc,n);   
		if(ninc == nincold)done=1;   /*  If there are no new cells incremented  
									 stop - this is the case when a flat has
									 no higher ground around it.  */
		nincold=ninc;
	}

}

//************************************************************************

int flatrout(int n,int *sloc, short *s, int **spos,int iter,float *elev1,
			  float *elev2, float *fact, int ns, short useflowfile)
{
	int ip,nu, *sloc2,ipp,err=TD_NO_ERROR;
	float *elev3;
	
	incfall(n,elev1,s,spos,iter,sloc);
	for(ip=0; ip < n; ip++)
	{
		elev2[sloc[ip]]=(float)(s[sloc[ip]]);
		s[sloc[ip]]=0;   /*  Initialize for pass 2  */
	}
	
	incrise(n,elev1,s,spos,iter,sloc);
	for(ip=0; ip < n; ip++)
	{
		elev2[sloc[ip]] += (float)(s[sloc[ip]]);
	}
	
	nu=0;
	for(ip=0; ip < n; ip++)
	{
 		set2(is[sloc[ip]],js[sloc[ip]],fact,elev1,elev2,iter,spos,s,useflowfile);
 		if( sdir.d[js[sloc[ip]]][is[sloc[ip]]] == 0)nu++;
	}
	if(nu >= n)return(TD_NO_ERROR);  //  Here the recursion is not converging so bail
	//  DGT 7/31/04 No error reported because this usually occurs at pits and it is OK to have pits be no data
	if(nu > 0)
	{ 
		/*  Iterate Recursively   */
		/*  Now resolve flats following the Procedure of Garbrecht and Martz, Journal
		of Hydrology, 1997.  */
		iter=iter+1;
		//	  printf("Resolving %d Flats, Iteration: %d \n",nu,iter);
		sloc2 = (int *)malloc(sizeof(int) * nu);
		elev3 = (float *)malloc(sizeof(float) *ns);
		
		if(sloc2 == NULL || elev3 == NULL)
		{
			// printf("Unable to allocate at iteration %d\n",iter);
		}
		/*  Initialize elev3  */
		for(ip=0; ip < ns; ip++)elev3[ip]=0.;
		/*  Put unresolved pixels on new stacks - keeping in same positions  */
		ipp=0;
		for(ip=0; ip<n; ip++)
		{
			if(sdir.d[js[sloc[ip]]][is[sloc[ip]]] == 0)
			{
				sloc2[ipp]=sloc[ip];
				/*   Initialize the stage 1 array for flat routing   */
				s[sloc[ip]] = 1;
				ipp++;
				
			}
			else
			{
			s[sloc[ip]] = -1;  /*  Used to designate out of remaining flat on 
			higher iterations   */ 
			}
			dn[sloc[ip]]=0;  /*  Reinitialize for next time round.  */
		}
		err=flatrout(nu,sloc2,s,spos,iter,elev2,elev3,fact,ns,useflowfile);
		free(sloc2);
		free(elev3);
		//  printf("Done iteration %d\nFlats resolved %d\n",iter,n);
	} /*  end if nu > 0  */
   return(err);
}   /*  End flatrout  */


//************************************************************************

void set(int i,int j,float *fact,float mval,short useflowfile)
{
	float slope,smax;
	int k,amax,in,jn,aneigh=-1;
	short dirnb;
	sdir.d[j][i]=0; //This necessary for repeat passes after level raised */
	smax=0.;
	amax=0;
	for(k=1; k<=8; k=k+2)   // examine adjacent cells first
	{
		in=i+d1[k];
		jn=j+d2[k];
		if(felevg.d[jn][in] <= mval) sdir.d[j][i]=MISSINGSHORT; //-32767);
		if(sdir.d[j][i] != MISSINGSHORT)
		{
			slope=fact[k]*(felevg.d[j][i] - felevg.d[jn][in]); 
			if(useflowfile == 1)aneigh=larr.d[jn][in]; 
			if(aneigh > amax && slope >= 0.)  
			{
				amax=aneigh;
				dirnb=sdir.d[jn][in];
				if(dirnb > 0 && abs(dirnb-k) != 4)
				{
				sdir.d[j][i]=k;   // Dont set opposing pointers
				}
			}
			else if(slope > smax) // && amax <=0)
			{
				smax=slope;
				dirnb=sdir.d[jn][in];
				if(dirnb > 0 && abs(dirnb-k) ==4)  //Dont set an opposing pointer.
//  Here we have a definitive positive slope but a neighbor that drains uphill
//  This occurs with imposed flow directions
//  Resolve by setting the direction of the burned in direction to no data
				{
				sdir.d[jn][in]=MISSINGSHORT; //-32767
				}
				sdir.d[j][i]=k; 
			}
		}
	}
	for(k=2; k<=8; k=k+2)   // examine diagonal cells 
	{   
		in=i+d1[k];
		jn=j+d2[k];
		if(felevg.d[jn][in] <= mval)sdir.d[j][i]=MISSINGSHORT; //-32767
		if(sdir.d[j][i] != MISSINGSHORT) //-32767
		{
			slope=fact[k]* (felevg.d[j][i]-felevg.d[jn][in]);
			if(slope > smax && dontcross(k,i,j) ==0)  // Dont cross pointers
			{
				smax=slope;
				dirnb=sdir.d[jn][in];
				if(dirnb > 0 && abs(dirnb-k) ==4)  //Dont set an opposing pointer.
//  Here we have a definitive positive slope but a neighbor that drains uphill
//  This occurs with imposed flow directions
//  Resolve by setting the direction of the burned in direction to no data
				{
				sdir.d[jn][in] = MISSINGSHORT;	//-32767 
				}
				sdir.d[j][i]=k; 
			}
		}
	}

}


int setdfnoflood(float mval, float fdmval, short useflowfile)
/*  This version is stripped of pit filling  */
{
	int i,j,k,ip, n, iter, err=TD_NO_ERROR;
	float fact[9];
	short *s;  /*  variables for flat draining   */
	int **spos, *sloc;
	float *elev2;

	/*  initialize internal pointers to 0 except where elevation is no data*/
	for(i=i2+1; i< n2-1; i++)for(j=i1+1; j<n1-1; j++)
	{
		if(felevg.d[j][i] <= mval)
		{
			sdir.d[j][i]=MISSINGSHORT ; // -32767

		}
		else sdir.d[j][i]=0;
	}
	/*  Direction factors  */
	for(k=1; k<= 8; k++)
		fact[k]= (float) (1./sqrt(d1[k]*dy*d1[k]*dy+d2[k]*d2[k]*dx*dx));

//  Set stream overlay directions
	if( useflowfile == 1 )
for(i=i2+1; i< n2-1; i++)
	  for(j=i1+1; j<n1-1; j++)
	  {	
		  {
			  if(sapoolg.d[j][i] > fdmval )
			  {
				  sdir.d[j][i] = sapoolg.d[j][i];
				  larr.d[j][i]=0;
			  }
		  }
	  }

//   Compute contrib area using overlayed directions for direction setting

	ccheck=0;   // dont worry about edge contamination
	useww=0;// dont worry about weights
for(i=i2+1; i< n2-1; i++)
	  for(j=i1+1; j<n1-1; j++)
	  {	
		  //This allows for a stream overlay
		  if( sdir.d[j][i] > 0) darea(i,j);
	  }

/*  Set positive slope directions   */
	n=0;
	for(i=i2+1; i< n2-1; i++)
	  for(j=i1+1; j<n1-1; j++)
	  {	
		  if( sdir.d[j][i] == 0 )
		  {	
			  if(felevg.d[j][i]  > mval)
			  {
				  set(i,j,fact,mval,useflowfile);
 				  if(sdir.d[j][i] == 0)
					  n++;
			  }
		  }
	  }

  if(n > 0)
  { 
/*  Now resolve flats following the Procedure of Garbrecht and Martz, Journal
   of Hydrology, 1997.  */

/*  Memory is utilized as follows
is, js, dn, s and elev2 are unidimensional arrays storing information for flats.
sloc is a indirect addressing array for accessing these - used during
recursive iteration
spos is a grid of pointers for accessing these to facilitate finding neighbors

The routine flatrout is recursive and at each recursion allocates a new sloc for
addressing these arrays and a new elev for keeping track of the elevations
for that recursion level.  
  */
	  iter=1;
//	  printf("Resolving %d Flats, Iteration: %d \n",n,iter);
  spos = (int **) matalloc(nx, ny, RPINTDTYPE);
  dn = (short *)malloc(sizeof(short) * n);
  is = (short *)malloc(sizeof(short) * n);
  js = (short *)malloc(sizeof(short) * n);
  s = (short *)malloc(sizeof(short) * n);
	  sloc = (int *)malloc(sizeof(int) * n);
  elev2 = (float *)malloc(sizeof(float) *n);

  if(dn == NULL || is == NULL || js == NULL || s == NULL || 
	  spos == NULL || elev2 == NULL || sloc == NULL)
  {
// printf("Unable to allocate at iteration %d\n",iter);
  }
/*  Put unresolved pixels on stack  */
   ip=0;
   for(i=i2; i< n2; i++)
	  for(j=i1; j<n1; j++)
  {
spos[j][i]=-1;   /*  Initialize stack position  */
if(sdir.d[j][i] == 0)
{
  is[ip]=i;
  js[ip]=j;
	  dn[ip]=0;
	  sloc[ip]=ip;
	  /*   Initialize the stage 1 array for flat routing   */
	  s[ip] = 1;
	  spos[j][i]=ip;  /*  pointer for back tracking  */
	  ip++;
//	  if(ip > n)printf("PROBLEM - Stack logic\n");
}
  }
  err=flatrout(n,sloc,s,spos,iter,elev2,elev2,fact,n,useflowfile);
/*  The direction 9 was used to flag pits.  Set these to 0  */
  for(i=i2; i< n2; i++)
	  for(j=i1; j<n1; j++)
	  {
		  if(sdir.d[j][i] == 9) sdir.d[j][i]=sdir.nodata;
		  if(sdir.d[j][i] == 0) sdir.d[j][i]=sdir.nodata;
	  }
  free(spos[0]); free(spos);
  free(elev2);
  free(dn);
  free(is);
  free(js);
  free(s);
  free(sloc);
//  printf("Done iteration %d\nFlats resolved %d\n",iter,n);
  } /*  End if n > 0  */
   return(err);   /*  OK exit from setdir  */

}  /*  End setdfnoflood  */

//************************************************************************
/*

Converted from FORTRAN July 05, 1997  K. Tarbet

C
C---SUBROUTINE TO SET VARIABLE DIRECTIONS.
CDIRECTIONS MEASURED ANTICLOCKWISE FROM EAST IN RADIANS.
C
*/

  

void setdf2(short useflowfile)
{

	float FANG[9];
	float dxx[3],slp;
	int i,j,k;
	float dd;

/*   INITIALISE ANGLES and slopes */

	for (i=0; i<nx; i++)
{
	    fangle.d[i][0] = -2;
	    fangle.d[i][ny-1] = -2;
		
		fslopeg.d[i][0]=-1;
		fslopeg.d[i][ny-1]=-1;

}

	for(i=0; i<ny; i++)
	{
		fangle.d[0][i]=-2;
		fangle.d[nx-1][i]=-2;
		
		fslopeg.d[0][i]=-1;
		fslopeg.d[nx-1][i]=-1;

	}

 FANG[1]=0.;
 FANG[2]=(float)atan2(dy,dx);
 FANG[3]=(float) PI2; 
 FANG[4]=2*FANG[3]-FANG[2];
 FANG[5]=2*FANG[3];
 FANG[6]=2*FANG[3]+FANG[2];
 FANG[7]=3*FANG[3];
 FANG[8]=4*FANG[3]-FANG[2];

/* --INITIALISE INTERNAL POINTERS (-VE POINTER INDICATES OUTSIDE DOMAIN) */

	for(i=1; i<ny-1; i++)
	{
		for(j=1; j<nx-1; j++)
{
			if(sdir.d[j][i] <  0)   
{				fangle.d[j][i] = -2;
				fslopeg.d[j][i] = -1;

				 /*  !  -1 slope indicates no data */
}
else if(sdir.d[j][i] == 0)
{				
				fangle.d[j][i]=-1;
				
				/*  DGT 9/2/97 since without   */
				fslopeg.d[j][i] = 0;
				
/*  ang = -1 designates unfilled pit, ang = -2 designates no data.  */  
}
else			
				fangle.d[j][i] = 0;
		}
	}



/*TEST ALL INTERNAL ELEVATIONS AND SET SLOPE AND ANGLE */

   dxx[1]=dx;
   dxx[2]=dy;
   dd=(float)sqrt(dx*dx+dy*dy);
	   for(i=1; i<ny-1; i++)
	   {

		   for(j=1; j<nx-1; j++)
		   {
   if(sdir.d[j][i] >0 )
   {
   SET2(i,j,dxx,dd);
//  9/14/02 DGT decided to set flow directions along existing paths if they are given 
			if(useflowfile ==1){
				   k=sapoolg.d[j][i];
				   if(k > 0)
				   {					   
					 fangle.d[j][i]=FANG[ k ];
					  slp=(felevg.d[j][i]-felevg.d[j+d2[k]][i+d1[k]])
						  /sqrt(dx*dx*d2[k]*d2[k]+dy*dy*d1[k]*d1[k]);
					  fslopeg.d[j][i]=slp;
				   }
			}
   if(fangle.d[j][i] < -0.5)
				   {
					  fangle.d[j][i]=FANG[sdir.d[j][i]];
				   }
   }
			   
		   }
	   }
}

//************************************************************************

/*  C version of setdir.c. This does not fill pits, it is intended for use AFTER
d8.c which fills pits.  Input is a dem file name.  Output is
slope and dinf angle files.   This version implements Garbrecht flow directions
	for flats.
 
  David G Tarboton
  Utah State University   
 */

/*
This roughly follows the following pseudocode from my review of Garbrecht's paper

Pseudocode to iteratively resolve flow directions

Define arrays E(x), D(x), S1(x), S2(x), I(x) where x refers to grid location.
 where E is original elevation, D is total increment, S1 step 1 increment, 
 S2 step 2 increment, I a flag to indicate whether flow direction has been set.  

Initialize  
do for each x
	read in E(x)
	I(x) = 1
enddo
unresolved = 1
while(unresolved > 0)
	unresolved = 0
	do for each x
		if(I(x) > 0)then  /*  only apply to unresolved pixels  
			calculate slope to each neighbor using E(x) 
			if(max slope > 0)then
				assign flow direction to appropriate neighbor
				I(x) = 0   /*  flag it as resolved   
			else
				unresolved=unresolved + 1
				I(x) = 1   /* this flags the pixel as unresolved  
			endif
		endif
	enddo
	if(unresolved > 0)then
		for each x
			D(x) = S2(x) = S2(x)= 0  /*  reset arrays for iteration  
		for each x with I(x) > 0
			Calculate step 1 increment towards falling terrain, S1(x) 
		for each x with I(x) > 0
			Calculate step 2 increment away from rising terrain, S2(x)
		for each x with I(x) > 0
			D(x) = S1(x) + S2(x)
		for each x
			E(x) = D(x)   
/*  overwrite original elevations since they are no longer needed from now 
on slopes are based on D(x)  
	endif
endwhile
output results
end

  */

 

int setdir(char *demfile, char *angfile, char *slopefile, char * flowfile, 
			 short useflowfile, bool LoadIntoMemory)
{
	inRam = LoadIntoMemory;
   int i;
	char * tempFileName = NULL;
	
	int filetype;
	err = TD_NO_ERROR;
	float fdmval;
	
	/* define directions */
	d1[1]=0; d1[2]= -1; d1[3]= -1; d1[4]= -1; d1[5]=0; d1[6]=1; d1[7]=1; d1[8]=1;
	d2[1]=1; d2[2]=1; d2[3]=0; d2[4]= -1; d2[5]= -1; d2[6]= -1; d2[7]=0; d2[8]=1;
	

	if(gridread(demfile,&felevg,&filetype) ==0)
		err=TD_NO_ERROR;
	else
		err=TD_FAILED_GRID_OPEN;

	


	nx =felevg.head.nx;
	ny = felevg.head.ny;
	dx =felevg.head.dx; 
	dy = felevg.head.dy; 
	csize = dx;
	mval = felevg.nodata ; 

	
for(i=0;i<4;i++) bndbox[i] = felevg.head.bndbox[i];

	if(err != TD_NO_ERROR)goto ERROR2;  
	
	if( useflowfile == 1 )
	{
		
		if(gridread(flowfile,&sapoolg,&filetype)==0) 
			err=TD_NO_ERROR;
		else
			err=TD_FAILED_GRID_OPEN;

		fdmval = sapoolg.nodata; 
		if(err != TD_NO_ERROR)return(err);
/*		larr.head.dx=dx;
		larr.head.dy=dy;
		larr.head.nx=nx;
		larr.head.ny=ny;
		for(i=0;i<4;i++) larr.head.bndbox[i]=felevg.head.bndbox[i];   */
		larr.head=felevg.head;
		larr.nodata=-1;

		
		
		if (allocategrid(&larr,larr.head,larr.nodata)!=0) 
			err = TD_FAILED_GRD_INIT;


	}
	if(err != TD_NO_ERROR)goto ERROR4;
	
	/*  allocate  dir and stack arrays  */
	
/*	sdir.head.dx=dx;
	sdir.head.dy=dy;
	sdir.head.nx=nx;
	sdir.head.ny=ny;  */
	sdir.head=felevg.head;
	sdir.nodata=MISSINGSHORT; //(-32767)
	
	
	if (allocategrid(&sdir,sdir.head,sdir.nodata) !=0)
		err = TD_FAILED_GRD_INIT;

	if(err != TD_NO_ERROR)goto ERROR1;

	i1=0; i2=0; n1=nx; n2=ny;  /*  full grid  */ 
	
	err=setdfnoflood(mval,fdmval,useflowfile);
	
	if(err != TD_NO_ERROR)goto ERROR1;
	
	/*  allocate memory for slope and angle  */
	
	fslopeg.head=felevg.head;
	fslopeg.nodata=-1;
	if (allocategrid(&fslopeg,fslopeg.head,fslopeg.nodata)!=0) 
		err = TD_FAILED_GRD_INIT;


	if (err != TD_NO_ERROR) goto ERROR1;


	fangle.head=felevg.head;
	fangle.nodata=-2;

	if (allocategrid(&fangle,fangle.head,fangle.nodata)!=0) 
		err = TD_FAILED_GRD_INIT;

	if (err != TD_NO_ERROR) goto ERROR1;
	
	setdf2(useflowfile);
	
	if (gridwrite(slopefile,fslopeg,filetype)==0) 
		err=TD_NO_ERROR;
	else{
		err=TD_FAILED_GRID_SAVE;
		return err;
//		if (slopefile)
//			AfxMessageBox( LPCTSTR(strcat( "Failed to save file: ", slopefile) ));
	}

	
	if (err != TD_NO_ERROR)goto ERROR3;
	
	
	if (gridwrite(angfile,fangle,filetype)==0)
		err=TD_NO_ERROR;
	else 
		err=TD_FAILED_GRID_SAVE;
	
	if (err != TD_NO_ERROR)goto ERROR3;
	
	/*  Wrapping up  */
	err=TD_NO_ERROR;
ERROR3:
	
	free(fslopeg.d[0]);
	free(fslopeg.d);
	
	free(fangle.d[0]);
	free(fangle.d);
ERROR1:
	
	free(sdir.d[0]);
	free(sdir.d);
ERROR4:
	if(useflowfile == 1)
	{	
		free(larr.d[0]);
		free(larr.d);

		free(sapoolg.d[0]);
		free(sapoolg.d);
	}
ERROR2:
	free(felevg.d[0]);
	free(felevg.d);
	return(err);
}

//************************************************************************

 

int setdird8(char *demfile, char *pfile, char *slopefile, char * flowfile, 
			 short useflowfile,bool LoadIntoRam)
{

	inRam = LoadIntoRam;

	char * tempFileName = NULL;
	int i;
	err = TD_NO_ERROR;
	float mval, fdmval;

	
	
	/* define directions */
	d1[1]=0; d1[2]= -1; d1[3]= -1; d1[4]= -1; d1[5]=0; d1[6]=1; d1[7]=1; d1[8]=1;
	d2[1]=1; d2[2]=1; d2[3]=0; d2[4]= -1; d2[5]= -1; d2[6]= -1; d2[7]=0; d2[8]=1;
	
	
	
	if(gridread(demfile,&felevg,&filetype)==0)
		err=TD_NO_ERROR;
	else
		err=TD_FAILED_GRID_OPEN;
	
	

	nx =felevg.head.nx;
	ny = felevg.head.ny;
	dx =felevg.head.dx; 
	dy = felevg.head.dy;
	csize = dx;
	mval = felevg.nodata;

	::mval = mval; //for some reason there is a global and local copy of this variable...?

	//filetype = 0; // ASCI filetype

	//bndbox is working off of the corner of the cell, not the center
	//so it needs to remove 1/2 of a cell size to map down to the corner
	//instead of the center of the cell...
	for(i=0;i<4;i++) bndbox[i]=felevg.head.bndbox[i];

	if(err != TD_NO_ERROR)goto ERROR2;  
	
	/*  allocate  dir and stack arrays  */

	sdir.head.nx=nx;
	sdir.head.ny=ny;
	sdir.head.dx=dx;
	sdir.head.dy=dy;
	sdir.nodata= MISSINGSHORT; //-32767
	for(i=0;i<4;i++) sdir.head.bndbox[i]=felevg.head.bndbox[i];


	if (allocategrid(&sdir,sdir.head,sdir.nodata) !=0)
	{
		err = TD_FAILED_GRD_INIT;
		goto ERROR2;
		
	}
	
	if( useflowfile == 1 )
	{
		
		if(gridread(flowfile,&sapoolg,&filetype)==0)
			err=TD_NO_ERROR;
		else
			err=TD_FAILED_GRID_OPEN;
		
		fdmval = sapoolg.nodata;

		if(err != TD_NO_ERROR)return(err);
		larr.head.dx=dx;
		larr.head.dy=dy;
		larr.head.nx=nx;
		larr.head.ny=ny;
		for(i=0;i<4;i++) larr.head.bndbox[i]=felevg.head.bndbox[i];
		larr.nodata=-1;

	
		if (allocategrid(&larr,larr.head,larr.nodata) !=0)
			err = TD_FAILED_GRD_INIT;
	}
	if(err != TD_NO_ERROR)goto ERROR4;
	
	i1=0; i2=0; n1=nx; n2=ny;  /*  full grid  */ 
	err = setdfnoflood(mval, fdmval, useflowfile);
	if(err != TD_NO_ERROR)goto ERROR1;
	
	if (gridwrite(pfile,sdir,filetype)==0)
		err=TD_NO_ERROR;
	else{
		err=TD_FAILED_GRID_SAVE;
		return err;
	//	if (pfile)
	//		AfxMessageBox( LPCTSTR(strcat( "Failed to save file: ", pfile) ));
	}

	if(err != TD_NO_ERROR)goto ERROR1;

	
	/*  allocate memory for slope   */

	fslopeg.head.dx=dx;
	fslopeg.head.dy=dy;
	fslopeg.head.nx=nx;
	fslopeg.head.ny=ny;
	for(i=0;i<4;i++) fslopeg.head.bndbox[i]=felevg.head.bndbox[i];
	
	if (allocategrid(&fslopeg,fslopeg.head,	-1.0) !=0)
	{
		err = TD_FAILED_GRD_INIT;
		goto ERROR1;
	}
	sloped8(fslopeg.nodata);
	if (gridwrite(slopefile,fslopeg,filetype)==0)
		err=TD_NO_ERROR;
	else{
		err=TD_FAILED_GRID_SAVE;
		return err;

	}
	
	if (err != TD_NO_ERROR)goto ERROR3;
	
	err=TD_NO_ERROR;
ERROR3:
	free(fslopeg.d[0]);free(fslopeg.d);

ERROR1:
	free(sdir.d[0]);free(sdir.d);
	
ERROR4:
	if(useflowfile == 1)
	{
		free(larr.d[0]); free(larr.d);
	}
ERROR2:
	free(felevg.d[0]); free(felevg.d);
	return(err);

	

}



//************************************************************************

float prop(float a,int k)
{
float p=0.;
if(k == 1 && a > PI)a=a-2.*PI;
if(a > aref[k-1] && a < aref[k+1]){
  if( a > aref[k])
   p=(aref[k+1]-a)/(aref[k+1]-aref[k]);
  else 
  p=(a-aref[k-1])/(aref[k]-aref[k-1]);
  }
return(p);
}

//************************************************************************

 

/* function to compute partial areas recursively */
void dparea(int i,int j)
{
	

	int in,jn,k,kback,con=0;
/* con is a flag that signifies possible contaminatin of sca
	due to edge effects  */
	float p;

	if(fsca.d[j][i] ==0.0)/* i.e., if the cell hasn't been looked at yet */
	{
if(i!=0 && i!=ny-1 && j!=0 && j!=nx-1 && fangle.d[j][i]> ndva)
			/* i.e. the cell isn't outside domain */
		{
			fsca.d[j][i]=dx;
			
			
			if(useww == 1) fareaw.d[j][i] = fweight.d[j][i];
		
			for(k=1; k<=8; k++)
			{
				in = i + d1[k];
				jn = j + d2[k];
				/* for each neighboring cell, find the proportion of of outflow */
				/* draining back to the cell in question */
				kback = k + 4;
				if(kback>8)kback = kback - 8;
				p=prop(fangle.d[jn][in],kback);
				if(p>1e-6)  /*  Recurse to upslope neighbor only for grid cells 
					with proportion greater than 1e-6 to guards against
					numerical loops due to angle rounding  DGT 9/11/08*/	
{
					dparea(in,jn);
					if(fsca.d[jn][in] < -0.5) 
					{  // DGT 8/6/06.  Added the if/else logic here.  It was a bug to add fsca and fareaw.d 
						// values when fsca is no data.
						con = -1;
					}
					else
					{

					/*****
					 tempvalue (below) holds the value of the grid to be used in the 
					 multiplication of tempvalue with p.  This is done because of 
					 rounding errors that were encountered and had weird effects on the data.
					 By storing it before multiplying, I was able to get the function 
					 to give the correct results (don't ask why this,solved the problem, 
					 because I don't know).  --Jonathan Cole
					*****/
					float tempvalue = fsca.d[jn][in];
					fsca.d[j][i]=fsca.d[j][i] + tempvalue*p;
					
					if(useww == 1)
					{
						if(fareaw.d[jn][in] <= ndv  || fareaw.d[j][i] <=ndv)
						{  
							
							fareaw.d[j][i]=ndv;
						}
						else
						{
							tempvalue = fareaw.d[jn][in];
							fareaw.d[j][i]=fareaw.d[j][i] + tempvalue*p;
						}
					}
					}
}
				if(fangle.d[jn][in] <= ndva)con = -1; 
}
			if(con == -1 && ccheck == 1)
			{
				fsca.d[j][i] = -1.0;
				if(useww == 1) fareaw.d[j][i]=ndv;
			}
		}
	}

} 


  


int area(char *pfile, char *afile, double *x, double *y, long nxy, int doall, 
		 char *wfile, int usew, int contcheck, bool inRam)
{

	int i,j,row,col;
	err = TD_NO_ERROR;
	ccheck=contcheck;
	useww=usew;
	
	/* Define directions */
	/*  d1a=dd1-1; d2a=dd2-1;  */
	d1[1]=0; d1[2]= -1; d1[3]= -1; d1[4]= -1; d1[5]=0; d1[6]=1; d1[7]=1; d1[8]=1;
	d2[1]=1; d2[2]=1; d2[3]=0; d2[4]= -1; d2[5]= -1; d2[6]= -1; d2[7]=0; d2[8]=1;
	
	/* read angles */
	//gridread function would return false (0) if there were no errors, but
	//the new grid open function returns true(1) if there were no errors;
	//therefore, this next if/else statement corrects the difference
	
	if (gridread(pfile,&fangle,&filetype) == 0)
		err=TD_NO_ERROR;
	else
		err=TD_FAILED_GRID_OPEN;
	

	nx = fangle.head.nx;     
	ny = fangle.head.ny; 
	dx = fangle.head.dx; 
	dy = fangle.head.dy; 
	csize = dx; 
	ndva = fangle.nodata;

	
	//bndbox is working off of the corner of the cell, not the center
	//so it needs to remove 1/2 of a cell size to map down to the corner
	//instead of the center of the cell...

	 for (i=0;i<4;i++) bndbox[i]=fangle.head.bndbox[i];

	

	if(err != TD_NO_ERROR)goto ERROR1;   
		
	/*  read weights if necessary   */
	if(usew == 1)
	{

		if(gridread(wfile,&fweight,&filetype)==0)
			err=TD_NO_ERROR;
		else
		{
		    err=TD_FAILED_GRID_OPEN;
		}

		ndv = fweight.nodata;

		
		
	if(err != TD_NO_ERROR)goto ERROR2;
		  fareaw.head=fangle.head;
		fareaw.nodata=ndv;
		err=allocategrid(&fareaw,fareaw.head,fareaw.nodata);
		
	}
	
		
	/* Calculate angles for interpolation */  
	aref[0]= -atan2(dy,dx);
	aref[1]=0.;
	aref[2]= -aref[0];
	aref[3]=(float)(0.5*PI);
	aref[4]=PI-aref[2];
	aref[5]=(float)PI;
	aref[6]=PI+aref[2];
	aref[7]=(float)(1.5*PI);
	aref[8]=2.*PI-aref[2];
	aref[9]=(float)(2.*PI);
	
	
	/* initialize sca grid and header */
	
	fsca.head=fangle.head;
	fsca.nodata=-1;
	err=allocategrid(&fsca,fsca.head,fsca.nodata);
    for(i=0; i<ny; i++)
	{
		for(j=0; j<nx; j++)
		{
			if(i!=0 && i!=ny-1 && j!=0 && j!=nx-1 && fangle.d[j][i] > ndva)
			{
				fsca.d[j][i]=0.0;
			
			}
			
			
		}
	}

	row=0; col=0;


	if(doall == 0)   /*  Only compute area's for designated location  */
	{
		for (int CurXY = 0; CurXY < nxy; CurXY++)
		{
			col= (int)floor((x[CurXY]-bndbox[0])/csize);
			row= (int)floor((bndbox[3]-y[CurXY])/csize);
			if(row <0 || row > ny || col < 0 || col > nx)
			{
				//			printf("Given coords out of area - whole area will be calculated\n");
				row=0; col=0;
			}
			/* call drainage area subroutine for pixel to zero on  */
			dparea(row,col);
		}
	}
	else
	{
		
		/* Call drainage area subroutine for each cell   */
		/* working from the middle out to avoid deep recursions  */ 
		
		/* "lower block" */
		for(i=ny/2; i<ny-1; i++)
		{
			
			for(j=nx/2; j<nx-1; j++)
				dparea(i,j);
			for(j=nx/2-1; j>=1; j--)
				dparea(i,j);

		}
		/* "upper block" */
		for(i=ny/2-1; i>=1; i--)
		{
			
			for(j=nx/2; j<nx-1; j++)
				dparea(i,j);
			for(j=nx/2-1; j>=1; j--)
				dparea(i,j);
		}
}
	/*  Write out areas  */  
	if(usew == 0)
	{
		if (gridwrite(afile,fsca,filetype)==0)// fScaGrid.save(afile) )
			err=TD_NO_ERROR;
		else{
			err=TD_FAILED_GRID_SAVE;
			return err;
			//if (afile)
			//	AfxMessageBox( LPCTSTR(strcat( "Failed to save file: ", afile)) );
		}
	}
	else
	{
		if ( gridwrite(afile,fareaw,filetype)==0)
			err=TD_NO_ERROR;
		else{
			err=TD_FAILED_GRID_SAVE;
			return err;
		//	if (afile)
		//		AfxMessageBox(LPCTSTR( strcat( "Failed to save file: ", afile) ));
		}
		free(fareaw.d[0]);free(fareaw.d);
	}
  

	free(fsca.d[0]); free(fsca.d);
	free(fangle.d[0]);free(fangle.d);
	

	if(usew == 1){
ERROR2:
	return(err);
	}
ERROR1:
	return(err);  
}  

//************************************************************************

//  Function to climb recursively flow direction grid for min elevation to burn down
float climb(int i,int j)
{
	float emin,eneigh;
	int in,jn,k;
	emin=felevg.d[j][i];
	for(k=1; k<=8; k++)
	{
		in=i+d1[k];
jn=j+d2[k];
/* test if neighbor drains towards cell excluding boundaries */
if(sdir.d[jn][in]>0 && (sdir.d[jn][in]-k==4 || sdir.d[jn][in]-k ==-4))
		{
			eneigh=climb(in,jn);
			if(eneigh < emin)emin=eneigh;
		}
	}
	felevg.d[j][i] = emin;  // burn down elevation
	return(emin);

}

//************************************************************************

//  Function to burn down flow elevations for given flow directions.
//  This is a placeholder for what will eventually be simplex optimization
int fdsimplex()
{


	int in,jn,i,j;
	err = TD_NO_ERROR;
	float emin;
	for(i=1; i<ny-1; i++)
		for(j=1; j<nx-1; j++)
		{
			if(sdir.d[j][i] > 0)
			{
				in=i+d1[sdir.d[j][i]];
				jn=j+d2[sdir.d[j][i]];
				if(sdir.d[jn][in] <= 0)  // Neighbour is out
					emin=climb(i,j);
			}
		}
	return(err);

}

//************************************************************************

//  Function to clean given flow directions.
int dirclean(char *newflowfile, int writeflowfile)
{
/*  Find all locations that have an imposed flow direction that drain to a location 
	that does not have imposed flow directions.
	Evaluate contrib area for these locations not doing contamination checking.
	At the end we have contrib area evaluated for the skeleton of cells that drain 
	off the imposed raster.  Any other cells in the imposed raster are "cleaned"
	from it because they do not drain off the raster, i.e. have inconsistent directions
	*/
	

	int in,jn,i,j;
	err = TD_NO_ERROR;
	ccheck=0;  // These are globals to remove contamination checking
	useww=0;  //  for the darea function
	for(i=1; i<ny-1; i++)
		for(j=1; j<nx-1; j++)
		{
			if(sdir.d[j][i] > 0)
			{
				in=i+d1[sdir.d[j][i]];
				jn=j+d2[sdir.d[j][i]];
				if(sdir.d[jn][in] <= 0)  // Neighbour is out
					darea(i,j);
			}
		}
	//  At this point contrib area has been evaluated for all grid cells that drain to 
	//	an edge or internal end.  Set direction to no data for the rest.
	for(i=1; i<ny-1; i++)
		for(j=1; j<nx-1; j++)
		{
			if(larr.d[j][i] <= 0)
			{   // reinitialize   10/5/02.  DGT modified below to include elevation check
				if(felevg.d[j][i] <= mval) sdir.d[j][i] =MISSINGSHORT; //-32767
				else sdir.d[j][i]=0;   
			}
		}  
	if(writeflowfile == 1)
	{// Use pool array as placeholder to write directions
		for(i=0; i<ny; i++)
			for(j=0; j<nx; j++)
			{
				if(sdir.d[j][i] >0) sapoolg.d[j][i] = sdir.d[j][i];
				else sapoolg.d[j][i]= MISSINGSHORT; //-32767
			} 
			
			sapoolg.nodata=MISSINGSHORT; 
			
			//sApoolGrid.setHeader(sApoolHeader);
			if (gridwrite(newflowfile,sapoolg,filetype)==0 )
				err=TD_NO_ERROR;
			else{
				err=TD_FAILED_GRID_SAVE;
				return err;
			//	if (newflowfile)
			//		AfxMessageBox( LPCTSTR(strcat( "Failed to save file: ", newflowfile)) );
			}
	}
	return(err);
}

//************************************************************************

int addstack(int i, int j)
{
	/*  Routine to add entry to is, js stack, enlarging if necessary   */
	nis=nis+1;
	if(nis >= istack )
  {
/*  Try enlarging   */
 istack=(int) (istack + nx*ny*.1);
 if(istack > nx*ny)
 {
//   printf("\n is,js stack too large, exiting ...\n");
   return(4);
 }
// printf("\n Enlarging is,js stack\n");
 is = (short *)realloc(is, sizeof(short) * istack);
 js = (short *)realloc(js, sizeof(short) * istack);
 dn = (short *)realloc(dn, sizeof(short) * istack);
 if(is == NULL || js == NULL || dn == NULL)
{
// printf("\n Could not enlarge stack\n");
 return(5);
}
  }
	is[nis]=i;
	js[nis]=j;
	return(0);
}


//************************************************************************

int vdn(int n)
{

	

  int ip,k,imin;
  float ed;
  nis=n;
  do
  {
  n=nis;
  nis=0;
  for(ip=1; ip <=n; ip++)dn[ip]=0;
  for(k=1; k<=7; k=k+2)
		for(ip=1; ip<=n; ip++)
		{
		   ed= felevg.d[js[ip]][is[ip]] - felevg.d[js[ip]+d2[k]][is[ip]+d1[k]];
		   if(ed >= 0. && sdir.d[js[ip]+d2[k]][is[ip]+d1[k]] != 0 
				   && dn[ip] == 0)
		 dn[ip]=k;
		}
  for(k=2; k<=8; k=k+2)
for(ip=1; ip<=n; ip++)
{
   ed= felevg.d[js[ip]][is[ip]] - felevg.d[js[ip]+d2[k]][is[ip]+d1[k]];
   if(ed >= 0. && sdir.d[js[ip]+d2[k]][is[ip]+d1[k]] != 0 
		   && dn[ip] == 0 && dontcross(k,is[ip],js[ip])==0)
 dn[ip]=k;
}
  imin=1;  /*  location of point on stack with lowest elevation  */
  for(ip=1; ip <= n; ip++)
  {
 if(dn[ip] > 0) sdir.d[js[ip]][is[ip]] = dn[ip];
 else
 {
nis++;
is[nis]=is[ip];
js[nis]=js[ip];
if( felevg.d[js[nis]][is[nis]] < felevg.d[js[imin]][is[imin]] )imin=nis;
 }
  }
  
  }while(nis < n);
  return(imin);

}   

//************************************************************************

/* function to compute pool recursively and at the same time determine
   the minimum elevation of the edge. */
void pool(int i,int j)
{
	
int in,jn,k;
recursedepth=recursedepth+1;

if( sapoolg.d[j][i] <=0)   /* not already part of a pool  */
{
if(sdir.d[j][i]!= MISSINGSHORT)  /* check only dir since dir was initialized  */
 /* not on boundary  */
  {
sapoolg.d[j][i] = pooln;  /*  apool assigned pool number */
npool=npool+1;
if(npool >= pstack)
{
  if(pstack < (nx*ny))
  {
/*  Try enlarging   */
//printf("\n Enlarging pool stack\n");
pstack=(int) (pstack + nx*ny*.1);
if(pstack > nx*ny)
{
//  printf("\n Pool stack too large, exiting ...\n");
}
ipool = (short *)realloc(ipool, sizeof(short) * pstack);
jpool = (short *)realloc(jpool, sizeof(short) * pstack);
if(ipool == NULL || jpool == NULL)
{
//  printf("\n Cannot reallocate pool stack, exiting ...\n");
}
  }
  else
  {
//printf("\n Could not enlarge Pool stack\n");
  }
}
ipool[npool]=i;
jpool[npool]=j;
/*printf("%d %d Pool %d\n",i,j,pooln);  */
for(k=1; k<=8; k++)
{  
   in=i+d1[k];
   jn=j+d2[k];
/* test if neighbor drains towards cell excluding boundaries */
   if((sdir.d[jn][in] >0 && (sdir.d[jn][in]-k==4||sdir.d[jn][in]-k==-4))
  || (sdir.d[jn][in] == 0 && felevg.d[jn][in] >= felevg.d[j][i] 
			  && dontcross(k,i,j) ==0))  
   /* so that adjacent flats get included */
 {
				   pool(in,jn);
 }
}
  }
}
}

//************************************************************************

float max2(float e1,float e2)
{ 
  float em; 
  em=e1;
  if(e2 > em)em=e2;
  return(em);
}


bool notfdr(short useflowfile, int i, int j)
{
	if(useflowfile == 0)
		return(true);
	else if (larr.d[j][i] <= 0)return(true);
	return(false);
}


int setdf(float mval, float fdmval, short useflowfile, 
		  char * newflowfile, int writeflowfile)
{

	int i,j,k,n,nflat,ni,ip,imin,jn,in,np1,nt;
	err = TD_NO_ERROR;
	float fact[9],per=1.;

	/*  initialize internal pointers */
	for(i=i2+1; i< n2-1; i++)for(j=i1+1; j<n1-1; j++)
	{
		if(felevg.d[j][i] <= mval)sdir.d[j][i]=MISSINGSHORT; //-32767
		else sdir.d[j][i]=0;
	}

	//  Set stream overlay directions
	if( useflowfile == 1 )
	{
		for(i=i2+1; i< n2-1; i++)
			for(j=i1+1; j<n1-1; j++)
			{	
				
//   DGT 7/22/02  A problem occurs if direction is set adjacent to no data elevation.
//   Guard against this
				if(sapoolg.d[j][i] > fdmval && sdir.d[j][i] >=0 )
				{
					int isAdjNodata = 0;
					for(k=1; k<=8; k=k+1)   // examine adjacent cells first
					{
						in=i+d1[k];
						jn=j+d2[k];
						if(felevg.d[jn][in] <= mval)isAdjNodata = 1;
					}
					if(isAdjNodata==0)
					{
						sdir.d[j][i]= sapoolg.d[j][i];
						larr.d[j][i]=0;
					}
				}
//  End DGT modifications. 
			}
			//arr=flowdirection;  // to reuse space for area array for cleaning and in set
			err=dirclean(newflowfile,writeflowfile);
			//   Call function to optimize elevation adjustments based on flow directions already given
			err=fdsimplex();			
	}



	
	
	/*  Direction factors  */
	for(k=1; k<= 8; k++)
		fact[k]=(float)(1./sqrt(d1[k]*dy*d1[k]*dy+d2[k]*d2[k]*dx*dx));

	/*  Set positive slope directions - store unresolved on stack */
	nis=0;
	for(i=i2+1; i< n2-1; i++)for(j=i1+1; j<n1-1; j++)
	{
		if(felevg.d[j][i] > mval && sdir.d[j][i] == 0)set(i,j,fact,mval,useflowfile);
		/*  Put unresolved pixels on stack  */
		if(sdir.d[j][i] == 0)
		{
			err=addstack(i,j);
		}
	}
	nflat=nis;
	/* routine to drain flats to neighbors  */
	 
	imin=vdn(nflat);
	n=nis;
	//   printf("Number of pits to resolve: %d\n",n); 
	np1=n;
	nt=np1*(1-per/100);
	
	/*  initialize apool to zero  */
	for(i=i2; i< n2; i++)for(j=i1; j<n1; j++)
		sapoolg.d[j][i]=0;

	/*  store unresolved stack location in apool for easy deletion  */
	// debuggind

//	fdbg=fopen("c:\\dave\\scratch\\temp.txt","w");
	while(nis > 0)   /*  While AA */
	{
		
		i=is[imin];
		j=js[imin];
		pooln=1;
		npool=0;
		nf = 0;  /*  reset flag to that new min elev is found */
//		fprintf(fdbg,"Pool i,j %d %d\n",i,j);
//		fflush(fdbg);
		recursedepth=0;
		pool(i,j);  /*  Recursive call on unresolved point with lowest elevation */
		/*  Find the pour point of the pool  */
//		fprintf(fdbg,"Depth %d \n", recursedepth);
//		fflush(fdbg);
		for(ip=1; ip<=npool; ip++)
		{
			i=ipool[ip];
			j=jpool[ip];
			for(k=1; k <=8; k++)	
			{
				jn=j+d2[k];
				in=i+d1[k];  
				if(sapoolg.d[jn][in] != pooln && dontcross(k, i, j)==0)  /*  neighbor not in pool  */
				{
					et=max2(felevg.d[j][i],felevg.d[jn][in]);
					if(nf == 0)  /*  this is the first edge found  */
					{
						emin=et;
						nf=1;
					}
					else
					{
						if(emin > et)
						{
							emin = et;
						}
					}
				}
			}
		}
		/*  Fill the pool  */
//		fprintf(fdbg,"Fill the pool %d \n",npool);
//		fflush(fdbg);
		for(k=1; k<=npool; k++)
		{
			i=ipool[k];
			j=jpool[k];
			if(felevg.d[j][i] <= emin)
			{
				if(sdir.d[j][i] > 0 && notfdr(useflowfile,i,j))   /*  Can be in pool, but not flat   */
				{   // notfdr is used to avoid changing burned in directions
				sdir.d[j][i]=0;
					/*  Add to stack  */
					err=addstack(i,j);
				}
				for(ip=1; ip <=8; ip++)
				{
					jn=j+d2[ip];
					in=i+d1[ip];
					if(felevg.d[jn][in] > felevg.d[j][i] && sdir.d[jn][in] > 0
						&& notfdr(useflowfile,in,jn))  // 7/29/04  DGT Fixed from notfdr(useflowfile,i,j)
					/*Only zero direction of neighbors that are higher - because
					lower  or equal may be a pour point in a pit that must not be disrupted  */
					{
						sdir.d[jn][in]=0;
						err=addstack(in,jn);
					}
				}
				felevg.d[j][i]=emin;				
			}
			sapoolg.d[j][i]=0;  /*  Reinitialize for next time round  */
		}
		
		/* reset unresolved stack  */
		ni=0;
		for(ip=1; ip <= nis; ip++)
		{
			set(is[ip],js[ip],fact,mval,useflowfile);  
			if(sdir.d[js[ip]][is[ip]] == 0)  /* still on stack */
			{
				ni++;
				is[ni]=is[ip];
				js[ni]=js[ip];
			}
		}
		n=nis;
		imin=vdn(ni);
		if(nis < nt){
			//printf("Percentage done %f\n",per);
			//		 ocxCtrl->FloodEvent((short)per);
			feedback("Percentage done %f\n",per,FN_FLOOD);
			per=per+1;
			nt=np1*(1-per/100);
		}
	}  /*  end of while AA  */
	
	return(err);
}


int flood(char *demfile, char *newfile, char * flowfile, short useflowfile, 
		  char *newflowfile, bool LoadIntoRam)
{
	inRam = LoadIntoRam;
int i;
	err = TD_NO_ERROR;
	float fdmval;
	char * ctempFile = NULL;
	
	/* define directions */
	d1[1]=0; d1[2]= -1; d1[3]= -1; d1[4]= -1; d1[5]=0; d1[6]=1; d1[7]=1; d1[8]=1;
	d2[1]=1; d2[2]=1; d2[3]=0; d2[4]= -1; d2[5]= -1; d2[6]= -1; d2[7]=0; d2[8]=1;
	
	
	if (gridread(demfile,&felevg,&filetype)==0)
		err=TD_NO_ERROR;
	else 
	{ 	err=TD_FAILED_GRID_OPEN;
	}

	if(err != TD_NO_ERROR)return(err);
	
	//fElevHeader = fElevGrid.getHeader();

	nx = felevg.head.nx;
	ny = felevg.head.ny;
	dx = felevg.head.dx;
	dy = felevg.head.dy;
	csize = dx;
	mval = felevg.nodata;

	//bndbox is working off of the corner of the cell, not the center
	//so it needs to remove 1/2 of a cell size to map down to the corner
	//instead of the center of the cell...
	for(i=0;i<4;i++) bndbox[i]=felevg.head.bndbox[i];
	

	//GenerateTempFile(&ctempFile,"bgd");
	

	if( useflowfile == 1 )
	{
		if(gridread(flowfile,&sapoolg,&filetype)==0)
			err=TD_NO_ERROR;
		else
			err=TD_FAILED_GRID_OPEN;

		
		fdmval =sapoolg.nodata;
		if(err != TD_NO_ERROR)
			return(err);

		if ( nx != sapoolg.head.nx || ny != sapoolg.head.ny)
		{
			err = TD_GRID_SIZE_MISMATCH;
			return err;
		}
		
		larr.head.dx=dx;
		larr.head.dy=dy;
		larr.head.nx=nx;
		larr.head.ny=ny;
		for(i=0;i<4;i++) larr.head.bndbox[i]=felevg.head.bndbox[i];
		larr.nodata=-1;
		
		if (allocategrid(&larr,larr.head,larr.nodata)!=0){
			err = TD_FAILED_GRID_OPEN; //;lArrGrid.LastErrorCode();
			return err;
		}
	}
	else
	{
		sapoolg.head.dx=dx;
		sapoolg.head.dy=dy;
		sapoolg.head.nx=nx;
		sapoolg.head.ny=ny;
		for(i=0;i<4;i++) sapoolg.head.bndbox[i]= felevg.head.bndbox[i];
		sapoolg.nodata=-1;

		if (allocategrid(&sapoolg,sapoolg.head,sapoolg.nodata)!=0){
			err =TD_FAILED_GRID_OPEN; // lArrGrid.LastErrorCode();
			return err;
		}
	}
	
	/*  allocate  dir and stack arrays  */
	sdir.head.dx=dx;
	sdir.head.dy=dy;
	sdir.head.nx=nx;
	sdir.head.ny=ny;
	for(i=0;i<4;i++)sdir.head.bndbox[i]=felevg.head.bndbox[i];
      	sdir.nodata=MISSINGSHORT;

	if ( allocategrid(&sdir,sdir.head,sdir.nodata)!=0) {
		err = TD_FAILED_GRID_OPEN;
		return err;
	}

	
	istack = (int) (nx * ny * 0.1);
	pstack=istack;
	dn = (short *)malloc(sizeof(short) * istack);
	is = (short *)malloc(sizeof(short) * istack);
	js = (short *)malloc(sizeof(short) * istack);
	ipool = (short *)malloc(sizeof(short) * pstack);
	jpool = (short *)malloc(sizeof(short) * pstack);
	if(dn == NULL || is == NULL || js == NULL || ipool == NULL || jpool == NULL)
	{
		// printf("\nError:  Cannot allocate memory for stacks\n");
		return(11);
	}
	
	i1=0; i2=0; n1=nx; n2=ny;  /*  full grid  */ 
	//  printf("Grid size: %d x %d\n",n1,n2);
	int writeflowfile=1;

	
	err=setdf(mval, fdmval, useflowfile, newflowfile, writeflowfile);
	
	
	/*  free stacks  */
	free(dn); free(is); free(js); free(ipool); free(jpool); 




	if ( gridwrite(newfile,felevg,filetype)==0)
		err=TD_NO_ERROR;
	else{
		err=TD_FAILED_GRID_SAVE;
		//if (newfile)
		//	AfxMessageBox(LPCTSTR( strcat( "Failed to save file: ", newfile) ));
	}

	if(err != TD_NO_ERROR) return(err);
	

	free(felevg.d[0]);free(felevg.d);
	free(sdir.d[0]);free(sdir.d);
	free(sapoolg.d[0]); free(sapoolg.d);

	
	if(useflowfile == 1)
	{ 
		free(larr.d[0]);free(larr.d);
		
	}



	return(0);  /*  ALL OK return from flood  */
}

//************************************************************************

float min2(float e1,float e2)
{ 
  float em; 
  em=e1;
  if(e2 < em)em=e2;
  return(em);
}


//************************************************************************
//Function to add outlet to list of potential outlets - keeping in order
int addoutlist(int i,int j,int diff)
{
	long x,y;
	x=j*csize+bndbox[0]+csize/2.;
	y=bndbox[3]-i*csize-csize/2.;
	if(nout < maxno)
	{
		nout=nout+1;
		for(i=nout-1; i>0; i--)
		{
			if(diff > diffout[i-1])
			{
				diffout[i]=diffout[i-1];
				xout[i]=xout[i-1];
				yout[i]=yout[i-1];
			}
			else
			{
				diffout[i]=diff;
				xout[i]=x;
				yout[i]=y;
				return(nout);
			}
		}
		diffout[0]=diff;
		xout[0]=x;
		yout[0]=y;
		return(nout);
	}
	else
	{
		i=nout-1;
		while(i > 0)
		{
			if(diff > diffout[i-1])
			{
				diffout[i]=diffout[i-1];
				xout[i]=xout[i-1];
				yout[i]=yout[i-1];
				i=i-1;
			}
			else
			{
				diffout[i]=diff;
				xout[i]=x;
				yout[i]=y;
				return(nout);
			}
		}
		if(i>=0)
		{
			diffout[i]=diff;
			xout[i]=x;
			yout[i]=y;
			return(nout);
		}
	}
	return(0);  //should only come here if nout is 0 - so return 0.
}


//************************************************************************


/* function to compute area recursively */
void srcarea(int i,int j)
  {

int in,jn,k,anmax=0,diff,inmax,jnmax,con=0;
	if(i!=0 && i!=ny-1 && j!=0 && j!=nx-1 && sdir.d[j][i]!= -1)   /* not on boundary  */  
{
  if(larr.d[j][i]< -1)  // not touched yet.
  {

	  larr.d[j][i]=src[j][i];
for(k=1; k<=8; k++)
{  in=i+d1[k];
   jn=j+d2[k];
/* test if neighbor drains towards cell excluding boundaries */
   if( sdir.d[jn][in] >0 
			   && (sdir.d[jn][in]-k==4 || sdir.d[jn][in]-k==-4))
 {
srcarea(in,jn);
				if(larr.d[jn][in] < 0)con = -1;
				else
				{
					if(larr.d[jn][in] > anmax)
					{
						anmax = larr.d[jn][in];
						inmax = in;
						jnmax = jn;
					}
					larr.d[j][i]=larr.d[j][i] + larr.d[jn][in];
				}
 }
		   if(sdir.d[jn][in] < 0)con = -1;
}
		if(con == -1  && ccheck == 1)larr.d[j][i] = -1;
		diff=larr.d[j][i]-anmax;
		if(diff > SourceThreshold)
			nout=addoutlist(inmax,jnmax,diff);
  }
}
	else larr.d[j][i] = -1;
}

//************************************************************************
//  1/30/01  added int dropan to argument list as flag to indicate that output 
//  to be threshold for drop analysis


int source(char *areafile,char *slopefile,char *plenfile,char *dirfile, 
		   char *srcfile, char *elvfile, char *gordfile, char *scafile,
		   char *fdrfile, int ipar,float *p, int nxy, double *x, double *y, 
		   int contcheck, int dropan, int masksca)
{

	float ndvs,ndvp,ndvd,emax,ndve,ndvo,wsum,val;
	
	float **selev ;
	
	/**********Grid Declarations*************/
    fgrid faagrid;
	fgrid fplengrid;
	sgrid sgordgrid;
	//=============================
	int row, col, i,j,iomax,jomax,bound,ik,jk,k,itresh;
	err = TD_NO_ERROR;
	int rcgood=1;
    ccheck=contcheck;


	
	/* define directions */
	d1[1]=0; d1[2]= -1; d1[3]= -1; d1[4]= -1; d1[5]=0; d1[6]=1; d1[7]=1; d1[8]=1;
	d2[1]=1; d2[2]=1; d2[3]=0; d2[4]= -1; d2[5]= -1; d2[6]= -1; d2[7]=0; d2[8]=1;
 
	
	/* read grid files */
	if(ipar == 1)
	{


		if(gridread(areafile,&faagrid,&filetype)==0)  
			err=TD_NO_ERROR;
		else
		{
			  err=TD_FAILED_GRID_OPEN;
		}
		

		nx = faagrid.head.nx; 
		ny = faagrid.head.ny;
		dx = faagrid.head.dx;
		dy = faagrid.head.dy;
		csize = dx;
		ndva = faagrid.nodata; 
		for(i=0;i<4;i++) bndbox[i]=faagrid.head.bndbox[i];


		if(err != TD_NO_ERROR)goto ERROR1;
	}
	
	if(ipar == 2)
	{
	
		if ( gridread(scafile,&faagrid,&filetype)==0)  
			err=TD_NO_ERROR;
		else
			err=TD_FAILED_GRID_OPEN;
	

		nx =faagrid.head.nx;
		ny = faagrid.head.ny;
		dx = faagrid.head.dx;
		dy = faagrid.head.dy;
		csize = dx;
		ndva = faagrid.nodata;
	    for(i=0;i<4;i++) bndbox[i]=faagrid.head.bndbox[i];	
		
	  
		if(err != TD_NO_ERROR)goto ERROR1;
		
	
		if ( gridread(slopefile,&fslopeg,&filetype)==0) 
			err=TD_NO_ERROR;
		else
			err=TD_FAILED_GRID_OPEN;
		
		ndvs = fslopeg.nodata; 
		
		if(err != TD_NO_ERROR)goto ERROR1; 
	}
	if(ipar == 3)
	{
		//  11/17/08 DGT changed file read from scafile to area file to switch this method to using ad8 input file as the default
		if ( gridread(areafile,&faagrid,&filetype)==0) 
			err=TD_NO_ERROR;
		else 
			err=TD_FAILED_GRID_OPEN;


		nx = faagrid.head.nx;
		ny = faagrid.head.ny;
		dx = faagrid.head.dx;
		dy = faagrid.head.dy;
		csize = dx;
		ndva = faagrid.nodata;
		for(i=0;i<4;i++) bndbox[i]=faagrid.head.bndbox[i];
		
		
		if(err != TD_NO_ERROR)goto ERROR1;
		
		
		if ( gridread(plenfile,&fplengrid,&filetype)==0)
			err=TD_NO_ERROR;
		else
			err=TD_FAILED_GRID_OPEN;
		
		ndvp = fplengrid.nodata; 

		
		if(err != TD_NO_ERROR)goto ERROR1; 
	}
	if(ipar == 4)
	{
		
		if( gridread(elvfile,&felevg,&filetype)==0) 

			err=TD_NO_ERROR;
		else
			err=TD_FAILED_GRID_OPEN;


		nx = felevg.head.nx;
		ny = felevg.head.ny;
		dx = felevg.head.dx;
		dy = felevg.head.dy;
		csize = dx;
		ndve = felevg.nodata;
		for(i=0;i<4;i++) bndbox[i]=felevg.head.bndbox[i];
	

		if(err != TD_NO_ERROR)goto ERROR1;  
	}
	if(ipar == 5)
	{ 
		
		if ( gridread(gordfile,&sgordgrid,&filetype)==0) 
			err=TD_NO_ERROR;
		else
			err=TD_FAILED_GRID_OPEN;
		
		
		nx = sgordgrid.head.nx;
		ny = sgordgrid.head.ny;
		dx = sgordgrid.head.dx;
		dy = sgordgrid.head.dy;
		csize = dx;
		ndvo = sgordgrid.nodata;
		for(i=0;i<4;i++)bndbox[i]=sgordgrid.head.bndbox[i];
	
		if(err != TD_NO_ERROR)goto ERROR1; 
	}
	if(ipar == 6)
	{ 
		
		if (gridread(fdrfile,&sgordgrid,&filetype)==0) 
			err=TD_NO_ERROR;
		else
			err=TD_FAILED_GRID_OPEN;
		

		nx = sgordgrid.head.nx;
		ny = sgordgrid.head.ny;
		dx = sgordgrid.head.dx;
		dy = sgordgrid.head.dy;
		csize = dx;
		ndvo = sgordgrid.nodata;
		for(i=0;i<4;i++)bndbox[i]=sgordgrid.head.bndbox[i];
		

		if(err != TD_NO_ERROR)goto ERROR1; 
	} 
	
	
	if((src = (short **)matalloc(nx,ny, RPSHRDTYPE)) == NULL)
	{
		err=TD_FAILED_MEMORY_ALLOC;
		//  printf("Could not allocate memory for src\n");
		goto ERROR1;
	}  


/*  Flag sources  */
	for(i=0; i < ny; i++)
	  for(j=0; j< nx; j++)
	  {
		  src[j][i] = 0;
		  if(ipar == 1)   /*  Area threshold   */
		  {
			  src[j][i] = (faagrid.d[j][i] >= p[0]) ? 1 : 0;
		  }
		  else if(ipar == 2)   /*  Slope and area combination   */
		  {
			  if( fslopeg.d[j][i] > 0.)
			  {
				  val = (faagrid.d[j][i] * pow((double)fslopeg.d[j][i],(double)p[1])) ;
				  src[j][i] = (val >= p[0])	  ? 1: 0;
			  }
		  }else if(ipar == 3)  /*  Slope and Length combination   */
		  {
			  if(fplengrid.d[j][i] > 0.)
			  {
				  src[j][i] = (faagrid.d[j][i] >= p[0]* pow((double)fplengrid.d[j][i],(double)p[1]))
				  ? 1: 0;
			  }
		  }
		  else if(ipar == 5)  /*  Grid order threshold  */
			  src[j][i] = (sgordgrid.d[j][i] >= p[0]) ? 1: 0;
		  else if(ipar == 6)  /*  Given flow directions threshold  */
			  src[j][i] = (sgordgrid.d[j][i] > 0) ? 1: 0;
	  }
	  if(ipar == 4)  /* Peuker and Douglas algorithm  */
	  {
	/*  Initialize internal cells to 1 for Peuker and Douglas algorithm and smooth  */
	if((selev = (float **)matalloc(nx,ny, RPFLTDTYPE)) == NULL)
	{
	  err=TD_FAILED_MEMORY_ALLOC;
	//  printf("Could not allocate memory for selev\n");
	  goto ERROR1;
	}
	  for(i=0; i <ny; i++)
		 for(j=0; j<nx; j++)
		 {
			  if(i == 0 || i == (ny-1) || j == 0 || j == (nx-1) || felevg.d[j][i] <= ndve)
			  {
				  selev[j][i]=felevg.d[j][i];
			  }
			  else
			  {
				src[j][i] = 1;
				selev[j][i]=p[1] * felevg.d[j][i];
				wsum=p[1];
				if(p[2] > 0.)
				  for(k=1; k<=7; k=k+2)
				  {
					if(felevg.d[j+d2[k]][i+d1[k]] > ndve)
					{
			  		  selev[j][i] += felevg.d[j+d2[k]][i+d1[k]] *p[2];
					  wsum += p[2];
					}				  
				  }
				if(p[3] > 0.)
				  for(k=2; k<=8; k=k+2)
				  {
					if(felevg.d[j+d2[k]][i+d1[k]] > ndve)
					{
			  		  selev[j][i] += felevg.d[j+d2[k]][i+d1[k]] *p[3];
					  wsum += p[3];
					}				  
				  }
			  }
		  }
		
		for (int curcol = 0; curcol < nx; curcol++)
			for (int currow = 0; currow < ny; currow++)
		
				felevg.d[curcol][currow]=selev[curcol][currow];

		for(i=0; i <ny-1; i++)
		  for(j=0; j<nx-1; j++)
		  {
			  emax= felevg.d[j][i]; 
			  iomax=0;
			  jomax=0;   
			  bound= 0;  /*  .false.  */
	/*  --FIRST PASS FLAG MAX ELEVATION IN GROUP OF FOUR  */
			  for(ik=0; ik<2; ik++)
				  for(jk=1-ik; jk < 2; jk++)
				  {
					  if(felevg.d[j+jk][i+ik] > emax)
					  {
						  emax=felevg.d[j+jk][i+ik];
						  iomax=ik;
						  jomax=jk;
					  }
					  if( felevg.d[j+jk][i+ik] <= ndve)
						 bound= 1;  /*  .true.  */
				  }
	/*  c---Unflag max pixel */
				 src[j+jomax][i+iomax] = 0;
	/*  c---Unflag pixels where the group of 4 touches a boundary  */
			  if(bound == 1)
			  {
				for(ik=0; ik < 2; ik++)
				  for(jk=0; jk< 2; jk++)
				  {
					  src[j+jk][i+ik]=0;
				  }
			  }
	/* 		  i.e. unflag flats.  */
				for(ik=0; ik < 2; ik++)
				  for(jk=0; jk< 2; jk++)
				  {
					 if(felevg.d[j+jk][i+ik] == emax)src[j+jk][i+ik] = 0;
				  }
		  }
	  }
	  //  Debug write of src
/*	  sgrid stemp;
	  stemp.head.dx=dx;
	  stemp.head.dy=dy;
	  stemp.head.nx=nx;
	  stemp.head.ny=ny;
	  stemp.nodata=-2;
	  for(i=0;i<4;i++) stemp.head.bndbox[i]=bndbox[i];
	  stemp.d=src;
	  gridwrite("C:/Dave/Scratch/KMWNew/tempsrc",stemp,filetype);  */
	  
	  if(ipar == 2){
		  free(fslopeg.d[0]); free(fslopeg.d); 
		  free(faagrid.d[0]); free(faagrid.d);
	  }
	  if(ipar == 3){
		 free(fplengrid.d[0]); free(fplengrid.d);
		 free(faagrid.d[0]); free(faagrid.d);
	  }
	  if(ipar == 4){
		  free(felevg.d[0]); free(felevg.d); 
		  free(selev[0]); free(selev);  // DGT 9/14/2007
	  }
//  DGT 5/16/09  changed not to free for 5 as needed below
//	  if(ipar == 5 || ipar == 6){
	  if(ipar == 6){
		  free(sgordgrid.d[0]); free(sgordgrid.d); 
	  }


	/*  Now get directions and compute area's  */

	  
	
	  if ( gridread(dirfile,&sdir,&filetype)==0) 
		  err=TD_NO_ERROR;
	  else
		  err=TD_FAILED_GRID_OPEN;

	 
	  ndvd = sdir.nodata;  

	  if(err != TD_NO_ERROR)goto ERROR1; 

	//allocate memory and headers for larr
	  larr.head.dx=dx;
	  larr.head.dy=dy;
	  larr.head.nx=nx;
	  larr.head.ny=ny;
	  larr.nodata=-2;
	  for(i=0;i<4;i++) larr.head.bndbox[i]=bndbox[i];
	  
	  
	  larr.nodata = -2;
	  allocategrid(&larr,larr.head,larr.nodata);
	  
  		nout=0;
		itresh=1;
		if(ipar == 4)itresh = p[0];
		err=TD_CHANNEL_NETWORK_MISMATCH;   //This flag will indicate no outlet found  12/15/02  DGT moved to outside the if block 
		// so that code works for at least one outlet found
		if(nxy >0)
		{
			for(i=0; i<nxy; i++)
			{
				col= (int)floor((x[i]-bndbox[0])/csize);
				row= (int)floor((bndbox[3]-y[i])/csize);
 				if(row >0 && row < ny-1 && col > 0 && col < nx-1 
					&& sdir.d[col][row]>0)  // DGT* this condition added 12/15/02 to not do outlets outside the domain
					{
	/* call drainage area subroutine for pixel to zero on  */
						srcarea(row,col);

						if(larr.d[col][row] >= itresh)err=TD_NO_ERROR;  // an outlet found so no error
					}
			}
			if(err==TD_CHANNEL_NETWORK_MISMATCH)goto ERROR9;  //  no outlet error
		}
		else
		{
	//  Do all pixels
		  for(i=1; i < ny-1; i++)
			for(j=1; j<nx-1; j++)srcarea(i,j);
			rcgood=0;  // no outlet coordinates found
		}
	

	//  Now threshold the src file
		if(dropan == 0)
		{

		for(i=1; i < ny-1; i++)
			for(j=1; j<nx-1; j++)
			{
				if(larr.d[j][i] >= itresh && sdir.d[j][i]>0) larr.d[j][i]=1;
				//  8/13/04  DGT added condition on sdir.d
				else larr.d[j][i]=0;
			}
		}
		if(dropan == 1 && ipar == 1)  // overwrite accumulated source area with actual area
		{
			for(i=1; i < ny-1; i++)
			for(j=1; j<nx-1; j++)
			{
				if(larr.d[j][i] >= itresh && sdir.d[j][i]>0)larr.d[j][i]=faagrid.d[j][i];
				//  8/13/04  DGT added condition on sdir.d
				else larr.d[j][i]=0;
			}
		}
		//  DGT 5/16/09  Fix to overwrite accumulated source area with grid network orders
		if(dropan == 1 && ipar == 5)
		{
			for(i=1; i < ny-1; i++)
			for(j=1; j<nx-1; j++)
			{
				if(larr.d[j][i] >= itresh && sdir.d[j][i]>0)larr.d[j][i]=sgordgrid.d[j][i];
				else larr.d[j][i]=0;
			}
			//  Now free the memory that could not be free'd above
			free(sgordgrid.d[0]); free(sgordgrid.d); 
		}

//free memory for sdir	
		free(sdir.d[0]);free(sdir.d);

// Moved from below so that could reopen with sca file for sure
		if(ipar == 1) {free(faagrid.d[0]);free(faagrid.d);} //DGT 1/6/06 changed the conditional from <= 3 to ==1, because faagrid for ipar=2 and 3 is free'd above 
		// Exclude area with specific catchment area no data
		if(masksca == 1)
		{
			if(gridread(scafile,&faagrid,&filetype)==0)
				err=TD_NO_ERROR;
			else
			{
		  		err=TD_FAILED_GRID_OPEN;
				return err;
			//	AfxMessageBox( LPCTSTR(strcat( "Failed to open sca file for masking: ", scafile) ));
			}
			if(err != TD_NO_ERROR)goto ERROR9;
			for(i=1; i < ny-1; i++)
			for(j=1; j<nx-1; j++)
			{
				if(faagrid.d[j][i] < 0)larr.d[j][i]=0;
			}
			free(faagrid.d[0]);free(faagrid.d);  //DGT 9/14/07 to clean up
		}
		if ( gridwrite(srcfile,larr,filetype)==0)
			err=TD_NO_ERROR;
		else{
			err=TD_FAILED_GRID_SAVE;
			//if (srcfile)
			//	AfxMessageBox( LPCTSTR(strcat( "Failed to save file: ", srcfile) ));
		}

	free(src[0]); free(src);
    free(larr.d[0]); free(larr.d);

    return(err);
ERROR9:
   free(src[0]); free(src);
   //Kiran added the following statement to clean up.
    free(larr.d[0]); free(larr.d);
	if(faagrid.d != NULL) {free(faagrid.d[0]); free(faagrid.d);}
	if(sdir.d != NULL) {
		if(sdir.d[0] != NULL)
			{free(sdir.d[0]); free(sdir.d);
		}
	}
    return(err);

ERROR1:
   free(src[0]); free(src);
   free(larr.d[0]); free(larr.d);
   return(err);
}

int validateoutlet(long x,long y,char *aread8file, char *srcfile)
{
	int col,row;
	err = TD_NO_ERROR;
	//err=gridread(aread8file,(void ***)&arr,RPINTDTYPE,&nx,&ny,&dx,&dy,
	//   bndbox,&csize,&ndv,&filetype);
	if (gridread(aread8file,&larr,&filetype) != 0)
		err = TD_FAILED_GRID_OPEN;


	nx = larr.head.nx;
	ny = larr.head.ny;
	dx = larr.head.dx;
	dy = larr.head.dy;
	ndv =larr.nodata;
	csize = dx;
	for(int i=0;i<4;i++) bndbox[i]=larr.head.bndbox[i];

	if(err != TD_NO_ERROR)goto ERROR1;
	
	//err=gridread(srcfile,(void ***)&src,RPSHRDTYPE,&nx,&ny,&dx,&dy,
	//	bndbox,&csize,&ndv,&filetype);
	if (gridread(srcfile,&larr,&filetype) !=0)
		err = TD_FAILED_GRID_OPEN;

	if(err != TD_NO_ERROR)goto ERROR2;
	col= (int)floor((x-bndbox[0])/csize);
	row= (int)floor((bndbox[3]-y)/csize);
	if(row >0 && row < ny-1 && col > 0 && col < nx-1)
	{
		if(larr.d[col][row] <=0 || larr.d[col][row] <=0)
			err=109;
	}
	else err=109;
	
	
ERROR2:
	free(larr.d[0]);free(larr.d);
	
ERROR1:
	free(larr.d[0]);free(larr.d);
	return(err);
}



//************************************************************************
/* function to compute stream drops recursively */


short dropanrecursive(int i,int j, float *elevout, float *lengthout, float *s1,
					 float *s2, float *s1sq, float *s2sq, int *n1,int *n2, int ithreshold)
{

short ordermax=0,orderout,k,n=0,orderin[5],newst=1;
	float lengthin, elevin[5],drop; //  Assume no more than 5 join at a point.
	int in,jn;
	*lengthout = 0.;  //  initialize
	for(k=1; k<=8; k++)
{  in=i+d1[k];
   jn=j+d2[k];
/* test if neighbor drains towards cell excluding boundaries */
   if(sdir.d[jn][in]>0 && (sdir.d[jn][in]-k==4||sdir.d[jn][in]-k==-4))
 {
				if(larr.d[jn][in] >= ithreshold)
				{
					n=n+1;
					orderin[n-1]=dropanrecursive(in,jn,&elevin[n-1],&lengthin,s1,s2,s1sq,
						s2sq,n1,n2,ithreshold);
					if(orderin[n-1] > ordermax)
					{
						ordermax = orderin[n-1];
						orderout = ordermax;
					}
					else if(orderin[n-1] == ordermax)
					{
						orderout = ordermax+1;
					}
					*lengthout += lengthin+sqrt(dx*dx*(j-jn)*(j-jn)+dy*dy*(i-in)*(i-in));  
					//  increment length with lengthin plus the single pixel distance
				}
		   }
	}
	if(n < 1)
	{
		*elevout = felevg.d[j][i];
		return(1);
	}
	if(n == 1)
	{
		*elevout = elevin[n-1];
		return(orderin[n-1]);
	}
	//  Here n >= 2 so there is a junction 
	for(k = 1; k <= n; k++)
	{
		if(orderin[k-1] < orderout)
		{
			drop = elevin[k-1] - felevg.d[j][i];
			if(orderin[k-1] == 1)
			{
				*s1= *s1+drop;
				*s1sq= *s1sq+drop*drop;
				*n1= *n1+1;
			}
			else
			{
				*s2= *s2+drop;
				*s2sq= *s2sq+drop*drop;
				*n2= *n2+1;
			}
		}
		else
		{
			//  Here orderin = orderout so this is the main stream
			*elevout = elevin[k-1];
			newst = 0; 
		}
	}
	if(newst == 1)
	{
		*elevout= felevg.d[j][i];
	}
	return(orderout);
}
					

//************************************************************************
int dropan(char *areafile,char *slopefile,char *plenfile,char *dirfile, 
		   char *srcfile, char *elevfile, char *gordfile, char *scafile,
		   char *fdrfile, int ipar,float *p, int nxy, double *x, double *y, 
		   int contcheck,int nthresh,int *thresholds, int *n1, int *n2,
		   float *s1, float *s2, float *s1sq, float *s2sq, float *ddens,long masksca)
{

	float ndvd, elevin;
	int col, row,in,jn;
	float areat=0.;
	int ithreshold,i;//j;
	//  Function to do stream drop analysis.
	//  First define the sources in file srcfile
	p[0] = thresholds[0];
	err = TD_NO_ERROR;
	err= source(areafile, slopefile, plenfile, dirfile, 
		   srcfile, elevfile, gordfile, scafile,
		   fdrfile, ipar,p, nxy, x, y, 
		   contcheck, 1, masksca);
	if(err != TD_NO_ERROR)goto ERROR1;
	
	//  now reread the srcfile and dirfile

	if ( gridread(srcfile,&larr,&filetype)==0)
		err=TD_NO_ERROR;
	else err=TD_FAILED_GRID_OPEN;
	
	if(err != TD_NO_ERROR)goto ERROR1; 

	
	nx =larr.head.nx;
	ny = larr.head.ny; 
	dx = larr.head.dx; 
	dy = larr.head.dy;
	csize = dx;
	ndva = larr.nodata;
	for(i=0;i<4;i++) bndbox[i]=larr.head.bndbox[i];

	if ( gridread(dirfile,&sdir,&filetype)==0)
		err=TD_NO_ERROR;
	else 
		err=TD_FAILED_GRID_OPEN;

	if(err != TD_NO_ERROR)goto ERROR2; 

	
	ndvd = sdir.nodata;


	 
	//  read the areafile first to get total area
	
	if (gridread(areafile,&laag,&filetype)==0)
		err= TD_NO_ERROR;
	else 
		err=TD_FAILED_GRID_OPEN;

	if(err != TD_NO_ERROR)goto ERROR3;

	
	ndvd =laag.nodata; 
	//  DGT*  To modify here if dropan is to work without outlets
	//  The problem is that as the threshold changes the number of outlets and total area will change.  
	//  It does not make sense to have the domain changing which is why dropan is only run for specified outlets
	for(i=0; i<nxy; i++)
	{
		col= (int)floor((x[i]-bndbox[0])/csize);
		row= (int)floor((bndbox[3]-y[i])/csize);  //  This loop over the whole grid if there are no outlets to identify ends of streams
 		if(row >0 && row < ny-1 && col > 0 && col < nx-1
			&& sdir.d[col][row] > 0)   // DGT 12/15/02  This last condition so that outlets in no data domain
//  are ignored rather than resulting in an error
		{
			in=row+d1[sdir.d[col][row]];
			jn=col+d2[sdir.d[col][row]];
			if(larr.d[jn][in] <= 0)  //  This outlet is a terminal outlet
				areat= areat+ laag.d[col][row];
		}
	}
	areat=areat*dx*dy;   // This evaluates total area for use in calculating drainage density
	free(laag.d[0]);free(laag.d);


	//******************************



	if(gridread(elevfile,&felevg,&filetype)==0)
		err=TD_NO_ERROR;
	else 
		err=TD_FAILED_GRID_OPEN;

	if(err != TD_NO_ERROR)goto ERROR3; 
	
	
	ndvd =felevg.nodata;

	for(ithreshold =0; ithreshold < nthresh; ithreshold ++)
	{
		n1[ithreshold]=0;
		n2[ithreshold]=0;
		s1[ithreshold]=0.;
		s2[ithreshold]=0.;
		s1sq[ithreshold]=0.;
		s2sq[ithreshold]=0.;
		ddens[ithreshold]=0.;
float length=0.;
		for(i=0; i<nxy; i++)
		{
			col= (int)floor((x[i]-bndbox[0])/csize);
			row= (int)floor((bndbox[3]-y[i])/csize);
 			if(row >0 && row < ny-1 && col > 0 && col < nx-1)
			{
				in=row+d1[sdir.d[col][row]];
				jn=col+d2[sdir.d[col][row]];
				if(larr.d[jn][in] <= 0)  //  This outlet is a terminal outlet
				{
					dropanrecursive(row,col,&elevin,&length,&s1[ithreshold],&s2[ithreshold],
					&s1sq[ithreshold],&s2sq[ithreshold],&n1[ithreshold],&n2[ithreshold],thresholds[ithreshold]);
					ddens[ithreshold] += length;  // accumulate lengths for drainage density calculation
				}
			}
		}
		if(areat > 0.)
			ddens[ithreshold]=ddens[ithreshold]/areat;
		else
			err=TD_DROPAN_ENCOUNTERED_ZERO;
	}
	free(felevg.d[0]);free(felevg.d);

ERROR3:

	free(sdir.d[0]); free(sdir.d);
ERROR2:
free(larr.d[0]); free(larr.d);
	
ERROR1:
	return(err);
	
}
//************************************************************************
//*************************************************************
  //Code changed for new gridio:Dr David G Tarboton
  //*************************************************************
int netsetup(char *fnprefix, char *pfile, char *srcfile, char *ordfile, 
			 char *ad8file, char *elevfile, char *treefile, char *coordfile, 
			 double *xnode, double *ynode, int nxy, long usetrace, long *idnodes)
{
	int itresh,icr,icend,i;
	err = TD_NO_ERROR;

	sgrid dirg;
	
	/* define directions */
	d1[1]=0; d1[2]= -1; d1[3]= -1; d1[4]= -1; d1[5]=0; d1[6]=1; d1[7]=1; d1[8]=1;
	d2[1]=1; d2[2]=1; d2[3]=0; d2[4]= -1; d2[5]= -1; d2[6]= -1; d2[7]=0; d2[8]=1;


	
	itresh=1;  // Thresholding to 1 done in source
	/*read dirfile   */
	

	err=gridread(pfile,&dirg,&filetype);


		 
	if (err != TD_NO_ERROR)
	{
		//sprintf(errmessage,"Failed to open grid file %s",pfile);
		 //AfxMessageBox(errmessage);
		return err;
	}
	dx=dirg.head.dx;
	dy=dirg.head.dy;
	ny=dirg.head.ny;
	nx=dirg.head.nx;
	for(i=0; i<4; i++)bndbox[i]=dirg.head.bndbox[i];
	dir=dirg.d;


	
	/*read srcfile   */
	igrid area;
	if (gridread(srcfile,&area,&filetype)!=0) 
	{
		err=TD_FAILED_GRID_OPEN;
		return err;
	}

 	
	if(area.head.nx != nx || area.head.ny != ny)
	{	
		err=TD_GRID_SIZE_MISMATCH;
		return 1;
	}
	
	
	//  check for source values >= threshold else fortran crashes
	int   n=0,j;
	for(i=0; i<ny; i++){
		for(j=0; j<nx; j++){
			if(dir[j][i]==dirg.nodata)dir[j][i]=0;  // Set direction no data to 0. Netex needs this  DGT 8/13/04
			if(area.d[j][i] >=itresh)n=n+1;
		}
	}
	if(n <= 0)
	{
		err=TD_CHANNEL_NETWORK_MISMATCH;
	}
	else
	{
		short *inodes;
		short *jnodes;

//  If there are outlets set up inodes, jnodes arrays
		if(nxy > 0){
			inodes = new short [nxy];
			jnodes = new short [nxy];
		}else
		{	
			inodes = new short [1];
			jnodes = new short [1];
		}

		for(int inode = 0; inode<nxy; inode++)
		{  
			jnodes[inode]= (short)floor((xnode[inode]-bndbox[0])/dx);  
			inodes[inode]= (short)floor((bndbox[3]-ynode[inode])/dy);
//   Trace to raster if necessary but only for nodes that are inside the domain
			if(usetrace == 1  && inodes[inode]>0 && inodes[inode] < ny-1 && 
				jnodes[inode] > 0 && jnodes[inode] < nx-1) 
			{
				if(area.d[jnodes[inode]][inodes[inode]] < itresh)  // Not on grid
				{
					//   Next downslope 
					short dirn=dir[jnodes[inode]][inodes[inode]];
					int nexti=inodes[inode]+d1[dirn];
					int nextj=jnodes[inode]+d2[dirn];
					int loopcount = 0;
					while(nexti>0 && nexti < ny-1 && nextj > 0 && nextj < nx-1 && area.d[jnodes[inode]][inodes[inode]] < itresh)
					{
						inodes[inode]=nexti;
						jnodes[inode]=nextj;
						dirn = dir[jnodes[inode]][inodes[inode]];
						if(dirn < 1 || dirn > 8)break;   // Here have gone out of grid so terminate trace downwards
						nexti=nexti+d1[dirn];
						nextj=nextj+d2[dirn];
						loopcount=loopcount+1;
						if(loopcount > nx && loopcount > ny)break;   // Here possible infinite loop so terminate trace downwards
					}
				}
			}

		}

/*		err=Netex_c(dir,area.d,treefile,coordfile,
			ordfile,nx,ny,itresh,icr,icend,dx,dy,*bndbox,dx,
			filetype,inodes,jnodes,nxy,idnodes);
		if(err !=0) return(err);  //  */
 
/*  Call fortran NETEX subroutine  */
		//   5/5/09  DGT changed the below to double precision
		{		// float dxf = dx; float dyf = dy;
		NETEX(dir[0],area.d[0],treefile,strlen(treefile),coordfile,strlen(coordfile),
			ordfile,strlen(ordfile),&nx,&ny,&itresh,&icr,&icend,&dx,&dy,bndbox,&dx,
			&filetype,&err,inodes,jnodes,&nxy,idnodes); // */ 
		}
		
	
		area.nodata = -1;
		if (gridwrite(ordfile,area,filetype)!=0)
		{
			err=TD_FAILED_GRID_SAVE;
			return err;

		}
		else
		{
			err=TD_NO_ERROR;
		}

		free(area.d[0]); free(area.d);
		//  fix negative directions
		for(i=0; i<ny; i++){
			for(j=0; j<nx; j++)
				if(dir[j][i] < 0 && dir[j][i] > -9)
				{
					dir[j][i] = -dir[j][i];
				}
		}
		

		err=gridread(ad8file,&area,&filetype);
		if(err != TD_NO_ERROR)
		{
		 //	sprintf(errmessage,"Failed to open ad8 grid file %s", ad8file);
		//	AfxMessageBox(errmessage);
			return err;
		}

		/*****read elevfile   *****/

		fgrid elevg;
		err=gridread(elevfile,&elevg,&filetype);
		if(err != TD_NO_ERROR)
		{
		 //	sprintf(errmessage,"Failed to open elev grid file %s", elevfile);
		//	AfxMessageBox(errmessage);
			return err;
		}

		if(elevg.head.nx != nx || elevg.head.ny != ny)
		{
		//	sprintf(errmessage,"Incompatible grid sizes");
		//	AfxMessageBox(errmessage);
			return 1;
		}
		
		/*  Call fortran NETPROP subroutine  */
		//  DGT 5/5/09  Changed dx and dy to double
	//	float dxf = dx; float dyf = dy;
		NETPROP(dir[0],area.d[0],elevg.d[0],coordfile,strlen(coordfile),&icr,&icend,&dx,
			&dy,&nx,&ny,bndbox,&csize,&err);  
		
		free(dir[0]); free(dir);
		free(elevg.d[0]); free(elevg.d);
		free(area.d[0]); free(area.d);
	}  //  end if associated with err from source
	return(err);
}

//************************************************************************

/* function to compute area recursively */

void d2area(int i,int j)
{

int in,jn,k;
	short a1,a2;
float ld;


if( sordg.d[j][i] <= 0)
{
  if(i!=0 && i!=ny-1 && j!=0 && j!=nx-1 && sdir.d[j][i]!= MISSINGSHORT
		  && laag.d[j][i]>=tresh)
  /* not on boundary and in area considered  */
  {
		sordg.d[j][i] = 1;
		
		fltpg.d[j][i]=0;
		
		fleng.d[j][i]=0;
		
		a1=0;
		a2=0;
	
for(k=1; k<=8; k++)
{  in=i+d1[k];
   jn=j+d2[k];
	   /* test if neighbor drains towards cell excluding boundaries */
   if(sdir.d[jn][in] >=0) 
{
   if( laag.d[jn][in]>=tresh && (sdir.d[jn][in]-k==4 
			   || sdir.d[jn][in]-k==-4))
 {
d2area(in,jn);
				if(sordg.d[jn][in] >= a1)
				{
					a2=a1;
					a1=sordg.d[jn][in];
				}
				else if ( sordg.d[jn][in] > a2 )
					a2=sordg.d[jn][in];


				ld= fleng.d[jn][in] + dist[ sdir.d[jn][in] ];
fltpg.d[j][i] = fltpg.d[j][i] + fltpg.d[jn][in]
					+ dist[ sdir.d[jn][in] ] ;
				if( ld > fleng.d[j][i] )
				{
					fleng.d[j][i]=ld;
				}
			}
		}
	}
		 if(a2+1 > a1) sordg.d[j][i]=a2+1;
		 else sordg.d[j][i]=a1;
  }
}


}

//************************************************************************

int gridnet(char *pfile,char *plenfile,char *tlenfile,char *gordfile,char *afile,
			double *x,double *y,long nxy, int useMask, int useOutlets, int thresh)
 {

	int i,j,row,col;
	err = TD_NO_ERROR;
	tresh=thresh;
	/* define directions */
	d1[1]=0; d1[2]= -1; d1[3]= -1; d1[4]= -1; d1[5]=0; d1[6]=1; d1[7]=1; d1[8]=1;
	d2[1]=1; d2[2]=1; d2[3]=0; d2[4]= -1; d2[5]= -1; d2[6]= -1; d2[7]=0; d2[8]=1;

/*read pointers  */
 

	
	if (gridread(pfile,&sdir,&filetype)==0)
		err=TD_NO_ERROR;
	else
		err=TD_FAILED_GRID_OPEN;
	
	if (err !=TD_NO_ERROR) return (err);

	

	nx =sdir.head.nx; 
	ny = sdir.head.ny;
	dx = sdir.head.dx;
	dy = sdir.head.dy;
	ndvp = sdir.nodata;
	csize = dx;
	//filetype = 0;
	for(i=0;i<4;i++) bndbox[i]=sdir.head.bndbox[i];

	
	if(useMask == 1)
	{
		/*   read mask  */



		if (gridread(afile,&laag,&filetype)==0)
			err=TD_NO_ERROR;
		else
			err=TD_FAILED_GRID_OPEN;

		ndv =laag.nodata;
		
		if(err != TD_NO_ERROR)return(err);
	}

	else
	{
		laag.head.dx=dx;
		laag.head.dy=dy;
		laag.head.nx=nx;
		laag.head.ny=ny;
		laag.nodata=-1;
		for(i=0;i<4;i++)laag.head.bndbox[i]=sdir.head.bndbox[i];
		allocategrid(&laag,laag.head,0);

		

	}
	//  fprintf(stderr,"...now allocating memory...\n");
	/* allocate memory to arrays */
	sordg.head=sdir.head;
	sordg.nodata=-1;
	allocategrid(&sordg,sordg.head,sordg.nodata);
	

	fltpg.head.dx=dx;
	fltpg.head.dy=dy;
	fltpg.head.nx=nx;
	fltpg.head.ny=ny;
	fltpg.nodata=-1;
	for(i=0;i<4;i++)fltpg.head.bndbox[i]=sdir.head.bndbox[i];
	allocategrid(&fltpg,fltpg.head,fltpg.nodata);
	
	fleng.head=fltpg.head;
	fleng.nodata=-1;
	allocategrid(&fleng,fleng.head,fleng.nodata);
	
	/*  Calculate Distances  */
	for(i=1; i<=8; i++){
		dist[i]=sqrt(d1[i]*d1[i]*dy*dy+d2[i]*d2[i]*dx*dx);
	}
	
	
	
	if(useOutlets == 1)  /*  Only compute area's for designated locations  */
	{
		for (int curXY = 0; curXY < nxy; curXY++)
		{
			col= (int)floor((x[curXY]-bndbox[0])/csize);
			row= (int)floor((bndbox[3]-y[curXY])/csize);
			if(row <0 || row > ny || col < 0 || col > nx)
			{
				//			printf("Given coords out of area\n");
				row=0; col=0;
			}
			d2area(row,col);
		}
	}
	else
	{
		for(i=0; i<ny; i++)
			for(j=0; j<nx; j++)
				d2area(i,j);
	}
	
	
	
	//wirte plen file with fleng
	
	if ( gridwrite(plenfile,fleng,filetype)==0)
		err=TD_NO_ERROR;
	else{
		err=TD_FAILED_GRID_SAVE;
		return err;
		//if (plenfile)
		//	AfxMessageBox( LPCTSTR(strcat( "Failed to save file: ", plenfile)) );
	}

	
	
	if (gridwrite(tlenfile,fltpg,filetype)==0) 
		err=TD_NO_ERROR;
	else{
		err=TD_FAILED_GRID_SAVE;
		return err;
		//	if (tlenfile)
		//	AfxMessageBox( LPCTSTR(strcat( "Failed to save file: ", tlenfile) ));
	}

	
	
	if (gridwrite(gordfile,sordg,filetype)==0)
		err=TD_NO_ERROR;
	else{
		err=TD_FAILED_GRID_SAVE;
		return err;
		//if (gordfile)
		//	AfxMessageBox(LPCTSTR( strcat( "Failed to save file: ", gordfile) ));
	}
	
	free(sordg.d[0]);
	free(sordg.d);

	
	free(laag.d[0]);
	free(laag.d);

	
	free(sdir.d[0]);
	free(sdir.d);

	
	free(fltpg.d[0]);
	free(fltpg.d);  
	
	
	free(fleng.d[0]);
	free(fleng.d);

	return(err);

 }






int reachshape(int **cnet,float **coord, int ilink, int nb,int mag, long dsnodeid)
{	
	//BS// Shape * sh = new ShpPolyline();
	shape * sh = new shp_polyline();

	
	double x,y,length,glength,x1,y1,xlast,ylast,usarea,dsarea,dslast,dl,drop,slope;
	int istart,iend,j;

	istart=cnet[1][ilink];  //  start coord for first link
	iend=cnet[2][ilink];//  end coord for first link
	x1=coord[0][istart];
	y1=coord[1][istart];
	length=0.;
	xlast=x1;
	ylast=y1;
	usarea=coord[4][istart];
	dslast=usarea;
	dsarea=usarea;
	long prt = 0;



	for(j=0; j<=iend-istart; j++)  //  add first link
		{
			x=coord[0][j+istart];
			y=coord[1][j+istart];
			dl=sqrt((x-xlast)*(x-xlast)+(y-ylast)*(y-ylast));
			if(dl > 0.)
			{
				length=length+dl;
				xlast=x;  ylast=y;
				dsarea=dslast;   // keeps track of last ds area
				dslast=coord[4][j+istart];
			}

			api_point p(x,y);
			sh ->insertPoint(p,0);
						
		
		}
	if(iend==istart){   //  DGT 8/17/04 A polyline seems to require at least 2 data points
		//  so where there was only one repeat it.
		api_point p(x,y);
		sh ->insertPoint(p,0);
	}
	drop=coord[3][istart]-coord[3][iend];
	slope=0.;
	float dsdist=coord[2][iend];
	float usdist=coord[2][istart];
	float middist=(dsdist+usdist)*0.5;
	if(length > 0.)slope=drop/length;
	glength=sqrt((x-x1)*(x-x1)+(y-y1)*(y-y1));
		

	shp1.insertShape( sh, 0);	
	cell v;
	v.setValue(ilink);				shp1[0]->setCell(v,0);
	v.setValue(cnet[3][ilink]);		shp1[0]->setCell(v,1);
	v.setValue(cnet[4][ilink]);		shp1[0]->setCell(v,2);
	v.setValue(cnet[5][ilink]);		shp1[0]->setCell(v,3);
	v.setValue((double)dsnodeid);	shp1[0]->setCell(v,4);
	v.setValue(cnet[6][ilink]);		shp1[0]->setCell(v,5);
	v.setValue(length);				shp1[0]->setCell(v,6);
	v.setValue(mag);				shp1[0]->setCell(v,7);
	v.setValue(dsarea);				shp1[0]->setCell(v,8);
	v.setValue(drop);				shp1[0]->setCell(v,9);
	v.setValue(slope);				shp1[0]->setCell(v,10);
	v.setValue(glength);			shp1[0]->setCell(v,11);
	v.setValue(usarea);				shp1[0]->setCell(v,12);
	v.setValue(nb);					shp1[0]->setCell(v,13);
	v.setValue(dsdist);				shp1[0]->setCell(v,14);
	v.setValue(usdist);				shp1[0]->setCell(v,15);
	v.setValue(middist);			shp1[0]->setCell(v,16);

return(0);
}
/* ****************************** markbasin() ********************************* */

int markbasin(int *nb, int ilink, int **cnet, float **coord, int *nrch, 
			  int ordert, int subbno, int *mag, long *dsnodeid)
{
	int row,iup1,iup2,col,flag = 0,thisreach,nbp;
	//int col1;
	float x,y;
	int icend, icbeg, icarea;  /*  variables for coord positions  */
	
	/*  -1 in all subscripting for fortran compatibility  */
  icend = cnet[3-1][ilink];   /*  This is coord of end of link */
  icbeg = cnet[2-1][ilink];   /*  This is coord of beg of link */
 // ic1 = cnet[2-1][tree[0][ilink-1]];  /*  This is coord of beginning of last link */
/*  printf("%d %d %d\n",ic1,icbeg,icend);  */
  mag[ilink]=0;  // Initiaize magnitude recursion
  if(icbeg < icend)   //  Has physical length
  {
		*nb = *nb + 1;   /*  Increment basin counter  */
		if(ordert < 0) *nb=subbno;  //Basin counting circumvented for separate watershed delineation
//   if(*nb > 1)
	   if(cnet[4-1][ilink] != -1)   /*  For anything other than a downstream end
		   area is defined one grid cell back  */
	   {
		   icarea=icend-1;
	   }
	   else
	   {
		   icarea=icend;
	   }
	   
	  x=coord[1-1][icarea];
	  y=coord[2-1][icarea];
	  	col= (int)floor((x-bndbox[0])/csize);
		row= (int)floor((bndbox[3]-y)/csize);
		if(row <0 || row > ny || col < 0 || col > nx)
		{
//			printf("Error 1: encountered coordinate outside grid\n");
			return(-1);
		}
		larea(row,col, *nb);  /*  Label the region that drains to this pixel  */
  }
  else
  {
	  x=coord[1-1][icend];
	  y=coord[2-1][icend];
	  col= (int)floor((x-bndbox[0])/csize);
	  row= (int)floor((bndbox[3]-y)/csize);
	  if(row <0 || row > ny || col < 0 || col > nx)
	  {
		  //			printf("Error 2: encountered coordinate outside grid\n");
		  return( -2);
	  }
	  /*  	  printf("Dummy link\n",icend);  */
	  icarea=icend;
	  flag=1;
  }
  /*  Search for upstream basins   */
	  iup1=cnet[5-1][ilink]; /*  Pointers to upstream links  */
	  iup2=cnet[6-1][ilink]; 
		if(iup1 > 0 || iup2 > 0)
		{
/*  There are basins upstream  */
			if(flag == 1)
			{   /*   Dummy 0 length reach   */
				*nrch = *nrch+1;
				thisreach = *nrch;
				if(ordert <= 0)nbp=subbno;
				else nbp= 0;  // a dummy reach does not get a basin number
				//  write shape 
//				reachshape(cnet,coord,tree,ilink,0,mag,order);   // done later
				rtable[thisreach][1]=thisreach;
				if(iup1 > 0)
				{
					rtable[thisreach][2]=markbasin(nb, iup1, cnet,  coord, 
						nrch,ordert,subbno,mag,dsnodeid);
					mag[ilink]=mag[ilink]+mag[iup1];
				}
				else
					rtable[thisreach][2]=0;
				if(iup2 > 0)
				{
					rtable[thisreach][3]=markbasin(nb, iup2, cnet, coord, 
						nrch,ordert,subbno,mag,dsnodeid);
					mag[ilink]=mag[ilink]+mag[iup2];
				}
				else
					rtable[thisreach][3]=0;
				reachshape(cnet,coord,ilink,nbp,mag[ilink],dsnodeid[ilink]);
				/*  Assign properties to dummy reach  */
				rprops[thisreach][1]=(float)0.01;  /*  Slope  */
				rprops[thisreach][2]=coord[5-1][icarea];   /*  Contributing area  */
				rprops[thisreach][3]=0.;   /*  Length  */
				rprops[thisreach][4]=coord[0][icarea];  /*  End X coordinate */
				rprops[thisreach][5]=coord[1][icarea];  /*  End Y coordinate */
			}
			else
			{
				/*  Lower half reach  */
				*nrch = *nrch+1;
				thisreach = *nrch;
				rtable[thisreach][1]= thisreach;
				rtable[thisreach][2]= *nb; 
				rtable[thisreach][3]= *nrch+1;
				rprops[thisreach][3]= (coord[3-1][icbeg]-coord[3-1][icend])/2.;
				rprops[thisreach][2]= coord[5-1][icarea];
				rprops[thisreach][1]= (coord[4-1][icbeg]-coord[4-1][icend])
					/(2.* rprops[thisreach][3]);
				rprops[thisreach][4]=coord[0][icarea];  /*  End X coordinate */
				rprops[thisreach][5]=coord[1][icarea];  /*  End Y coordinate */
				if(ordert <= 0)nbp=subbno;
				else nbp= *nb;
//				reachshape(cnet,coord,tree,ilink,nbp,mag,order);  //  done later
				/*  Upper half reach  */
				*nrch = *nrch +1;
				rtable[thisreach+1][1]= *nrch;
				if(iup1 > 0)
				{
					rtable[thisreach+1][2]=markbasin(nb, iup1, cnet, coord, 
						nrch,ordert,subbno,mag,dsnodeid);
					mag[ilink]=mag[ilink]+mag[iup1];
				}
				else
					rtable[thisreach+1][2]=0;
				if(iup2 > 0)
				{
					rtable[thisreach+1][3]=markbasin(nb, iup2, cnet, coord, 
						nrch,ordert,subbno,mag,dsnodeid);
					mag[ilink]=mag[ilink]+mag[iup2];
				}
				else
					rtable[thisreach+1][3]=0;
				reachshape(cnet,coord,ilink,nbp,mag[ilink],dsnodeid[ilink]);
				rprops[thisreach+1][3]= rprops[thisreach][3];
				rprops[thisreach+1][2]= rprops[thisreach][2];
				rprops[thisreach+1][1]= rprops[thisreach][1];
				rprops[thisreach+1][4]= coord[0][(icarea+icbeg)/2];  /*  approx midpoint*/
				rprops[thisreach+1][5]= coord[1][(icarea+icbeg)/2];  
			}
		}
		else
		{
			/*  This is an external basin  */
				*nrch = *nrch+1;
				thisreach = *nrch;
				rtable[*nrch][1]= *nrch;
				rtable[*nrch][2]= *nb; 
				rtable[*nrch][3]= 0;
				rprops[thisreach][3]= (coord[3-1][icbeg]-coord[3-1][icend])/2.;
				rprops[thisreach][2]= coord[5-1][icarea];
				if(rprops[thisreach][3] <= 0.)
				{
					rprops[thisreach][1]=(float)0.01;
				}
				else
				{
					rprops[thisreach][1]= (coord[4-1][icbeg]-coord[4-1][icend])
					/(2.* rprops[thisreach][3]);
				}
				rprops[thisreach][4]=coord[0][icarea];  /*  End X coordinate */
				rprops[thisreach][5]=coord[1][icarea];  /*  End Y coordinate */
				if(ordert <= 0)nbp=subbno;
				else nbp= *nb;
				mag[ilink]=1;   // magnitude of external basin
				reachshape(cnet,coord,ilink,nbp,mag[ilink],dsnodeid[ilink]);
		}
		return(thisreach);
}

//*************************** subbasinsetup ******************************************

int subbasinsetup(char *pfile, char *wfile,char *treefile,char *coordfile, 
				  char *shpfile,int ordert,int subbno)
{

  char dummy[MAXLN];
  int nlmax,i,j,nc,nb,nrc,nrch,
	  nhc,thisreach;
  err = TD_NO_ERROR;
  int **cnet, *mag;
  float **coord, ndv;

  FILE *fp;
/* define directions */
   d1[1]=0; d1[2]= -1; d1[3]= -1; d1[4]= -1; d1[5]=0; d1[6]=1; d1[7]=1; d1[8]=1;
   d2[1]=1; d2[2]=1; d2[3]=0; d2[4]= -1; d2[5]= -1; d2[6]= -1; d2[7]=0; d2[8]=1;

/*  reading Tree file  */
	fp = fopen(treefile,"r");
if (fp == NULL) return 1; //printf("Unable to open treefile\n");
/*  count length of file  */
	i=0;
	nlmax= -1;
	while(i != EOF)
	{
		i=readline(fp, dummy);
		nlmax++;
	}
/*	printf("%d\n",nlmax);   */
	fclose(fp);
/*  Allocate arrays for use in fortran  */
	cnet = (int **) matalloc(7,nlmax+1,RPINTDTYPE);
	mag = (int *) malloc(sizeof(int)*(nlmax+1));

	fp = fopen(treefile,"r");
if (fp == NULL) return 1; //printf("Unable to open treefile\n");
	long * dsnodeid;//Downstream node identifier
	dsnodeid =  new long [nlmax];
	for(j = 0; j < nlmax; j++)
	{
		for(i=0; i<7; i++)
			fscanf(fp,"%d",&cnet[i][j]); 
		fscanf(fp,"%d",&dsnodeid[j]);   // Clear any remnants at end of line
		eol(fp);
	}
	fclose(fp);

/*  Reading coordfile  */
	fp = fopen(coordfile,"r");
if (fp == NULL) return 1; //printf("Unable to open coordfile\n");
/*  count length of file  */
	i=0;
	nc= 0;  /*  nc ends up actually being one more than the number of coords 
	retained for consistency with original linkan code */
	while(i != EOF)
	{
		i=readline(fp, dummy);
		nc++;
	}

/*	printf("%d\n",nc);  */
	coord = (float **) matalloc(5,nc,RPFLTDTYPE);
	nc=nc-1;
	fclose(fp);
	fp = fopen(coordfile,"r");
if (fp == NULL) return 1;//printf("Unable to open coordfile\n");
	for(j = 0; j < nc; j++)
		for(i=0; i<5; i++)fscanf(fp,"%f",&coord[i][j]);
	fclose(fp);


	if (gridread(pfile,&sdir,&filetype)==0)
		err=TD_NO_ERROR;
	else 
		err=TD_FAILED_GRID_OPEN;
	
	
	nx = sdir.head.nx;
	ny = sdir.head.ny;
	dx = sdir.head.dx;
	dy = sdir.head.dy;
	csize = dx;
	ndv = sdir.nodata;
	for(i=0;i<4;i++) bndbox[i]=sdir.head.bndbox[i];
	sawg.head=sdir.head;
	sawg.nodata=-1;
	
	if (allocategrid(&sawg,sawg.head,sawg.nodata)!=0)
		err=TD_FAILED_MEMORY_ALLOC;

	nb=0;
	nrc = 2*nlmax;  /* The number that reach counting starts from  */
	nrch = nrc-1;   /* Initialize current reach number  */
	nhc = 5*nlmax-2;  /*  The maximum number of reaches possible in binary tree */
	rtable =  imatrix(nrc,nhc,1,3);   /*  Numerical Recipes matrix allocation  */
	rprops =  matrix(nrc,nhc,1,5); 
	
	

	shp1.create(shpfile,API_SHP_POLYLINE);
	{field f("DOUT_MID",FTDouble,16,1);  shp1.insertField(f,0);}
	{field f("DOUT_START",FTDouble,16,1);  shp1.insertField(f,0);}
	{field f("DOUT_END",FTDouble,16,1);  shp1.insertField(f,0);}
	{field f("WSNO",FTInteger,6,0);  shp1.insertField(f,0);}
	{field f("US_Cont_Area",FTDouble,16,1);  shp1.insertField(f,0);}  //dgt 3/6/2006 changed 4th argument to 1 to force to store as double  
	{field f("Straight_Length",FTDouble,16,1);  shp1.insertField(f,0);}  //dgt 3/6/2006 changed 4th argument to 1 to force to store as double
	{field f("Slope",FTDouble,16,12);  shp1.insertField(f,0);}
	{field f("Drop",FTDouble,16,2);  shp1.insertField(f,0);}
	{field f("DS_Cont_Area",FTDouble,16,1);  shp1.insertField(f,0);}  //dgt 3/6/2006 changed 4th argument to 1 to force to store as double
	{field f("Magnitude",FTInteger,6,0);  shp1.insertField(f,0);}
	{field f("Length",FTDouble,16,1);  shp1.insertField(f,0);}
	{field f("Order",FTInteger,6,0);  shp1.insertField(f,0);}
	{field f("DSNODEID",FTDouble,12,0);  shp1.insertField(f,0);}  
	{field f("USLINKNO2",FTInteger,6,0);  shp1.insertField(f,0);}
	{field f("USLINKNO1",FTInteger,6,0);  shp1.insertField(f,0);}
	{field f("DSLINKNO",FTInteger,6,0);  shp1.insertField(f,0);}
	{field f("LINKNO",FTInteger,6,0);  shp1.insertField(f,0);}

// 1/30/05 Logic has been added here to delineate separate watersheds as distinct areas if ordert < 0
    if(ordert < 0)subbno=0;  // Initialize subbasin number for separate basin counting	
		
	for(i =0; i < nlmax; i++)
	{
		if(cnet[3][i] == -1)  // this is a root link
		{
			if(ordert >=0)
		    	thisreach=markbasin(&nb,i,cnet,coord, &nrch,ordert,subbno,mag,dsnodeid);  
			else
			{
				subbno=subbno+1;
				thisreach=markbasin(&nb,i,cnet,coord, &nrch,ordert,subbno,mag,dsnodeid); 
			}
			/*  Recursive call to mark basins  */
		}
	}

	shp1.close(shpfile);

	if(ordert ==0)
	{
		if(subbno > 0) // logic to add to an existing watershed file
		{
			
		
			sgrid sawtg;

			int count=0;
			
			
			if (gridread(wfile,&sawtg,&filetype)==0)
				err=TD_NO_ERROR;
			else
				err =TD_FAILED_GRID_OPEN;
			
			if(err == TD_NO_ERROR)
			{
			for(i=0; i<ny; i++)
				for(j=0; j<nx;j++)
				{
					if(sawg.d[j][i] >=1)
					{
						sawg.d[j][i]=subbno;
						count=count+1;
						// code here for other watershed attributes
					}
					else sawg.d[j][i]= sawtg.d[j][i];
				}
				free(sawtg.d[0]);free(sawtg.d);
				
			}
			else
			{
				ndv= -1;
				
				for(i=0; i<ny; i++)
					for(j=0; j<nx; j++)
						if(sawg.d[j][i]>=1)
						{
							sawg.d[j][i]=subbno;
							
							count = count + 1;
						// code here for other watershed attributes
						}
			}
			printf("%d %d\n",subbno,count);
		}
		else
		{
			ndv= -1;
		for(i=0; i<ny; i++)
			for(j=0; j<nx; j++)
				if(sawg.d[j][i]>=1)sawg.d[j][i]=1;
		}
	}
	


	if (gridwrite(wfile,sawg,filetype)==0)
		err=TD_NO_ERROR;
	else{
		err=TD_FAILED_GRID_SAVE;
		return err;
		//if (wfile)
		//	AfxMessageBox( LPCTSTR(strcat( "Failed to save file: ", wfile)) );
	}


	free(sawg.d[0]);free(sawg.d);
	free(sdir.d[0]); free(sdir.d);
	

	return(err);

} 


/*  ***************************** larea()  *************************** */

/* function to label area recursively */
void larea(int i,int j,int lab)
{
int in,jn,k;
if(sawg.d[j][i] != (short)lab)
{
		if(i!=0 && i!=ny-1 && j!=0 && j!=nx-1 && sdir.d[j][i]!= -1)
			/* not on boundary  */
		{
			sawg.d[j][i] = (short)lab;   
			for(k=1; k<=8; k++)
			{  
				in=i+d1[k];
				jn=j+d2[k];
				/* test if neighbor drains towards cell excluding boundaries */
				if(sdir.d[jn][in]>=0 && (sdir.d[jn][in]-k==4 || sdir.d[jn][in]-k==-4))
				{
					larea(in,jn,lab);
				}
			}
		}
}
}

//*******************************deprec**************************************
/* function to compute dependencies recursively.
   This invokes symmetry and works the opposite way round to dparea*/

void deprec(int i,int j)
{
   int in,jn,k;
   float p;
   if(apool[j][i] == 0)
   {
	   apool[j][i]=1;  /*  Use this to flag pixel as done   */
	   /*   printf("%d %d %g\n",i,j,pout);   */
	   if(i!=0 && i!=ny-1 && j!=0 && j!=nx-1 && fangle.d[j][i] >= -0.5)
		   /* i.e. the cell isn't outside domain */
	   { 
		   /*add proportion out to dependence  */
   for(k=1; k<=8; k++)
   {
			   /* Find the proportion draining to each neighboring cell  */
			   p=prop(fangle.d[j][i],k);
			   if(p>1e-6)  /*  Recurse only for grid cells 
					with proportion greater than 1e-6 to guards against
					numerical loops due to angle rounding  DGT 9/12/08*/	
//			   if(p>0.0)  /*  Recurse to downslope neighbor */	
			   {
				   in = i + d1[k];
				   jn = j + d2[k];
				   if(apool[jn][in] == 0)
				   {
					   deprec(in,jn);
				   }
				   fdepg.d[j][i] = fdepg.d[j][i] + p * fdepg.d[jn][in] ;
			   }
		   }
	   }
   }
} 

//****************************************dependence***************************

/* Subroutine to compute Dependence map   */
int dependence(int *infi,int *infj,int ninf)
{
	int i,j, k;
	/* Define directions */
	
	d1[1]=0; d1[2]= -1; d1[3]= -1; d1[4]= -1; d1[5]=0; d1[6]=1; d1[7]=1; d1[8]=1;
	d2[1]=1; d2[2]=1; d2[3]=0; d2[4]= -1; d2[5]= -1; d2[6]= -1; d2[7]=0; d2[8]=1;
	
	/* Calculate angles for interpolation */  
	aref[0]= -atan2(dy,dx);
	aref[1]=0.;
	aref[2]= -aref[0];
	aref[3]=(float)(0.5*PI);
	aref[4]=PI-aref[2];
	aref[5]=(float)PI;
	aref[6]=PI+aref[2];
	aref[7]=(float)(1.5*PI);
	aref[8]=2.*PI-aref[2];
	aref[9]=(float)(2.*PI);
	/*  Initialize dependence to 0  */
for(i=0; i<ny; i++){
		for(j=0; j<nx; j++)
		{
		 //Already initialized to zero at memory allocation time
			apool[j][i]=0;
		}
	}
	
		/*  For each pixel in dependence set initialize dep */
for(k=0; k<ninf; k++)
	{
		fdepg.d[infj[k]][infi[k]]=1.;
		apool[infj[k]][infi[k]]=1;   /*  Set this to mark as done and avoid 
									 double counting in case these drain towards 
									 another in the dependence set */
	}

/* "lower block" */
	for(i=ny/2; i<ny-1; i++)
	{
		for(j=nx/2; j<nx-1; j++)
			deprec(i,j);
		for(j=nx/2-1; j>=1; j--)
			deprec(i,j);
	}
 /* "upper block" */
	for(i=ny/2-1; i>=1; i--)
	{
		for(j=nx/2; j<nx-1; j++)
			deprec(i,j);
		for(j=nx/2-1; j>=1; j--)
			deprec(i,j);
	}
	return(0);
}  


int depgrd(char *angfile,char *dgfile, char *depfile)
{
	int *rows, *cols,n,i,j;//,**dg;
	err = TD_NO_ERROR;
	
	//read angles 
	if (gridread(angfile,&fangle,&filetype)!=0)
		err = TD_FAILED_GRID_OPEN;
	if(err != TD_NO_ERROR)goto ERROR1;  


	nx =fangle.head.nx;
	ny = fangle.head.ny;
	dx = fangle.head.dx;
	dy = fangle.head.dy;
	ndv = fangle.nodata;
	csize = dx;
	for(i=0;i<4;i++) bndbox[i]=fangle.head.bndbox[i];

	//read disturbance grid file   
	if (gridread(dgfile,&ldisturbg,&filetype)!=0)
		err = TD_FAILED_GRID_OPEN;

	if(err != TD_NO_ERROR)goto ERROR2;   
	//Allocate memory  
	
	fangle.nodata=-1;	//temporarily change nodatavalue for init of depgrid

	fdepg.head=fangle.head;
	if (allocategrid(&fdepg,fdepg.head,0)!=0)
		err = TD_FAILED_MEMORY_ALLOC;

	fdepg.nodata=fangle.nodata;
	fangle.nodata=ndv;	//set nodatavalue back to what it was...
	
	if((apool = (short **) matalloc(nx, ny, RPSHRDTYPE)) == NULL)
	{
		err=TD_FAILED_MEMORY_ALLOC;
//		AfxMessageBox( LPCTSTR(strcat( "Could not allocate memory for apool", "323")) );
		return err;
	}  
	
	n=0;
	for(i=0; i<ny; i++){
		for(j=0; j<nx; j++)
		{
			if(ldisturbg.d[j][i] >= 1)n=n+1;
		}
	}
	rows = (int *)malloc(sizeof(int)*(n));
	cols = (int *)malloc(sizeof(int)*(n));
	n=0;
	for(i=0; i<ny; i++){
		for(j=0; j<nx; j++)
		{
			if(ldisturbg.d[j][i] >= 1)
			{
				rows[n]=i;
				cols[n]=j;
				n=n+1;
			}
		}
	}
	err=dependence(rows,cols,n);
	//Write out areas  

	if (gridwrite(depfile,fdepg,filetype)!=0)
	{
		err = TD_FAILED_GRID_SAVE;
		return err;
	}

	free(fdepg.d[0]);free(fdepg.d);
	free(apool[0]);
	free(apool);
ERROR2:
	
	free(ldisturbg.d[0]);free(ldisturbg.d);
	
ERROR1:

	free(fangle.d[0]);free(fangle.d);
	
	return(err);
} 

  


/*  Function to compute Topmodel A/tan Slope

This subroutine takes as input 3 grid files, 
	slopefile - the slope grid
	areafile - the specific catchment area grid

David G Tarboton
Utah State University  
 */
//#include "gridio.h"
/*********************************************************************/
int atanbgrid(char *slopefile,char *areafile,char *atanbfile, int diskbasedflag)
{
  float ndvs, ndva;
  int i,j;
  err = TD_NO_ERROR;

/* read grid files */

/*  bool inRam;
  if (diskbasedflag == 1) //open as diskbased
	  inRam = false;
  else
	  inRam = true;

  inRam = true; //********** TEMPORARY HARD CODED VALUE UNTIL BUG IS FIXED IN GRID DLL ************/

  if (gridread(slopefile,&fslopeg,&filetype) != 0) 
  {
	  err = TD_FAILED_GRID_OPEN;
	  return err;
  }
  else
	  err = TD_NO_ERROR;

  
  nx =fslopeg.head.nx; 
  ny = fslopeg.head.ny; 
  dx = fslopeg.head.dx; 
  dy = fslopeg.head.dy; 
  ndvs = fslopeg.nodata;
  csize = dx;
  for(i=0;i<4;i++) bndbox[i]=fslopeg.head.bndbox[i];
 

if(err != TD_NO_ERROR)goto ERROR0;



//  DGT logic to deal with disk based grid being written
/*	if(inRam == false)
	{
	
		fgrid tempg;
		tempg.head.dx=dx;
		tempg.head.dy=dy;
		tempg.head.nx=nx;
		tempg.head.ny=ny;
		for(i=0;i<4;i++) tempg.head.bndbox[i]=bndbox[i];
		tempg.nodata=-1;
		allocategrid(&tempg,tempg.head,0.0);

		if(gridread(areafile,&fsca,&filetype)!=0) {free(fsca.d[0]);free(fsca.d); return 1;}

		for(i=0; i<nx; i++)for(j=0; j<ny; j++)
		tempg.d[j][i]=fsca.d[j][i];
		if (gridwrite(atanbfile,tempg,filetype)!=0)
		{
			err=TD_FAILED_GRID_SAVE;
			return err;
		}
		free(tempg.d[0]);free(tempg.d);
	
		if (gridread(atanbfile,&fsca,&filetype)!=0){free(fslopeg.d[0]);free(fslopeg.d); return TD_FAILED_GRID_OPEN;} //close the previously opened grid befor leaving function
	}
	else  */
		if(gridread(areafile,&fsca,&filetype) !=0){free(fslopeg.d[0]);free(fslopeg.d); return TD_FAILED_GRID_OPEN;} //close the previously opened grid befor leaving function


  
  ndva = fsca.nodata; 

if(err != TD_NO_ERROR)goto ERROR1;  

for(i=0; i < ny; i++)
  {
  for(j=0; j<nx; j++)
{
if(fsca.d[j][i] > 0.0 && fsca.d[j][i] > ndvs)
  {
		
	fsca.d[j][i]=fslopeg.d[j][i]/fsca.d[j][i];
	  /*  Work with slope/area to avoid divide by zero's   */
  }
	else
		
		fsca.d[j][i]=-1;
}
  } 

	
 

fsca.nodata=-1;


	if (gridwrite(atanbfile,fsca,filetype) !=0) 
	{
		err = TD_FAILED_GRID_SAVE; //file save error
		return err;
	}
	else
		err = TD_NO_ERROR; //no errors


/*  wrapping up  */
  free(fsca.d[0]); free(fsca.d);
	
ERROR1:
  free(fslopeg.d[0]);
  free(fslopeg.d);
ERROR0:
  return(err);
}
 

/*  Function to compute hillslope flow distances

Takes as input 2 grid files, 
	pfile - the d8 directions
	srcfile - the channel definition file
	thresh is the integer value used to define channels in srcfile
	(a greater than or equal to test is used)

distfile is the output grid of distances to channels
 */

float **darr;
/* function to compute distance recursively */

void dists(int i,int j)
{
int in,jn,k;
  if(i!=0 && i!=ny-1 && j!=0 && j!=nx-1 && sdir.d[j][i] != -1)
  /* not on boundary  */
  {
in=i+d1[sdir.d[j][i]];
		jn=j+d2[sdir.d[j][i]];
if(larr.d[j][i] < tresh)   /*  Only add to cells on hillside  */
		fdarr.d[j][i] = dist[sdir.d[j][i]]+ fdarr.d[jn][in]; 	
for(k=1; k<=8; k++)
{  in=i+d1[k];
   jn=j+d2[k];
	   /* test if neighbor drains towards cell excluding boundaries */
   if(sdir.d[jn][in]>=0 && larr.d[jn][in] < tresh &&   /*  Only recurse upslope */
		 (sdir.d[jn][in]-k==4|| sdir.d[jn][in]-k==-4))/*  from cells on hillslope  */
		   {
dists(in,jn);
		   }
}
  }
}

int distgrid(char *pfile, char *srcfile, char *distfile, int thresh, int diskbasedflag)
 {
   int i,j;
   err = TD_NO_ERROR;
   tresh = thresh;  /*  Passing to global variable for recursive call  */

/* define directions */
   d1[1]=0; d1[2]= -1; d1[3]= -1; d1[4]= -1; d1[5]=0; d1[6]=1; d1[7]=1; d1[8]=1;
   d2[1]=1; d2[2]=1; d2[3]=0; d2[4]= -1; d2[5]= -1; d2[6]= -1; d2[7]=0; d2[8]=1;
//  JOHN If diskbasedflag has the value 1 then open the grids disk based 
//   - otherwise work with them in memory


   if (diskbasedflag == 1)
	   inRam = false;
   else
	   inRam = true;

   if ( gridread(pfile,&sdir,&filetype) !=0) return TD_FAILED_GRID_OPEN;
   else err = TD_NO_ERROR;

  

/*   read areas  */
   
   
   
   if ( gridread(srcfile,&larr,&filetype) !=0)
   {	
	   free(sdir.d[0]);free(sdir.d);
	   return TD_FAILED_GRID_OPEN;
   }
   else 
	   err = TD_NO_ERROR;

   
   
   nx = sdir.head.nx; 
   ny =sdir.head.ny; 
   dx = sdir.head.dx;
   dy = sdir.head.dy; 
   ndv = sdir.nodata;

   csize = dx;
   for (i=0;i<4;i++)bndbox[i]=sdir.head.bndbox[i];
  



   
   
   fdarr.head.dx=dx;
   fdarr.head.dy=dy;
   fdarr.head.nx=nx;
   fdarr.head.ny=ny;
   for(i=0;i<4;i++) fdarr.head.bndbox[i]=bndbox[i];

   fdarr.nodata=-1;
   allocategrid(&fdarr,fdarr.head,fdarr.nodata);

   

   

/*  Calculate Distances  */
for(i=1; i<=8; i++){
 dist[i]=sqrt(d1[i]*d1[i]*dy*dy+d2[i]*d2[i]*dx*dx);
	}

 
/*  Recursive call to compute distances for each source pixel  */
	 for(i=1; i<ny-1; i++)
 {  for(j=1; j<nx-1; j++)
{
  if(larr.d[j][i] >= thresh)
		  {
				  fdarr.d[j][i]=0;
	
			  dists(i,j);
		  }
}
 }
 
/* write out to grid files */


   if (gridwrite(distfile,fdarr,filetype) !=0 )
   {
	   err = TD_FAILED_GRID_SAVE;
	   return err;

   }
   else 
	   err = TD_NO_ERROR;


   free(fdarr.d[0]);
   free(fdarr.d);
   

   free(larr.d[0]);
   free(larr.d);


   free(sdir.d[0]);
   free(sdir.d);
   

   return(err);
}

/* function to compute partial areas recursively */

void dpmarea(int i,int j)
{

	int in,jn,k,kback,con=0;
/* con is a flag that signifies possible contaminatin of sca
	due to edge effects  */
	float p;

	if(fsca.d[j][i] ==0.0)/* i.e., if the cell hasn't been looked at yet */
	{
if(i!=0 && i!=ny-1 && j!=0 && j!=nx-1 && fangle.d[j][i] > ndva)
			/* i.e. the cell isn't outside domain */
		{
			fsca.d[j][i] = dx;
			
			
			if(useww == 1)fareaw.d[j][i] = fweight.d[j][i];
			for(k=1; k<=8; k++)
			{
				in = i + d1[k];
				jn = j + d2[k];
				/* for each neighboring cell, find the proportion of of outflow */
				/* draining back to the cell in question */
				kback = k + 4;
				if(kback>8)kback = kback - 8;
				p=prop(fangle.d[jn][in],kback);
				if(p>1e-6)  /*  Recurse only for grid cells 
					with proportion greater than 1e-6 to guards against
					numerical loops due to angle rounding  DGT 9/12/08*/	
//				if(p>0.)
					{
					if (err != TD_NO_ERROR)
						return; //avoid future recursion

					dpmarea(in,jn);
					
					if ( err != TD_NO_ERROR )
						return; //exit function without continuing if err occured in recursion

					if( fsca.d[jn][in] < -0.5)
						con = -1;

					/*****
					 tempvalue (below) holds the value of the grid to be used in the 
					 multiplication of tempvalue with p.  This is done because of 
					 rounding errors that were encountered and had weird effects on the data.
					 By storing it before multiplying, I was able to get the function 
					 to give the correct results (don't ask why this solved the problem, 
					 because I don't know).  --Jonathan Cole
					*****/
					float tempvalue;
					if ( fdmg.d[jn][in] != ndv) 
						tempvalue = fsca.d[jn][in] * fdmg.d[jn][in];
					else{
						err = TD_UNEXPECTED_NODATA_IN_GRD;
						return;
					}

					fsca.d[j][i] = fsca.d[j][i] + tempvalue*p ;

					
					if(useww == 1)
					{
						if(fareaw.d[jn][in] <= ndv  || fareaw.d[j][i] <=ndv)
						{  
							fareaw.d[j][i] = ndv;
						}
						else
						{
							tempvalue =fareaw.d[jn][in] * fdmg.d[jn][in];  // DGT 10/26/02 added decay multiplier here
							fareaw.d[j][i] = fareaw.d[j][i] + tempvalue*p;
						}
					}
}
				if(fangle.d[jn][in] <= ndva)con = -1; 
}
			if(con == -1 && ccheck == 1)
			{
				fsca.d[j][i]= -1.0;
				if(useww == 1)fareaw.d[j][i]=ndv;
			}
		}
	}
}

int dmarea(char *pfile, char *afile, char *dmfile, double *x, double *y, long nxy, 
		 char *wfile, int usew, int contcheck)
{
	

	int i,j,row,col;
	err = TD_NO_ERROR;
	ccheck=contcheck;
	useww=usew;

 
	/* Define directions */
	/*  d1a=dd1-1; d2a=dd2-1;  */
	d1[1]=0; d1[2]= -1; d1[3]= -1; d1[4]= -1; d1[5]=0; d1[6]=1; d1[7]=1; d1[8]=1;
	d2[1]=1; d2[2]=1; d2[3]=0; d2[4]= -1; d2[5]= -1; d2[6]= -1; d2[7]=0; d2[8]=1;
	
	/* read angles */


	
	
	//gridread function would return false (0) if there were no errors, but
	//the new grid open function returns true(1) if there were no errors;
	//therefore, this next if/else statement corrects the difference
	if (gridread(pfile,&fangle,&filetype)==0)
	{
		err=TD_NO_ERROR;
	}
	else
	{
		err=TD_FAILED_GRID_OPEN;
		return err;
	}
	
	nx = fangle.head.nx;
	ny = fangle.head.ny;
	dx =fangle.head.dx;
	dy = fangle.head.dy;
	csize = dx; 
	ndva = fangle.nodata;

	if(gridread(dmfile,&fdmg,&filetype)!=0)
	{	err = TD_FAILED_GRID_OPEN;
		return err;
	}
	else
		err = headcheck(fdmg.head,fangle.head);

	if (err != TD_NO_ERROR)	return err;
	
	dmndv = fdmg.nodata;
	
	//bndbox is working off of the corner of the cell, not the center
	//so it needs to remove 1/2 of a cell size to map down to the corner
	//instead of the center of the cell...
	for(i=0;i<4;i++)bndbox[i]=fangle.head.bndbox[i];
	

	if(err != TD_NO_ERROR)goto ERROR1;   
	
	/*  read weights if necessary   */
	if(usew == 1)
	{
		

		if(gridread(wfile,&fweight,&filetype) !=0)
		{	err=TD_FAILED_GRID_OPEN;
				return err;
		}
		else
			err = headcheck(fweight.head,fangle.head);
			
		
		
		ndv = fweight.nodata;

						
		if(err != TD_NO_ERROR)goto ERROR2;
		fareaw.head=fangle.head;
		fareaw.nodata=ndv;
		allocategrid(&fareaw,fareaw.head,ndv);
		
		
	}
	
		
	/* Calculate angles for interpolation */  
	aref[0]= -atan2(dy,dx);
	aref[1]=0.;
	aref[2]= -aref[0];
	aref[3]=(float)(0.5*PI);
	aref[4]=PI-aref[2];
	aref[5]=(float)PI;
	aref[6]=PI+aref[2];
	aref[7]=(float)(1.5*PI);
	aref[8]=2.*PI-aref[2];
	aref[9]=(float)(2.*PI);
	
	/* initialize sca grid and header */
	fsca.head=fangle.head;
	fsca.nodata=-1;
		/* initialize sca grid to -1 */
	if(allocategrid(&fsca,fsca.head,-1)!=0) return 1;
	

/* initialize sca grid to 0 */
for(i=0; i<ny; i++)
	{
		for(j=0; j<nx; j++)
		{
			if(i!=0 && i!=ny-1 && j!=0 && j!=nx-1 && fangle.d[j][i]> ndva)
			{
			   fsca.d[j][i]=0.0; 
			}
		}
	}

		
	if(nxy > 0)   /*  Only compute area's for designated location  */
	{
		for (int CurXY = 0; CurXY < nxy; CurXY++)
		{
			col= (int)floor((x[CurXY]-bndbox[0])/csize);
			row= (int)floor((bndbox[3]-y[CurXY])/csize);
			if(row <0 || row > ny || col < 0 || col > nx)
			{
			//			printf("Given coords out of area - whole area will be calculated\n");
				row=0; col=0;
			}
			if(row > 0 && col > 0)
			{
		/* call drainage area subroutine for pixel to zero on  */
				dpmarea(row,col);
			}
		}
	}
	else
	{
		
		/* Call drainage area subroutine for each cell   */
		/* working from the middle out to avoid deep recursions  */ 
		
		/* "lower block" */
		for(i=ny/2; i<ny-1; i++)
		{
			
			for(j=nx/2; j<nx-1; j++)
				dpmarea(i,j);
			for(j=nx/2-1; j>=1; j--)
				dpmarea(i,j);

		}
		/* "upper block" */
		for(i=ny/2-1; i>=1; i--)
		{
		
			for(j=nx/2; j<nx-1; j++)
				dpmarea(i,j);
			for(j=nx/2-1; j>=1; j--)
				dpmarea(i,j);
		}
}
	/*  Write out areas  */  
	if(usew == 0)
	{
		if (gridwrite(afile,fsca,filetype)==0)
			err=TD_NO_ERROR;
		else{
			err=TD_FAILED_GRID_SAVE;
			return err;
			//if (afile)
			//	AfxMessageBox( LPCTSTR(strcat( "Failed to save file: ", afile)) );
		}
	}
	else
	{
		if ( gridwrite(afile,fareaw,filetype)==0)
			err=TD_NO_ERROR;
		else{
			err=TD_FAILED_GRID_SAVE;
			return err;
			//if (afile)
			//	AfxMessageBox(LPCTSTR( strcat( "Failed to save file: ", afile) ));
		}
	
		free(fareaw.d[0]);free(fareaw.d);
	}
	free(fsca.d[0]);free(fsca.d);
	

	if(usew == 1){
ERROR2:
	free(fweight.d[0]); free(fweight.d);
	
	}
ERROR1:
	free(fangle.d[0]);free(fangle.d);
	
	return(err);  
}


// function to compute dependencies recursively.
//This invokes symmetry and works the opposite way round to dparea

void dsaccumrec(int i, int j, int touch, float wgval)
{
	int in,jn,k;
	float p;
	if(apool[j][i] == 0)
	{
		apool[j][i]=1;  //Use this to flag pixel as done   
		//printf("%d %d %g\n",i,j,pout);   
		if(i!=0 && i!=ny-1 && j!=0 && j!=nx-1 && fangle.d[j][i] >= -0.5 && fdisturbg.d[j][i] > ndv)
		{
			//i.e. the cell isn't outside domain 
			fdepg.d[j][i]= fdisturbg.d[j][i];  // Count self
			fdmaxg.d[j][i]= fdisturbg.d[j][i];  
			
			//add proportion out to dependence  
			for(k=1; k<=8; k++)
			{
				//Find the proportion draining to each neighboring cell  
				p=prop(fangle.d[j][i],k);
				if(p>1e-6)  /*  Recurse only for grid cells 
					with proportion greater than 1e-6 to guards against
					numerical loops due to angle rounding  DGT 9/12/08*/	
//				if(p>0.0)  //Recurse to downslope neighbor 
				{
					in = i + d1[k];
					jn = j + d2[k];
					if(apool[jn][in] == 0 && touch == 1)
					{
						if (fdisturbg.d[jn][in] >= wgval)dsaccumrec(in,jn,touch,wgval);
					}
					else if (fdisturbg.d[jn][in] > ndv)
					{
						dsaccumrec(in,jn,touch,wgval);
						fdepg.d[j][i]= fdepg.d[j][i]+p*fdepg.d[jn][in];
						if(fdmaxg.d[jn][in] > fdmaxg.d[j][i])fdmaxg.d[j][i]= fdmaxg.d[jn][in];
					}
				}
}
		}
	}
} 


int dsaccum(char *angfile,char *wgfile, char *depfile, char *maxfile, int touch, float wgval)
{
   int err,/* *rows, *cols,n,*/i,j/*,k*/;

   err = TD_NO_ERROR; //assume no errors until one occurs
 
   //read angles 
//err=gridread(angfile,(void ***)&ang,RPFLTDTYPE,&nx,&ny,&dx,&dy,
// bndbox,&csize,&ndv,&filetype);
	if (gridread(angfile,&fangle,&filetype) !=0)
	{	err = TD_FAILED_GRID_OPEN;
		return err;
	}
		
	
	nx = fangle.head.nx;
	ny = fangle.head.ny;
	dx = fangle.head.dx;
	dy = fangle.head.dy;
	csize = dx;
	ndv = fangle.nodata;
	

if(err != TD_NO_ERROR)goto ERROR1;  
  
//read disturbance grid file   



	if(gridread(wgfile,&fdisturbg,&filetype)!=0)
	{
		err = TD_FAILED_GRID_OPEN;
	}
		
	else
	{
		err= TD_NO_ERROR;
	}
	
	if(err != TD_NO_ERROR)goto ERROR2;
	
	//Allocate memory  
	fangle.nodata=-1;	//temporarily change the ndv for init of these grids
	allocategrid(&fdepg,fangle.head,0);
	allocategrid(&fdmaxg,fangle.head,0);
	
	fdepg.nodata=fangle.nodata;
	fdmaxg.nodata=fangle.nodata;

	if((apool = (short **) matalloc(nx, ny, RPSHRDTYPE)) == NULL)
	{
		err=TD_FAILED_MEMORY_ALLOC;
	//	AfxMessageBox( LPCTSTR(strcat( "Could not allocate memory for apool 2", "232")) );		
		return err;
	}  
	fangle.nodata=ndv;	 //change it back just in case
	//Define directions 
	
	d1[1]=0; d1[2]= -1; d1[3]= -1; d1[4]= -1; d1[5]=0; d1[6]=1; d1[7]=1; d1[8]=1;
	d2[1]=1; d2[2]=1; d2[3]=0; d2[4]= -1; d2[5]= -1; d2[6]= -1; d2[7]=0; d2[8]=1;
	
	//Calculate angles for interpolation 
	aref[0]= (float)-atan2(dy,dx);
	aref[1]=0.;
	aref[2]= -aref[0];
	aref[3]=(float)(0.5*PI);
	aref[4]=PI-aref[2];
	aref[5]=(float)PI;
	aref[6]=PI+aref[2];
	aref[7]=(float)(1.5*PI);
	aref[8]=2.*PI-aref[2];
	aref[9]=(float)(2.*PI);
//Initialize dependence to 0  
for(i=0; i<ny; i++){
		for(j=0; j<nx; j++)
		{
			//dep[j][i]= 0.; //initialization to zero performed earlier
			//dmax[j][i]=0.; //initialization to zero performed earlier
			apool[j][i]=0;
			if(fdisturbg.d[j][i] >= wgval && touch == 1)apool[j][i]=1;
			//  Set this to mark as done and avoid 
			//									 double counting in case these drain towards 
			//									 another in the dependence set and touch is set 
		}
	}
	
	//"lower block" 
	for(i=ny/2; i<ny-1; i++)
	{
		for(j=nx/2; j<nx-1; j++)
			dsaccumrec(i,j,touch,wgval);
		for(j=nx/2-1; j>=1; j--)
			dsaccumrec(i,j,touch,wgval);
	}
	//"upper block" 
	for(i=ny/2-1; i>=1; i--)
	{
		for(j=nx/2; j<nx-1; j++)
			dsaccumrec(i,j,touch,wgval);
		for(j=nx/2-1; j>=1; j--)
			dsaccumrec(i,j,touch,wgval);
	}
	
	//Write out areas   
	
	
	if (gridwrite(depfile,fdepg,filetype)!=0)
	{
		err = TD_FAILED_GRID_SAVE;
		return err;
	}
	
	
	
	if (gridwrite(maxfile,fdmaxg,filetype) !=0)
		err = TD_FAILED_GRID_SAVE;
	//free memory
	free(fdepg.d[0]);free(fdepg.d);
	
	free(fdmaxg.d[0]); free(fdmaxg.d);
	
	free(apool[0]);
	free(apool);
	apool = NULL;
ERROR2:
	
	free(fdisturbg.d[0]);free(fdisturbg.d);
	
ERROR1:
	
	free(fangle.d[0]); free(fangle.d);
	
	return(err);
} 

//************************************************************************

//function to compute partial areas recursively 

void sedacum(int i,int j)
{

   int in,jn,k,kback,con=0;
// con is a flag that signifies possible contaminatin of sca
	//due to edge effects  
   float p, mout;
   if(sca[j][i]==0.0)//i.e., if the cell hasn't been looked at yet 
  {
if(i!=0 && i!=ny-1 && j!=0 && j!=nx-1 && fangle.d[j][i] > ndva)
   //i.e. the cell isn't outside domain 
 {
   sca[j][i] = dx;
		   fareaw.d[j][i]=0;
		   if(usecc == 1)mout = 0;
   for(k=1; k<=8; k++)
   {
 in = i + d1[k];
 jn = j + d2[k];
//for each neighboring cell, find the proportion of of  outflow 
//draining back to the cell in question 
 kback = k + 4;
 if(kback>8)kback = kback - 8;
 p=prop(fangle.d[jn][in],kback);
 if(p>1e-6)  /*  Recurse only for grid cells 
					with proportion greater than 1e-6 to guards against
					numerical loops due to angle rounding  DGT 9/12/08*/	
// if(p>0.)
{
 sedacum(in,jn);
 if(sca[jn][in] < -0.5)con = -1;
 sca[j][i] = sca[j][i] + sca[jn][in]*p;
   if(fareaw.d[jn][in] <= ndv  || fweight.d[j][i] <=ndv)
 {  
fareaw.d[j][i]=ndv;
 }
   else
						   {
							   fareaw.d[j][i]=fareaw.d[j][i] + fareaw.d[jn][in]*p;
							   if(usecc==1)mout = mout+fareaw.d[jn][in]*p*fcoutg.d[jn][in];
						   }
}
 if(fangle.d[jn][in] <= ndva)con = -1; 
}
 //   ***********   addition for sediment transport capacity limitation
		   fdepg.d[j][i]= 0;   // initial value by default
		   if(fareaw.d[j][i] + fweight.d[j][i]> ftcapg.d[j][i])
		   {
			   fdepg.d[j][i] = fareaw.d[j][i]+fweight.d[j][i]- ftcapg.d[j][i];
			   if(fareaw.d[j][i] > ftcapg.d[j][i])
			   {  // this is the case where no concentration originates at cell
				   if(usecc==1)
				   {
					   if (fareaw.d[j][i] <= 0)fcoutg.d[j][i]=0;
					   else fcoutg.d[j][i]=mout/fareaw.d[j][i];
				   }
				  fareaw.d[j][i] = ftcapg.d[j][i];
			   }
			   else
			   {   //  this is the case where some concentration originates at cell
				   if(usecc==1)mout=(ftcapg.d[j][i] - fareaw.d[j][i])*fcing.d[j][i]+mout;
				   fareaw.d[j][i] = ftcapg.d[j][i];
				   if(usecc==1)
				   {
					   if(fareaw.d[j][i] <= 0) fcoutg.d[j][i] = 0;
						else fcoutg.d[j][i]=mout/fareaw.d[j][i];
				   }
			   }
		   }
		   else
		   {   // case where all quantity originating at cell is removed
			   if(fweight.d[j][i] > ndv)
			   {
			   fareaw.d[j][i]= fareaw.d[j][i] + fweight.d[j][i];
			   if(usecc==1)
			   {
				   if(fareaw.d[j][i] <= 0)fcoutg.d[j][i] = 0;
				   else fcoutg.d[j][i]=(mout+fweight.d[j][i]*fcing.d[j][i])/(fareaw.d[j][i]);
			   }
			   }
			   else
			   {
				   fareaw.d[j][i] = ndv;
				   if(usecc=1)fcoutg.d[j][i] = ndv;
			   }
		   }
//**********  end addition for sediment transport capacity limitation
   if(con == -1 && ccheck == 1)
		   {
			   sca[j][i]= -1.;
			   fareaw.d[j][i] = ndv;
			   if(usecc == 1)
			   {
				   fcoutg.d[j][i] = ndv;
			   }
		   }
 }
		else
		{  // on boundaries
			fareaw.d[j][i] = ndv;
			fcoutg.d[j][i] = ndv;
			fdepg.d[j][i] = ndv;
		}
 }
   
} 

//************function to do transport limited accumulation

int tlaccum(char *pfile, char *wfile, char *tcfile, char *tlafile, char *depfile, 
			char *cfile, char *coutfile, double  *x, double *y, int nxy, int usec, 
			int contcheck, ItkCallback * callback)
{	
	//variables used for calculating progress
	double totalCount; //number of rows
	double CurCount = 0;
	int Percent = 0;
	int NewPercent = 0;
	
	if (callback)
		cbProgress(callback,"",0,"Calculating transport limited accumulation...");


	int i,j,err,row,col;
	ccheck=contcheck;
	usecc=usec;
	err = TD_NO_ERROR; //assume no errors until they occur
	
	//Define directions 
	//d1a=dd1-1; d2a=dd2-1;  
	d1[1]=0; d1[2]= -1; d1[3]= -1; d1[4]= -1; d1[5]=0; d1[6]=1; d1[7]=1; d1[8]=1;
	d2[1]=1; d2[2]=1; d2[3]=0; d2[4]= -1; d2[5]= -1; d2[6]= -1; d2[7]=0; d2[8]=1;
	
	//read angles 
	//err=gridread(pfile,(void ***)&ang,RPFLTDTYPE,&nx,&ny,&dx,&dy,
	//	bndbox,&csize,&ndva,&filetype);
	if ( gridread(pfile,&fangle,&filetype)!=0)
	{
		err = TD_FAILED_GRID_OPEN;
		//if (callback)
		//	cbError(callback,pfile,err,"Unable to open angle grid file...");
	}
	
	if(err != TD_NO_ERROR)goto ERROR1;

	
	nx =fangle.head.nx;
	ny = fangle.head.ny;
	dx = fangle.head.dx;
	dy = fangle.head.dy;
	csize = dx;
	ndva = fangle.nodata;
	for(i=0;i<4;i++) bndbox[i]=fangle.head.bndbox[i];

	//read weights   




	if(gridread(wfile,&fweight,&filetype)!=0)
	{
		err = TD_FAILED_GRID_OPEN;
		//if (callback)
		//	cbError(callback,wfile,err,"Unable to open weight grid file...");
	}

	if(err != TD_NO_ERROR)goto ERROR2;
	
	
	
	ndv =fweight.nodata;


	//Allocate areaw memory  
	fareaw.head = fangle.head;
	fareaw.nodata=ndv;
	fdepg.head=fareaw.head;
	
	allocategrid(&fareaw,fareaw.head,fareaw.nodata);
	allocategrid(&fdepg,fareaw.head,0);

	fdepg.nodata=fareaw.nodata;


	
	


	if(gridread(tcfile,&ftcapg,&filetype)!=0)
	{	
		err = TD_FAILED_GRID_OPEN;
	//	if (callback)
	//		cbError(callback,tcfile,err,"Unable to open grid file...");
	}

	if(err != TD_NO_ERROR)goto ERROR2;
	if(usec==1)
	{
	

		if(gridread(cfile,&fcing,&filetype) !=0)
		{
			err = TD_FAILED_GRID_OPEN;
		//	if (callback)
		//		cbError(callback,cfile,err,"Unable to open grid file...");
		}
		

		if (err !=TD_NO_ERROR)
			goto ERROR2;
		


		fcoutg.head=fcing.head;
		
		
		allocategrid(&fcoutg,fcoutg.head,0);
		fcoutg.nodata=ndv;
		
		// initialize out of range to ndv, the rest to 0.  There are some small rounding errors that
		//		  sometimes result in a cell feeding back on itself.  A large negative ndv blows this up
		for(i=0; i<ny; i++){
			for(j=0; j<nx; j++)
			{
				if(fangle.d[j][i] <=ndv)fcoutg.d[j][i]=ndv;
			}
		}
	}
	//Allocate memory  
	if((sca = (float **) matalloc(nx, ny, RPFLTDTYPE))== NULL)
	{
		err=TD_FAILED_MEMORY_ALLOC;
		//AfxMessageBox( LPCTSTR(strcat( "Could not allocate memory for sca ", "563")) );
		return err;
	}
	if((src = (short **)matalloc(nx,ny, RPSHRDTYPE)) == NULL)
	{
		err=TD_FAILED_MEMORY_ALLOC;
		//AfxMessageBox( LPCTSTR(strcat( "Could not allocate memory for src ", "653")) );
		return err;
	}  
	
	//Calculate angles for interpolation
	aref[0]= -atan2(dy,dx);
	aref[1]=0.;
	aref[2]= -aref[0];
	aref[3]=(float)(0.5*PI);
	aref[4]=PI-aref[2];
	aref[5]=(float)PI;
	aref[6]=PI+aref[2];
	aref[7]=(float)(1.5*PI);
	aref[8]=2.*PI-aref[2];
	aref[9]=(float)(2.*PI);
	//initialize sca array to 0 
for(i=0; i<ny; i++)
	{
		for(j=0; j<nx; j++)
		{
			if(i!=0 && i!=ny-1 && j!=0 && j!=nx-1 && fangle.d[j][i]> ndva)
				sca[j][i]=0;
			else sca[j][i]= -1;
		}
	}

	

	if(nxy > 0){   //Only compute area's for designated location  
		totalCount = nxy;
		for(i=0; i<nxy; i++)
		{
			col= (int)floor((x[i]-bndbox[0])/csize);
			row= (int)floor((bndbox[3]-y[i])/csize);
			if(row >0 && row < ny-1 && col > 0 && col < nx-1)
			{  // call transport limited accumulation function for designated outlet
				sedacum(row,col);
				if (callback){
				NewPercent = (CurCount / totalCount) * 100;
				if (NewPercent > Percent){//only send back events when a change has occured
					cbProgress(callback,"",NewPercent,"Computing transport limited accumulation for specified outlets...");
					Percent = NewPercent;
				}
				CurCount++;
			}
			}
		}
	}
	else
	{
		
		//Call drainage area subroutine for each cell   
		// working from the middle out to avoid deep recursions  
		//"lower block" 
		totalCount = ny; 
		for(i=ny/2; i<ny-1; i++)
		{
			
			for(j=nx/2; j<nx-1; j++)
				sedacum(i,j);
			for(j=nx/2-1; j>=1; j--)
				sedacum(i,j);
			
			//calculate progress
			if (callback){
				NewPercent = (CurCount / totalCount) * 100;
				if (NewPercent > Percent){//only send back events when a change has occured
					cbProgress(callback,"",NewPercent,"Computing transport limited accumulation");
					Percent = NewPercent;
				}
				CurCount++;
			}
				

		}
		//"upper block" 
		for(i=ny/2-1; i>=1; i--)
		{
			
			for(j=nx/2; j<nx-1; j++)
				sedacum(i,j);
			for(j=nx/2-1; j>=1; j--)
				sedacum(i,j);

			//calculate progress
			if (callback){
				NewPercent = (CurCount / totalCount) * 100;
				if (NewPercent > Percent){//only send back events when a change has occured
					cbProgress(callback,"",NewPercent,"Computing transport limited accumulation");
					Percent = NewPercent;
				}
				CurCount++;
			}
		}
	}
	//Write out results  
	if(usec==1)
	{
		
		
		if (gridwrite(coutfile,fcoutg,filetype)!=0) 
		{
			err = TD_FAILED_GRID_SAVE;
			if (callback)
				cbError(callback,coutfile,err,"Unable to save grid file...");
		}
		free(fcoutg.d[0]);free(fcoutg.d);
		free(fcing.d[0]);free(fcing.d);  // DGT 10/10/07 corrected from fcoutg being free's twice.
			
		
	}
	
	

	if (gridwrite(tlafile,fareaw,filetype) !=0)
	{
		err = TD_FAILED_GRID_SAVE;
		goto ERROR1;
		//if (callback)
		//	cbError(callback,tlafile,err,"Unable to save grid file");
	}

	if (gridwrite(depfile,fdepg,filetype)!= 0)
	{
		err = TD_FAILED_GRID_SAVE;
		goto ERROR1;
		//if (callback)
		//	cbError(callback,depfile,err,"Unable to save grid file");
	}

	
	
	free(fareaw.d[0]);free(fareaw.d);
	
	free(fdepg.d[0]);free(fdepg.d);
	

	free(sca[0]);
	free(sca);
ERROR2:
	free(fweight.d[0]);free(fweight.d);
	
ERROR1:
	free(fangle.d[0]); free(fangle.d);
	
	return(err);  
	
}  

/* function to compute partial areas recursively */
void dpsllArea(int i,int j)
{
	if (err != TD_NO_ERROR)
		return; //stop processing

	int in,jn,k,kback,con=0;
/* con is a flag that signifies possible contaminatin of sca(specific catchment area)
	due to edge effects  */
	float p, Concentration;
	
	if(fsca.d[j][i] ==0.0)/* i.e., if the cell hasn't been looked at yet */
	{
if(i!=0 && i!=ny-1 && j!=0 && j!=nx-1 && fangle.d[j][i] > ndva)
			/* i.e. the cell isn't outside domain */
		{
			fsca.d[j][i]=dx;
			fareaw.d[j][i]=0.0;
			Concentration = 0;
			for(k=1; k<=8; k++)
			{
				in = i + d1[k]; //find the row of the neighbor cell we are looking at
				jn = j + d2[k]; //find the column of the neighbor cell we are looking at

				/* for each neighboring cell, find the proportion of outflow */
				/* draining back to the cell in question */
				kback = k + 4;
				if(kback>8)kback = kback - 8;
				p = prop(fangle.d[jn][in],kback);				
				if(p>1e-6)  /*  Recurse only for grid cells 
					with proportion greater than 1e-6 to guards against
					numerical loops due to angle rounding  DGT 9/12/08*/	
//				if(p>0.)
{
					dpsllArea(in,jn);
					if (err != TD_NO_ERROR)
						return; //if an error occured, we want to exit without continuing
					
					if( fsca.d[jn][in] < -0.5)
						con = -1;
					
					if ( fdmg.d[jn][in] == ndv) 
					{
						err = TD_UNEXPECTED_NODATA_IN_GRD;
					return;
					}
					
					if(fsca.d[jn][in] <= -1  || fsca.d[j][i] <= -1)
					{  
						fsca.d[j][i]=-1;
						fareaw.d[j][i]=ndv;
					}
					else
					{
						Concentration += p * fareaw.d[jn][in] * fqg.d[jn][in] * fdmg.d[jn][in];
					}
				}			
				if(fangle.d[jn][in] <= ndva)con = -1;  
			}
		}
		
		Concentration = Concentration / fqg.d[j][i];
		
		if ( scontag.d[j][i] > 0)
			Concentration = cSolG;
		
		fareaw.d[j][i] = Concentration;
		
		if(con == -1 && ccheck == 1)
		{
			fsca.d[j][i] =  -1.0;
			fareaw.d[j][i]=ndv;
		}
	}
}



//Saturated Limited Loading Accumulation

//int dsllArea(char *pfile, char *afile, char *dmfile, double x, double y, int doall,
int dsllArea(char *pfile, char *afile, char *dmfile, double *x, double *y, long nxy, 
		 char *wfile, char *indicatorFile, int contcheck, float cSol, ItkCallback * callback)
{
	int i,j,row,col;
	err = TD_NO_ERROR;
	ccheck=contcheck;
	cSolG = cSol;
	
	/* Define directions */
	/*  d1a=dd1-1; d2a=dd2-1;  */
	d1[1]=0; d1[2]= -1; d1[3]= -1; d1[4]= -1; d1[5]=0; d1[6]=1; d1[7]=1; d1[8]=1;
	d2[1]=1; d2[2]=1; d2[3]=0; d2[4]= -1; d2[5]= -1; d2[6]= -1; d2[7]=0; d2[8]=1;
	
	/* read angles */
	
	//gridread function would return false (0) if there were no errors, but
	//the new grid open function returns true(1) if there were no errors;
	//therefore, this next if/else statement corrects the difference
	if (gridread(pfile,&fangle,&filetype)==0)
		err=TD_NO_ERROR;
	else
	{
		err=TD_FAILED_GRID_OPEN;
		return err;
	}

	
	nx = fangle.head.nx;
	ny = fangle.head.ny;
	dx = fangle.head.dx;
	dy = fangle.head.dy;
	csize = dx; 
	ndva = fangle.nodata;

//	if (fDmGrid.open(dmfile,false) == false)
	if(gridread(dmfile,&fdmg,&filetype) !=0)
		err = TD_FAILED_GRID_OPEN;
	else
		err = TD_NO_ERROR;

	if(err != TD_NO_ERROR)goto ERROR1; 
	
	
	dmndv = fdmg.nodata;
	
	//bndbox is working off of the corner of the cell, not the center
	//so it needs to remove 1/2 of a cell size to map down to the corner
	//instead of the center of the cell...
	for(i=0;i<4;i++)bndbox[i]=fangle.head.bndbox[i];
	
	


	if(gridread(wfile,&fqg,&filetype)==0)
		err=TD_NO_ERROR;
	else
		err=TD_FAILED_GRID_OPEN;

	if(err != TD_NO_ERROR)goto ERROR2;
	

	
	ndv = fqg.nodata;
	fareaw.head=fangle.head;
	fareaw.nodata=ndv;
	if(allocategrid(&fareaw,fareaw.head,fareaw.nodata)!=0) return TD_FAILED_MEMORY_ALLOC;
	
	
	
	//open the contamination grid as a short grid

	if (gridread(indicatorFile,&scontag,&filetype) !=0)
		err = TD_FAILED_GRID_OPEN;
	else
		err = TD_NO_ERROR;

	if (err != TD_NO_ERROR)
		goto ERROR2;
		
	//Calculate angles for interpolation  
	aref[0]= -atan2(dy,dx);
	aref[1]=0.;
	aref[2]= -aref[0];
	aref[3]=(float)(0.5*PI);
	aref[4]=PI-aref[2];
	aref[5]=(float)PI;
	aref[6]=PI+aref[2];
	aref[7]=(float)(1.5*PI);
	aref[8]=2.*PI-aref[2];
	aref[9]=(float)(2.*PI);
	
	//initialize sca grid and header 
	fsca.head=fangle.head;
	fsca.nodata=-1;
	if(allocategrid(&fsca,fsca.head,fsca.nodata) !=0) return TD_FAILED_MEMORY_ALLOC;
	
	
	
	


	//initialize sca grid to 0 
for(i=0; i<ny; i++)
	{
		for(j=0; j<nx; j++)
		{
			if(i!=0 && i!=ny-1 && j!=0 && j!=nx-1 && fangle.d[j][i]> ndva)
			{
				fsca.d[j][i]= 0.0;
			}
		}
	}

		
//	if(doall == 0)   //Only compute area's for designated location  
	if(nxy > 0)  //The logic is now that if outlets are provided use them, otherwise do all
	{
		for(i=0; i<nxy; i++)
		{
			col= (int)floor((x[i]-bndbox[0])/csize);
			row= (int)floor((bndbox[3]-y[i])/csize);
			if(row >0 && row < ny-1 && col > 0 && col < nx-1)
			{
			//call drainage area subroutine for pixel to zero on  
				dpsllArea(row,col);			
			}
		}
	}
	else
	{
		
		long Percent = -1;
		long newPercent = 0;
		long curCell = 0;
		long NumCells = ny * nx;
		//Call drainage area subroutine for each cell   
		//working from the middle out to avoid deep recursions  
		
	
		//"lower block" 
		for(i=ny/2; i<ny-1; i++)
		{	
			
			
			for(j=nx/2; j<nx-1; j++){
				if (callback != NULL){
					newPercent = 100 * (curCell / (double)NumCells);
					if (newPercent > Percent){
						Percent = newPercent;
						callback->Progress(A2BSTR(""),Percent,A2BSTR("Calculating Saturated Limited Loading"));
					}
					curCell++;
				}	
				dpsllArea(i,j);
				if (err != TD_NO_ERROR) 
					goto ERROR3;
			}
			
			for(j=nx/2-1; j>=1; j--){
				if (callback != NULL){
					newPercent = 100 * (curCell / (double)NumCells);
					if (newPercent > Percent){
						Percent = newPercent;
						callback->Progress(A2BSTR(""),Percent,A2BSTR("Calculating Saturated Limited Loading"));
					}
					curCell++;
				}	
				dpsllArea(i,j);
				if (err != TD_NO_ERROR) goto ERROR3;
			}
		}
		//"upper block" 
		for(i=ny/2-1; i>=1; i--)
		{
			
			for(j=nx/2; j<nx-1; j++){
				if (callback != NULL){
					newPercent = 100 * (curCell / (double)NumCells);
					if (newPercent > Percent){
						Percent = newPercent;
						callback->Progress(A2BSTR(""),Percent,A2BSTR("Calculating Saturated Limited Loading"));
					}
					curCell++;
				}	
				dpsllArea(i,j);
				if (err != TD_NO_ERROR) goto ERROR3;
			}
			for(j=nx/2-1; j>=1; j--){
				if (callback != NULL){
					newPercent = 100 * (curCell / (double)NumCells);
					if (newPercent > Percent){
						Percent = newPercent;
						callback->Progress(A2BSTR(""),Percent,A2BSTR("Calculating Saturated Limited Loading"));
					}
					curCell++;
				}	
				dpsllArea(i,j);
				if (err != TD_NO_ERROR) goto ERROR3;
			}
		}
}
	
	if (gridwrite(afile,fareaw,filetype)==0)
		err= TD_NO_ERROR;
	else
	{
		err=TD_FAILED_GRID_SAVE;
		goto ERROR2;
		//if (afile)
		//	AfxMessageBox(LPCTSTR( strcat( "Failed to save file: ", afile) ));
	}


ERROR3:
	free(fareaw.d[0]);free(fareaw.d);
	free(fqg.d[0]); free(fqg.d);
	free(fdmg.d[0]);free(fdmg.d);
	free(fsca.d[0]);free(fsca.d);
	return(err);
	
ERROR2:
	free(fqg.d[0]); free(fqg.d);
	free(scontag.d[0]); free(scontag.d);
	return(err);
	
ERROR1:
	free(fangle.d[0]); free(fangle.d);
	return(err);  
}


void GenerateTempFile(char ** fnameBuffer, char * ext)
{
	if (*fnameBuffer != NULL)
		delete [] (*fnameBuffer);

	const long MaxBuffer = 1024;

	char * filePath = new char [MaxBuffer];
	*fnameBuffer = new char [MaxBuffer];
	CString c;
	GetTempPath(MaxBuffer,filePath);
	GetTempFileName(filePath,"tau",0,*fnameBuffer);

	c = (*fnameBuffer);

	if (ext != NULL) //replace existing extension with parameter extension
		c = c.Left(c.GetLength() - 3) + ext;

	delete [] filePath;

	strcpy(*fnameBuffer,c);
	return;
}

//==================================
//New grid conversion function:Ajay
int gridconvert(char *ipFile,char *opFile,int ipdataformat,int opdataformat,int opfiletype)
{	

	int filetype;
	int err=0;
	//Grid conv 
	sgrid sGridConv,opsgrid;
	igrid iGridConv,opigrid;
	fgrid fGridConv,opfgrid;
	ghead iphead;
		
	//For ASCII file: where the file can be read as Short,int or long(user's choice)
//=======================================================================================
	CString strFileName= CString(ipFile); 
	CString strFileExtension= strFileName.Right(4); 

	if (strFileExtension.CompareNoCase(CString(".asc"))==0)
	{
			if (opdataformat==1)
			{ 
				
				if(ipdataformat == 1)
					{
						if (gridread(ipFile,&sGridConv,&filetype) != 0) return (TD_FAILED_GRID_OPEN);
						iphead=sGridConv.head;
						if (allocategrid(&opsgrid,iphead,sGridConv.nodata)!=0) return (TD_FAILED_MEMORY_ALLOC);
						opsgrid.d=sGridConv.d;
						opsgrid.nodata=sGridConv.nodata;
					}
				
				else if(ipdataformat == 2)
					{	
						if (gridread(ipFile,&iGridConv,&filetype) != 0) return (TD_FAILED_GRID_OPEN);
						iphead=iGridConv.head;
						if (allocategrid(&opsgrid,iphead,MISSINGSHORT)!=0)  return (TD_FAILED_MEMORY_ALLOC);
						for( int i = 0; i < iphead.ny; i++ )
						{	for( int j = 0; j < iphead.nx; j++ )
							{
							 opsgrid.d[j][i]=(iGridConv.d[j][i]==iGridConv.nodata)? MISSINGSHORT :(short)(iGridConv.d[j][i]);
							}				
						}
						opsgrid.nodata=MISSINGSHORT;
					}
					  
				else 
					{	
						if (gridread(ipFile,&fGridConv,&filetype) != 0) return (TD_FAILED_GRID_OPEN);
						iphead=fGridConv.head;
						if(allocategrid(&opsgrid,iphead,MISSINGSHORT)!=0)  return (TD_FAILED_MEMORY_ALLOC);
						for( int i = 0; i < iphead.ny; i++ )
						{	for( int j = 0; j < iphead.nx; j++ )
							{
							opsgrid.d[j][i]=(fGridConv.d[j][i]==fGridConv.nodata)? MISSINGSHORT :(short)(fGridConv.d[j][i]);
							}				
						}
						opsgrid.nodata=MISSINGSHORT;
					}	
					
					if(gridwrite(opFile,opsgrid,opfiletype)!=0) return TD_FAILED_GRID_SAVE;
					free(opsgrid.d[0]); free(opsgrid.d);
					return TD_NO_ERROR;
			} 
			else if (opdataformat==2)
			{
			
				if (ipdataformat==1)
					{	
						if (gridread(ipFile,&sGridConv,&filetype) != 0) return (TD_FAILED_GRID_OPEN);
						iphead=sGridConv.head;
						if (allocategrid(&opigrid,iphead,(int)sGridConv.nodata)!=0)  return (TD_FAILED_MEMORY_ALLOC);
					   for( int i = 0; i < iphead.ny; i++ )
						{	for( int j = 0; j < iphead.nx; j++ )
							{
						   opigrid.d[j][i]=(sGridConv.d[j][i]==sGridConv.nodata) ? MISSINGINT:(int)sGridConv.d[j][i];
						   
							}				
						}	 
						opigrid.nodata=MISSINGINT;
					}
				else if(ipdataformat==2)
					{		
							if (gridread(ipFile,&iGridConv,&filetype) != 0) return (TD_FAILED_GRID_OPEN);
							iphead=iGridConv.head;
							if (allocategrid(&opigrid,iphead,iGridConv.nodata)!=0) return TD_FAILED_MEMORY_ALLOC;
							opigrid.d=iGridConv.d;
							opigrid.nodata=(int)iGridConv.nodata;
					} 
				else
					{ 
						if (gridread(ipFile,&fGridConv,&filetype) != 0) return (TD_FAILED_GRID_OPEN);
						iphead=fGridConv.head;
						if (allocategrid(&opigrid,iphead,MISSINGINT)!=0) return (TD_FAILED_MEMORY_ALLOC);
					for (int i = 0; i < iphead.ny; i++ )
						{	for (int j = 0; j < iphead.nx; j++ )
							{
							opigrid.d[j][i]=(fGridConv.d[j][i] ==fGridConv.nodata) ? MISSINGINT : (int)fGridConv.d[j][i];
							}				
						}
						
					}
				opigrid.nodata=MISSINGINT;
				if(gridwrite(opFile,opigrid,opfiletype)!=0) return TD_FAILED_GRID_SAVE;
				free(opigrid.d[0]); free(opigrid.d);
				return TD_NO_ERROR;
			}
			else if (opdataformat==3)
			{
			
				if (ipdataformat==1)
					{  
						if (gridread(ipFile,&sGridConv,&filetype) != 0) return (TD_FAILED_GRID_OPEN);
						 iphead=sGridConv.head;
						if(allocategrid(&opfgrid,iphead,(float)sGridConv.nodata)!=0) return (TD_FAILED_MEMORY_ALLOC);
					   for( int i = 0; i < iphead.ny; i++ )
						{	for( int j = 0; j < iphead.nx; j++ )
							{
							opfgrid.d[j][i]=(float)sGridConv.d[j][i];
							
							}				
						}	
					   opfgrid.nodata=(float)sGridConv.nodata;
					}
				else if(ipdataformat==2)
					{  
						if (gridread(ipFile,&iGridConv,&filetype) != 0) return (TD_FAILED_GRID_OPEN);
						iphead=iGridConv.head;
						if(allocategrid(&opfgrid,iphead,(float)iGridConv.nodata)!=0) return (TD_FAILED_MEMORY_ALLOC);
						for( int i = 0; i < iphead.ny; i++ )
						{	for( int j = 0; j < iphead.nx; j++ )
							{
							opfgrid.d[j][i]=(float)iGridConv.d[j][i];
							
							}				
						}	 
						opfgrid.nodata=(float)iGridConv.nodata;
					}
				else
					{   
						if (gridread(ipFile,&fGridConv,&filetype) != 0) return (TD_FAILED_GRID_OPEN);
						iphead=fGridConv.head;
						if (allocategrid(&opfgrid,iphead,fGridConv.nodata)!=0) return (TD_FAILED_MEMORY_ALLOC);
						opfgrid.d=fGridConv.d;
						opfgrid.nodata=fGridConv.nodata;
					}
						
				    if(gridwrite(opFile,opfgrid,opfiletype)!=0) return TD_FAILED_GRID_SAVE ;
					free(opfgrid.d[0]); free(opfgrid.d);
					return TD_NO_ERROR;
			}

	}
//For ESRI and Binary where input datatype is read from the grid accordinginly.
	else
	{
		if (opdataformat==1)
		{ 	
			 if(gridread(ipFile,&sGridConv,&filetype)!=0)  return (TD_FAILED_GRID_OPEN);

			 if(gridwrite(opFile,sGridConv,opfiletype)!=0) return (TD_FAILED_GRID_SAVE);
			 free(sGridConv.d[0]); free(sGridConv.d);
			 
		}
		else if (opdataformat==2)
		{	
			if(gridread(ipFile,&iGridConv,&filetype)!=0)  return (TD_FAILED_GRID_OPEN);
			if(gridwrite(opFile,iGridConv,opfiletype)!=0) return (TD_FAILED_GRID_SAVE);
			free(iGridConv.d[0]); free(iGridConv.d);
		}
		else if (opdataformat==3)
		{	 
			if(gridread(ipFile,&fGridConv,&filetype)!=0)  return (TD_FAILED_GRID_OPEN);
			if(gridwrite(opFile,fGridConv,opfiletype)!=0) return (TD_FAILED_GRID_SAVE);
			free(fGridConv.d[0]); free(fGridConv.d);
		}
		else return (TD_FAILED_GRID_OPEN);
	}
	
	
	
	
	return TD_NO_ERROR;
}


//======================================

// Slope D
///////////////////////////////////////////////////////////////////////
int sloped(char *pfile,char* felfile,char* slpdfile, double dn)
{
	sgrid spg;
	fgrid ffelg,fslpdg;
	int  ftype,k,i,j,in,jn,in1,jn1,nx,ny;
	double dx,dy,da=0,d=0;
	float tempvalue=0.0;
	int firsttime=0;
	float fact[9],ed,e,e1,e2;
	int dd1[8],dd2[8];				
//Offset pointers d1 and d2 
	int *d1 = dd1-1;
	int *d2 = dd2-1;

	d1[1]=0; d1[2]= -1; d1[3]= -1; d1[4]= -1; d1[5]=0; d1[6]=1; d1[7]=1; d1[8]=1;
	d2[1]=1; d2[2]=1; d2[3]=0; d2[4]= -1; d2[5]= -1; d2[6]= -1; d2[7]=0; d2[8]=1;
	d=dn/1000; 
	if (gridread(pfile,&spg,&ftype)!=0) return TD_FAILED_GRID_OPEN;
	if(gridread(felfile,&ffelg,&ftype)!=0)return TD_FAILED_GRID_OPEN;
	
	fslpdg.head=ffelg.head;
	fslpdg.nodata=ffelg.nodata;
	dx=fslpdg.head.dx;
	dy=fslpdg.head.dy;
	nx=fslpdg.head.nx;
	ny=fslpdg.head.ny;
	
	if(allocategrid(&fslpdg,fslpdg.head,fslpdg.nodata)!=0) return TD_FAILED_MEMORY_ALLOC;
	for(k=1; k<= 8; k++)
		fact[k]= (float) (1./sqrt(d1[k]*dy*d1[k]*dy+d2[k]*d2[k]*dx*dx));

	for(i=0;i<ny;i++)
	{
		for(j=0;j<nx;j++)
		{
			if(spg.d[j][i] > 0)
				{	do
						{
						if(firsttime==0)
						  {
							  jn=j+d2[spg.d[j][i]];
							  in=i+d1[spg.d[j][i]];
							  jn1=jn;
							  in1=in;
							  firsttime=1;
							  da=da+fact[spg.d[j][i]];
							  
						  }
						  else
						  { 
									if (spg.d[jn1][in1]>0  && spg.d[jn][in]>0)
									{
										jn1=jn+d2[spg.d[jn][in]];
										in1=in+d1[spg.d[jn][in]];
										jn=jn1;in=in1;
										if(spg.d[jn1][in1]<0)
										{
											tempvalue=ffelg.nodata;
											break;
										}
										da=da+fact[spg.d[jn1][in1]];
									}
									else
									{
										tempvalue= ffelg.nodata;
										break;
									}
							}

						} while(da<=d);
				}
				else
				{
					tempvalue= ffelg.nodata;
				}

			if(tempvalue != ffelg.nodata && da>0.0)
			{
				e2 = ffelg.d[j][i] ;
				e1 = ffelg.d[jn1][in1];
				e=e2-(d/da)*(e2-e1);
				ed=e2-e;
				tempvalue =  ed/(d*1000);
			}
			fslpdg.d[j][i]=tempvalue;
			tempvalue=0.;
			firsttime=0;
			da=0.0;
		}
	}
	if(gridwrite(slpdfile,fslpdg,ftype)!=0) return TD_FAILED_GRID_SAVE;
	return 0;
}
///////////////////////////////////////////////////////////////////////////////////////////////

//***************************************************************************************
//  Kiran added this 1/22/04
int outletstosrc(char *pfile, char *srcfile,char *ad8file, long *xnode, long *ynode, int nxy, int max_dist,long *ismoved,  long *dist_moved)
{
	int itresh, **area,nyn,nxn;
	int err = TD_NO_ERROR;
	short **dir;
	int usetrace =1;
	sgrid sdirgrid;
	int ftype;
	/* define directions */
	d1[1]=0; d1[2]= -1; d1[3]= -1; d1[4]= -1; d1[5]=0; d1[6]=1; d1[7]=1; d1[8]=1;
	d2[1]=1; d2[2]=1; d2[3]=0; d2[4]= -1; d2[5]= -1; d2[6]= -1; d2[7]=0; d2[8]=1;

//	FILE *fp;
	
	itresh=1;  // Thresholding to 1 done in source

	err=gridread(pfile,&sdirgrid,&ftype);
	if(err != TD_NO_ERROR){
		return err;
	}
		
		
	int nx =sdirgrid.head.nx;// sDirHeader.getNumberCols();
	int ny = sdirgrid.head.ny;
	double dx = sdirgrid.head.dx;
	double dy = sdirgrid.head.dy;
	double csize = dx;
	short ndv = sdirgrid.nodata;
	double bndbox[4];
	for(int i=0;i<4;i++)bndbox[i]=sdirgrid.head.bndbox[i];

	
	//copy data into dir array for use in fortran code
	if((dir = (short**) matalloc(nx,ny, RPSHRDTYPE)) == NULL)
	{
		err=TD_FAILED_MEMORY_ALLOC;
		//  printf("Could not allocate memory for src\n");
		//goto ERROR1;
		return err;
	}  


	for (int curRow = 0; curRow < ny; curRow++){
		for (int curCol = 0; curCol < nx; curCol++)
			dir[curCol][curRow] = sdirgrid.d[curCol][curRow];
	}
	
	
	/*    read srcfile   */
	igrid iareagrid;
			
	if (gridread(srcfile,&iareagrid,&ftype)==0)// lAreaGrid.open(srcfile) )
		err= TD_NO_ERROR;
	else
		err=TD_FAILED_GRID_OPEN;
	
	if (err != TD_NO_ERROR)
		return err;
	

	nxn =iareagrid.head.nx; 
	nyn = iareagrid.head.ny;
	
	//copy area data into area array for use in fortran code
	
	if((area = (int**) matalloc(nxn,nyn, RPINTDTYPE)) == NULL)
	{
		err=TD_FAILED_MEMORY_ALLOC;
		//  printf("Could not allocate memory for src\n");
		return err;
	}  
	for (curRow = 0; curRow < ny; curRow++){
		for (int curCol = 0; curCol < nx; curCol++)
			area[curCol][curRow] =iareagrid.d[curCol][curRow];
	}
	
	
	if(nxn != nx || nyn != ny)
	{
		//	   printf("Incompatible grid sizes\n");
		//	   exit(0);
		return 1;
	}
	
	
	//  check for source values >= threshold else fortran crashes
	int   n=0,j;
	for(i=0; i<ny; i++){
		for(j=0; j<nx; j++)
			if(area[j][i] >=itresh)n=n+1;
	}
	if(n <= 0)
	{
		err=TD_CHANNEL_NETWORK_MISMATCH;
	}
	else
	{
		short *inodes;
		short *jnodes;

//  If there are outlets set up inodes, jnodes arrays
		if(nxy > 0){
			inodes = new short [nxy];
			jnodes = new short [nxy];
		}else
		{	
			inodes = new short [1];
			jnodes = new short [1];
		}

		for(int inode = 0; inode<nxy; inode++)
		{  
			jnodes[inode]= (short)floor((xnode[inode]-bndbox[0])/csize);  
			inodes[inode]= (short)floor((bndbox[3]-ynode[inode])/csize);
			int loopcount=0; //Kiran declared before the loop
//   Trace to raster if necessary but only for nodes that are inside the domain
			if(usetrace == 1  && inodes[inode]>0 && inodes[inode] < ny-1 && 
				jnodes[inode] > 0 && jnodes[inode] < nx-1) 
			{
				if(area[jnodes[inode]][inodes[inode]] < itresh)  // Not on grid
				{
					//   Next downslope 
					short dirn=dir[jnodes[inode]][inodes[inode]];
					int nexti=inodes[inode]+d1[dirn];
					int nextj=jnodes[inode]+d2[dirn];
					loopcount = 0;
					while(nexti>0 && nexti < ny-1 && nextj > 0 && nextj < nx-1 && area[jnodes[inode]][inodes[inode]] < itresh)
					{
						inodes[inode]=nexti;
						jnodes[inode]=nextj;
						dirn = dir[jnodes[inode]][inodes[inode]];
						if(dirn < 1 || dirn > 8)break;   // Here have gone out of grid so terminate trace downwards
						nexti=nexti+d1[dirn];
						nextj=nextj+d2[dirn];
						loopcount=loopcount+1;
						if(loopcount > nx && loopcount > ny && loopcount > max_dist)break;   // Here possible infinite loop so terminate trace downwards
					}
				}
			}
			if (loopcount < max_dist){
				xnode[inode] = (long)(jnodes[inode]*csize + bndbox[0]+csize/2.);
				ynode[inode] = (long)(bndbox[3]-(inodes[inode]*csize)-csize/2.);
				dist_moved[inode]=loopcount;//Kiran..to keep track of how many cells each outlet has moved
				ismoved[inode] = 1;
			}
			else{
				dist_moved[inode]=max_dist;
				ismoved[inode] = 0;
				
			}
			
		}
		free(dir[0]); free(dir);
		free(sdirgrid.d[0]); free(sdirgrid.d);
		free(area[0]); free(area);
		free(iareagrid.d[0]); free(iareagrid.d);
	}
		return err;

}
//XXXXXXXXXXXXXXXXXXXXXX converted FORTRAN tardemflib.f ..Kiran XXXXXXXXXXXXXXXXXXXXXXXXX
//These functions are required by the Netex function
//strt(), isnode_c() and isnode2()
bool strt(int i,int j,int **area,short **dir, long nx, long ny, int igx, int igy,int itresh){

int ind,jnd,in,jn;
//strt = true;
 /* define directions */
 d1[1]=0; d1[2]= -1; d1[3]= -1; d1[4]= -1; d1[5]=0; d1[6]=1; d1[7]=1; d1[8]=1;
 d2[1]=1; d2[2]=1; d2[3]=0; d2[4]= -1; d2[5]= -1; d2[6]= -1; d2[7]=0; d2[8]=1;

if (area[j][i] < itresh){
		if(area[j][i] <= 0){
			dir[j][i] = 0; // ZERO DIRECTIONS OUTSIDE AREA
			}
		return false;
}
else{ //check upstream pixels
	int k;
	for (k=1; k<= 8 ;k++)
	{
		in=i+d1[k];    // NEIGHBOUR PIXEL
        jn=j+d2[k];       
        if(dir[jn][in] > 0)
		{
		  ind=in+d1[dir[jn][in]] ;  // PIXEL DOWNSTREAM FROM NEIGHBOUR
		  jnd = jn + d2[dir[jn][in]];
		  if(ind == i && jnd == j)
		  { //NEIGHBOUR DRAINS INTO (I,J)
			  if(area[jn][in] >= itresh) return false;
		  }
		}
	}
	in=i+d1[dir[j][i]];
	jn=j+d2[dir[j][i]];
	if(area[jn][in] < itresh)return false;
}
return true;
}
bool isnode_c(int mnext, int mag, int i , int j , short *inodes, short *jnodes, int nnodes){
	int k;
	for (k= 0; k < nnodes;k++){
		if( inodes[k] == i && jnodes[k] == j ){
			if( mnext != mag) return false;
			return true;
		}
	}
	return false; 
}
//This function is required by the Netex function
bool isnode2(int i , int j , short *inodes, short *jnodes, int nnodes,int &nodeno){
	int k;
	for (k= 0; k < nnodes ;k++){
		if( inodes[k]  == i && jnodes[k] == j ){
			nodeno=k;//for return to use in indexing
			return true;
		}
	}
	return false; 
}
//XXXXXXXXXXXXXXXXXXXXXXXX converted FORTRAN tardemflib.f XXXXXXXXXXXXXXXXXXXXXXXXX

int Netex_c(short **dir, int **area, char *treefile, char *coordfile, char *ordfile, int nx, int ny, int itresh,
			 int& icr, int& icend, double dx, double dy, double bndbox, double csize, int iftype, 
			 short *inodes, short *jnodes, int nnodes, long *idnodes){

/*	NETEX(dir[0],area[0],treefile,strlen(treefile),coordfile,strlen(coordfile),
			ordfile,strlen(ordfile),&nx,&ny,&itresh,&icr,&icend,&fdx,&fdy,bndbox,&csize,
			&filetype,&err,inodes,jnodes,&nxy,idnodes);  */
	int ipoint[7],iordup[7];
	
	int ishrdt=1, iintdt=2,ifltdt=3;//these are declared as paramters in FORTRAN
	

	//Read input
	int igy=ny,i,j;
	int igx=nx;
	int err=0;
	int n=0;
	int nmax=0;
	bool k=0;
	/* Count all start pixels */
	for( i= 0; i< ny;i++)
	{
		for(j= 0; j< nx;j++)
		{
			if(strt(i,j,area,dir,nx,ny,igx,igy,itresh))
				n=n+1;
		}
	}
	nmax=n;
	int mnl=2*n+1+nnodes;
//  Allocate memory
	int *nextl = new int[mnl+1];
	int *prevL1=new int[mnl+1];
	int	*prevL2=new int[mnl+1];
	int *ist=new int[nmax+1];
	int *jst=new int[nmax+1];
	int *iord=new int[mnl+1];
	int	*istart=new int[mnl+1];
	int *jstart=new int[mnl+1];
	int *iend=new int[mnl+1];
	int *jend=new int[mnl+1];
	int *mag=new int[mnl+1];
	n=0;
	for( i= 0; i< ny;i++)
	{
		for(j= 0; j< nx;j++)
		{
			if(strt(i,j,area,dir,nx,ny,igx,igy,itresh))
			{    
				n=n+1;
				ist[n]=i;
				jst[n]=j;
			}
		}
	}
	if ( n > nmax)return(2);
/*Zero area array*/
	for(i= 0; i< ny;i++){
		for(j= 0; j< nx;j++){
			area[j][i]=0;
		}
	}

	/*Trace streams downwards adding 1 to magnitude of each pixel.  
	(Magnitude stored in area array)*/
	int is,inext,jnext;
	for(is=1; is<=n; is++){           //This was DO 3
		i =ist[is];
		j=jst[is];
		while(dir[j][i] > 0){
			area[j][i]=area[j][i]+1;
			inext=i+d1[dir[j][i]];
			jnext=j+d2[dir[j][i]];
			i=inext;
			j=jnext;
		}

	}
//Identify links by different magnitude
	int ilink=1;
	for(is = 1;is<=n;is++){     
		istart[ilink]=ist[is];
		jstart[ilink]=jst[is];
		//initialize pointers
		prevL1[ilink]=0;
		prevL2[ilink]=0;
		i=ist[is];
		j=jst[is];
		mag[ilink]=area[j][i];
		iord[ilink]= 1;
		int mnext;
START6: inext=i+d1[abs(dir[j][i])];
		jnext=j+d2[abs(dir[j][i])];
		//check for the end
		int intemp,jntemp;
		if(dir[jnext][inext] == 0){ //end of path
			iend[ilink]=i;
			jend[ilink]=j;
			nextl[ilink]=-1;
			if(is < n)
				ilink=ilink+1; //if not last on list increment link counter
		}
		else{
			mnext=area[jnext][inext];
			i=inext;
			j=jnext;
			iend[ilink]=i;
			jend[ilink]=j;
			//modes allow insertion of nodes DGT 7/17/02 as commented in the original FORTRAN code
			if(isnode_c(mnext,mag[ilink],i,j,inodes,jnodes,nnodes)){
				//check here that this is not the end of a path because then it will be a node anyway
				intemp=i+d1[abs(dir[j][i])];
				jntemp=j+d2[abs(dir[j][i])];
				if(dir[jntemp][intemp] != 0){
					ilink=ilink+1;
					istart[ilink]=i;
					jstart[ilink]=j;
					prevL1[ilink]=ilink-1;
					prevL2[ilink]=0;
					nextl[ilink-1]=ilink;
					mag[ilink]=mag[ilink-1];
					iord[ilink]=iord[ilink-1];
					iend[ilink]=i;
					jend[ilink]=j;
				}
			}
			//end mods to allow insertion of nodes DGT
			if(mnext == mag[ilink])goto START6;
			//continue here for new link
			//check if junction already reached (flagged by negative direction)
			if(dir[j][i] < 0){
				//check if all links converging are have been done by summing magnitude
				int msum=0;
				int iordm=0;
				int iconv=0, il;
				for (il=1;il<=ilink;il++){    //  This was DO 7
					if(iend[il] == i && jend[il] == j){
						iconv=iconv+1;//counter of number of links that converge
						ipoint[iconv]=il;
						iordup[iconv]=iord[il];
						msum=msum+mag[il];
					}
				}
				if(msum == mnext){ //yes all links have been processed
					//sort irdup, ipoint into descending stream order
					for(int ic=1;ic <= iconv-1;ic++){     // These were DO 11
						for(int iic=ic+1;iic <=iconv;iic++){
							if(iordup[iic] > iordup[ic]){
								int itemp=iord[iic];
								iordup[iic]=iordup[ic];
								iordup[ic]=itemp;
								itemp=ipoint[iic];
								ipoint[iic]=ipoint[ic];
								ipoint[ic]=itemp;
							}
						}
					}
					for(ic=1;ic <= iconv-1;ic++){   //DO 17
						ilink=ilink+1;
						istart[ilink]=i;
						jstart[ilink]=j;
						prevL1[ilink]=ipoint[ic];
						prevL2[ilink]=ipoint[ic+1];
						nextl[ipoint[ic]]=ilink;
						nextl[ipoint[ic+1]]=ilink;
						mag[ilink]=mag[prevL1[ilink]]+mag[prevL2[ilink]];
//						iord[ilink]=max(iordup[0],iordup[1]+1);
						iord[ilink]= (iordup[1] >(iordup[2]+1))? iordup[1]:(iordup[2]+1);
						ipoint[ic+1]=ilink;
						iend[ilink]=i;
						jend[ilink]=j;
					}
					goto START6;
				}
				else{
					ilink=ilink+1;
				}
				
			}
			else{
				dir[j][i]=-dir[j][i];
				ilink=ilink+1;
			}
		}//end of path closure
		
	}
	FILE *fcoord,*ftree;
	fcoord=fopen(coordfile,"w");
	ftree=fopen(treefile,"w");
	//reinitialize area.  For output it will contain order
	for(i= 0; i<= ny-1;i++){
		for(j= 0; j<= nx-1;j++){
			area[j][i]=0;
		}
	}
	int icord=0,ics;
	int inodeid=0,nodeno;
	//write root link first
	for(int il=ilink;il<=ilink;il++){   // DO 10
		i=istart[il];
		j=jstart[il];
		ics=icord;
		while ((i != iend[il]) || (j != jend[il]) ){
			fprintf(fcoord,"%d %d\n",i+1,j+1);	fflush(fcoord);
//			area[j][i]=max(iord[il-1],area[j][i]);
			area[j][i]=(iord[il]>area[j][i])? iord[il]: area[j][i];
			icend=icord;
			icord=icord+1;
			inext=i+d1[abs(dir[j][i])];
			jnext=j+d2[abs(dir[j][i])];
			i=inext;
			j=jnext;
		}
		fprintf(fcoord,"%d %d\n",i+1,j+1);	fflush(fcoord);
//		area[j][i]=max(iord[il-1],area[j][i]);
		area[j][i]=(iord[il]>area[j][i])? iord[il]: area[j][i];
		icend=icord;
		icord=icord+1;

		inodeid=0;		//this will be the first one so will be the most downstream
		if(isnode2(iend[il],jend[il],inodes,jnodes,nnodes,nodeno)){
			if(idnodes[nodeno] >= 0){
				inodeid=idnodes[nodeno];
				idnodes[nodeno]=-1;//this logic to pick only the first one if there are multiple at a junction
			}
		}
		fprintf(ftree,"\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\n",0,ics,icend,-1,prevL1[il],prevL2[il],iord[il],inodeid);
		fflush(ftree);
	}
	icr=icord;
	//write reminder links
	for(il=1; il<=ilink-1;il++){
		i=istart[il];
		j=jstart[il];
		ics=icord;
		
		while(i != iend[il] || j != jend[il])
		{
			fprintf(fcoord,"%d %d\n",i+1,j+1);	fflush(fcoord);
//			area[j][i]=max(iord[il],area[j][i]);
			area[j][i]=(iord[il]>area[j][i])? iord[il] : area[j][i];
			icend=icord;
			icord=icord+1;
			inext=i+d1[abs(dir[j][i])];
			jnext=j+d2[abs(dir[j][i])];
			i=inext;
			j=jnext;
		}
		fprintf(fcoord,"%d %d\n",i+1,j+1);	fflush(fcoord);
		area[j][i]=(iord[il]>area[j][i])? iord[il] : area[j][i];
		icend=icord;
		icord=icord+1;
		if(nextl[il] == ilink)nextl[il]=0;
		inodeid=-1;
		if(nextl[il] < 0)inodeid=0;//this is a most downstream link so flagged by inodeid=0
		if(isnode2(iend[il],jend[il],inodes,jnodes,nnodes,nodeno)){
			if(idnodes[nodeno] >= 0){
				inodeid=idnodes[nodeno];
				idnodes[nodeno]=-1;//this logic to pick only the first one if there are multiple at a junction
			}
		}
		fprintf(ftree,"\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\n",il,ics,icend,nextl[il],prevL1[il],prevL2[il],iord[il],inodeid);
		fflush(ftree);
	}
	fclose(ftree);
	fclose(fcoord);
	return(err);
}
//XXXXXXXXXXXXXXXXXXXXXXXX converted FORTRAN tardemflib.f XXXXXXXXXXXXXXXXXXXXXXXXX

void Netprop_c(short **dir, int **area, float **elev, char *coordfile, int& icr, 
			   int& icmax,double& dx, double& dy,int& nx, int& ny, 
			   double bndbox, double& csize,int& err){	

/*NETPROP(dir[0],area[0],elev[0],coordfile,strlen(coordfile),&icr,&icend,&fdx,
			&fdy,&nx,&ny,bndbox,&csize,&err); 
			
*/	/*		
	int MNL = 5000;//This is hard coded in FORTRAN for memory allocation for tha arrays and may be changed to dynamic allocation
	char *pointfile, *areafile;
	int *nextl = new int[MNL];
	int *prevL1=new int[MNL];
	int	*prevL2=new int[MNL];
	int ipoint[7],iordup[7],d1[8],d2[8];
	int *ist=new int[(MNL+1)/2];
	int *jst=new int[(MNL+1)/2];
	int *iord=new int[MNL];
	int	*istart=new int[MNL];
	int *jstart=new int[MNL];
	int *iend=new int[MNL];
	int *jend=new int[MNL];
	int *mag=new int[MNL];
	*/

}


/*  
  Distance to Stream.  
  
  David G Tarboton
  Utah State University   
  
  3/26/07
   */
  
//	int iy,jy,ccheck;  // Could make these local variables. (Not necessary to declare here because already a global in tardemlib.cpp)
//  float aref[10];  // Not necessary to declare here because already a global in tardemlib.cpp
//  float dist[9];
// int dd1[8],dd2[8];	// Not necessary to declare here because already a global in tardemlib.cpp			
/* Offset pointers d1 and d2 */
/*int *d1 = dd1-1;
int *d2 = dd2-1;  */
//int nx,ny;  // Not necessary to declare here because already a global in tardemlib.cpp
float idv;
int gweight;


/*  Global grids  */
//fgrid ang;
fgrid wgrd;
fgrid dts;
sgrid srcdts;


/* function to compute average distance to stream recursively.  */
void adistrec(int i, int j)
{
	int in,jn,k,con=0;
	float p,sumn,sump;
	if(dts.d[j][i] == idv)  // idv (initial data value) is used to flag that the cell has not been done
	{
		/*  Make sure the cell is in the domain   */
		if(i!=0 && i!=ny-1 && j!=0 && j!=nx-1 && fangle.d[j][i] > fangle.nodata)
		{
			/*    look in all directions  */
			sumn=0.;
			sump=0.;
			float wf=1.;
			if(gweight == 1)
			{
				wf=wgrd.d[j][i];
			}
			for(k=1; k<=8; k++)
			{
				in = i + d1[k];
				jn = j + d2[k];
				/* Find the proportion draining to a downslope neighboring cell  */
				p=prop(fangle.d[j][i],k);
				if(p>1e-6)  /*  Recurse to downslope neighbor only for grid cells 
					with proportion greater than 0.  Numerically the threshold 1e-6 guards against
					numerical loops due to angle rounding*/	
				{
					adistrec(in,jn);
					if(dts.d[jn][in] <= dts.nodata)
					{
						con = -1;
					}
					else
					{
						sumn=sumn+p*(dist[k]*wf+dts.d[jn][in]);
						sump=sump+p;
					}
				}
				if(fangle.d[jn][in] <= fangle.nodata) con= -1;
            }
			if(sump > 0.)
			{
				dts.d[j][i]=sumn/sump;
			}
			else
				dts.d[j][i]=dts.nodata;
			if (con == -1 && ccheck == 1)
			{
				dts.d[j][i]=dts.nodata;
			}
		}
		else
		dts.d[j][i]=dts.nodata;
	}
} 

/* function to compute longest distance to stream recursively.  */
void ldistrec(int i, int j)
{
	int in,jn,k,con=0;
	float p,sumn,sump;
	if(dts.d[j][i] == idv)  // idv (initial data value) is used to flag that the cell has not been done
	{
		/*  Make sure the cell is in the domain   */
		if(i!=0 && i!=ny-1 && j!=0 && j!=nx-1 && fangle.d[j][i] > fangle.nodata)
		{
			/*    look in all directions  */
			sumn=0.;
			sump=0.;
			float wf=1.;
			if(gweight == 1)
			{
				wf=wgrd.d[j][i];
			}
			for(k=1; k<=8; k++)
			{
				in = i + d1[k];
				jn = j + d2[k];
				/* Find the proportion draining to a downslope neighboring cell  */
				p=prop(fangle.d[j][i],k);
				if(p>1e-6)  /*  Recurse to downslope neighbor only for grid cells 
					with proportion greater than 0.  Numerically the threshold 1e-6 guards against
					numerical loops due to angle rounding*/	
				{
					ldistrec(in,jn);
					if(dts.d[jn][in] <= dts.nodata)
					{
						con = -1;
					}
					else
					{
						float ed=wf*dist[k]+dts.d[jn][in];
						if(ed > sumn)sumn=ed;  //  Keep track of longest distance
						sump=sump+p;
					}
				}
				if(fangle.d[jn][in] <= fangle.nodata) con= -1;
            }
			if(sump > 0)
			{
				dts.d[j][i]=sumn;
			}
			else
				dts.d[j][i]=dts.nodata;
			if (con == -1 && ccheck == 1)
			{
				dts.d[j][i]=dts.nodata;
			}
		}
		else
		dts.d[j][i]=dts.nodata;
	}
} 

/* function to compute shortest distance to stream recursively.  */
void sdistrec(int i, int j)
{
	int in,jn,k,con=0;
	float p,sumn;
	if(dts.d[j][i] == idv)  // idv (initial data value) is used to flag that the cell has not been done
	{
		/*  Make sure the cell is in the domain   */
		if(i!=0 && i!=ny-1 && j!=0 && j!=nx-1 && fangle.d[j][i] > fangle.nodata)
		{
			/*    look in all directions  */
			float wf=1.;
			if(gweight == 1)
			{
				wf=wgrd.d[j][i];
			}
			int ik=0;  //  This records the number of positive p values
			for(k=1; k<=8; k++)
			{
				in = i + d1[k];
				jn = j + d2[k];
				/* Find the proportion draining to a downslope neighboring cell  */
				p=prop(fangle.d[j][i],k);
				if(p>1e-6)  /*  Recurse to downslope neighbor only for grid cells 
					with proportion greater than 0.  Numerically the threshold 1e-6 guards against
					numerical loops due to angle rounding*/	
				{
					sdistrec(in,jn);
					if(dts.d[jn][in] <= dts.nodata)
					{
						con = -1;
					}
					else
					{
						ik=ik+1;
						float ed=wf*dist[k]+dts.d[jn][in];
						if(ik==1)  //  first pass through
						{
							sumn=ed;
						}
						else
						{
							if(ed < sumn)sumn=ed;  //  Keep track of shortest distance
						}
					}
				}
				if(fangle.d[jn][in] <= fangle.nodata) con= -1;
            }
			if(ik > 0)
			{
				dts.d[j][i]=sumn;
			}
			else
				dts.d[j][i]=dts.nodata;
			if (con == -1 && ccheck == 1)
			{
				dts.d[j][i]=dts.nodata;
			}
		}
		else
		dts.d[j][i]=dts.nodata;
	}
} 

int disttostreamgrd(char *angfile, char *wfile, char *srcfile, char *dtsfile, int method, int useweight, int concheck)
{
   int err,i,j,filetype;
 
   /* read angles */
	err=gridread(angfile,&fangle,&filetype);
    if(err != 0)goto ERROR1; 
	nx=fangle.head.nx;
	ny=fangle.head.ny;

 	/* read Stream Raster Grid */
	err=gridread(srcfile,&srcdts,&filetype);
    if(err != 0)goto ERROR2;  
	//Check that grid dimensions are the same
	if(nx != srcdts.head.nx || ny != srcdts.head.ny)
	{
		printf("Grid dimensions are inconsistent\n");
		printf("%d %d %s\n",nx,ny,angfile);
		printf("%d %d %s\n",srcdts.head.nx,srcdts.head.ny,srcfile);
		goto ERROR2;
	}
  /* read weights if necessary */
	if(useweight == 1)
	{
	err=gridread(wfile,&wgrd,&filetype);
    if(err != 0)goto ERROR3;  
	//Check that grid dimensions are the same
	if(nx != wgrd.head.nx || ny != wgrd.head.ny)
	{
		printf("Grid dimensions are inconsistent\n");
		printf("%d %d %s\n",nx,ny,angfile);
		printf("%d %d %s\n",wgrd.head.nx,wgrd.head.ny,wfile);
		goto ERROR3;
	}
	}
	gweight=useweight;   //  Assign to global for recursion
	ccheck=concheck;   // Another global for recursion

/*  Allocate memory  */
	err=allocategrid(&dts,fangle.head,-2.);
    if(err != 0)goto ERROR3;  

/* Define directions */

d1[1]=0; d1[2]= -1; d1[3]= -1; d1[4]= -1; d1[5]=0; d1[6]=1; d1[7]=1; d1[8]=1;
d2[1]=1; d2[2]=1; d2[3]=0; d2[4]= -1; d2[5]= -1; d2[6]= -1; d2[7]=0; d2[8]=1;

/* Calculate angles for interpolation */                                      
aref[0]= -atan2(fangle.head.dy,fangle.head.dx);
aref[1]=0.;
aref[2]= -aref[0];
aref[3]=0.5*PI;
aref[4]=PI-aref[2];
aref[5]=PI;
aref[6]=PI+aref[2];
aref[7]=1.5*PI;
aref[8]=2.*PI-aref[2];
aref[9]=2.*PI;

//  Calculate distances in each direction
int k;
for(k=1; k<=8; k++)
{
	dist[k]=sqrt(pow(fangle.head.dx*d2[k],2)+pow(fangle.head.dy*d1[k],2));
}
idv = -1;
/*  Initialize distance to idv, an initial data value that is not the same as the no data value
    to indicate that this has not been done yet.  On same parse through initialize dts values on streams  */
    for(i=0; i<ny; i++)
      for(j=0; j<nx; j++)
	  {
         dts.d[j][i]= idv;  // Use idv to indicate that this has not been done yet.
		 if(srcdts.d[j][i] >=1)
		 {
			 dts.d[j][i]=0.;  // Initialize dts values on stream to 0.
		 }
	  }

switch (method)
{
case 0:
/* "lower block" */
for(i=ny/2; i<ny-1; i++)
   {for(j=nx/2; j<nx-1; j++)
      adistrec(i,j);
    for(j=nx/2-1; j>=1; j--)
      adistrec(i,j);
   }
 /* "upper block" */
for(i=ny/2-1; i>=1; i--)
   {for(j=nx/2; j<nx-1; j++)
      adistrec(i,j);
    for(j=nx/2-1; j>=1; j--)
      adistrec(i,j);
   }
break;
case 1:
	/* "lower block" */
for(i=ny/2; i<ny-1; i++)
   {for(j=nx/2; j<nx-1; j++)
      ldistrec(i,j);
    for(j=nx/2-1; j>=1; j--)
      ldistrec(i,j);
   }
 /* "upper block" */
for(i=ny/2-1; i>=1; i--)
   {for(j=nx/2; j<nx-1; j++)
      ldistrec(i,j);
    for(j=nx/2-1; j>=1; j--)
      ldistrec(i,j);
   }
break;
case 2:
	/* "lower block" */
for(i=ny/2; i<ny-1; i++)
   {for(j=nx/2; j<nx-1; j++)
      sdistrec(i,j);
    for(j=nx/2-1; j>=1; j--)
      sdistrec(i,j);
   }
 /* "upper block" */
for(i=ny/2-1; i>=1; i--)
   {for(j=nx/2; j<nx-1; j++)
      sdistrec(i,j);
    for(j=nx/2-1; j>=1; j--)
      sdistrec(i,j);
   }
break;
}
//  Parse through the grid again setting unchanged initial data values to no data
    for(i=0; i<ny; i++)
      for(j=0; j<nx; j++)
	  {
         if(dts.d[j][i]== idv)dts.d[j][i]=dts.nodata;  
	  }


	/*  Write out results  */ 
	err= gridwrite(dtsfile,dts,filetype);
 
/*    err = gridwrite(rtrfile,(void **)rtr,RPFLTDTYPE,nx,ny,dx,dy,
             bndbox,csize,-1.,filetype);  */
	free(dts.d[0]);
    free(dts.d); 
ERROR3:
    free(srcdts.d[0]);
    free(srcdts.d);
ERROR2:
    free(fangle.d[0]);
    free(fangle.d);
ERROR1:	
	return(err);
} 


//  End of block of code associated with disttostream 



/*
  Rise to Ridge.  
  
  David G Tarboton
  Utah State University   
  
  3/23/07
*/
  

/*  Globals   */
// Not necessary to declare many of these variables here because they have already been declared
// globals in tardemlib.cpp

//int iy,jy,ccheck;
//float aref[10];
//int dd1[8],dd2[8];				

/* Offset pointers d1 and d2 */
//int *d1 = dd1-1;
//int *d2 = dd2-1;
//int nx,ny;
//float idv;
float gthresh;

//float prop(float,int);
void artrrec(int,int);
void mxrtrrec(int,int);
void mnrtrrec(int,int);

/*  Global grids  */
//fgrid ang;  //used fangle instead
//fgrid elev;  //used felevg instead
fgrid rtr;

/* Function to compute average rise to ridge recursively  */
void artrrec(int i, int j)
{
	int in,jn,k,kback,con=0;
	float p,ed,sumn,sumd;
	if(rtr.d[j][i] == idv)  // idv (initial data value) is used to flag that the cell has not been done
	{
		/*  Make sure the cell is in the domain   */
		if(i!=0 && i!=ny-1 && j!=0 && j!=nx-1 && fangle.d[j][i] > fangle.nodata)
		{
			rtr.d[j][i]=0;  //Initialize to 0
			/*    look in all directions  */
			sumn=0.;
			sumd=0.;
//			if(i==164 && j==215)
//				i=i;
			for(k=1; k<=8; k++)
			{
				in = i + d1[k];
				jn = j + d2[k];
				/* Find the proportion draining from each neighboring cell  */
				kback=k+4;
				if(kback>8)kback=kback-8;
				p=prop(fangle.d[jn][in],kback);
				/*  Recurse only for grid cells 
					with proportion greater than 1e-6 to guards against
					numerical loops due to angle rounding  DGT 9/12/08*/	
				if(p>1e-6 && p >= gthresh)  /*  Recurse to upslope neighbor only for grid cells 
					with proportion greater than or equal to the threshold and greater than 0 */	
				{
					artrrec(in,jn);
					if(rtr.d[jn][in] <= felevg.nodata)
					{
						con = -1;
					}
					else
					{
						ed=felevg.d[jn][in]-felevg.d[j][i];
						if (ed < 0.)ed=0.; // Do not allow negative elevation differences (that arise due to angle rounding)
						sumn=sumn+p*(ed+rtr.d[jn][in]);
						sumd=sumd+p;
					}
				}
				if(fangle.d[jn][in] <= fangle.nodata) con= -1;
            }
			if(sumd>0.)
				rtr.d[j][i]=sumn/sumd;
			if (con == -1 && ccheck == 1)
			{
				rtr.d[j][i]=felevg.nodata;
			}
		}
	}
} 

/* function to compute maximum rise to ridge recursively.  */
void mxrtrrec(int i, int j)
{
	int in,jn,k,kback,con=0;
	float p,ed;
	if(rtr.d[j][i] == idv)  // idv (initial data value) is used to flag that the cell has not been done
	{
		/*  Make sure the cell is in the domain   */
		if(i!=0 && i!=ny-1 && j!=0 && j!=nx-1 && fangle.d[j][i] > fangle.nodata)
		{
			rtr.d[j][i]=0;  //Initialize to 0
			/*    look in all directions  */
			for(k=1; k<=8; k++)
			{
				in = i + d1[k];
				jn = j + d2[k];
				/* Find the proportion draining from each neighboring cell  */
				kback=k+4;
				if(kback>8)kback=kback-8;
				p=prop(fangle.d[jn][in],kback);
				 /*  Recurse only for grid cells 
					with proportion greater than 1e-6 to guards against
					numerical loops due to angle rounding  DGT 9/12/08*/	
				if(p>1e-6 && p >= gthresh)  /*  Recurse to upslope neighbor only for grid cells 
					with proportion greater than or equal to the threshold and greater than 0 */	
				{
					mxrtrrec(in,jn);
					if(rtr.d[jn][in] < felevg.nodata)
					{
						con = -1;
					}
					else
					{
						ed=felevg.d[jn][in]-felevg.d[j][i]+rtr.d[jn][in];
						if(ed > rtr.d[j][i])rtr.d[j][i]=ed;
					}
				}
				if(fangle.d[jn][in] <= fangle.nodata) con= -1;
            }
			if (con == -1 && ccheck == 1)
			{
				rtr.d[j][i]=felevg.nodata;
			}
		}
	}
} 

/* function to compute minimum rise to ridge recursively.  */
void mnrtrrec(int i, int j)
{
	int in,jn,k,kback,con=0;
	float p,ed;
	if(rtr.d[j][i] == idv)  // idv (initial data value) is used to flag that the cell has not been done
	{
		/*  Make sure the cell is in the domain   */
		if(i!=0 && i!=ny-1 && j!=0 && j!=nx-1 && fangle.d[j][i] > fangle.nodata)
		{
			/*    look in all directions  */
			int ik=0;  // This counts how many upslope grid cells there are
			for(k=1; k<=8; k++)
			{
				in = i + d1[k];
				jn = j + d2[k];
				/* Find the proportion draining from each neighboring cell  */
				kback=k+4;
				if(kback>8)kback=kback-8;
				p=prop(fangle.d[jn][in],kback);
				if(p>1e-6 && p >= gthresh)  /*  Recurse to upslope neighbor only for grid cells 
					with proportion greater than or equal to the threshold and greater than 0.
					1e-6 is used instead of 0 to avoid numerical loops due to angle rounding.*/	
				{
					mnrtrrec(in,jn);
					if(rtr.d[jn][in] < rtr.nodata)
					{
						con = -1;
					}
					else
					{
						ik=ik+1;
						ed=felevg.d[jn][in]-felevg.d[j][i]+rtr.d[jn][in];
						if(ik == 1)
						{
							rtr.d[j][i]=ed;
						}
						else if(ed < rtr.d[j][i])
							rtr.d[j][i]=ed;
					}
				}
				if(fangle.d[jn][in] <= fangle.nodata) con= -1;
            }
			if(ik == 0)
				rtr.d[j][i]=0.;
			if (con == -1 && ccheck == 1)
			{
				rtr.d[j][i]=rtr.nodata;
			}
		}
	}
} 


int risetoridgegrd(char *angfile, char *felfile, char *rtrfile, int method, int contcheck, float thresh)
{
   int err,i,j,filetype;
 
   /* read angles */
   	//New Gridread function:	
	err=gridread(angfile,&fangle,&filetype);

/*    err=gridread(angfile,(void ***)&ang,RPFLTDTYPE,&nx,&ny,&dx,&dy,
             bndbox,&csize,&ndv,&filetype);  */
    if(err != 0)goto ERROR1; 
	nx=fangle.head.nx;
	ny=fangle.head.ny;

   /* read elevations */
	err=gridread(felfile,&felevg,&filetype);
/*    err=gridread(felfile,(void ***)&felevg,RPFLTDTYPE,&nx,&ny,&dx,&dy,
             bndbox,&csize,&ndv,&filetype);  */
    if(err != 0)goto ERROR2;  
	//Check that grid dimensions are the same
	if(nx != felevg.head.nx || ny != felevg.head.ny)
	{
		printf("Grid dimensions are inconsistent\n");
		printf("%d %d %s\n",nx,ny,angfile);
		printf("%d %d %s\n",felevg.head.nx,felevg.head.ny,felfile);
		goto ERROR2;
	}

/*  Allocate memory  */
	err=allocategrid(&rtr,felevg.head,felevg.nodata);
    if(err != 0)goto ERROR2;  

//	rtr = (float **) matalloc(nx, ny, RPFLTDTYPE);

/* Define directions */

d1[1]=0; d1[2]= -1; d1[3]= -1; d1[4]= -1; d1[5]=0; d1[6]=1; d1[7]=1; d1[8]=1;
d2[1]=1; d2[2]=1; d2[3]=0; d2[4]= -1; d2[5]= -1; d2[6]= -1; d2[7]=0; d2[8]=1;

/* Calculate angles for interpolation */                                      
aref[0]= -atan2(felevg.head.dy,felevg.head.dx);
aref[1]=0.;
aref[2]= -aref[0];
aref[3]=0.5*PI;
aref[4]=PI-aref[2];
aref[5]=PI;
aref[6]=PI+aref[2];
aref[7]=1.5*PI;
aref[8]=2.*PI-aref[2];
aref[9]=2.*PI;
idv = -1;
if(rtr.nodata == -1)
  idv= -2;
/*  Initialize rise to ridge to idv, an initial data value that is not the same as the no data value
    to indicate that this has not been done yet  */
    for(i=0; i<ny; i++)
      for(j=0; j<nx; j++)
	  {
         rtr.d[j][i]= idv;  //Use idv to indicate that this has not been done yet.
	  }
	  ccheck=contcheck;  // Edge contamination checking
	  gthresh=thresh;

switch (method)
{
case 0:
/* "lower block" */
for(i=ny/2; i<ny-1; i++)
   {for(j=nx/2; j<nx-1; j++)
      artrrec(i,j);
    for(j=nx/2-1; j>=1; j--)
      artrrec(i,j);
   }
 /* "upper block" */
for(i=ny/2-1; i>=1; i--)
   {for(j=nx/2; j<nx-1; j++)
      artrrec(i,j);
    for(j=nx/2-1; j>=1; j--)
      artrrec(i,j);
   }
break;
case 1:
/* "lower block" */
for(i=ny/2; i<ny-1; i++)
   {for(j=nx/2; j<nx-1; j++)
      mxrtrrec(i,j);
    for(j=nx/2-1; j>=1; j--)
      mxrtrrec(i,j);
   }
 /* "upper block" */
for(i=ny/2-1; i>=1; i--)
   {for(j=nx/2; j<nx-1; j++)
      mxrtrrec(i,j);
    for(j=nx/2-1; j>=1; j--)
      mxrtrrec(i,j);
   }
break;
case 2:
/* "lower block" */
for(i=ny/2; i<ny-1; i++)
   {for(j=nx/2; j<nx-1; j++)
      mnrtrrec(i,j);
    for(j=nx/2-1; j>=1; j--)
      mnrtrrec(i,j);
   }
 /* "upper block" */
for(i=ny/2-1; i>=1; i--)
   {for(j=nx/2; j<nx-1; j++)
      mnrtrrec(i,j);
    for(j=nx/2-1; j>=1; j--)
      mnrtrrec(i,j);
   }
break;
}

//  Parse through the grid again setting unchanged initial data values to no data
    for(i=0; i<ny; i++)
      for(j=0; j<nx; j++)
	  {
         if(rtr.d[j][i]== idv)rtr.d[j][i]=rtr.nodata;  
	  }


	/*  Write out results  */ 
	err= gridwrite(rtrfile,rtr,filetype);
 
/*    err = gridwrite(rtrfile,(void **)rtr,RPFLTDTYPE,nx,ny,dx,dy,
             bndbox,csize,-1.,filetype);  */
	free(rtr.d[0]);
    free(rtr.d); 
ERROR2:
    free(felevg.d[0]);
    free(felevg.d);
ERROR1:
    free(fangle.d[0]);
    free(fangle.d);
	return(err);
} 

                
// End of block of code associated with Rise to Ridge

/*  DropToStream.cpp. 

  Drop to Stream.  
  
  David G Tarboton
  Utah State University   
  
  3/26/07
   */
  

/*  Globals   */
// Not necessary to declare many of these variables here because they have already been declared
// globals in tardemlib.cpp

//int iy,jy,ccheck;
//float aref[10];
//int dd1[8],dd2[8];				

/* Offset pointers d1 and d2 */
//int *d1 = dd1-1;
//int *d2 = dd2-1;
//int nx,ny;
//float idv
float semin;

//float prop(float,int);
void adtsrec(int,int);
void ldtsrec(int,int);
void sdtsrec(int,int);

/*  Global grids  */
//fgrid ang;  //used fangle instead
//fgrid elev;  //used felevg instead
fgrid vs;
sgrid ssrc;

/* function to compute average drop to stream recursively.  */
void adtsrec(int i, int j)
{
	int in,jn,k,con=0;
	float p,sumn;
	if(vs.d[j][i] == idv)  // idv (initial data value) is used to flag that the cell has not been done
	{
		/*  Make sure the cell is in the domain   */
		if(i!=0 && i!=ny-1 && j!=0 && j!=nx-1 && fangle.d[j][i] > fangle.nodata)
		{
			/*    look in all directions  */
			sumn=0.;
			for(k=1; k<=8; k++)
			{
				in = i + d1[k];
				jn = j + d2[k];
				/* Find the proportion draining to a downslope neighboring cell  */
				p=prop(fangle.d[j][i],k);
				if(p>1e-6)  /*  Recurse to downslope neighbor only for grid cells 
					with proportion greater than 0.  Numerically the threshold 1e-6 guards against
					numerical loops due to angle rounding*/	
				{
					adtsrec(in,jn);
					if(vs.d[jn][in] <= felevg.nodata)
					{
						con = -1;
					}
					else
					{
						float ed=felevg.d[j][i]-felevg.d[jn][in];
						if(ed < 0.)ed = 0.;  //  Do not average in uphill steps drops
						sumn=sumn+p*(ed+vs.d[jn][in]);
					}
				}
				if(fangle.d[jn][in] <= fangle.nodata) con= -1;
            }
			vs.d[j][i]=sumn;
			if (con == -1)
			{
				vs.d[j][i]=felevg.nodata;
			}
		}
	}
} 

/* function to compute longest drop to stream recursively.  */
void ldtsrec(int i, int j)
{
	int in,jn,k,con=0;
	float p,sumn;
	if(vs.d[j][i] == idv)  // idv (initial data value) is used to flag that the cell has not been done
	{
		/*  Make sure the cell is in the domain   */
		if(i!=0 && i!=ny-1 && j!=0 && j!=nx-1 && fangle.d[j][i] > fangle.nodata)
		{
			/*    look in all directions  */
			sumn=0.;
			for(k=1; k<=8; k++)
			{
				in = i + d1[k];
				jn = j + d2[k];
				/* Find the proportion draining to a downslope neighboring cell  */
				p=prop(fangle.d[j][i],k);
				if(p>1e-6)  /*  Recurse to downslope neighbor only for grid cells 
					with proportion greater than 0.  Numerically the threshold 1e-6 guards against
					numerical loops due to angle rounding*/	
				{
					ldtsrec(in,jn);
					if(vs.d[jn][in] <= felevg.nodata)
					{
						con = -1;
					}
					else
					{
						float ed=felevg.d[j][i]-felevg.d[jn][in]+vs.d[jn][in];
						if(ed > sumn)sumn=ed;  //  Keep track of longest drop
					}
				}
				if(fangle.d[jn][in] <= fangle.nodata) con= -1;
            }
			vs.d[j][i]=sumn;
			if (con == -1)
			{
				vs.d[j][i]=felevg.nodata;
			}
		}
	}
} 

/* function to compute shortest drop to stream recursively.  */
void sdtsrec(int i, int j)
{
	int in,jn,k,con=0;
	float p,sumn;
	if(vs.d[j][i] == idv)  // idv (initial data value) is used to flag that the cell has not been done
	{
		/*  Make sure the cell is in the domain   */
		if(i!=0 && i!=ny-1 && j!=0 && j!=nx-1 && fangle.d[j][i] > fangle.nodata)
		{
			/*    look in all directions  */
			sumn=semin;  //  Min elevation on streams
			for(k=1; k<=8; k++)
			{
				in = i + d1[k];
				jn = j + d2[k];
				/* Find the proportion draining to a downslope neighboring cell  */
				p=prop(fangle.d[j][i],k);
				if(p>1e-6)  /*  Recurse to downslope neighbor only for grid cells 
					with proportion greater than 0.  Numerically the threshold 1e-6 guards against
					numerical loops due to angle rounding*/	
				{
					sdtsrec(in,jn);
					if(vs.d[jn][in] <= felevg.nodata)
					{
						con = -1;
					}
					else
					{
						float ed=felevg.d[jn][in]-vs.d[jn][in];
						if(ed > sumn)sumn=ed;  //  Keep track of elevation of highest point on stream that this flows to
					}
				}
				if(fangle.d[jn][in] <= fangle.nodata) con= -1;
            }
			vs.d[j][i]=felevg.d[j][i]-sumn;
			if (con == -1)
			{
				vs.d[j][i]=felevg.nodata;
			}
		}
	}
} 

int droptostreamgrd(char *angfile, char *felfile, char *srcfile, char *vsfile, int method)
{
   int err,i,j,filetype,ifirst;
 
   /* read angles */
	err=gridread(angfile,&fangle,&filetype);
    if(err != 0)goto ERROR1; 
	nx=fangle.head.nx;
	ny=fangle.head.ny;

   /* read elevations */
	err=gridread(felfile,&felevg,&filetype);
    if(err != 0)goto ERROR2;  
	//Check that grid dimensions are the same
	if(nx != felevg.head.nx || ny != felevg.head.ny)
	{
		printf("Grid dimensions are inconsistent\n");
		printf("%d %d %s\n",nx,ny,angfile);
		printf("%d %d %s\n",felevg.head.nx,felevg.head.ny,felfile);
		goto ERROR2;
	}
	/* read Stream Raster Grid */
	err=gridread(srcfile,&ssrc,&filetype);
    if(err != 0)goto ERROR3;  
	//Check that grid dimensions are the same
	if(nx != ssrc.head.nx || ny != ssrc.head.ny)
	{
		printf("Grid dimensions are inconsistent\n");
		printf("%d %d %s\n",nx,ny,angfile);
		printf("%d %d %s\n",ssrc.head.nx,ssrc.head.ny,srcfile);
		goto ERROR3;
	}


/*  Allocate memory  */
	err=allocategrid(&vs,felevg.head,felevg.nodata);
    if(err != 0)goto ERROR4;  

/* Define directions */

d1[1]=0; d1[2]= -1; d1[3]= -1; d1[4]= -1; d1[5]=0; d1[6]=1; d1[7]=1; d1[8]=1;
d2[1]=1; d2[2]=1; d2[3]=0; d2[4]= -1; d2[5]= -1; d2[6]= -1; d2[7]=0; d2[8]=1;

/* Calculate angles for interpolation */                                      
aref[0]= -atan2(felevg.head.dy,felevg.head.dx);
aref[1]=0.;
aref[2]= -aref[0];
aref[3]=0.5*PI;
aref[4]=PI-aref[2];
aref[5]=PI;
aref[6]=PI+aref[2];
aref[7]=1.5*PI;
aref[8]=2.*PI-aref[2];
aref[9]=2.*PI;
idv = -1;
if(vs.nodata == -1)
  idv= -2;
/*  Initialize rise to ridge to idv, an initial data value that is not the same as the no data value
    to indicate that this has not been done yet.  On same parse through initialize vs values on streams
    and find lowest elevation point on stream  */
    ifirst=1;
    for(i=0; i<ny; i++)
      for(j=0; j<nx; j++)
	  {
         vs.d[j][i]= idv;  // Use idv to indicate that this has not been done yet.
		 if(ssrc.d[j][i] >=1)
		 {
			 vs.d[j][i]=0.;  // Initialize vs values on stream to 0.
			 if (ifirst==1 && felevg.d[j][i] > felevg.nodata)
			 {
				 semin=felevg.d[j][i];
				 ifirst=0;
			 }
			 else if (felevg.d[j][i] > felevg.nodata && felevg.d[j][i] < semin)
			 {
				 semin=felevg.d[j][i];
			 }
		 }
	  }

switch (method)
{
case 0:
/* "lower block" */
for(i=ny/2; i<ny-1; i++)
   {for(j=nx/2; j<nx-1; j++)
      adtsrec(i,j);
    for(j=nx/2-1; j>=1; j--)
      adtsrec(i,j);
   }
 /* "upper block" */
for(i=ny/2-1; i>=1; i--)
   {for(j=nx/2; j<nx-1; j++)
      adtsrec(i,j);
    for(j=nx/2-1; j>=1; j--)
      adtsrec(i,j);
   }
break;
case 1:
	/* "lower block" */
for(i=ny/2; i<ny-1; i++)
   {for(j=nx/2; j<nx-1; j++)
      ldtsrec(i,j);
    for(j=nx/2-1; j>=1; j--)
      ldtsrec(i,j);
   }
 /* "upper block" */
for(i=ny/2-1; i>=1; i--)
   {for(j=nx/2; j<nx-1; j++)
      ldtsrec(i,j);
    for(j=nx/2-1; j>=1; j--)
      ldtsrec(i,j);
   }
break;
case 2:
	/* "lower block" */
for(i=ny/2; i<ny-1; i++)
   {for(j=nx/2; j<nx-1; j++)
      sdtsrec(i,j);
    for(j=nx/2-1; j>=1; j--)
      sdtsrec(i,j);
   }
 /* "upper block" */
for(i=ny/2-1; i>=1; i--)
   {for(j=nx/2; j<nx-1; j++)
      sdtsrec(i,j);
    for(j=nx/2-1; j>=1; j--)
      sdtsrec(i,j);
   }
break;
}
//  Parse through the grid again setting unchanged initial data values to no data
    for(i=0; i<ny; i++)
      for(j=0; j<nx; j++)
	  {
         if(vs.d[j][i]== idv)vs.d[j][i]=vs.nodata;  
	  }


	/*  Write out results  */ 
	err= gridwrite(vsfile,vs,filetype);
 
/*    err = gridwrite(rtrfile,(void **)rtr,RPFLTDTYPE,nx,ny,dx,dy,
             bndbox,csize,-1.,filetype);  */
	free(vs.d[0]);
    free(vs.d); 
ERROR4:
    free(ssrc.d[0]);
    free(ssrc.d);
ERROR3:
    free(felevg.d[0]);
    free(felevg.d);
ERROR2:
    free(fangle.d[0]);
    free(fangle.d);
ERROR1:	
	return(err);
} 

// End of block of code associated with Drop to Stream

/*  AvalancheRunout.cpp. 

  Avalanche Runout.  
  
  David G Tarboton
  Utah State University   
  
  4/9/07
  See NewTauDEMfunctions.doc for a fuller development of the algorithm.

  Edge contamination checking is not implemented in this function because we presume that interest is in
  computing runout zones only from avalanches in the source sites grid which has to be the same
  dimensions as the other grids.
   */
  

/*  Globals   */
// Not necessary to declare many of these variables here because they have already been declared
// globals in tardemlib.cpp

// int iy,jy;
// float aref[10],dist[9];
//int dd1[8],dd2[8];				

/* Offset pointers d1 and d2 */
//int *d1 = dd1-1;
//int *d2 = dd2-1;
//int nx,ny;
//float idv;
//float gthresh;
float galpha;
int gpath;
float *xcoord;
float *ycoord;

//float prop(float,int);
void avalancherunoutrec(int,int);

/*  Global grids  */
//fgrid ang;  //used fangle instead
//fgrid elev;  //used felevg instead
fgrid rz;
fgrid dm;
sgrid ass;
sgrid im;
sgrid jm;

/* Function to compute avalanche runout recursively.  */
void avalancherunoutrec(int i, int j)
{
	int in,jn,k,kback;
	float p;
	if(rz.d[j][i] == idv)  // idv (initial data value) is used to flag that the cell has not been done
	{
		/*  Make sure the cell is in the domain   */
		if(i!=0 && i!=ny-1 && j!=0 && j!=nx-1 && fangle.d[j][i] > fangle.nodata)
		{
			rz.d[j][i]=rz.nodata;  //Initialize to no data
			if(ass.d[j][i] > 0)
			{
				rz.d[j][i]=galpha;
				im.d[j][i]=i;
				jm.d[j][i]=j;
				dm.d[j][i]=0;
			}
			// The logic above assures that rz is set for non edge cells the first time this is called
			// so if there is circularity in the flow directions, which can result from errors or numerical rounding
			// the procedure will terminate 
			/*    look in all directions  */
			for(k=1; k<=8; k++)
			{
				in = i + d1[k];
				jn = j + d2[k];
				/* Find the proportion draining from each neighboring cell  */
				kback=k+4;
				if(kback>8)kback=kback-8;
				p=prop(fangle.d[jn][in],kback);
				if(p>4e-6 && p >= gthresh)  /*  Recurse to upslope neighbor only for grid cells 
					with proportion greater than or equal to the threshold and greater than 4e-6 to avoid loops 
					due to direction rounding */	
				{
					avalancherunoutrec(in,jn);
					if (rz.d[jn][in] >= galpha)  //  The upslope neighbor is in the runout zone
					{
						float d;
						if(gpath==1)
							d=dm.d[jn][in]+dist[k];
						else
						{
							float dx=xcoord[j]-xcoord[jm.d[jn][in]];
							float dy=ycoord[i]-ycoord[im.d[jn][in]];
							d=sqrt(dx*dx+dy*dy);
						}
						float zd=felevg.d[jm.d[jn][in]][im.d[jn][in]]-felevg.d[j][i];
						float beta=atan(zd/d)*180/PI;
						if(beta >= galpha && beta > rz.d[j][i])
						{
							rz.d[j][i]=beta;
							im.d[j][i]=im.d[jn][in];
							jm.d[j][i]=jm.d[jn][in];
							dm.d[j][i]=d;
						}
					}
				}
            }
//			i=i;   This was for debugging
		}
	}
} 


int avalancherunoutgrd(char *angfile, char *felfile, char *assfile, char *rzfile, char *dmfile, float thresh, float alpha, int path)
{
   int err,i,j,filetype;
 
   /* read angles */
 	err=gridread(angfile,&fangle,&filetype);
    if(err != 0)goto ERROR1; 
	nx=fangle.head.nx;
	ny=fangle.head.ny;

   /* read elevations */
	err=gridread(felfile,&felevg,&filetype);
    if(err != 0)goto ERROR2;  
	//Check that grid dimensions are the same
	if(nx != felevg.head.nx || ny != felevg.head.ny)
	{
		printf("Grid dimensions are inconsistent\n");
		printf("%d %d %s\n",nx,ny,angfile);
		printf("%d %d %s\n",felevg.head.nx,felevg.head.ny,felfile);
		goto ERROR3;
	}

	/* read avalanche source sites */
	err=gridread(assfile,&ass,&filetype);
    if(err != 0)goto ERROR3;  
	//Check that grid dimensions are the same
	if(nx != ass.head.nx || ny != ass.head.ny)
	{
		printf("Grid dimensions are inconsistent\n");
		printf("%d %d %s\n",nx,ny,assfile);
		printf("%d %d %s\n",ass.head.nx,ass.head.ny,felfile);
		goto ERROR4;
	}

/*  Allocate memory  */
	err=allocategrid(&rz,felevg.head,-2.);  //  Set no data value to -2.
    if(err != 0)goto ERROR4;  

/*  Allocate memory for row index*/
	err=allocategrid(&im,felevg.head,-2);  //  Set no data value to -2.
    if(err != 0)goto ERROR5;  

/*  Allocate memory  */
	err=allocategrid(&jm,felevg.head,-2);  //  Set no data value to -2.
    if(err != 0)goto ERROR6;  

/*  Allocate memory  */
	err=allocategrid(&dm,felevg.head,-2);  //  Set no data value to -2.
    if(err != 0)goto ERROR7;  


/* Define directions */

d1[1]=0; d1[2]= -1; d1[3]= -1; d1[4]= -1; d1[5]=0; d1[6]=1; d1[7]=1; d1[8]=1;
d2[1]=1; d2[2]=1; d2[3]=0; d2[4]= -1; d2[5]= -1; d2[6]= -1; d2[7]=0; d2[8]=1;

/* Calculate angles for interpolation */                                      
aref[0]= -atan2(felevg.head.dy,felevg.head.dx);
aref[1]=0.;
aref[2]= -aref[0];
aref[3]=0.5*PI;
aref[4]=PI-aref[2];
aref[5]=PI;
aref[6]=PI+aref[2];
aref[7]=1.5*PI;
aref[8]=2.*PI-aref[2];
aref[9]=2.*PI;
//  Calculate distances in each direction
int k;
for(k=1; k<=8; k++)
{
	dist[k]=sqrt(pow(fangle.head.dx*d2[k],2)+pow(fangle.head.dy*d1[k],2));
}

// Define coordinate tables
xcoord=new float[nx];
ycoord=new float[ny];
for(i=0; i<ny; i++)
	ycoord[i]=fangle.head.bndbox[3]-i*fangle.head.dy;
for(j=0; j<nx; j++)
	xcoord[j]=fangle.head.bndbox[0]+j*fangle.head.dx;

idv = -1;
/*  Initialize runout zone to idv, an initial data value that is not the same as the no data value
    to indicate that this has not been done yet  */
    for(i=0; i<ny; i++)
      for(j=0; j<nx; j++)
	  {
         rz.d[j][i]= idv;  //Use idv to indicate that this has not been done yet.
	  }
	  gthresh=thresh;
	  galpha=alpha;
	  gpath=path;
/* "lower block" */
for(i=ny/2; i<ny-1; i++)
   {for(j=nx/2; j<nx-1; j++)
      avalancherunoutrec(i,j);
    for(j=nx/2-1; j>=1; j--)
      avalancherunoutrec(i,j);
   }
 /* "upper block" */
for(i=ny/2-1; i>=1; i--)
   {for(j=nx/2; j<nx-1; j++)
      avalancherunoutrec(i,j);
    for(j=nx/2-1; j>=1; j--)
      avalancherunoutrec(i,j);
   }
//  Parse through the grid again setting unchanged initial data values to no data
    for(i=0; i<ny; i++)
      for(j=0; j<nx; j++)
	  {
         if(rz.d[j][i]== idv)rz.d[j][i]=rz.nodata;  
	  }


	/*  Write out results  */ 
	err= gridwrite(rzfile,rz,filetype);
	err= gridwrite(dmfile,dm,filetype);

	free(dm.d[0]);
    free(dm.d); 
ERROR7:
	free(jm.d[0]);
    free(jm.d); 
ERROR6:
	free(im.d[0]);
    free(im.d); 
ERROR5:
	free(rz.d[0]);
    free(rz.d); 
ERROR4:
    free(ass.d[0]);
    free(ass.d);
ERROR3:
    free(felevg.d[0]);
    free(felevg.d);
ERROR2:
    free(fangle.d[0]);
    free(fangle.d);
ERROR1:
	return(err);
} 

// End of block of code associated with Avalanche Runout
